"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteFile = exports.fileReaderUsers = exports.fileReader = void 0;
const fs_1 = __importDefault(require("fs"));
const promises_1 = require("fs/promises");
const csv_parser_1 = __importDefault(require("csv-parser"));
const fileReader = (filePath) => {
    return new Promise((resolve, reject) => {
        const data = [];
        fs_1.default.createReadStream(filePath)
            .pipe((0, csv_parser_1.default)())
            .on('data', (row) => {
            row.regular_spots = row.regular_spots.split(',').map((spot) => spot.trim());
            row.designated_spots = row.designated_spots.split(',').map((spot) => spot.trim());
            row.electric_spots = row.electric_spots.split(',').map((spot) => spot.trim());
            data.push(row);
        })
            .on('end', () => {
            resolve(data);
        })
            .on('error', (err) => {
            reject(err);
        });
    });
};
exports.fileReader = fileReader;
const fileReaderUsers = (filePath) => {
    return new Promise((resolve, reject) => {
        const data = [];
        fs_1.default.createReadStream(filePath)
            .pipe((0, csv_parser_1.default)())
            .on('data', (row) => {
            // console.log(row.dateOfBirth.replace(/\//g, "-"))
            const dob = new Date();
            // console.log(dob,'dob')
            dob.setUTCHours(12);
            dob.setUTCMinutes(0);
            dob.setUTCSeconds(0);
            row.allowedParkingLots = row.allowedParkingLots.split(',').map((spot) => spot.trim()).filter((spot) => spot !== '');
            row.designatedSpots = row.designatedSpots.split(',').map((spot) => spot.trim()).filter((spot) => spot !== '');
            // row.permanentDesignated = row.permanentDesignated.split(',').map((spot: any) => spot.trim()).filter((spot: string) => spot !== '')
            row.dateOfBirth = dob.toISOString();
            row.electricCharging = row.electricCharging == 'FALSE' ? false : true;
            row.accessible = row.accessible == 'FALSE' ? false : true;
            data.push(row);
        })
            .on('end', () => {
            resolve(data);
        })
            .on('error', (err) => {
            reject(err);
        });
    });
};
exports.fileReaderUsers = fileReaderUsers;
const deleteFile = (filePath) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        return yield (0, promises_1.unlink)(filePath);
    }
    catch (err) {
        return err;
    }
});
exports.deleteFile = deleteFile;
