"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateParkingLotReportCsv = exports.generateGuestUserReportCsv = exports.generateUserReportCsv = exports.generateComplaintsReportCsv = exports.doubleParkingNotificationTextGenerator = exports.registrationSmsGenerator = exports.generateParkingCodes = exports.generateLevelCode = exports.generateHashPassword = exports.generateTemporaryPinCode = exports.generateTemporaryPassword = void 0;
const bcrypt_1 = __importDefault(require("bcrypt"));
const json2csv_1 = require("json2csv");
const path_1 = __importDefault(require("path"));
const fs_1 = require("fs");
const generateTemporaryPassword = () => {
    const uppercaseLetters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZ';
    const lowercaseLetters = 'abcdefghijklmnopqrstuvwxyz';
    const digits = '0123456789';
    let result = '';
    let typesCount = 0; // Counter to track the number of character types used in the password
    // Function to check if the password contains characters from at least three different types
    const hasThreeTypes = () => {
        return (/[A-Z]/.test(result) && // Check for uppercase letters
            /[a-z]/.test(result) && // Check for lowercase letters
            /\d/.test(result) // Check for digits
        );
    };
    // Generate password until it contains characters from at least three different types
    while (!hasThreeTypes()) {
        result = '';
        typesCount = 0;
        for (let i = 0; i < 10; i++) {
            const randomIndex = Math.floor(Math.random() * 3); // Randomly select a character type
            switch (randomIndex) {
                case 0:
                    result += uppercaseLetters.charAt(Math.floor(Math.random() * uppercaseLetters.length));
                    typesCount++;
                    break;
                case 1:
                    result += lowercaseLetters.charAt(Math.floor(Math.random() * lowercaseLetters.length));
                    typesCount++;
                    break;
                case 2:
                    result += digits.charAt(Math.floor(Math.random() * digits.length));
                    typesCount++;
                    break;
            }
        }
    }
    const date = new Date();
    date.setMinutes(date.getMinutes() + 30);
    return { expiry: date, password: result };
};
exports.generateTemporaryPassword = generateTemporaryPassword;
const generateTemporaryPinCode = () => {
    let result = '';
    for (let i = 0; i < 5; i++) {
        result += Math.floor(Math.random() * 10);
    }
    const date = new Date();
    date.setMinutes(date.getMinutes() + 30);
    return { expiry: date, password: result };
};
exports.generateTemporaryPinCode = generateTemporaryPinCode;
const generateHashPassword = (password) => __awaiter(void 0, void 0, void 0, function* () {
    const salt = yield bcrypt_1.default.genSalt(10);
    return yield bcrypt_1.default.hash(password, salt);
});
exports.generateHashPassword = generateHashPassword;
const generateLevelCode = (floorNumber) => {
    return String.fromCharCode(floorNumber + 64);
};
exports.generateLevelCode = generateLevelCode;
const generateParkingCodes = (floorNumber, numSpotsPerFloor, startFrom) => {
    const floorCodes = [];
    for (let spot = startFrom ? startFrom : 1; spot <= numSpotsPerFloor; spot++) {
        const floorCode = String.fromCharCode(floorNumber + 64);
        const spotCode = spot.toString().padStart(2, '0');
        const parkingCode = `${floorCode}${spotCode}`;
        floorCodes.push(parkingCode);
    }
    return floorCodes;
};
exports.generateParkingCodes = generateParkingCodes;
const registrationSmsGenerator = (firstName, generateRandom, generatedPassword, password) => {
    return `
${firstName}, ברוך הבא לספוטפארק, אפליקציית ניהול החניונים החדשה של חברת החשמל.\n
הוראות ההתקנה והשימוש הועברו אליך במצגת במייל הארגוני.\n
יש להוריד את האפליקציה החדשה מהחנות , מצ"ב קישורים:\n
קישור לבעלי אייפון: https://apps.apple.com/us/app/spotpark/id6475667550 \n
קישור לבעלי אנדרואיד: https://play.google.com/store/apps/details?id=com.spotpark.ysb&pli=1 \n
לאחר הורדת האפליקציה החדשה מהחנות יש למלא את  שם המשתמש שלך (מספר הטלפון הנייד שלך) והסיסמה הזמנית שלך היא: ${generateRandom ? generatedPassword.password : password}.\n
לאחר מכן, יש לשנות סיסמא כפי שמוסבר במצגת שקיבלת.\n
בכל בעיה שאתה נתקל אתה מוזמן לפנות לאדמיניסטרטור שלך כפי שמופיע במצגת שקיבלת.\n
לאחר ההתחברות לאפליקציה החדשה יש להסיר את האפליקציה הישנה שיצאה משימוש.\n
שים לב כי מספר החניה שלך נשאר ללא שינוי.\n
בהצלחה לכולנו!`;
};
exports.registrationSmsGenerator = registrationSmsGenerator;
const doubleParkingNotificationTextGenerator = (registration, parkingLot, name, phoneNumber) => {
    return `רכב ${registration} חוסם אותך בחניה כפולה בחניון ${parkingLot}. מספר הטלפון שלו הוא ${phoneNumber === null || phoneNumber === void 0 ? void 0 : phoneNumber.number}. שם הנהג: ${name}`;
};
exports.doubleParkingNotificationTextGenerator = doubleParkingNotificationTextGenerator;
const generateComplaintsReportCsv = (complaints) => __awaiter(void 0, void 0, void 0, function* () {
    const fields = [
        // { label: 'מזהה תלונה', value: '_id' },
        { label: 'שם חניון', value: 'parkingLot.name' },
        // { label: 'זהות המשתמש', value: 'complaintBy._id' },
        { label: 'שם מלא', value: 'complaintBy.firstName' },
        { label: 'טלפון', value: 'complaintBy.phoneNumber.number' },
        { label: 'מייל', value: 'complaintBy.email' },
        { label: 'תיאור הבעיה', value: 'details' },
        { label: 'סטטוס', value: 'status' },
        { label: 'נוצר ב', value: 'createdAt' },
        // { label: 'מעודכן ב', value: 'updatedAt' },
    ];
    const parser = new json2csv_1.Parser({ fields, });
    const csv = parser.parse(complaints);
    const directory = path_1.default.join(__dirname, '../../../public/reports/complaints');
    yield fs_1.promises.mkdir(directory, { recursive: true });
    const filePath = path_1.default.join(directory, `complaint-report-${Date.now()}${Date.now()}.csv`);
    const createdPath = 'public/' + (filePath === null || filePath === void 0 ? void 0 : filePath.split('public/')[1]);
    const file = yield fs_1.promises.writeFile(filePath, csv, 'utf-8');
    return {
        file,
        filePath: createdPath,
    };
});
exports.generateComplaintsReportCsv = generateComplaintsReportCsv;
const generateUserReportCsv = (users) => __awaiter(void 0, void 0, void 0, function* () {
    const newUsers = yield Promise.all(users.map((user) => __awaiter(void 0, void 0, void 0, function* () {
        var _a, _b, _c, _d, _e;
        const object = Object.assign(Object.assign({}, user === null || user === void 0 ? void 0 : user.toObject()), { isSpotFrozen: '', levels: '' });
        if (((_a = user === null || user === void 0 ? void 0 : user.parkingLots) === null || _a === void 0 ? void 0 : _a.length) > 0) {
            const parkingLots = yield Promise.all((_b = user === null || user === void 0 ? void 0 : user.parkingLots) === null || _b === void 0 ? void 0 : _b.map((parkingLot) => parkingLot === null || parkingLot === void 0 ? void 0 : parkingLot.name));
            object.parkingLots = parkingLots.join(', ');
        }
        else
            object.parkingLots = '';
        if (((_c = user === null || user === void 0 ? void 0 : user.designatedParkingSpot) === null || _c === void 0 ? void 0 : _c.length) > 0) {
            const frozenArray = [];
            const levelsArray = [];
            const designatedParkingSpot = yield Promise.all((_d = user === null || user === void 0 ? void 0 : user.designatedParkingSpot) === null || _d === void 0 ? void 0 : _d.map((spot) => {
                frozenArray.push(spot.isPermanent ? 'כן' : 'לא');
                // frozenArray.push(spot.isPermanent ? 'true' : 'false')
                levelsArray.push(spot.level.name);
                return spot === null || spot === void 0 ? void 0 : spot.parkingCode;
            }));
            object.isSpotFrozen = frozenArray.join(', ');
            object.designatedParkingSpot = designatedParkingSpot.join(', ');
            object.levels = levelsArray.join(', ');
        }
        else
            object.designatedParkingSpot = '';
        object.deviceToken = (user === null || user === void 0 ? void 0 : user.deviceToken) ? true : false;
        object.electricCharging = (user === null || user === void 0 ? void 0 : user.electricCharging) ? 'כן' : 'לא';
        object.disabled = (user === null || user === void 0 ? void 0 : user.disabled) ? 'כן' : 'לא';
        object.phoneNumber = (_e = user === null || user === void 0 ? void 0 : user.phoneNumber) === null || _e === void 0 ? void 0 : _e.number;
        return object;
    })));
    const fields = [
        // { label: 'זהות המשתמש', value: '_id' },
        { label: 'שם מלא', value: 'firstName' },
        { label: 'טלפון', value: 'phoneNumber' },
        { label: 'חניונים', value: 'parkingLots' },
        { label: 'אזורים', value: 'levels' },
        { label: 'מספר חניה', value: 'designatedParkingSpot' },
        { label: 'קפואה?', value: 'isSpotFrozen' },
        { label: 'תאריך התחברות ראשון', value: 'firstLoginDate' },
        { label: 'חשמלי', value: 'electricCharging' },
        { label: 'נגיש', value: 'disabled' },
        { label: 'אישר התראות', value: 'deviceToken' },
        // { label: 'נוצר ב', value: 'createdAt' },
        // { label: 'מעודכן ב', value: 'updatedAt' },
    ];
    const parser = new json2csv_1.Parser({ fields, });
    const csv = parser.parse(newUsers);
    const directory = path_1.default.join(__dirname, '../../../public/reports/users');
    yield fs_1.promises.mkdir(directory, { recursive: true });
    const filePath = path_1.default.join(directory, `users-report-${Date.now()}${Date.now()}.csv`);
    const createdPath = 'public/' + (filePath === null || filePath === void 0 ? void 0 : filePath.split('public/')[1]);
    const file = yield fs_1.promises.writeFile(filePath, csv, 'utf-8');
    return {
        file: file,
        filePath: createdPath,
    };
});
exports.generateUserReportCsv = generateUserReportCsv;
const generateGuestUserReportCsv = (users) => __awaiter(void 0, void 0, void 0, function* () {
    const fields = [
        { label: 'שם מלא', value: 'firstName' },
        { label: 'מספר טלפון', value: 'phoneNumber.number' },
        { label: 'רישום רכב', value: 'vehicles' },
        { label: 'נגיש', value: 'disabled' },
        { label: 'מקום חניה שמור', value: 'designatedParkingSpot' },
        { label: 'תאריך הגעה', value: 'reservations' }
    ];
    const parser = new json2csv_1.Parser({ fields, });
    const csv = parser.parse(users);
    const directory = path_1.default.join(__dirname, '../../../public/reports/guest-users');
    yield fs_1.promises.mkdir(directory, { recursive: true });
    const filePath = path_1.default.join(directory, `guest-users-report-${Date.now()}${Date.now()}.csv`);
    const createdPath = 'public/' + (filePath === null || filePath === void 0 ? void 0 : filePath.split('public/')[1]);
    const file = yield fs_1.promises.writeFile(filePath, csv, 'utf-8');
    return {
        file,
        filePath: createdPath,
    };
});
exports.generateGuestUserReportCsv = generateGuestUserReportCsv;
const generateParkingLotReportCsv = (parkingSpots) => __awaiter(void 0, void 0, void 0, function* () {
    const fields = [
        { label: 'שם חניון', value: 'parkingLot.name' },
        { label: 'שם אזור', value: 'level.name' },
        { label: 'מספר חניה', value: 'parkingCode' },
        { label: 'סוג חניה', value: 'spotType' },
        { label: 'קפואה?', value: 'isPermanent' },
        { label: 'סטטוס', value: 'status' },
        { label: 'שמור למשתמש', value: 'designatedTo.firstName' },
        { label: 'תפוס על ידי', value: 'isOccupied' },
    ];
    const parser = new json2csv_1.Parser({ fields, });
    const csv = parser.parse(parkingSpots);
    const directory = path_1.default.join(__dirname, '../../../public/reports/parking-lot');
    yield fs_1.promises.mkdir(directory, { recursive: true });
    const filePath = path_1.default.join(directory, `parking-lot-report-${Date.now()}${Date.now()}.csv`);
    const createdPath = 'public/' + (filePath === null || filePath === void 0 ? void 0 : filePath.split('public/')[1]);
    const file = yield fs_1.promises.writeFile(filePath, csv, 'utf-8');
    return {
        file,
        filePath: createdPath,
    };
});
exports.generateParkingLotReportCsv = generateParkingLotReportCsv;
