"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.performDesignatedSpotAction = exports.customSortRegistrationDescending = exports.customSortRegistrationAscending = exports.findAvailableSpots = exports.customSortStatusDescending = exports.customSortStatusAscending = exports.customSortPhoneNumberDesignatedToDescending = exports.customSortPhoneNumberDesignatedToAscending = exports.customSortPhoneNumberDescending = exports.customSortPhoneNumberAscending = exports.customSortNameReservedByDescending = exports.customSortNameReservedByAscending = exports.customSortNameDescending = exports.customSortNameAscending = exports.customSortDescending = exports.customSort = void 0;
const dates_1 = require("./dates");
const mongodb_1 = require("mongodb");
const parking_spot_1 = __importDefault(require("../models/parking-spot"));
const reservation_1 = __importDefault(require("../models/reservation"));
const absence_1 = __importDefault(require("../models/absence"));
const user_1 = __importDefault(require("../models/user"));
const log_1 = __importDefault(require("../models/log"));
const log_types_1 = __importDefault(require("../services/constants/log-types"));
const notifications_1 = require("../middlewares/notifications");
const notification_1 = require("../services/constants/notification");
const controllers_1 = require("../v2.9.3/reservations/controllers");
// Assuming availableSpots is an array of objects with a property named 'parkingCode'
// Define a custom sorting function
const customSort = (a, b) => {
    // Extract the numeric parts of the parking codes
    const numA = parseInt(a.parkingCode.substring(1));
    const numB = parseInt(b.parkingCode.substring(1));
    // Extract the alphabetical parts of the parking codes
    const alphaA = a.parkingCode.substring(0, 1);
    const alphaB = b.parkingCode.substring(0, 1);
    // First, sort alphabetically
    if (alphaA < alphaB)
        return -1;
    if (alphaA > alphaB)
        return 1;
    // If the alphabetical parts are equal, sort numerically
    if (numA < numB)
        return -1;
    if (numA > numB)
        return 1;
    // If both parts are equal, retain the original order
    return 0;
};
exports.customSort = customSort;
const customSortDescending = (a, b) => {
    // Extract the numeric parts of the parking codes
    const numA = parseInt(a.parkingCode.substring(1));
    const numB = parseInt(b.parkingCode.substring(1));
    // Extract the alphabetical parts of the parking codes
    const alphaA = a.parkingCode.substring(0, 1);
    const alphaB = b.parkingCode.substring(0, 1);
    // First, sort alphabetically in descending order
    if (alphaA > alphaB)
        return -1;
    if (alphaA < alphaB)
        return 1;
    // If the alphabetical parts are equal, sort numerically in descending order
    if (numA > numB)
        return -1;
    if (numA < numB)
        return 1;
    // If both parts are equal, retain the original order
    return 0;
};
exports.customSortDescending = customSortDescending;
const customSortNameAscending = (a, b) => {
    // Handle null values in designatedTo
    if (a.designatedTo === null && b.designatedTo === null) {
        return 0; // If both are null, retain the original order
    }
    else if (a.designatedTo === null) {
        return -1; // If only a is null, place it before b
    }
    else if (b.designatedTo === null) {
        return 1; // If only b is null, place it before a
    }
    else {
        // Compare by designatedTo.firstName
        return a.designatedTo.firstName.localeCompare(b.designatedTo.firstName);
    }
};
exports.customSortNameAscending = customSortNameAscending;
const customSortNameDescending = (a, b) => {
    // Handle null values in designatedTo
    if (a.designatedTo === null && b.designatedTo === null) {
        return 0; // If both are null, retain the original order
    }
    else if (a.designatedTo === null) {
        return 1; // If only a is null, place it after b
    }
    else if (b.designatedTo === null) {
        return -1; // If only b is null, place it after a
    }
    else {
        // Compare by designatedTo.firstName in reverse order
        return b.designatedTo.firstName.localeCompare(a.designatedTo.firstName);
    }
};
exports.customSortNameDescending = customSortNameDescending;
const customSortNameReservedByAscending = (a, b) => {
    // Handle null values in designatedTo
    if (a.reservedBy === null && b.reservedBy === null) {
        return 0; // If both are null, retain the original order
    }
    else if (a.reservedBy === null) {
        return -1; // If only a is null, place it before b
    }
    else if (b.reservedBy === null) {
        return 1; // If only b is null, place it before a
    }
    else {
        // Compare by designatedTo.firstName
        return a.reservedBy.firstName.localeCompare(b.reservedBy.firstName);
    }
};
exports.customSortNameReservedByAscending = customSortNameReservedByAscending;
const customSortNameReservedByDescending = (a, b) => {
    // Handle null values in designatedTo
    if (a.reservedBy === null && b.reservedBy === null) {
        return 0; // If both are null, retain the original order
    }
    else if (a.reservedBy === null) {
        return 1; // If only a is null, place it after b
    }
    else if (b.reservedBy === null) {
        return -1; // If only b is null, place it after a
    }
    else {
        // Compare by designatedTo.firstName in reverse order
        return b.reservedBy.firstName.localeCompare(a.reservedBy.firstName);
    }
};
exports.customSortNameReservedByDescending = customSortNameReservedByDescending;
const customSortPhoneNumberAscending = (a, b) => {
    if (a.reservedBy === null)
        return 1;
    if (b.reservedBy === null)
        return -1;
    if (a.reservedBy.phoneNumber.number < b.reservedBy.phoneNumber.number)
        return -1;
    if (a.reservedBy.phoneNumber.number > b.reservedBy.phoneNumber.number)
        return 1;
    return 0;
};
exports.customSortPhoneNumberAscending = customSortPhoneNumberAscending;
const customSortPhoneNumberDescending = (a, b) => {
    if (a.reservedBy === null)
        return 1;
    if (b.reservedBy === null)
        return -1;
    if (a.reservedBy.phoneNumber.number < b.reservedBy.phoneNumber.number)
        return 1;
    if (a.reservedBy.phoneNumber.number > b.reservedBy.phoneNumber.number)
        return -1;
    return 0;
};
exports.customSortPhoneNumberDescending = customSortPhoneNumberDescending;
const customSortPhoneNumberDesignatedToAscending = (a, b) => {
    if (a.designatedTo === null)
        return 1;
    if (b.designatedTo === null)
        return -1;
    if (a.designatedTo.phoneNumber.number < b.designatedTo.phoneNumber.number)
        return -1;
    if (a.designatedTo.phoneNumber.number > b.designatedTo.phoneNumber.number)
        return 1;
    return 0;
};
exports.customSortPhoneNumberDesignatedToAscending = customSortPhoneNumberDesignatedToAscending;
const customSortPhoneNumberDesignatedToDescending = (a, b) => {
    if (a.designatedTo === null && b.designatedTo === null)
        return 0;
    if (a.designatedTo === null)
        return -1;
    if (b.designatedTo === null)
        return 1;
    if (a.designatedTo.phoneNumber.number < b.designatedTo.phoneNumber.number)
        return 1;
    if (a.designatedTo.phoneNumber.number > b.designatedTo.phoneNumber.number)
        return -1;
    return 0;
};
exports.customSortPhoneNumberDesignatedToDescending = customSortPhoneNumberDesignatedToDescending;
// Custom sort function for sorting in ascending order based on "status"
const customSortStatusAscending = (a, b) => {
    console.log(a.status, b.status, 'customSortStatusAscending');
    const statusOrder = {
        'reserved': 4,
        'released': 3,
        'free': 2,
        'designated-not-released': 1
    };
    return statusOrder[a.status] - statusOrder[b.status];
};
exports.customSortStatusAscending = customSortStatusAscending;
// Custom sort function for sorting in descending order based on "status"
const customSortStatusDescending = (a, b) => {
    const statusOrder = {
        'reserved': 1,
        'released': 2,
        'free': 3,
        'designated-not-released': 4
    };
    return statusOrder[b.status] - statusOrder[a.status];
};
exports.customSortStatusDescending = customSortStatusDescending;
const findAvailableSpots = (req, level) => __awaiter(void 0, void 0, void 0, function* () {
    const { _id } = req.params;
    const { reservationDate, type, isDisabled, isElectric, } = req.query;
    const { startOfDay, endOfDay } = (0, dates_1.getStartAndEndOfDay)(reservationDate ? reservationDate : new Date());
    const sort = { parkingCode: 1 };
    let filter = {
        level: level ? level : new mongodb_1.ObjectId(_id),
        $or: [{ isPermanent: false }, { isPermanent: undefined }]
    };
    if (type != undefined)
        filter.spotType = type;
    if (isDisabled == 'false')
        filter.spotType = { $nin: ['disabled'] };
    if (isElectric == 'false')
        filter.spotType = { $nin: ['electric'] };
    if (isDisabled == 'false' && isElectric == 'false')
        filter.spotType = { $nin: ['disabled', 'electric'] };
    let outerWithNoInnerReserved = 0;
    const totalSpots = yield parking_spot_1.default.find(filter).sort(sort);
    const reservedSpots = yield reservation_1.default.find({ level: level ? level : new mongodb_1.ObjectId(_id), reservationDate: { $gte: startOfDay, $lte: endOfDay }, parkingStatus: { $in: [controllers_1.reservationEnums[0], controllers_1.reservationEnums[1]] } });
    const availableSpots = yield Promise.all(totalSpots.map((spot) => __awaiter(void 0, void 0, void 0, function* () {
        const isIncludedInReserved = reservedSpots.findIndex((reservedSpot) => reservedSpot.parkingSpot.toString() == spot._id.toString());
        // If No Reservation
        // if (isIncludedInReserved < 0) {
        //     const isOuter = spot.outerToSpot != null
        //     console.log(isOuter, 'isOuter')
        //     // If 17-B
        //     if (isOuter) {
        //         // Check Reservation on 17-A
        //         const isReservationOnInner = await Promise.resolve(reservedSpots.findIndex((reservedSpot: any) => reservedSpot.parkingSpot.toString() == spot.outerToSpot.toString()))
        //         // If Reservation on 17-A, return 17-B
        //         console.log(isReservationOnInner, 'isReservationOnInner')
        //         if (isReservationOnInner >= 0) return spot
        //         else outerWithNoInnerReserved += 1      // else return nothing and increase count by one for double spots i.e returned will be 17-A and count will be +1 for double
        //     } else {
        //         if (spot.spotType == 'designated' && spot.designatedTo != null) {
        //             const { startOfDay, endOfDay } = getStartAndEndOfDay(reservationDate ? reservationDate : new Date())
        //             const spotAbsence = await Absence.findOne({ designatedSpot: spot._id, $and: [{ startDate: { $lte: reservationDate }, endDate: { $gte: reservationDate } },] })
        //             const isUserPresent = spot.userPresenceDate >= startOfDay && spot.userPresenceDate <= endOfDay
        //             if (spotAbsence && !isUserPresent) return spot
        //         } else return spot
        //     }
        // }
        if (isIncludedInReserved < 0) {
            if (spot.spotType == 'designated' && spot.designatedTo != null) {
                const { startOfDay, endOfDay } = (0, dates_1.getStartAndEndOfDay)(reservationDate ? reservationDate : new Date());
                const spotAbsence = yield absence_1.default.findOne({ designatedSpot: spot._id, $and: [{ startDate: { $lte: reservationDate }, endDate: { $gte: reservationDate } },] });
                const isUserPresent = spot.userPresenceDate >= startOfDay && spot.userPresenceDate <= endOfDay;
                if (spotAbsence && !isUserPresent)
                    return spot;
            }
            else {
                const isOuter = spot.outerToSpot != null;
                // If 17-B
                if (isOuter) {
                    // Check Reservation on 17-A
                    const isReservationOnInner = yield Promise.resolve(reservedSpots.findIndex((reservedSpot) => reservedSpot.parkingSpot.toString() == spot.outerToSpot.toString()));
                    // If Reservation on 17-A, return 17-B
                    if (isReservationOnInner >= 0)
                        return spot;
                    else
                        outerWithNoInnerReserved += 1;
                }
                else
                    return spot;
            }
        }
    })));
    const filteredSpots = yield Promise.all(availableSpots.filter((spot) => spot != null));
    const sorted = filteredSpots.sort(exports.customSort);
    return { availableSpots: sorted, totalCount: sorted.length + outerWithNoInnerReserved };
});
exports.findAvailableSpots = findAvailableSpots;
const getFirstRegistration = (item) => {
    if (item.reservedBy && item.reservedBy.vehicles.length > 0) {
        console.log('reservedBy', item.reservedBy.vehicles);
        return item.reservedBy.vehicles[0].registration;
    }
    else if (item.designatedTo && item.designatedTo.vehicles.length > 0) {
        console.log('designatedTo', item.designatedTo.vehicles);
        return item.designatedTo.vehicles[0].registration;
    }
    else if ((item.reservedBy && item.reservedBy.vehicles.length === 0) || (item.designatedTo && item.designatedTo.vehicles.length === 0)) {
        return '';
    }
    return null;
};
const customSortRegistrationAscending = (a, b) => {
    const aReg = getFirstRegistration(a);
    const bReg = getFirstRegistration(b);
    if (aReg === null)
        return 1;
    if (bReg === null)
        return -1;
    if (aReg === '')
        return 1;
    if (bReg === '')
        return -1;
    return aReg.localeCompare(bReg);
};
exports.customSortRegistrationAscending = customSortRegistrationAscending;
const customSortRegistrationDescending = (a, b) => {
    const aReg = getFirstRegistration(a);
    const bReg = getFirstRegistration(b);
    if (aReg === null)
        return -1;
    if (bReg === null)
        return 1;
    if (aReg === '')
        return -1;
    if (bReg === '')
        return 1;
    return bReg.localeCompare(aReg);
};
exports.customSortRegistrationDescending = customSortRegistrationDescending;
const sendNotificationForDesignatedSpotConfirmation = (user, spot) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m;
    if ((user === null || user === void 0 ? void 0 : user.deviceToken) && ((_a = spot === null || spot === void 0 ? void 0 : spot.parkingLot) === null || _a === void 0 ? void 0 : _a.messageSendTime)) {
        {
            try {
                const notification = (user === null || user === void 0 ? void 0 : user.deviceToken) ? yield (0, notifications_1.sendNotification)(user === null || user === void 0 ? void 0 : user.deviceToken, 
                // `נא לאשר האם אתה משתמש חניה השמורה שלך בחניון ${spot?.parkingLot?.name}`,
                `נא לאשר האם אתה משתמש בחניה השמורה שלך בחניון  ${(_b = spot === null || spot === void 0 ? void 0 : spot.parkingLot) === null || _b === void 0 ? void 0 : _b.name}`, notification_1.NotificationTypes.designatedSpotConfirmation, null, [user === null || user === void 0 ? void 0 : user._id]) : null;
                yield log_1.default.create({
                    user: user._id,
                    parkingLot: (_c = spot === null || spot === void 0 ? void 0 : spot.parkingLot) === null || _c === void 0 ? void 0 : _c._id,
                    level: spot === null || spot === void 0 ? void 0 : spot.level,
                    parkingSpot: spot === null || spot === void 0 ? void 0 : spot._id,
                    status: 'success',
                    request: {
                        messageSendTime: (_d = spot === null || spot === void 0 ? void 0 : spot.parkingLot) === null || _d === void 0 ? void 0 : _d.messageSendTime,
                        absenceSendTime: (_e = spot === null || spot === void 0 ? void 0 : spot.parkingLot) === null || _e === void 0 ? void 0 : _e.absenceSendTime,
                        userPresenceDate: spot === null || spot === void 0 ? void 0 : spot.userPresenceDate,
                        deviceToken: user === null || user === void 0 ? void 0 : user.deviceToken,
                    },
                    response: notification,
                    type: log_types_1.default.designatedCronJob.notificationSent,
                });
                if (!((_f = process.env.DB_COLLECTION) === null || _f === void 0 ? void 0 : _f.includes('-')))
                    console.log('designated notification send status', notification);
            }
            catch (error) {
                yield log_1.default.create({
                    user: user._id,
                    parkingLot: (_g = spot === null || spot === void 0 ? void 0 : spot.parkingLot) === null || _g === void 0 ? void 0 : _g._id,
                    level: spot === null || spot === void 0 ? void 0 : spot.level,
                    parkingSpot: spot === null || spot === void 0 ? void 0 : spot._id,
                    status: 'error',
                    request: {
                        messageSendTime: (_h = spot === null || spot === void 0 ? void 0 : spot.parkingLot) === null || _h === void 0 ? void 0 : _h.messageSendTime,
                        absenceSendTime: (_j = spot === null || spot === void 0 ? void 0 : spot.parkingLot) === null || _j === void 0 ? void 0 : _j.absenceSendTime,
                        userPresenceDate: spot === null || spot === void 0 ? void 0 : spot.userPresenceDate,
                    },
                    type: log_types_1.default.designatedCronJob.notificationFailedToSend,
                });
            }
        }
    }
    else {
        yield log_1.default.create({
            user: user._id,
            parkingLot: (_k = spot === null || spot === void 0 ? void 0 : spot.parkingLot) === null || _k === void 0 ? void 0 : _k._id,
            level: spot === null || spot === void 0 ? void 0 : spot.level,
            parkingSpot: spot === null || spot === void 0 ? void 0 : spot._id,
            status: 'error',
            request: {
                messageSendTime: (_l = spot === null || spot === void 0 ? void 0 : spot.parkingLot) === null || _l === void 0 ? void 0 : _l.messageSendTime,
                absenceSendTime: (_m = spot === null || spot === void 0 ? void 0 : spot.parkingLot) === null || _m === void 0 ? void 0 : _m.absenceSendTime,
                userPresenceDate: spot === null || spot === void 0 ? void 0 : spot.userPresenceDate,
            },
            type: log_types_1.default.designatedCronJob.notificationDetailsMissing,
        });
    }
});
const timeMatchCheck = (spot, time, sendMessage, createAbsence) => {
    var _a, _b;
    return (((_a = spot === null || spot === void 0 ? void 0 : spot.parkingLot) === null || _a === void 0 ? void 0 : _a.messageSendTime) == time && sendMessage) || (((_b = spot === null || spot === void 0 ? void 0 : spot.parkingLot) === null || _b === void 0 ? void 0 : _b.absenceSendTime) == time && createAbsence);
};
const userPresenceCheck = (spot) => {
    var _a;
    const { today, tomorrow } = (0, dates_1.getTodayTomorrowYesterday)();
    const { startOfDay, endOfDay } = (0, dates_1.getStartAndEndOfDay)(((_a = spot === null || spot === void 0 ? void 0 : spot.parkingLot) === null || _a === void 0 ? void 0 : _a.reservationTime) == 'same-day' ? today : tomorrow);
    return (spot.userPresenceDate == null || !(spot.userPresenceDate >= startOfDay && endOfDay >= spot.userPresenceDate));
};
const absenceCheck = (spot) => __awaiter(void 0, void 0, void 0, function* () {
    var _o, _p;
    const { today, tomorrow } = (0, dates_1.getTodayTomorrowYesterday)();
    return yield absence_1.default.findOne({
        designatedSpot: spot === null || spot === void 0 ? void 0 : spot._id,
        startDate: { $lte: ((_o = spot === null || spot === void 0 ? void 0 : spot.parkingLot) === null || _o === void 0 ? void 0 : _o.reservationTime) == 'same-day' ? today : tomorrow },
        endDate: { $gte: ((_p = spot === null || spot === void 0 ? void 0 : spot.parkingLot) === null || _p === void 0 ? void 0 : _p.reservationTime) == 'same-day' ? today : tomorrow },
        status: 'pending'
    });
});
// export const performDesignatedSpotAction = async (time: string, sendMessage: boolean, createAbsence: boolean) => {
//     try {
//         const today = getCorrectDateTime(new Date())
//         const yesterday = getCorrectDateTime(new Date())
//         yesterday.setUTCDate(yesterday.getUTCDate() - 1)
//         const shabbatCheck = await isJewishHolidayShabbat(today)
//         const holiday = await getJewishHolidayEndDate(today)
//         const holidayYesterday = await getJewishHolidayEndDate(yesterday)
//         if (shabbatCheck && shabbatCheck.isShabbat) {
//             console.log('It is Shabbat, not doing anything')
//         } else {
//             const users = await User.find({
//                 $and: [{ designatedParkingSpot: { $exists: true, $ne: null } }, { designatedParkingSpot: { $ne: [] } }]
//             }).populate({
//                 path: 'designatedParkingSpot',
//                 populate: {
//                     path: 'parkingLot',
//                     select: ['messageSendTime', 'reservationTime', 'absenceSendTime', 'name']
//                 }
//             })
//             if (users?.length > 0) {
//                 await Promise.all(users.map(async (user: IUserInterface) => {
//                     try {
//                         const spotsNoPresence: any[] = [];
//                         if (user.designatedParkingSpot?.length > 0) {
//                             await Promise.all(user.designatedParkingSpot.map(async (spot: any) => {
//                                 try {
//                                     // Check All Conditions
//                                     const findUserAbsenceForToday = await absenceCheck(spot)
//                                     const isTimeMatching = timeMatchCheck(spot, time, sendMessage, createAbsence)
//                                     const isUserAbsentOnDate = userPresenceCheck(spot)
//                                     // Check if the spot is valid for notification/absence
//                                     const isSpotValidForNotification = isTimeMatching && findUserAbsenceForToday == null && isUserAbsentOnDate
//                                     // Check if Holiday was today and spot is valid
//                                     const isHolidaySpotValid = holiday && findUserAbsenceForToday == null && isUserAbsentOnDate
//                                     if (!process.env.DB_COLLECTION?.includes('-')) {
//                                         if (!holiday) {
//                                             console.log(createAbsence ? `isSpotValidForAbsence ${spot?._id}` : `isSpotValidForNotification ${spot?._id}`, isSpotValidForNotification)
//                                             console.log('--------------------------------')
//                                         }
//                                         console.log(spot?.parkingLot?.messageSendTime, 'spot?.parkingLot?.messageSendTime', spot?.parkingLot?.name)
//                                         console.log('--------------------------------')
//                                     }
//                                     if (isSpotValidForNotification && !spot?.isPermanent) {
//                                         const index = spotsNoPresence.findIndex(item => item._id == spot._id)
//                                         if (index < 0) {
//                                             await Log.create({
//                                                 user: user._id,
//                                                 parkingLot: spot?.parkingLot?._id,
//                                                 level: spot?.level,
//                                                 parkingSpot: spot?._id,
//                                                 status: 'success',
//                                                 request: {
//                                                     sendNotification: sendMessage,
//                                                     createAbsence,
//                                                     isSpotValidForNotification,
//                                                     currentTime: time,
//                                                     messageSendTime: spot?.parkingLot?.messageSendTime,
//                                                     absenceSendTime: spot?.parkingLot?.absenceSendTime,
//                                                     userPresenceDate: spot.userPresenceDate,
//                                                     spotsNoPresence,
//                                                     spotsNoPresenceIndex: index,
//                                                 },
//                                                 type: sendMessage ? LogTypes.designatedCronJob.notificationValidity : LogTypes.designatedCronJob.absenceValidity,
//                                             })
//                                             spotsNoPresence.push(spot)
//                                         } else {
//                                             await Log.create({
//                                                 user: user._id,
//                                                 parkingLot: spot?.parkingLot?._id,
//                                                 level: spot?.level,
//                                                 parkingSpot: spot?._id,
//                                                 status: 'error',
//                                                 request: {
//                                                     sendNotification: sendMessage,
//                                                     createAbsence,
//                                                     isSpotValidForNotification,
//                                                     currentTime: time,
//                                                     messageSendTime: spot?.parkingLot?.messageSendTime,
//                                                     absenceSendTime: spot?.parkingLot?.absenceSendTime,
//                                                     userPresenceDate: spot.userPresenceDate,
//                                                     spotsNoPresenceIndex: index,
//                                                     spotsNoPresence,
//                                                 },
//                                                 type: sendMessage ? LogTypes.designatedCronJob.notificationValidity : LogTypes.designatedCronJob.absenceValidity,
//                                             })
//                                         }
//                                     } else if (isHolidaySpotValid && !spot?.isPermanent) {
//                                         const parkingLotReservationTime = getTimeDifference(spot?.parkingLot?.messageSendTime, holiday.endTime)
//                                         const isParkingLotTimePassed = parkingLotReservationTime.minutes < 0 ? true : false
//                                         if (spot?.parkingLot?.reservationTime == 'same-day' && isParkingLotTimePassed) {
//                                             await Log.create({
//                                                 user: user._id,
//                                                 parkingLot: spot?.parkingLot?._id,
//                                                 level: spot?.level,
//                                                 parkingSpot: spot?._id,
//                                                 status: 'error',
//                                                 request: {
//                                                     sendNotification: sendMessage,
//                                                     createAbsence,
//                                                     isSpotValidForNotification,
//                                                     currentTime: time,
//                                                     messageSendTime: spot?.parkingLot?.messageSendTime,
//                                                     absenceSendTime: spot?.parkingLot?.absenceSendTime,
//                                                     userPresenceDate: spot.userPresenceDate,
//                                                     spotsNoPresence,
//                                                     isParkingLotTimePassed,
//                                                     parkingLotReservationTime,
//                                                     holiday,
//                                                 },
//                                                 type: LogTypes.designatedCronJob.parkingLotTimePassedOnHoliday,
//                                             })
//                                             console.log('Parking lot time has passed for same-day and today holiday ended at', holiday.endTime)
//                                             console.log('--------------------------------')
//                                         }
//                                         else {
//                                             const timeDifference = getTimeDifference(time, holiday.endTime)
//                                             if (timeDifference.minutes == 60 && sendMessage) {
//                                                 await Log.create({
//                                                     user: user._id,
//                                                     parkingLot: spot?.parkingLot?._id,
//                                                     level: spot?.level,
//                                                     parkingSpot: spot?._id,
//                                                     status: 'success',
//                                                     request: {
//                                                         sendNotification: sendMessage,
//                                                         createAbsence,
//                                                         isSpotValidForNotification,
//                                                         currentTime: time,
//                                                         messageSendTime: spot?.parkingLot?.messageSendTime,
//                                                         absenceSendTime: spot?.parkingLot?.absenceSendTime,
//                                                         userPresenceDate: spot.userPresenceDate,
//                                                         spotsNoPresence,
//                                                         timeDifference,
//                                                         holiday,
//                                                     },
//                                                     type: sendMessage ? LogTypes.designatedCronJob.notificationValidityHoliday : LogTypes.designatedCronJob.absenceValidityHoliday,
//                                                 })
//                                                 spotsNoPresence.push(spot)
//                                                 if (!process.env.DB_COLLECTION?.includes('-')) {
//                                                     console.log(isHolidaySpotValid, 'isHolidaySpotValidForNotification', spot?._id)
//                                                     console.log('--------------------------------')
//                                                 }
//                                             } else if (createAbsence) {
//                                                 const timerDuration = getTimeDifference(spot?.parkingLot.absenceSendTime, spot?.parkingLot.messageSendTime)
//                                                 const timeAfterOneHourOfShabbat = addTime(holiday.endTime, '01:00')
//                                                 const durationEndTime = addTime(timeAfterOneHourOfShabbat, timerDuration.formatted)
//                                                 if (durationEndTime == time) {
//                                                     spotsNoPresence.push(spot)
//                                                 }
//                                                 await Log.create({
//                                                     user: user._id,
//                                                     parkingLot: spot?.parkingLot?._id,
//                                                     level: spot?.level,
//                                                     parkingSpot: spot?._id,
//                                                     status: durationEndTime == time ? 'success' : 'error',
//                                                     request: {
//                                                         sendNotification: sendMessage,
//                                                         createAbsence,
//                                                         isSpotValidForNotification,
//                                                         isHolidaySpotValid,
//                                                         currentTime: time,
//                                                         messageSendTime: spot?.parkingLot?.messageSendTime,
//                                                         absenceSendTime: spot?.parkingLot?.absenceSendTime,
//                                                         userPresenceDate: spot.userPresenceDate,
//                                                         spotsNoPresence,
//                                                         timeDifference,
//                                                         holiday,
//                                                     },
//                                                     type: LogTypes.designatedCronJob.absenceValidityHoliday,
//                                                 })
//                                                 // console.log(isHolidaySpotValid, 'isHolidaySpotValidForAbsence', spot?._id)
//                                                 // console.log('--------------------------------')
//                                             } else {
//                                                 if (!process.env.DB_COLLECTION?.includes('-')) {
//                                                     console.log(createAbsence ? `isSpotValidForAbsenceOnHoliday ${spot?._id}` : `isSpotValidForNotificationOnHoliday ${spot?._id}`, isSpotValidForNotification)
//                                                     console.log('--------------------------------')
//                                                 }
//                                                 await Log.create({
//                                                     user: user._id,
//                                                     parkingLot: spot?.parkingLot?._id,
//                                                     level: spot?.level,
//                                                     parkingSpot: spot?._id,
//                                                     status: 'error',
//                                                     request: {
//                                                         sendNotification: sendMessage,
//                                                         createAbsence,
//                                                         isSpotValidForNotification,
//                                                         isHolidaySpotValid,
//                                                         currentTime: time,
//                                                         messageSendTime: spot?.parkingLot?.messageSendTime,
//                                                         absenceSendTime: spot?.parkingLot?.absenceSendTime,
//                                                         userPresenceDate: spot.userPresenceDate,
//                                                         spotsNoPresence,
//                                                         timeDifference,
//                                                         holiday,
//                                                     },
//                                                     type: LogTypes.designatedCronJob.notificationValidityHoliday,
//                                                 })
//                                             }
//                                         }
//                                     } else if (spot?.parkingLot?.messageSendTime == time || spot?.parkingLot?.absenceSendTime == time) {
//                                         if (!process.env.DB_COLLECTION?.includes('-')) {
//                                             console.log(`Not eligible to send ${sendMessage ? 'Notification' : 'Absence'}`, spot?._id)
//                                             console.log('--------------------------------')
//                                         }
//                                         await Log.create({
//                                             user: user._id,
//                                             parkingLot: spot?.parkingLot?._id,
//                                             level: spot?.level,
//                                             parkingSpot: spot?._id,
//                                             status: 'error',
//                                             request: {
//                                                 sendNotification: sendMessage,
//                                                 createAbsence: createAbsence,
//                                                 isSpotValidForNotification,
//                                                 isHolidaySpotValid,
//                                                 currentTime: time,
//                                                 messageSendTime: spot?.parkingLot?.messageSendTime,
//                                                 absenceSendTime: spot?.parkingLot?.absenceSendTime,
//                                                 userPresenceDate: spot.userPresenceDate,
//                                                 spotsNoPresence,
//                                                 findUserAbsenceForToday,
//                                                 isSpotPermanent: spot?.isPermanent,
//                                             },
//                                             type: sendMessage ? LogTypes.designatedCronJob.notificationNotEligible : LogTypes.designatedCronJob.absenceNotEligible,
//                                         })
//                                     }
//                                 } catch (err) {
//                                     console.log('Error in mapping user designated spots in performAbsenceAction', err)
//                                 }
//                             }))
//                             if (spotsNoPresence.length > 0) {
//                                 if (sendMessage) {
//                                     console.log('Sending notification to user: ' + user?._id)
//                                     await Promise.all(spotsNoPresence.map(async (spot: any) => {
//                                         if (user?.deviceToken) await sendNotificationForDesignatedSpotConfirmation(user, spot)
//                                         else console.log(user?._id, 'does not have device token')
//                                     }))
//                                 }
//                                 if (createAbsence && user?._id && user?.designatedParkingSpot.length > 0) {
//                                     await Promise.all(spotsNoPresence.map(async (spot: any) => {
//                                         console.log('Sending absence to user: ' + user?._id)
//                                         const dateForAbsence = new Date()
//                                         if (spot?.parkingLot?.reservationTime == 'day-before') dateForAbsence.setUTCDate(dateForAbsence.getUTCDate() + 1)
//                                         const { startOfDay, endOfDay } = getStartAndEndOfDay(dateForAbsence)
//                                         const absence = new Absence({
//                                             user: user?._id,
//                                             designatedSpot: spot?._id,
//                                             startDate: startOfDay,
//                                             endDate: endOfDay,
//                                             note: 'המשתמש לא הגיב בתוך פרק הזמן שצוין.',
//                                             type: 'system-generated'
//                                         })
//                                         await absence.save()
//                                         await ParkingSpot.findByIdAndUpdate(spot?._id, { userPresenceDate: null });
//                                     }));
//                                 }
//                             }
//                         }
//                     } catch (err) {
//                         console.log('Error in mapping users in performAbsenceAction', err)
//                     }
//                 }));
//             }
//             if (!process.env.DB_COLLECTION?.includes('-')) {
//                 console.log('All spots checked', users?.length);
//                 console.log('===================');
//                 console.log('===================');
//             }
//         }
//     } catch (err) {
//         console.log('Error executing performAbsenceAction', err)
//     }
// }
const processUserDesignatedSpots = (user, time, sendMessage, createAbsence, holiday, holidayYesterday) => __awaiter(void 0, void 0, void 0, function* () {
    var _q;
    const spotsNoPresence = [];
    if (((_q = user.designatedParkingSpot) === null || _q === void 0 ? void 0 : _q.length) > 0) {
        yield Promise.all(user.designatedParkingSpot.map((spot) => __awaiter(void 0, void 0, void 0, function* () {
            var _r, _s, _t, _u, _v, _w, _x, _y, _z;
            try {
                // Check All Conditions
                const findUserAbsenceForToday = yield absenceCheck(spot);
                const isTimeMatching = timeMatchCheck(spot, time, sendMessage, createAbsence);
                const isUserAbsentOnDate = userPresenceCheck(spot);
                let parkingLotReservationTime = { minutes: 0, formatted: '00:00' };
                if (holiday)
                    parkingLotReservationTime = (0, dates_1.getTimeDifference)((_r = spot === null || spot === void 0 ? void 0 : spot.parkingLot) === null || _r === void 0 ? void 0 : _r.messageSendTime, holiday.endTime);
                const isParkingLotTimePassed = parkingLotReservationTime.minutes < 0 ? true : false;
                // Check if the spot is valid for notification/absence
                const isSpotValidForNotification = isTimeMatching && findUserAbsenceForToday == null && isUserAbsentOnDate;
                // Check if Holiday was today and spot is valid
                const isHolidaySpotValid = holiday && findUserAbsenceForToday == null && isUserAbsentOnDate;
                // Check if the time is between mid night and 3 hours after
                const isTimeAfterMidNight = (0, dates_1.isTimeBetween)(time, '00:00', '03:00');
                // Check if parking lot notification is on
                const isParkingLotNotificationAllowed = ((_s = spot === null || spot === void 0 ? void 0 : spot.parkingLot) === null || _s === void 0 ? void 0 : _s.sendPushNotification) != false ? true : false;
                if (isSpotValidForNotification && isParkingLotNotificationAllowed && !(spot === null || spot === void 0 ? void 0 : spot.isPermanent) && !isParkingLotTimePassed) {
                    const index = spotsNoPresence.findIndex(item => item._id == spot._id);
                    if (index < 0) {
                        // Log here
                        spotsNoPresence.push(spot);
                    }
                    else {
                        // Log here
                    }
                }
                else if (isHolidaySpotValid && isParkingLotNotificationAllowed && !(spot === null || spot === void 0 ? void 0 : spot.isPermanent)) {
                    if (((_t = spot === null || spot === void 0 ? void 0 : spot.parkingLot) === null || _t === void 0 ? void 0 : _t.reservationTime) == 'same-day' && isParkingLotTimePassed) {
                        console.log('Parking lot time has passed for same-day and today holiday ended at', holiday.endTime);
                        console.log('--------------------------------');
                    }
                    else {
                        const timeDifference = (0, dates_1.getTimeDifference)(time, holiday.endTime);
                        if (timeDifference.minutes == 60 && isParkingLotTimePassed && sendMessage) {
                            spotsNoPresence.push(spot);
                            if (!((_u = process.env.DB_COLLECTION) === null || _u === void 0 ? void 0 : _u.includes('-'))) {
                                console.log(isHolidaySpotValid, 'isHolidaySpotValidForNotification', spot === null || spot === void 0 ? void 0 : spot._id);
                                console.log('--------------------------------');
                            }
                        }
                        else if (createAbsence) {
                            const timerDuration = (0, dates_1.getTimeDifference)(spot === null || spot === void 0 ? void 0 : spot.parkingLot.absenceSendTime, spot === null || spot === void 0 ? void 0 : spot.parkingLot.messageSendTime);
                            const timeAfterOneHourOfShabbat = (0, dates_1.addTime)(holiday.endTime, '01:00');
                            const durationEndTime = (0, dates_1.addTime)(timeAfterOneHourOfShabbat, timerDuration.formatted);
                            if (durationEndTime == time) {
                                spotsNoPresence.push(spot);
                            }
                            // console.log(isHolidaySpotValid, 'isHolidaySpotValidForAbsence', spot?._id)
                            // console.log('--------------------------------')
                        }
                        else {
                            if (!((_v = process.env.DB_COLLECTION) === null || _v === void 0 ? void 0 : _v.includes('-'))) {
                                console.log(createAbsence ? `isSpotValidForAbsenceOnHoliday ${spot === null || spot === void 0 ? void 0 : spot._id}` : `isSpotValidForNotificationOnHoliday ${spot === null || spot === void 0 ? void 0 : spot._id}`, isSpotValidForNotification);
                                console.log('--------------------------------');
                            }
                        }
                    }
                }
                else if (((_w = spot === null || spot === void 0 ? void 0 : spot.parkingLot) === null || _w === void 0 ? void 0 : _w.messageSendTime) == time || ((_x = spot === null || spot === void 0 ? void 0 : spot.parkingLot) === null || _x === void 0 ? void 0 : _x.absenceSendTime) == time) {
                    if (!((_y = process.env.DB_COLLECTION) === null || _y === void 0 ? void 0 : _y.includes('-'))) {
                        console.log(`Not eligible to send ${sendMessage ? 'Notification' : 'Absence'}`, spot === null || spot === void 0 ? void 0 : spot._id);
                        console.log('--------------------------------');
                    }
                }
                else if (isTimeAfterMidNight && holidayYesterday && createAbsence && isParkingLotNotificationAllowed) {
                    const timeAfterOneHourOfShabbat = (0, dates_1.addTime)(holidayYesterday.endTime, '01:00');
                    const durationEndTime = (0, dates_1.addTime)(timeAfterOneHourOfShabbat, (_z = spot === null || spot === void 0 ? void 0 : spot.parkingLot) === null || _z === void 0 ? void 0 : _z.designatedReservationDuration);
                    const isDurationEndTimeAfterMidNight = (0, dates_1.isTimeBetween)(durationEndTime, '00:00', '03:00');
                    // console.log(isDurationEndTimeAfterMidNight,'isDurationEndTimeAfterMidNight')
                    // console.log(durationEndTime,'durationEndTime')
                    // console.log(time,'time')
                    if (isDurationEndTimeAfterMidNight) {
                        if (durationEndTime == time) {
                            // spotsNoPresence.push(spot)
                            console.log('Sending absence to user: ' + (user === null || user === void 0 ? void 0 : user._id));
                            const dateForAbsence = new Date();
                            const { startOfDay, endOfDay } = (0, dates_1.getStartAndEndOfDay)(dateForAbsence);
                            const absence = new absence_1.default({
                                user: user === null || user === void 0 ? void 0 : user._id,
                                designatedSpot: spot === null || spot === void 0 ? void 0 : spot._id,
                                startDate: startOfDay,
                                endDate: endOfDay,
                                note: 'המשתמש לא הגיב בתוך פרק הזמן שצוין.',
                                type: 'system-generated'
                            });
                            yield absence.save();
                            yield parking_spot_1.default.findByIdAndUpdate(spot === null || spot === void 0 ? void 0 : spot._id, { userPresenceDate: null });
                        }
                    }
                }
            }
            catch (err) {
                console.log('Error in mapping user designated spots in performAbsenceAction', err);
            }
        })));
        if (spotsNoPresence.length > 0) {
            if (sendMessage) {
                console.log('Sending notification to user: ' + (user === null || user === void 0 ? void 0 : user._id));
                yield Promise.all(spotsNoPresence.map((spot) => __awaiter(void 0, void 0, void 0, function* () {
                    if (user === null || user === void 0 ? void 0 : user.deviceToken)
                        yield sendNotificationForDesignatedSpotConfirmation(user, spot);
                    else
                        console.log(user === null || user === void 0 ? void 0 : user._id, 'does not have device token');
                })));
            }
            if (createAbsence && (user === null || user === void 0 ? void 0 : user._id) && (user === null || user === void 0 ? void 0 : user.designatedParkingSpot.length) > 0) {
                yield Promise.all(spotsNoPresence.map((spot) => __awaiter(void 0, void 0, void 0, function* () {
                    var _0;
                    console.log('Sending absence to user: ' + (user === null || user === void 0 ? void 0 : user._id));
                    const dateForAbsence = new Date();
                    if (((_0 = spot === null || spot === void 0 ? void 0 : spot.parkingLot) === null || _0 === void 0 ? void 0 : _0.reservationTime) == 'day-before')
                        dateForAbsence.setUTCDate(dateForAbsence.getUTCDate() + 1);
                    const { startOfDay, endOfDay } = (0, dates_1.getStartAndEndOfDay)(dateForAbsence);
                    const absence = new absence_1.default({
                        user: user === null || user === void 0 ? void 0 : user._id,
                        designatedSpot: spot === null || spot === void 0 ? void 0 : spot._id,
                        startDate: startOfDay,
                        endDate: endOfDay,
                        note: 'המשתמש לא הגיב בתוך פרק הזמן שצוין.',
                        type: 'system-generated'
                    });
                    yield absence.save();
                    yield parking_spot_1.default.findByIdAndUpdate(spot === null || spot === void 0 ? void 0 : spot._id, { userPresenceDate: null });
                })));
            }
        }
    }
});
const performDesignatedSpotAction = (time, sendMessage, createAbsence) => __awaiter(void 0, void 0, void 0, function* () {
    var _1;
    try {
        const { today, yesterday } = (0, dates_1.getTodayTomorrowYesterday)();
        const shabbatCheck = yield (0, dates_1.isJewishHolidayShabbat)(today);
        const holiday = yield (0, dates_1.getJewishHolidayEndDate)(today);
        const holidayYesterday = yield (0, dates_1.getJewishHolidayEndDate)(yesterday);
        if (shabbatCheck && shabbatCheck.isShabbat) {
            console.log('It is Shabbat, not doing anything');
        }
        else {
            const users = yield user_1.default.find({ $and: [{ designatedParkingSpot: { $exists: true, $ne: null } }, { designatedParkingSpot: { $ne: [] } }] })
                .populate({
                path: 'designatedParkingSpot',
                populate: {
                    path: 'parkingLot',
                    select: ['name', 'reservationTime', 'messageSendTime', 'absenceSendTime', 'designatedReservationDuration', 'sendPushNotification',],
                }
            });
            if ((users === null || users === void 0 ? void 0 : users.length) > 0) {
                yield Promise.all(users.map((user) => __awaiter(void 0, void 0, void 0, function* () {
                    try {
                        yield processUserDesignatedSpots(user, time, sendMessage, createAbsence, holiday, holidayYesterday);
                    }
                    catch (err) {
                        console.log('Error in mapping users in performAbsenceAction', err);
                    }
                })));
            }
            if (!((_1 = process.env.DB_COLLECTION) === null || _1 === void 0 ? void 0 : _1.includes('-'))) {
                console.log('All spots checked', users === null || users === void 0 ? void 0 : users.length);
                console.log('===================');
                console.log('===================');
            }
        }
    }
    catch (err) {
        console.log('Error executing performAbsenceAction', err);
    }
});
exports.performDesignatedSpotAction = performDesignatedSpotAction;
