"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.uploadVideos = exports.uploadFiles = exports.uploadImages = void 0;
const multer_1 = __importDefault(require("multer"));
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
const uploaders_1 = require("../../helpers/uploaders");
const imagesStorage = multer_1.default.diskStorage({
    destination: (req, file, cb) => {
        const filePath = path_1.default.join(__dirname, "../../../public/images");
        // const filePath = path.join(__dirname, "../../public/images")
        fs_1.default.mkdirSync(filePath, { recursive: true });
        cb(null, filePath);
    },
    filename: (req, file, cb) => {
        const name = `${path_1.default.parse(file.originalname).name}-${Date.now()}${Date.now()}${path_1.default.extname(file.originalname)}`;
        cb(null, name);
    },
});
const filesStorage = multer_1.default.diskStorage({
    destination: (req, file, cb) => {
        const filePath = path_1.default.join(__dirname, "../../../public/files");
        // const filePath = path.join(__dirname, "../../public/files")
        fs_1.default.mkdirSync(filePath, { recursive: true });
        cb(null, filePath);
    },
    filename: (req, file, cb) => {
        const name = `${path_1.default.parse(file.originalname).name}-${Date.now()}${Date.now()}${path_1.default.extname(file.originalname)}`;
        cb(null, name);
    },
});
const videosStorage = multer_1.default.diskStorage({
    destination: (req, file, cb) => {
        const filePath = path_1.default.join(__dirname, "../../../public/videos");
        // const filePath = path.join(__dirname, "../../public/videos")
        fs_1.default.mkdirSync(filePath, { recursive: true });
        cb(null, filePath);
    },
    filename: (req, file, cb) => {
        const name = `${path_1.default.parse(file.originalname).name}-${Date.now()}${Date.now()}${path_1.default.extname(file.originalname)}`;
        cb(null, name);
    },
});
const imageUploader = (0, multer_1.default)({
    storage: imagesStorage,
    limits: { fileSize: (0, uploaders_1.mbToBytes)(5) },
    fileFilter: uploaders_1.imageFileFilter
});
const fileUploader = (0, multer_1.default)({
    storage: filesStorage,
    limits: { fileSize: (0, uploaders_1.mbToBytes)(5) },
    fileFilter: uploaders_1.fileFilter,
});
const videoUploader = (0, multer_1.default)({
    storage: videosStorage,
    limits: { fileSize: (0, uploaders_1.mbToBytes)(25) },
    fileFilter: uploaders_1.videoFilter,
});
exports.uploadImages = imageUploader.array('images', 10);
exports.uploadFiles = fileUploader.array('files', 10);
exports.uploadVideos = videoUploader.array('videos', 10);
