"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const mongoose_1 = require("mongoose");
const adminSchema = new mongoose_1.Schema({
    email: {
        type: String,
        required: true,
        unique: true,
    },
    phoneNumber: {
        countryCode: {
            type: String,
            required: true,
        },
        number: {
            type: String,
            required: true,
        },
    },
    password: {
        type: String,
        required: true,
    },
    avatar: {
        type: String,
        required: false,
        default: null,
    },
    temporaryPassword: {
        type: String,
        required: false,
        default: null,
    },
    firstName: {
        type: String,
        required: true,
        default: "",
    },
    middleName: {
        type: String,
        required: false,
        default: "",
    },
    lastName: {
        type: String,
        required: false,
        default: "",
    },
    failedLoginAttempts: {
        type: Number,
        default: 0,
    },
    failedAttemptsDates: [{
            type: Date,
            default: []
        }],
    passwordUpdatedAt: {
        type: Date,
        default: Date.now,
    },
    temporaryPasswordExpiry: {
        type: Date,
    },
    parkingLots: [{
            type: mongoose_1.Schema.Types.ObjectId,
            ref: 'ParkingLot',
            required: true,
            default: [],
        }],
    role: {
        type: String,
        required: true,
        enum: ['admin', 'sub-admin'],
        default: 'admin',
    },
    accountLockInstances: [{
            type: Date,
            default: []
        }],
}, { timestamps: true });
const Admin = (0, mongoose_1.model)('Admin', adminSchema);
exports.default = Admin;
