"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const mongoose_1 = require("mongoose");
const parkingLotSchema = new mongoose_1.Schema({
    name: {
        type: String,
        required: true,
    },
    chargingTimeLimit: {
        type: Boolean,
        required: true,
    },
    chargingTimeStart: {
        type: String,
        required: false,
        default: null,
    },
    chargingTimeEnd: {
        type: String,
        required: false,
        default: null,
    },
    location: {
        name: {
            type: String,
            required: true,
        },
        latitude: {
            type: Number,
            required: true,
        },
        longitude: {
            type: Number,
            required: true,
        },
    },
    admins: [{
            type: mongoose_1.Schema.Types.ObjectId,
            ref: 'Admin',
            required: true,
            default: [],
        }],
    allotmentMethod: {
        type: String,
        required: false,
        enum: ['random', 'manual', 'user-selected'],
        default: 'random',
    },
    allotmentFile: {
        type: String,
        required: false,
        default: null,
    },
    accessMapFile: {
        type: String,
        required: false,
        default: null,
    },
    avatar: {
        type: String,
        required: true,
    },
    maxReservationDistance: {
        type: Number,
        required: true,
        default: 1,
    },
    messageSendTime: {
        type: String,
        required: true,
        default: '05:00',
    },
    absenceSendTime: {
        type: String,
        required: true,
        default: '08:00',
    },
    messageSendTimeOfDay: {
        type: String,
        required: false,
        default: 'morning',
    },
    reservationTime: {
        type: String,
        required: true,
        default: 'same-day',
        enum: ['same-day', 'day-before']
    },
    smsSendTime: {
        type: String,
        required: false,
    },
    designatedReservationDuration: {
        type: String,
        required: true,
        default: '03:00', // Hours
    },
    sendPushNotification: {
        type: Boolean,
        required: false,
        default: true,
    },
}, { timestamps: true });
const ParkingLot = (0, mongoose_1.model)('ParkingLot', parkingLotSchema);
exports.default = ParkingLot;
