"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const mongoose_1 = require("mongoose");
const vehicle_1 = require("./vehicle");
const userSchema = new mongoose_1.Schema({
    email: {
        type: String,
        required: false,
        unique: false,
    },
    phoneNumber: {
        countryCode: {
            type: String,
            required: true,
        },
        number: {
            type: String,
            required: true,
            unique: true,
        },
    },
    password: {
        type: String,
        required: false,
        default: '',
    },
    avatar: {
        type: String,
        required: false,
        default: null,
    },
    temporaryPassword: {
        type: String,
        required: false,
        default: null,
    },
    firstName: {
        type: String,
        required: true,
    },
    middleName: {
        type: String,
        required: false,
    },
    lastName: {
        type: String,
        required: false,
    },
    failedLoginAttempts: {
        type: Number,
        default: 0,
    },
    failedAttemptsDates: [{
            type: Date,
            default: []
        }],
    passwordUpdatedAt: {
        type: Date,
        default: Date.now,
    },
    temporaryPasswordExpiry: {
        type: Date,
    },
    electricCharging: {
        type: Boolean,
        required: false,
        default: false,
    },
    designatedParkingSpot: [{
            type: mongoose_1.Schema.Types.ObjectId,
            ref: 'ParkingSpot',
            required: false,
            default: [],
        }],
    userSpot: {
        type: mongoose_1.Schema.Types.String,
        required: false,
        default: null,
    },
    accountLockInstances: [{
            type: Date,
            default: []
        }],
    parkingLots: [{
            type: mongoose_1.Schema.Types.ObjectId,
            default: [],
            ref: 'ParkingLot',
        }],
    mainParkingLot: {
        type: mongoose_1.Schema.Types.ObjectId,
        default: null,
        ref: 'ParkingLot',
    },
    disabled: {
        type: mongoose_1.Schema.Types.Boolean,
        required: false,
        default: false,
    },
    deviceToken: {
        type: mongoose_1.Schema.Types.String,
        required: false,
        default: null,
    },
    vehicle: vehicle_1.vehiclesSchema,
    dateOfBirth: {
        type: Date,
        required: false,
        default: Date.now()
    },
    firstLoginDate: {
        type: Date,
        required: false,
        default: null
    },
    vehicles: [{
            ref: 'Vehicle',
            type: mongoose_1.Schema.Types.ObjectId,
            default: []
        }],
    lastLogin: {
        type: Date,
        required: false,
        default: null,
    },
    isGuest: {
        type: mongoose_1.Schema.Types.Boolean,
        required: false,
        default: false,
    }
}, { timestamps: true });
const User = (0, mongoose_1.model)('User', userSchema);
exports.default = User;
