"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const HttpMessages = {
    // ------- Validations -------
    validations: {
        INVALID_LOGIN_CREDENTIALS: 'Please provide login credentials.',
        INVALID_EMAIL: 'Email must be a valid email.',
        INVALID_PASSWORD: `Password must contain at lease three of the following:
        - Small letter (a-z).
        - Capital letter (A-Z).
        - Special character (!@#$%A&*()-+).
        - Number (1-9)`,
        INVALID_TOKEN: 'Invalid Token.',
        PASSWORDS_NOT_MATCH: "Passwords do not match.",
        TEMP_PASSWORD_EXPIRED: "Temporary Password has expired.",
        INVALID_PHONE_NUMBER: "Phone number is not valid.",
    },
    // ------- User -------
    user: {
        USER_DETAILS_MISSING: 'Please provide all the user details.',
        USER_ALREADY_EXISTS: 'Email/Phone No. already in use.',
        USER_NOT_FOUND: 'User not found.',
        USER_DELETED_SUCCESS: 'User deleted successfully.',
        ALL_USERS_DELETED: 'All users deleted successfully.',
        ACCOUNT_LOCKED: 'Too many failed login attempts. Your account has been locked, please contact the administration.',
        FAILED_LOGIN_ATTEMPT: 'Incorrect Password. ',
        PASSWORD_EXPIRED: 'Your password is more than 90 days old, please reset to continue.',
        FAILED_TEMPORARY_LOGIN: 'The temporary password you entered is incorrect, please check your email and enter the temporary password mentioned in the email.',
        WRONG_OLD_PASSWORD: 'The old password you entered is incorrect, please enter the correct old password to continue.',
        TEMPORARY_PASSWORD_CREATED: 'A temporary password has been sent to your email. Please check your inbox.',
        TEMPORARY_PASSWORD_CREATED_PHONE: 'A temporary password has been sent to your mobile number. Please check your inbox.',
        PASSWORD_UPDATE_SUCCESS: 'Your password has been updated. Use the new password to login.',
        ACCOUNT_UNLOCKED: 'The account has been unlocked.',
        ACCOUNT_LOGGED_OUT: 'Your account has been logged out successfully.',
        IS_TEMPORARY_PASSWORD: `You've entered a temporary password, please change the password to continue.`,
        FAILED_TO_CREATE_MULTIPLE_USERS: 'Email/Phone number/Registration already exits at row ',
        EMAIL_ALREADY_EXISTS: 'Email already exists',
        PHONE_ALREADY_EXISTS: 'Phone Number already exists',
        REG_ALREADY_EXISTS: 'Vehicle registration already exists',
        EMAIL_ALREADY_EXISTS_HEB: 'האימייל כבר קיים',
        PHONE_ALREADY_EXISTS_HEB: 'מספר טלפון כבר קיים',
        REG_ALREADY_EXISTS_HEB: 'רישום הרכב כבר קיים',
        FAILED_TO_CREATE_MULTIPLE_USERS_HEB: `דוא"ל/מספר טלפון/רישום כבר יוצא בשורה `
    },
    // ------- Vehicle -------
    vehicle: {
        VEHICLE_DETAILS_MISSING: 'Please provide all the vehicle details.',
        VEHICLE_NOT_FOUND: 'Vehicle not found.',
        VEHICLE_DELETED_SUCCESS: 'Vehicle deleted successfully.',
        VEHICLE_REGISTRATION_DUPLICATE: 'Vehicle Registration already in use.',
    },
    // ------- Parking Lot -------
    parkingLot: {
        PARKING_LOT_DETAILS_MISSING: 'Please provide all the parking lot details.',
        PARKING_LOT_NOT_FOUND: 'Parking lot not found.',
        PARKING_LOT_DELETED_SUCCESS: 'Parking lot deleted successfully.',
        DELETE_FAILURE_REASON_UNCOMPLETED_RESERVATION: 'Failed to delete parking lot due to incomplete reservations.',
        DELETE_FAILURE_REASON_DESIGNATED: 'Failed to delete parking lot. Users are assigned designated spots from this parking lot.',
    },
    // ------- Levels -------
    lotLevel: {
        LEVEL_DETAILS_MISSING: 'Please provide all the level details.',
        LEVEL_ALREADY_EXIST: 'Level already exists for this parking lot. ',
        LEVEL_NOT_FOUND: 'Level not found.',
        LEVEL_DELETED_SUCCESS: 'Level deleted successfully.',
        LEVEL_NOT_FOUND_IN_FILE: 'The selected level is not in your uploaded file, please check and try again or upload a new file.',
        UPDATE_FAILURE_REASON_INCOMPLETE_RESERVATION: 'Failed to update level due to incomplete reservations.',
        UPDATE_FAILURE_REASON_DESIGNATED: 'Failed to update level due to the spots are assigned to some user.',
        DELETE_FAILURE_REASON_UNCOMPLETED_RESERVATION: 'Failed to delete level due to reservations. If you want to delete the level, please delete the reservations first.',
    },
    // ------- Parking Spot -------
    parkingSpot: {
        PARKING_SPOT_DETAILS_MISSING: 'Please provide all the parking spot details.',
        PARKING_SPOT_NOT_FOUND: 'Parking spot not found.',
        PARKING_SPOT_NOT_FOUND_HEB: 'מקום חניה לא נמצא.',
        PARKING_SPOT_DELETED_SUCCESS: 'Parking spot deleted successfully.',
        REG_PARKING_SPOT_UNAVAILABLE: 'Regular Parking Spots unavailable. ',
        DESIGNATED_PARKING_SPOT_UNAVAILABLE: 'Designated Parking Spots unavailable.',
        DESIGNATED_SPOT_ALREADY_ASSIGNED_TO_USER: 'Designated Parking Spot is already assigned to another user.',
        DESIGNATED_SPOT_HAS_RESERVATIONS: 'Designated Parking Spot has reservations against it. You will be able to designate it to another user after the reservation completed.',
        ELECTRIC_STATION_UNAVAILABLE: 'Electric Parking Spots unavailable. ',
        PARKING_SPOT_IN_FILE_NOT_MATCH_REQUESTED: 'The total parking spots your requested exceeds the parking spots in the file you uploaded.',
        PARKING_CODE_ALREADY_IN_DB: 'The parking code selected is already in the database against this parking lot and level.',
        FAILURE_CREATING_DUPLICATE_VALUES: 'Failed to create the parking spots. You have entered invalid parking codes or the parking codes are overlapping.',
        FAILURE_CREATING_DUPLICATE_VALUES_HEB: 'יצירת מקומות החניה נכשלה. הזנת קודי חניה לא חוקיים או שקודי החניה חופפים.',
        FAILURE_UPDATING_DUPLICATE_VALUES: 'Failed to update the parking spots of the area. You have entered invalid parking codes or the parking codes are overlapping.',
        FAILURE_UPDATING_DUPLICATE_VALUES_HEB: 'עדכון מקומות החניה של האזור נכשל. הזנת קודי חניה לא חוקיים או שקודי החניה חופפים.',
        SPOT_ID_NOT_ENTERED: 'Please enter a valid spot ID.'
    },
    // ------- Reservation -------
    reservation: {
        RESERVATION_DETAILS_MISSING: 'Please provide all the reservation details.',
        RESERVATION_NOT_FOUND: 'Reservation not found.',
        RESERVATION_NOT_FOUND_HEB: 'ההזמנה לא נמצאה.',
        RESERVATION_UPDATE_SUCCESS: 'Reservation updated successfully.',
        RESERVATION_INVALID_STATUS: `Reservation status can only be of the following ['reserved', 'occupied', 'completed', 'canceled'].`,
        RESERVATION_DELETED_SUCCESS: 'Reservation deleted successfully.',
        RESERVATION_ALREADY_EXISTS: 'Reservation already exists for the date.',
        RESERVATION_SPOTS_ALL_BOOKED: 'There are no free\nspots available for the\nchosen date.',
        RESERVATION_SELECTED_SPOT_ALREADY_BOOKED: 'The selected parking spot is already booked for the date.',
        // RESERVATION_SELECTED_SPOT_ALREADY_BOOKED_HEB: 'מקום החניה שנבחר כבר מוזמן לתאריך.',
        RESERVATION_SELECTED_SPOT_ALREADY_BOOKED_HEB: 'החניה שבחרת תפוסה כבר.',
        RESERVATION_SPOT_NOT_RELEASED: 'The selected parking spot is not released.',
        RESERVATION_SPOT_NOT_RELEASED_HEB: 'מקום החניה שבחרת נתפס כבר על ידי עובד אחר.',
        RESERVATION_SPOTS_ALL_BOOKED_ELECTRIC: 'There are no free electric\ncharging station parking\nspots available for the\nchosen date.',
        RESERVATION_SPOTS_ALL_BOOKED_ELECTRIC_HEB: 'אין עמדות טעינה חשמליות פנויות\nלמועד הנבחר.',
        // RESERVATION_SPOTS_ALL_BOOKED : 'No spot available for reservation in the selected parking lot.',
        RESERVATION_NOT_IN_RESERVATION_RADIUS: 'You are too far to\ncreate a reservation\nfor this parking space.',
        RESERVATION_NOT_IN_RESERVATION_RADIUS_HEB: 'אתה רחוק מדי מהחניון\nבכדי לבצע הזמנה\nעבור שמירת מקום חניה זה.',
        // RESERVATION_NOT_IN_RESERVATION_RADIUS : 'You are out of the maximum reservation distance of the lot.',
        RESERVATION_CHARGER_NOT_AVAILABLE: 'No Charging station available for reservation in the selected parking lot level.',
        RESERVATION_ALREADY_IN_PARKING_LOT: 'You already have a reservation in the selected parking lot. Please complete it to make another reservation.',
        USER_MAX_RESERVATION: 'User has multiple reservations. Only two reservations allowed per user.',
        USER_MAX_RESERVATION_HEB: 'למשתמש יש מספר הזמנות. רק שתי הזמנות מותרות לכל משתמש.',
        DESIGNATED_ALREADY_IN_PARKING_LOT: 'You already have a designated parking spot in the selected parking lot.',
        DESIGNATED_ALREADY_IN_PARKING_LOT_HEB: 'אינך יכול להזמין חניה מפני שיש לך כבר חניה שמורה מוקצית.',
        RESERVATION_ALREADY_IN_PARKING_LOT_HEB: 'יש לך כבר הזמנה קיימת בחניון הזה.',
        RESERVATION_EXTRA_SPOT_NOT_FOUND: 'There are no more\nparking spots available\nfor the chosen date.',
        RESERVATION_EXTRA_SPOT_NOT_FOUND_HEB: 'אין עוד מקומות חניה פנויים\nלתאריך הנבחר.',
        // RESERVATION_EXTRA_SPOT_NOT_FOUND : 'Charging station is available but parking lot has time limit for charging, there is no available spot for parking.',
    },
    // ------- Announcement -------
    announcement: {
        ANNOUNCEMENT_DETAILS_MISSING: 'Please provide all the announcement details.',
        ANNOUNCEMENT_NOT_FOUND: 'Announcement not found.',
        ANNOUNCEMENT_DELETED_SUCCESS: 'Announcement deleted successfully.',
        ANNOUNCEMENT_WRONG_DATE: 'Please provide a valid date, start date cannot be greater than end date.',
    },
    // ------- Complaints -------
    complaint: {
        COMPLAINT_DETAILS_MISSING: 'Please provide all the complaint details.',
        COMPLAINT_NOT_FOUND: 'Complaint not found.',
        COMPLAINT_INVALID_STATUS: `Complaint status can only be of the following ['pending', 'opened', 'in-progress', 'closed', 'completed'].`,
        COMPLAINT_DELETED_SUCCESS: 'Complaint deleted successfully.',
    },
    holiday: {
        HOLIDAY_DETAILS_MISSING: 'Please provide all the holiday details.',
        HOLIDAY_NOT_FOUND: 'Holiday not found.',
        HOLIDAY_DELETED_SUCCESS: 'Holiday deleted successfully.',
    },
    // ------- Upload -------
    upload: {
        FILE_UPLOAD_FAILED: 'Failed to upload file(s), please check the format of the uploaded file(s), it should be comma separated CSV file.',
        VIDEO_UPLOAD_FAILED: 'Failed to upload file(s), please check the format of the uploaded file(s), it should be one of the following: mp4, mpeg, avi or webm.',
        IMAGE_UPLOAD_FAILED: 'Failed to upload image(s), please check the format of the uploaded file(s), it should be one of the following: PNG, JPG or JPEG.',
    },
    // ------- Admin -------
    admin: {
        ADMIN_DELETE_DEPENDENCIES: 'The admin you are trying to delete has multiple dependencies in Parking Lot, please delete the dependencies to continue.',
        ADMIN_DELETED_SUCCESS: 'Admin deleted successfully.',
        ADMIN_DETAILS_MISSING: 'Please provide all the user details.',
        ADMIN_ALREADY_EXISTS: 'Email already in use.',
        ADMIN_NOT_FOUND: 'Admin not found.',
        ACCOUNT_LOCKED: 'Too many failed login attempts. Your account has been locked, please contact the administration.',
        FAILED_LOGIN_ATTEMPT: 'Incorrect Password. ',
        PASSWORD_EXPIRED: 'Your password is more than 90 days old, please reset to continue.',
        FAILED_TEMPORARY_LOGIN: 'The temporary password you entered is incorrect, please check your email and enter the temporary password mentioned in the email.',
        WRONG_OLD_PASSWORD: 'The old password you entered is incorrect, please enter the correct old password to continue.',
        TEMPORARY_PASSWORD_CREATED: 'A temporary password has been sent to your email. Please check your inbox.',
        PASSWORD_UPDATE_SUCCESS: 'Your password has been updated. Use the new password to login.',
        ACCOUNT_UNLOCKED: 'The account has been unlocked.',
    },
    // ------- Sms/Email -------
    emailSmsNotification: {
        SMS_SENT_SUCCESS: `Message has been successfully sent to the user's phone number.`,
        SMS_SENT_FAILURE: `Message failed to send.`,
        EMAIL_SENT_SUCCESS: `Email has been successfully sent to the user's email address.`,
        EMAIL_SENT_FAILURE: `Email failed to send.`,
        NOTIFICATION_SENT_SUCCESS: `Notification sent successfully.`,
        NOTIFICATION_SENT_FAILURE: `Failed to send notification.`,
    },
    // ------- Absence -------
    absence: {
        ABSENCE_DETAILS_MISSING: 'Please provide all the Absence details.',
        ABSENCE_NOT_FOUND: 'Absence not found.',
        CANNOT_CANCEL_ABSENCE: 'Absence cannot be canceled as there are reservations already made against this spot.',
        CANNOT_DELETE_ABSENCE: 'Absence cannot be deleted as there are reservations already made against this spot.',
        ABSENCE_INVALID_STATUS: `Absence status can only be of the following ['pending', 'canceled', 'completed'].`,
        ABSENCE_DELETED_SUCCESS: 'Absence deleted successfully.',
        ABSENCE_UPDATE_SUCCESS: 'Absence updated successfully.',
        ALREADY_ABSENCE_EXISTS: 'An absence already exists for the specified date and spot number.',
    },
};
exports.default = HttpMessages;
