"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.scheduleCronJob = exports.findUsersWithMultipleReservations = void 0;
const node_cron_1 = __importDefault(require("node-cron"));
const dates_1 = require("../../helpers/dates");
const reservation_1 = __importDefault(require("../../models/reservation"));
const moment_1 = __importDefault(require("moment"));
const user_1 = __importDefault(require("../../models/user"));
const absence_1 = __importDefault(require("../../models/absence"));
const notifications_1 = require("../../middlewares/notifications");
const notification_1 = require("../constants/notification");
const announcement_1 = __importDefault(require("../../models/announcement"));
const helpers_1 = require("../../helpers");
const controllers_1 = require("../../v2.9.3/reservations/controllers");
const parking_spot_1 = __importDefault(require("../../models/parking-spot"));
const findUsersWithMultipleReservations = (getReservations) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { startOfDay, endOfDay } = (0, dates_1.getStartAndEndOfDay)(new Date());
        const filters = [
            {
                $match: {
                    reservationDate: {
                        $gte: new Date(startOfDay),
                        $lte: new Date(endOfDay),
                    },
                    parkingStatus: { $in: ["reserved", "occupied"] },
                },
            },
            {
                $lookup: {
                    from: "users",
                    localField: "user",
                    foreignField: "_id",
                    as: "userData",
                },
            },
            {
                $unwind: "$userData", // Flatten the userData array
            },
            {
                $group: {
                    _id: "$user",
                    count: { $sum: 1 },
                    user: { $first: "$userData" },
                    reservations: getReservations && { $push: "$$ROOT" },
                },
            },
            {
                $match: {
                    count: { $gt: 1 },
                },
            },
            {
                $project: {
                    _id: 1,
                    count: 1,
                    user: {
                        _id: 1,
                        email: 1,
                        phoneNumber: 1,
                    },
                    reservations: {
                        $cond: {
                            if: getReservations,
                            then: "$reservations",
                            else: [], // Set reservations to null if count is not greater than 1
                        },
                    },
                },
            },
        ];
        const reservations = yield reservation_1.default.aggregate(filters);
        return reservations;
    }
    catch (err) {
        console.log(err);
        return [];
    }
});
exports.findUsersWithMultipleReservations = findUsersWithMultipleReservations;
const findAbsencesEndedLastNight = () => __awaiter(void 0, void 0, void 0, function* () {
    const today = (0, dates_1.getCorrectDateTime)(new Date());
    yield absence_1.default.updateMany({ endDate: { $lte: today } }, { status: "completed" });
});
const testingLastDayUsers = () => __awaiter(void 0, void 0, void 0, function* () {
    console.log("Here");
});
const findReservationsWithElectricLimit = (time) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const today = new Date();
        const { startOfDay, endOfDay } = (0, dates_1.getStartAndEndOfDay)(today);
        const reservations = yield reservation_1.default.find({
            forMoving: false,
            parkingStatus: { $in: [controllers_1.reservationEnums[0], controllers_1.reservationEnums[1]] },
            reservationDate: { $gte: startOfDay, $lte: endOfDay },
        })
            .populate({
            path: "parkingLot",
            select: ["chargingTimeLimit", "chargingTimeEnd"],
            // match: {
            //     $and: [
            //         { parkingLot: { $exists: true } },
            //         { chargingTimeLimit: true },
            //         { chargingTimeEnd: time }
            //     ]
            // },
        })
            .populate({
            path: "user",
            select: ["phoneNumber", "deviceToken"],
        })
            .populate({
            path: "moveTo",
            select: ["parkingSpot"],
            populate: {
                path: "parkingSpot",
                select: "parkingCode",
            },
        });
        if ((reservations === null || reservations === void 0 ? void 0 : reservations.length) > 0) {
            yield Promise.all(reservations.map((reservation) => __awaiter(void 0, void 0, void 0, function* () {
                var _a, _b, _c, _d, _e, _f, _g;
                if (((_a = reservation === null || reservation === void 0 ? void 0 : reservation.user) === null || _a === void 0 ? void 0 : _a.deviceToken) &&
                    (reservation === null || reservation === void 0 ? void 0 : reservation.parkingLot) &&
                    ((_b = reservation === null || reservation === void 0 ? void 0 : reservation.parkingLot) === null || _b === void 0 ? void 0 : _b.chargingTimeEnd) == time &&
                    ((_d = (_c = reservation === null || reservation === void 0 ? void 0 : reservation.moveTo) === null || _c === void 0 ? void 0 : _c.parkingSpot) === null || _d === void 0 ? void 0 : _d.parkingCode)) {
                    const notification = ((_e = reservation === null || reservation === void 0 ? void 0 : reservation.user) === null || _e === void 0 ? void 0 : _e.deviceToken)
                        ? yield (0, notifications_1.sendNotification)((_f = reservation === null || reservation === void 0 ? void 0 : reservation.user) === null || _f === void 0 ? void 0 : _f.deviceToken, "הזמן המוקצה עבור החניה החשמלית שלך הסתיים, אנא פנה את הרכב למקום חניה פנוי", notification_1.NotificationTypes.electricReservationTimeLimitEnd, { reservation: String(reservation === null || reservation === void 0 ? void 0 : reservation._id) }, [(_g = reservation === null || reservation === void 0 ? void 0 : reservation.user) === null || _g === void 0 ? void 0 : _g._id])
                        : null;
                    // console.log(notification, 'findReservationsWithElectricLimitNotification')
                    console.log(`notification send status(${notification}) \nfor reservation ID: ` +
                        (reservation === null || reservation === void 0 ? void 0 : reservation._id));
                }
            })));
            // console.log(reservations?.length, 'findReservationsWithElectricLimit')
        }
    }
    catch (error) {
        console.log(error, "error while finding electric with limit");
    }
});
const findAnnouncementsWithTime = () => __awaiter(void 0, void 0, void 0, function* () {
    const currentDate = (0, dates_1.getCorrectDateTime)(new Date());
    let filter = {
        startDate: currentDate,
    };
    const announcements = yield announcement_1.default.find(filter).sort({ updatedAt: -1 });
    if (announcements.length > 0) {
        const announcementsText = [];
        const parkingLots = yield Promise.all(announcements.map(announcement => {
            announcementsText.push(announcement.content);
            return announcement.parkingLots;
        }));
        if (parkingLots.length > 0) {
            const allParkingLots = parkingLots.flat();
            const users = yield user_1.default.find({
                parkingLots: { $in: allParkingLots },
                $and: [
                    { deviceToken: { $ne: null } },
                    { deviceToken: { $ne: undefined } },
                ],
            });
            if (users.length > 0) {
                const deviceTokens = [];
                const userIds = [];
                yield Promise.all(users.map(user => {
                    userIds === null || userIds === void 0 ? void 0 : userIds.push(user === null || user === void 0 ? void 0 : user._id);
                    if (user.deviceToken != null)
                        deviceTokens.push(user.deviceToken);
                }));
                announcementsText.map((content) => __awaiter(void 0, void 0, void 0, function* () {
                    const notifications = yield (0, notifications_1.sendNotification)(deviceTokens, content, notification_1.NotificationTypes.announcement, null, userIds);
                }));
            }
        }
    }
});
const deleteGuestUsers = () => __awaiter(void 0, void 0, void 0, function* () {
    const { yesterday } = (0, dates_1.getTodayTomorrowYesterday)();
    const { endOfDay } = (0, dates_1.getStartAndEndOfDay)(yesterday);
    const filter = { reservationDate: { $lte: endOfDay } };
    const users = yield user_1.default.find({
        isGuest: true,
    });
    users.forEach((user) => __awaiter(void 0, void 0, void 0, function* () {
        const reservations = yield reservation_1.default.find(Object.assign({ user: user === null || user === void 0 ? void 0 : user._id }, filter));
        yield Promise.all(reservations === null || reservations === void 0 ? void 0 : reservations.map((reservation) => __awaiter(void 0, void 0, void 0, function* () {
            yield parking_spot_1.default.findByIdAndUpdate(reservation === null || reservation === void 0 ? void 0 : reservation.parkingSpot, {
                designatedTo: null,
                userPresenceDate: null,
            });
            yield reservation_1.default.findByIdAndDelete(reservation === null || reservation === void 0 ? void 0 : reservation._id);
        })));
        const remainingReservations = yield reservation_1.default.find({ user: user === null || user === void 0 ? void 0 : user._id });
        if (!remainingReservations.length) {
            yield user_1.default.deleteOne({ _id: user._id });
        }
    }));
});
const scheduleCronJob = () => {
    const oneAmTask = node_cron_1.default.schedule("0 1 * * *", () => __awaiter(void 0, void 0, void 0, function* () {
        console.log("Cron Job Started, 1am");
        yield findAbsencesEndedLastNight();
        yield deleteGuestUsers();
    }));
    const everyHourTask = node_cron_1.default.schedule("0 * * * *", () => __awaiter(void 0, void 0, void 0, function* () {
        console.log("Cron Job Started, hourly,", (0, dates_1.getCorrectDateTime)(new Date()));
        const date = (0, moment_1.default)();
        yield findReservationsWithElectricLimit(date.format("HH:mm"));
        // const currentHour = date.format('HH:mm')
        // const threeHoursBefore = date.subtract(3, 'hours').format('HH:mm')
        // console.log('performAbsenceAction sending notification for', currentHour)
        // console.log('performAbsenceAction creating absence for', threeHoursBefore)
        // await performAbsenceAction(currentHour, true)
        // await performAbsenceAction(threeHoursBefore, false, true)
    }));
    const everyMinuteTask = node_cron_1.default.schedule("* * * * *", () => __awaiter(void 0, void 0, void 0, function* () {
        console.log("Cron Job Started, every minute,", (0, dates_1.getCorrectDateTime)(new Date()));
        const date = (0, moment_1.default)();
        const currentTime = date.format("HH:mm");
        yield findAnnouncementsWithTime();
        yield (0, helpers_1.performDesignatedSpotAction)(currentTime, true, false);
        yield (0, helpers_1.performDesignatedSpotAction)(currentTime, false, true);
    }));
    console.log("Cron job scheduled");
};
exports.scheduleCronJob = scheduleCronJob;
