"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteAbsenceService = exports.updateAbsenceService = exports.getUserAbsenceStatusForDateService = exports.getSpotAbsencesService = exports.getSingleAbsenceService = exports.getAllAbsencesService = exports.createAbsenceService = void 0;
const absence_1 = __importDefault(require("../../models/absence"));
const http_messages_1 = __importDefault(require("../../services/constants/http-messages"));
const http_statuses_1 = __importDefault(require("../../services/constants/http-statuses"));
const dates_1 = require("../../helpers/dates");
const parking_spot_1 = __importDefault(require("../../models/parking-spot"));
const controllers_1 = require("../reservations/controllers");
const reservation_1 = __importDefault(require("../../models/reservation"));
const moment_1 = __importDefault(require("moment"));
const absenceEnums = ['pending', 'completed', 'canceled'];
const createAbsenceService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { user, designatedSpot, startDate, endDate, note, type, } = req.body;
        if (user && designatedSpot && startDate && endDate) {
            const absence = new absence_1.default({
                user,
                designatedSpot,
                startDate: new Date(startDate).setUTCHours(0, 0, 0, 1),
                endDate: new Date(endDate).setUTCHours(23, 59, 59, 999),
                note,
                type: type ? type : 'user-created',
            });
            const saved = yield absence.save();
            res.status(http_statuses_1.default.CREATED).json(saved);
        }
        else
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.absence.ABSENCE_DETAILS_MISSING });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.createAbsenceService = createAbsenceService;
const getAllAbsencesService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { page = 1, limit = 10, status, user, designatedSpot } = req.query;
        let filter = {
            endDate: { $gte: (0, dates_1.getCorrectDateTime)(new Date()) },
        };
        if (status)
            filter.status = status;
        if (user) {
            filter.user = user;
            filter.type = 'user-created';
        }
        if (designatedSpot)
            filter.designatedSpot = designatedSpot;
        const totalCount = yield absence_1.default.countDocuments(filter);
        const totalPages = Math.ceil(totalCount / limit);
        const absences = yield absence_1.default.find(filter).sort({ startDate: -1, }).limit(limit).skip(limit * (page - 1)).populate({ path: 'designatedSpot', select: ['parkingLot', 'parkingCode'], populate: { path: 'parkingLot', select: ['name'] } });
        res.status(http_statuses_1.default.OK).json({ totalPages, totalCount, absences });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getAllAbsencesService = getAllAbsencesService;
const getSingleAbsenceService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const absence = yield absence_1.default.findById(_id);
        if (absence)
            res.status(http_statuses_1.default.OK).json(absence);
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.absence.ABSENCE_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getSingleAbsenceService = getSingleAbsenceService;
const getSpotAbsencesService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const absences = yield absence_1.default.find({ designatedSpot: _id });
        if (absences)
            res.status(http_statuses_1.default.OK).json({ absences });
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.absence.ABSENCE_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getSpotAbsencesService = getSpotAbsencesService;
const getUserAbsenceStatusForDateService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { checkDate, checkTime } = req.query;
        const date = checkDate ? new Date(checkDate) : new Date();
        if (!_id || _id == undefined || _id == 'undefined')
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.parkingSpot.SPOT_ID_NOT_ENTERED });
        else {
            const spot = yield parking_spot_1.default.findById(_id).populate({ path: 'parkingLot', select: 'messageSendTime' });
            if (spot) {
                const startOfDay = new Date(date.getFullYear(), date.getMonth(), date.getDate(), 0, 0, 0);
                const endOfDay = new Date(date.getFullYear(), date.getMonth(), date.getDate(), 23, 59, 59);
                const userPresenceDate = spot.userPresenceDate;
                if (userPresenceDate >= startOfDay && userPresenceDate <= endOfDay)
                    res.status(http_statuses_1.default.OK).json({ status: true, message: 'Already submitted' });
                else {
                    const absence = yield absence_1.default.findOne({ designatedSpot: _id, startDate: { $lte: date }, endDate: { $gte: date }, status: 'pending' });
                    if (absence)
                        res.status(http_statuses_1.default.OK).json({ status: true, message: 'Already have an absence' });
                    else {
                        const timeDiff = (0, dates_1.getTimeRelation)(checkTime ? checkTime : '05:00', spot === null || spot === void 0 ? void 0 : spot.parkingLot.messageSendTime);
                        if (timeDiff == 'within-2-hours')
                            res.status(http_statuses_1.default.OK).json({ status: false, message: 'Time is in between two hours' });
                        else if (timeDiff == 'before')
                            res.status(http_statuses_1.default.OK).json({ status: true, message: 'It is too soon' });
                        else if (timeDiff == 'after' || timeDiff == 'within-4-hours')
                            res.status(http_statuses_1.default.OK).json({ status: true, message: 'Time has passed' });
                    }
                }
            }
            else
                res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.parkingSpot.PARKING_SPOT_NOT_FOUND });
        }
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getUserAbsenceStatusForDateService = getUserAbsenceStatusForDateService;
const updateAbsenceService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { status } = req.body;
        if (absenceEnums.includes(status)) {
            if (status == 'canceled') {
                const absence = yield absence_1.default.findById(_id);
                if (absence) {
                    const query = {
                        $and: [
                            {
                                parkingSpot: absence.designatedSpot
                            },
                            {
                                $or: [
                                    { parkingStatus: { $ne: controllers_1.reservationEnums[0] } },
                                    { parkingStatus: { $ne: controllers_1.reservationEnums[1] } }
                                ]
                            },
                            {
                                reservationDate: { $gte: absence.startDate, $lte: absence.endDate }
                            }
                        ]
                    };
                    const reservations = yield reservation_1.default.find(query);
                    if (reservations.length > 0) {
                        res.status(http_statuses_1.default.FORBIDDEN).json({ message: http_messages_1.default.absence.CANNOT_CANCEL_ABSENCE + ` Reservation Date(s): ${reservations.map(reservation => (0, moment_1.default)(reservation.reservationDate).format('DD-MM-YYYY')).join(', ')}.` });
                    }
                    else {
                        const absence = yield absence_1.default.findByIdAndUpdate(_id, { status, }, { new: true, omitUndefined: true, });
                        res.status(http_statuses_1.default.OK).json(absence);
                    }
                }
                else
                    res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.absence.ABSENCE_NOT_FOUND });
            }
            else {
                const absence = yield absence_1.default.findByIdAndUpdate(_id, { status, }, { new: true, omitUndefined: true, });
                if (absence)
                    res.status(http_statuses_1.default.OK).json({ message: http_messages_1.default.absence.ABSENCE_UPDATE_SUCCESS });
                else
                    res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.absence.ABSENCE_NOT_FOUND });
            }
        }
        else
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.absence.ABSENCE_INVALID_STATUS });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.updateAbsenceService = updateAbsenceService;
const deleteAbsenceService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const absence = yield absence_1.default.findById(_id);
        if (absence) {
            // const query = {
            //     $and: [
            //         { parkingSpot: absence.designatedSpot },
            //         { parkingStatus: { $in: [reservationEnums[0], reservationEnums[1]] } },
            //         { reservationDate: { $gte: absence.startDate, $lte: absence.endDate } }
            //     ]
            // }
            const reservations = yield reservation_1.default.find({ parkingSpot: absence.designatedSpot, parkingStatus: { $in: [controllers_1.reservationEnums[0], controllers_1.reservationEnums[1]] }, reservationDate: { $gte: absence.startDate, $lte: absence.endDate } });
            if ((reservations === null || reservations === void 0 ? void 0 : reservations.length) > 0) {
                res.status(http_statuses_1.default.FORBIDDEN).json({ message: http_messages_1.default.absence.CANNOT_DELETE_ABSENCE });
            }
            else {
                yield absence_1.default.findByIdAndDelete(_id);
                res.status(http_statuses_1.default.OK).json({ message: http_messages_1.default.absence.ABSENCE_DELETED_SUCCESS });
            }
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.absence.ABSENCE_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.deleteAbsenceService = deleteAbsenceService;
