"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteComplaintService = exports.spotTakenComplaintService = exports.updateComplaintService = exports.getSingleComplaintService = exports.getComplaintsReportService = exports.getAllComplaintsService = exports.createComplaintService = void 0;
const http_messages_1 = __importDefault(require("../../services/constants/http-messages"));
const http_statuses_1 = __importDefault(require("../../services/constants/http-statuses"));
const complaint_1 = __importDefault(require("../../models/complaint"));
const parking_spot_1 = __importDefault(require("../../models/parking-spot"));
const user_1 = __importDefault(require("../../models/user"));
const admin_1 = __importDefault(require("../../models/admin"));
const parking_lot_1 = __importDefault(require("../../models/parking-lot"));
const email_messaging_1 = require("../../middlewares/email-messaging");
const vehicle_1 = __importDefault(require("../../models/vehicle"));
const generators_1 = require("../../helpers/generators");
const complaintEnums = ['pending', 'opened', 'in-progress', 'closed', 'completed'];
const createComplaintService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { parkingLot, complaintBy, details, } = req.body;
        if (complaintBy && details) {
            const complaint = new complaint_1.default({
                parkingLot,
                complaintBy,
                details,
            });
            if (parkingLot) {
                const found = yield parking_lot_1.default.findById(parkingLot).populate({ path: 'admins', select: ['phoneNumber'] }).select(['name']);
                const user = yield user_1.default.findById(complaintBy).select(['firstName', 'phoneNumber']);
                if (found && user) {
                    const { admins } = found;
                    if (admins.length > 0) {
                        const message = `דיווח משתמש בחניון ${found.name}, ממשתמש בשם: ${user.firstName} טלפון: ${user.phoneNumber.number}. ההודעה: ${details}`;
                        yield Promise.all(admins.map((admin) => __awaiter(void 0, void 0, void 0, function* () {
                            return yield (0, email_messaging_1.sendMessage)(admin.phoneNumber, message);
                        })));
                    }
                }
            }
            yield complaint.save();
            res.status(http_statuses_1.default.CREATED).json(complaint);
        }
        else
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.complaint.COMPLAINT_DETAILS_MISSING });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.createComplaintService = createComplaintService;
const getAllComplaintsService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { page = 1, limit = 10, status, complaintBy, currentAdmin } = req.query;
        let filter = {};
        if (currentAdmin) {
            const admin = yield admin_1.default.findById(currentAdmin).select(['parkingLots', 'role']);
            if (admin && admin.role == 'sub-admin') {
                const { parkingLots } = admin;
                filter.parkingLot = { $in: parkingLots };
            }
        }
        if (status)
            filter.status = status;
        if (complaintBy)
            filter.complaintBy = complaintBy;
        const totalCount = yield complaint_1.default.countDocuments(filter);
        const totalPages = Math.ceil(totalCount / limit);
        const complaints = yield complaint_1.default.find(filter).sort({ updatedAt: -1, }).limit(limit).skip(limit * (page - 1))
            .populate('parkingLot', ['name',])
            .populate('complaintBy', ['firstName', 'middleName', 'lastName', 'phoneNumber', 'email',]);
        res.status(http_statuses_1.default.OK).json({ totalPages, totalCount, complaints });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getAllComplaintsService = getAllComplaintsService;
const getComplaintsReportService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { currentAdmin } = req.query;
        let filter = {};
        if (currentAdmin) {
            const admin = yield admin_1.default.findById(currentAdmin).select(['parkingLots', 'role']);
            if (admin && admin.role == 'sub-admin') {
                const { parkingLots } = admin;
                filter.parkingLot = { $in: parkingLots };
            }
        }
        const complaints = yield complaint_1.default.find(filter).sort({ createdAt: -1, })
            .populate('parkingLot', ['name',])
            .populate('complaintBy', ['firstName', 'middleName', 'lastName', 'phoneNumber', 'email',]);
        const finalComplaints = yield Promise.all(complaints.map((complaint) => {
            return Object.assign(Object.assign({}, complaint.toObject()), { status: (complaint === null || complaint === void 0 ? void 0 : complaint.status) == 'pending' ? 'פתוח' : (complaint === null || complaint === void 0 ? void 0 : complaint.status) == 'completed' ? 'סגור' : complaint === null || complaint === void 0 ? void 0 : complaint.status });
        }));
        const { filePath } = yield (0, generators_1.generateComplaintsReportCsv)(finalComplaints);
        res.status(http_statuses_1.default.OK).json({ path: process.env.LIVE_BASE_URL + filePath });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getComplaintsReportService = getComplaintsReportService;
const getSingleComplaintService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const complaint = yield complaint_1.default.findById(_id);
        if (complaint)
            res.status(http_statuses_1.default.OK).json(complaint);
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.complaint.COMPLAINT_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getSingleComplaintService = getSingleComplaintService;
const updateComplaintService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { status } = req.body;
        if (complaintEnums.includes(status)) {
            const complaint = yield complaint_1.default.findByIdAndUpdate(_id, { status, }, { new: true, omitUndefined: true, });
            if (complaint)
                res.status(http_statuses_1.default.OK).json(complaint);
            else
                res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.complaint.COMPLAINT_NOT_FOUND });
        }
        else
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.complaint.COMPLAINT_INVALID_STATUS });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.updateComplaintService = updateComplaintService;
const spotTakenComplaintService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    var _a, _b, _c, _d;
    try {
        const { complaintBy, registration, parkingSpot, lang = "heb", } = req.body;
        const complaintByUser = yield user_1.default.findById(complaintBy).populate('vehicles');
        if (complaintByUser) {
            const complaintAgainstVehicle = yield vehicle_1.default.findOne({ registration: { $regex: new RegExp(registration.toUpperCase(), 'i') } });
            if (complaintAgainstVehicle) {
                const user = yield user_1.default.findById(complaintAgainstVehicle.user);
                if (user) {
                    const spot = yield parking_spot_1.default.findById(parkingSpot).populate({ path: 'parkingLot', select: ['admins'], populate: { path: 'admins', select: 'phoneNumber' } });
                    if (spot) {
                        const complaintAgainstUser = yield user_1.default.findById(user._id).populate('vehicles');
                        if (complaintAgainstUser) {
                            const sms = `שם המתלונן: ${complaintByUser.firstName} ${complaintByUser.lastName}
מספר טלפון המתלונן:  ${complaintByUser.phoneNumber.number}
מספר רכב חוסם: ${complaintAgainstVehicle.registration}
שם רכב החוסם: ${complaintAgainstUser.firstName} ${complaintAgainstUser.lastName}
מספר טלפון רכב חוסם:  ${complaintAgainstUser.phoneNumber.number}
מספר חניה שדווחה כחסומה: ${spot === null || spot === void 0 ? void 0 : spot.parkingCode}`;
                            //                             const sms = `שם המתלונן: ${complaintByUser.firstName} ${complaintByUser.lastName}
                            // מספר טלפון המתלונן:  ${complaintByUser.phoneNumber.number}
                            // מספר רכב חוסם: ${complaintByUser.vehicles?.[0]?.registration}
                            // שם רכב החוסם: ${complaintAgainstUser.firstName} ${complaintAgainstUser.lastName}
                            // מספר טלפון רכב חוסם:  ${complaintAgainstUser.phoneNumber.number}
                            // מספר חניה שדווחה כחסומה: ${spot?.parkingCode}`
                            //                             const sms = `שם המתלונן: ${complaintByUser.firstName} ${complaintByUser.lastName}\n
                            // מספר טלפון המתלונן: ${complaintByUser.phoneNumber.countryCode + complaintByUser.phoneNumber.number}
                            // מספר רכב חוסם: ${registration}
                            // מספר טלפון רכב חוסם: ${complaintAgainstUser.phoneNumber.countryCode + complaintAgainstUser.phoneNumber.number}
                            // מספר חניה שדווחה כחסומה: ${spot?.parkingCode}`
                            const complaint = new complaint_1.default({
                                parkingLot: (_a = spot.parkingLot) === null || _a === void 0 ? void 0 : _a._id,
                                complaintBy,
                                // details: `Car with the plate number ${registration} was reported to unauthorizedly park in parking spot ${spot?.parkingCode}.\n\n${complaintAgainstUser?.phoneNumber?.countryCode + complaintAgainstUser?.phoneNumber?.number}`,
                                details: sms
                            });
                            const messagesSent = [];
                            if ((_b = spot === null || spot === void 0 ? void 0 : spot.parkingLot) === null || _b === void 0 ? void 0 : _b.admins) {
                                yield Promise.all((_d = (_c = spot === null || spot === void 0 ? void 0 : spot.parkingLot) === null || _c === void 0 ? void 0 : _c.admins) === null || _d === void 0 ? void 0 : _d.map((admin) => __awaiter(void 0, void 0, void 0, function* () {
                                    const messageSent = yield (0, email_messaging_1.sendMessage)(admin === null || admin === void 0 ? void 0 : admin.phoneNumber, sms);
                                    messagesSent.push(messageSent);
                                })));
                            }
                            const saved = yield complaint.save();
                            res.status(http_statuses_1.default.CREATED).json({ complaint: saved, messagesSent });
                        }
                        else
                            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.user.USER_NOT_FOUND });
                    }
                    else
                        res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.parkingSpot.PARKING_SPOT_NOT_FOUND });
                }
                else
                    res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.user.USER_NOT_FOUND });
            }
            else
                res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.vehicle.VEHICLE_NOT_FOUND });
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.user.USER_NOT_FOUND });
        // if (spot) {
        //     if (complaintByUser) {
        //         const complaintAgainstUser = await User.findById(vehicle?.user)
        //         if (complaintAgainstUser) {
        //             const complaint = new Complaint({
        //                 parkingLot: spot.parkingLot,
        //                 complaintBy,
        //                 details: `Car with the plate number ${registration} was reported to unauthorizedly park in parking spot ${spot?.parkingCode}.\n\n${complaintAgainstUser?.phoneNumber?.countryCode + complaintAgainstUser?.phoneNumber?.number}`,
        //             })
        //             const saved = await complaint.save()
        //             res.status(HttpStatus.CREATED).json({ complaint: saved })
        //         } else res.status(HttpStatus.NOT_FOUND).json({ message: HttpMessages.user.USER_NOT_FOUND })
        //     } else res.status(HttpStatus.NOT_FOUND).json({ message: HttpMessages.user.USER_NOT_FOUND })
        // } else res.status(HttpStatus.NOT_FOUND).json({ message: HttpMessages.parkingSpot.PARKING_SPOT_NOT_FOUND })
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.spotTakenComplaintService = spotTakenComplaintService;
const deleteComplaintService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const complaint = yield complaint_1.default.findByIdAndDelete(_id);
        if (complaint)
            res.status(http_statuses_1.default.OK).json({ message: http_messages_1.default.complaint.COMPLAINT_DELETED_SUCCESS });
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.complaint.COMPLAINT_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.deleteComplaintService = deleteComplaintService;
