"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteHolidayService = exports.updateHolidayService = exports.getSingleHolidayService = exports.getAllHolidaysService = exports.createHolidayService = void 0;
const http_messages_1 = __importDefault(require("../../services/constants/http-messages"));
const http_statuses_1 = __importDefault(require("../../services/constants/http-statuses"));
const holiday_1 = __importDefault(require("../../models/holiday"));
const createHolidayService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { name, startDate, endDate, startTime, endTime } = req.body;
        if (startDate && endDate && startTime && endTime) {
            const holiday = new holiday_1.default({
                name,
                startDate,
                endDate,
                startTime,
                endTime
            });
            yield holiday.save();
            res.status(http_statuses_1.default.CREATED).json(holiday);
        }
        else
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.holiday.HOLIDAY_DETAILS_MISSING });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.createHolidayService = createHolidayService;
const getAllHolidaysService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const holidays = yield holiday_1.default.find().sort({ startDate: 1, });
        res.status(http_statuses_1.default.OK).json(holidays);
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getAllHolidaysService = getAllHolidaysService;
const getSingleHolidayService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const holiday = yield holiday_1.default.findById(_id);
        if (holiday)
            res.status(http_statuses_1.default.OK).json(holiday);
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.complaint.COMPLAINT_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getSingleHolidayService = getSingleHolidayService;
const updateHolidayService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { name, startDate, endDate, startTime, endTime } = req.body;
        const holiday = yield holiday_1.default.findByIdAndUpdate(_id, {
            name,
            startDate,
            endDate,
            startTime,
            endTime
        }, { new: true, omitUndefined: true });
        if (holiday)
            res.status(http_statuses_1.default.OK).json(holiday);
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.holiday.HOLIDAY_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.updateHolidayService = updateHolidayService;
const deleteHolidayService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const holiday = yield holiday_1.default.findByIdAndDelete(_id);
        if (holiday)
            res.status(http_statuses_1.default.OK).json({ message: http_messages_1.default.holiday.HOLIDAY_DELETED_SUCCESS });
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.holiday.HOLIDAY_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.deleteHolidayService = deleteHolidayService;
