"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteParkingSpotService = exports.updateParkingSpotService = exports.getLevelAvailableSpotsService = exports.getLotLevelSpotsService = exports.getLotMonitorSpotsService = exports.getParkingLotParkingSpotsService = exports.getSingleParkingSpotService = exports.getAllParkingSpotsService = exports.createParkingSpotService = void 0;
const http_messages_1 = __importDefault(require("../../services/constants/http-messages"));
const http_statuses_1 = __importDefault(require("../../services/constants/http-statuses"));
const parking_lot_1 = __importDefault(require("../../models/parking-lot"));
const lot_level_1 = __importDefault(require("../../models/lot-level"));
const parking_spot_1 = __importDefault(require("../../models/parking-spot"));
const absence_1 = __importDefault(require("../../models/absence"));
const dates_1 = require("../../helpers/dates");
const reservation_1 = __importDefault(require("../../models/reservation"));
const controllers_1 = require("../reservations/controllers");
const helpers_1 = require("../../helpers");
const createParkingSpotService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { parkingLot, name, avatar, level, parkingCode, spotType } = req.body;
        if (parkingLot && level && parkingCode) {
            const lot = yield parking_lot_1.default.findById(parkingLot);
            if (lot) {
                const parkingLevel = yield lot_level_1.default.findById(level);
                if (parkingLevel) {
                    const code = yield parking_spot_1.default.findOne({ parkingLot, level, parkingCode });
                    if (code) {
                        res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.parkingSpot.PARKING_CODE_ALREADY_IN_DB });
                    }
                    else {
                        const parkingSpot = new parking_spot_1.default({
                            name,
                            avatar,
                            parkingLot,
                            level,
                            spotType,
                            parkingCode,
                        });
                        yield parkingSpot.save();
                        res.status(http_statuses_1.default.CREATED).json(parkingSpot);
                    }
                }
                else
                    res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.lotLevel.LEVEL_NOT_FOUND });
            }
            else
                res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.parkingLot.PARKING_LOT_NOT_FOUND });
        }
        else
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.parkingSpot.PARKING_SPOT_DETAILS_MISSING });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.createParkingSpotService = createParkingSpotService;
const getAllParkingSpotsService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { page = 1, limit = 10, type, isAvailable } = req.query;
        let filter = {};
        if (type != undefined)
            filter.spotType = type;
        if (type == 'designated' && (isAvailable == 'true' || isAvailable == true))
            filter.designatedTo = null;
        if (type == 'designated' && (isAvailable == 'false' || isAvailable == false))
            filter.designatedTo = { $ne: null };
        const totalCount = yield parking_spot_1.default.countDocuments(filter);
        const totalPages = Math.ceil(totalCount / limit);
        const parkingSpots = yield parking_spot_1.default.find(filter).sort({ updatedAt: -1 }).limit(limit).skip(limit * (page - 1));
        res.status(http_statuses_1.default.OK).json({ totalPages, totalCount, parkingSpots });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getAllParkingSpotsService = getAllParkingSpotsService;
const getSingleParkingSpotService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const parkingSpot = yield parking_spot_1.default.findById(_id);
        if (parkingSpot) {
            res.status(http_statuses_1.default.OK).json(parkingSpot);
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.parkingSpot.PARKING_SPOT_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getSingleParkingSpotService = getSingleParkingSpotService;
const getParkingLotParkingSpotsService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { page = 1, limit = 10, type, isAvailable, sortParkingCodes } = req.query;
        let filter = { parkingLot: _id };
        if (type != undefined)
            filter.spotType = type;
        if (type == 'designated' && (isAvailable == 'true' || isAvailable == true))
            filter.designatedTo = null;
        if (type == 'designated' && (isAvailable == 'false' || isAvailable == false))
            filter.designatedTo = { $ne: null };
        const parkingLot = yield parking_lot_1.default.findById(_id);
        if (parkingLot) {
            const sort = sortParkingCodes == 'true' || sortParkingCodes == true ? { parkingCode: 1 } : { _id: 1 };
            // const sort: {} = parkingLot.allotmentMethod == 'manual' ? { createdAt: 1 } : { parkingCode: 1 }
            const totalCount = yield parking_spot_1.default.countDocuments(filter);
            const totalPages = Math.ceil(totalCount / limit);
            const parkingSpots = yield parking_spot_1.default.find(filter).sort(sort).limit(limit).skip(limit * (page - 1));
            res.status(http_statuses_1.default.OK).json({ totalPages, totalCount, parkingSpots });
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.parkingLot.PARKING_LOT_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getParkingLotParkingSpotsService = getParkingLotParkingSpotsService;
const getLotMonitorSpotsService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { startDate = new Date(), endDate = new Date(), } = req.query;
        let filter = { parkingLot: _id };
        const parkingLot = yield parking_lot_1.default.findById(_id);
        if (parkingLot) {
            const sort = { parkingCode: 1 };
            const { startOfDay, } = (0, dates_1.getStartAndEndOfDay)(startDate);
            const { endOfDay } = (0, dates_1.getStartAndEndOfDay)(endDate);
            const levels = yield lot_level_1.default.find({ parkingLot: _id });
            const parkingSpots = yield parking_spot_1.default.find(filter).sort(sort).populate({
                path: 'designatedTo',
                select: ['-password', '-temporaryPassword'],
                populate: [{
                        path: 'parkingLots',
                    }, {
                        path: 'designatedParkingSpot',
                        populate: 'parkingLot'
                    }, {
                        path: 'vehicles'
                    }]
            });
            const reservations = yield reservation_1.default.find({ parkingStatus: { $nin: [controllers_1.reservationEnums[3], controllers_1.reservationEnums[4]] }, reservationDate: { $gte: startOfDay, $lte: endOfDay } })
                .populate({
                path: 'user',
                select: ['-password', '-temporaryPassword'],
                populate: [{
                        path: 'parkingLots',
                    }, {
                        path: 'designatedParkingSpot',
                        populate: 'parkingLot'
                    }, {
                        path: 'vehicles'
                    }]
            });
            const finalSpots = yield Promise.all(parkingSpots.map((spot) => __awaiter(void 0, void 0, void 0, function* () {
                var _a, _b, _c, _d, _e;
                const reservationIndex = yield Promise.resolve(reservations.findIndex((reservation) => String(reservation.parkingSpot) == String(spot._id)));
                const absence = yield absence_1.default.findOne({ designatedSpot: spot === null || spot === void 0 ? void 0 : spot._id, startDate: { $lte: startDate }, endDate: { $gte: endDate } });
                if (reservationIndex < 0) {
                    if (((_a = spot === null || spot === void 0 ? void 0 : spot.designatedTo) === null || _a === void 0 ? void 0 : _a.parkingLots) && ((_c = (_b = spot === null || spot === void 0 ? void 0 : spot.designatedTo) === null || _b === void 0 ? void 0 : _b.parkingLots) === null || _c === void 0 ? void 0 : _c.length) > 0) {
                        const userWithDesignatedSpots = yield Promise.all((_d = spot === null || spot === void 0 ? void 0 : spot.designatedTo) === null || _d === void 0 ? void 0 : _d.parkingLots.map((parkingLot) => __awaiter(void 0, void 0, void 0, function* () {
                            var _f;
                            const designatedSpots = yield parking_spot_1.default.find({ parkingLot: parkingLot._id, spotType: 'designated', designatedTo: (_f = spot === null || spot === void 0 ? void 0 : spot.designatedTo) === null || _f === void 0 ? void 0 : _f._id });
                            return Object.assign(Object.assign({}, parkingLot.toObject()), { designatedSpots });
                        })));
                        const designatedTo = (spot === null || spot === void 0 ? void 0 : spot.designatedTo) ? Object.assign(Object.assign({}, (_e = spot === null || spot === void 0 ? void 0 : spot.designatedTo) === null || _e === void 0 ? void 0 : _e.toObject()), { parkingLots: userWithDesignatedSpots }) : null;
                        return Object.assign(Object.assign({}, spot.toObject()), { status: absence && designatedTo ? 'released' : 'designated-not-released', designatedTo, reservedBy: null });
                    }
                    else
                        return Object.assign(Object.assign({}, spot.toObject()), { status: 'free', reservedBy: null });
                }
                else {
                    const userWithDesignatedSpots = yield Promise.all(reservations[reservationIndex].user.parkingLots.map((parkingLot) => __awaiter(void 0, void 0, void 0, function* () {
                        const designatedSpots = yield parking_spot_1.default.find({ parkingLot: parkingLot._id, spotType: 'designated', designatedTo: null });
                        return Object.assign(Object.assign({}, parkingLot.toObject()), { designatedSpots });
                    })));
                    return Object.assign(Object.assign({}, spot.toObject()), { status: 'reserved', reservedBy: Object.assign(Object.assign({}, reservations[reservationIndex].user.toObject()), { parkingLots: userWithDesignatedSpots }) });
                }
            })));
            let totalCount = 0;
            let totalReservedCount = 0;
            let totalFreeCount = 0;
            let totalElectricCount = 0;
            let totalDesignatedCount = 0;
            let totalRegularCount = 0;
            let totalDisabledCount = 0;
            const levelsWithSpots = yield Promise.all(levels.map((level) => __awaiter(void 0, void 0, void 0, function* () {
                const spots = yield Promise.all(finalSpots.filter((spot) => String(spot.level) == String(level._id)));
                const reserved = yield Promise.all(spots.filter((spot) => spot.status == 'reserved'));
                const free = yield Promise.all(spots.filter((spot) => spot.status == 'free'));
                const electric = yield Promise.all(spots.filter((spot) => spot.spotType == 'electric'));
                const designated = yield Promise.all(spots.filter((spot) => spot.spotType == 'designated'));
                const designatedReleased = yield Promise.all(spots.filter((spot) => spot.spotType == 'designated' && (spot === null || spot === void 0 ? void 0 : spot.status) == 'released'));
                const regular = yield Promise.all(spots.filter((spot) => spot.spotType == 'regular'));
                const disabled = yield Promise.all(spots.filter((spot) => spot.spotType == 'disabled'));
                totalCount = totalCount + spots.length;
                totalReservedCount = totalReservedCount + reserved.length;
                totalFreeCount = totalFreeCount + free.length + (designatedReleased === null || designatedReleased === void 0 ? void 0 : designatedReleased.length);
                totalElectricCount = totalElectricCount + electric.length;
                totalDesignatedCount = totalDesignatedCount + designated.length;
                totalRegularCount = totalRegularCount + regular.length;
                totalDisabledCount = totalDisabledCount + disabled.length;
                return {
                    level: Object.assign(Object.assign({}, level.toObject()), { totalCount: spots.length, reservedCount: reserved.length, electricCount: electric.length, designatedCount: designated.length, regularCount: regular.length, disabledCount: disabled.length, freeCount: free.length + (designatedReleased === null || designatedReleased === void 0 ? void 0 : designatedReleased.length) }),
                    parkingSpots: spots.sort(helpers_1.customSort)
                };
            })));
            res.status(http_statuses_1.default.OK).json({
                totalCount,
                totalReservedCount,
                totalFreeCount,
                totalElectricCount,
                totalDesignatedCount,
                totalRegularCount,
                totalDisabledCount,
                levels: levelsWithSpots,
            });
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.parkingLot.PARKING_LOT_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getLotMonitorSpotsService = getLotMonitorSpotsService;
const getLotLevelSpotsService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { page = 1, limit = 10, type, isAvailable, isDisabled, isElectric } = req.query;
        let filter = { level: _id, };
        if (type != undefined)
            filter.spotType = type;
        if (type == 'designated' && isAvailable == 'true')
            filter.designatedTo = null;
        if (type == 'designated' && isAvailable == 'false')
            filter.designatedTo = { $ne: null };
        if (isDisabled == 'false')
            filter.spotType = Object.assign(Object.assign({}, filter.spotType), { $nin: ['disabled'] });
        if (isElectric == 'false')
            filter.spotType = Object.assign(Object.assign({}, filter.spotType), { $nin: ['electric'] });
        if (isDisabled == 'false' && isElectric == 'false')
            filter.spotType = { $nin: ['disabled', 'electric'] };
        const level = yield lot_level_1.default.findById(_id);
        if (level) {
            const parkingLot = yield parking_lot_1.default.findById(level.parkingLot);
            if (parkingLot) {
                const sort = { _id: 1 };
                const totalCount = yield parking_spot_1.default.countDocuments(filter);
                const totalPages = Math.ceil(totalCount / limit);
                const parkingSpots = yield parking_spot_1.default.find(filter).sort(sort).limit(limit).skip(limit * (page - 1));
                if (parkingSpots) {
                    res.status(http_statuses_1.default.OK).json({ totalPages, totalCount, parkingSpots });
                }
                else
                    res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.lotLevel.LEVEL_NOT_FOUND });
            }
            else
                res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.parkingLot.PARKING_LOT_NOT_FOUND });
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.lotLevel.LEVEL_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getLotLevelSpotsService = getLotLevelSpotsService;
const getLevelAvailableSpotsService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const level = yield lot_level_1.default.findById(_id);
        if (level) {
            const parkingLot = yield parking_lot_1.default.findById(level.parkingLot);
            if (parkingLot) {
                const { availableSpots, totalCount } = yield (0, helpers_1.findAvailableSpots)(req);
                res.status(http_statuses_1.default.OK).json({ parkingSpots: availableSpots, totalCount });
            }
            else
                res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.parkingLot.PARKING_LOT_NOT_FOUND });
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.lotLevel.LEVEL_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getLevelAvailableSpotsService = getLevelAvailableSpotsService;
const updateParkingSpotService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { name, avatar, userPresenceDate } = req.body;
        const parkingSpot = yield parking_spot_1.default.findByIdAndUpdate(_id, { name, avatar, userPresenceDate }, { omitUndefined: true, new: true });
        if (parkingSpot) {
            let response = { parkingSpot };
            if (userPresenceDate == null) {
                const absence = new absence_1.default({
                    user: parkingSpot.designatedTo,
                    designatedSpot: parkingSpot._id,
                    startDate: new Date().setUTCHours(0, 0, 0, 0),
                    endDate: new Date().setUTCHours(23, 59, 59, 999),
                    type: 'system-generated',
                });
                const saved = yield absence.save();
                if (saved)
                    response.absence = saved;
            }
            res.status(http_statuses_1.default.OK).json(response);
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.parkingSpot.PARKING_SPOT_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.updateParkingSpotService = updateParkingSpotService;
const deleteParkingSpotService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const parkingSpot = yield parking_spot_1.default.findByIdAndDelete(_id);
        if (parkingSpot) {
            res.status(http_statuses_1.default.OK).json({ message: http_messages_1.default.parkingSpot.PARKING_SPOT_DELETED_SUCCESS });
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.parkingSpot.PARKING_SPOT_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.deleteParkingSpotService = deleteParkingSpotService;
