"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteUserService = exports.deleteAllUsersService = exports.unlockUserAccountService = exports.resetPasswordService = exports.loginWithTemporaryPasswordService = exports.forgotPasswordService = exports.logoutUserService = exports.loginUserService = exports.updateUserService = exports.getUsersReportService = exports.getAllUsersService = exports.getAdminLotsUsersService = exports.getSingleUserService = exports.createMultipleUsersService = exports.createUserService = void 0;
const express_1 = __importDefault(require("express"));
const http_messages_1 = __importDefault(require("../../services/constants/http-messages"));
const http_statuses_1 = __importDefault(require("../../services/constants/http-statuses"));
const validators_1 = require("../../helpers/validators");
const generators_1 = require("../../helpers/generators");
const authentication_1 = require("../../middlewares/authentication");
const app = (0, express_1.default)();
const user_1 = __importDefault(require("../../models/user"));
const parking_spot_1 = __importDefault(require("../../models/parking-spot"));
const vehicle_1 = __importDefault(require("../../models/vehicle"));
const email_messaging_1 = require("../../middlewares/email-messaging");
const reservation_1 = __importDefault(require("../../models/reservation"));
const dates_1 = require("../../helpers/dates");
const files_1 = require("../../helpers/files");
const parking_lot_1 = __importDefault(require("../../models/parking-lot"));
const admin_1 = __importDefault(require("../../models/admin"));
const controllers_1 = require("../reservations/controllers");
const updateUserDetailsHelper = (req) => __awaiter(void 0, void 0, void 0, function* () {
    const { _id } = req.params;
    const { email, firstName, middleName, lastName, avatar, phoneNumber, electricCharging, parkingLots, mainParkingLot, disabled, } = req.body;
    return yield user_1.default.findByIdAndUpdate(_id, {
        email: email ? email.toLowerCase() : undefined,
        firstName,
        middleName,
        lastName,
        avatar,
        phoneNumber,
        electricCharging,
        parkingLots,
        mainParkingLot,
        disabled,
    }, { omitUndefined: true, new: true, }).select(['-password', '-temporaryPassword']);
});
const updateDesignatedSpotsHelper = (req) => __awaiter(void 0, void 0, void 0, function* () {
    const { _id } = req.params;
    const { designatedParkingSpot } = req.body;
    if (designatedParkingSpot) {
        const foundUser = yield user_1.default.findById(_id).select('designatedParkingSpot');
        if (designatedParkingSpot.length == 0 && (foundUser === null || foundUser === void 0 ? void 0 : foundUser.designatedParkingSpot) && (foundUser === null || foundUser === void 0 ? void 0 : foundUser.designatedParkingSpot.length) > 0) {
            yield Promise.all(foundUser.designatedParkingSpot.map((spot) => __awaiter(void 0, void 0, void 0, function* () {
                yield reservation_1.default.updateMany({ parkingSpot: spot, user: foundUser._id, parkingStatus: { $in: [controllers_1.reservationEnums[0], controllers_1.reservationEnums[1]] }, }, { parkingStatus: controllers_1.reservationEnums[4] });
                return yield parking_spot_1.default.findByIdAndUpdate(spot, { designatedTo: null, userPresenceDate: null });
            })));
            yield user_1.default.findByIdAndUpdate(_id, { designatedParkingSpot: [] }, { new: true });
        }
        else if (designatedParkingSpot.length > 0) {
            if (foundUser) {
                yield reservation_1.default.updateMany({ parkingSpot: { $nin: designatedParkingSpot }, user: foundUser._id, parkingStatus: { $in: [controllers_1.reservationEnums[0], controllers_1.reservationEnums[1]] }, }, { parkingStatus: controllers_1.reservationEnums[4] });
                yield Promise.all(foundUser.designatedParkingSpot.map((spot) => __awaiter(void 0, void 0, void 0, function* () {
                    const updated = yield parking_spot_1.default.findByIdAndUpdate(spot, { designatedTo: null, userPresenceDate: null });
                    return updated;
                })));
            }
            yield Promise.all(designatedParkingSpot.map((spot) => __awaiter(void 0, void 0, void 0, function* () {
                return yield parking_spot_1.default.findByIdAndUpdate(spot, { designatedTo: foundUser === null || foundUser === void 0 ? void 0 : foundUser._id, userPresenceDate: null });
            })));
            yield user_1.default.findByIdAndUpdate(_id, { designatedParkingSpot: designatedParkingSpot }, { new: true });
        }
    }
});
const updateVehiclesHelper = (req, _id, vehicles) => __awaiter(void 0, void 0, void 0, function* () {
    const errors = [];
    const updatedVehicles = yield Promise.all(vehicles.map((vehicle) => __awaiter(void 0, void 0, void 0, function* () {
        try {
            let updatedVehicle;
            const existingVehicle = yield vehicle_1.default.findOne({ registration: vehicle.registration, _id: { $ne: vehicle === null || vehicle === void 0 ? void 0 : vehicle._id } });
            if (existingVehicle) {
                errors.push(`Vehicle with registration ${vehicle.registration} already exists.`);
                return null;
            }
            else {
                if (vehicle._id)
                    updatedVehicle = yield vehicle_1.default.findByIdAndUpdate(vehicle._id, vehicle, { new: true });
                else
                    updatedVehicle = yield vehicle_1.default.create(vehicle);
            }
            return updatedVehicle;
        }
        catch (error) {
            console.error('Error updating/creating vehicle:', error);
            return null;
        }
    })));
    if (errors.length > 0) {
        return { vehicles: [], errors, };
    }
    else {
        const updatedVehiclesIds = updatedVehicles.filter(vehicle => vehicle !== null).map((vehicle) => vehicle._id);
        yield vehicle_1.default.deleteMany({ _id: { $nin: updatedVehiclesIds }, user: _id });
        const updated = (yield vehicle_1.default.find({ user: _id }));
        yield user_1.default.findByIdAndUpdate(_id, { vehicles: updated.map(vehicle => vehicle._id) });
        return { vehicles: updated, errors, };
    }
});
const createUserService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { email, password, confirmPassword, firstName, middleName, lastName, avatar, phoneNumber, electricCharging, designatedParkingSpot, generateRandom, make, model, registration, userSpot, parkingLots, mainParkingLot, disabled, vehicles, } = req.body;
        if (!email && email != '' && !(0, validators_1.validateEmail)(email)) {
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.validations.INVALID_EMAIL });
        }
        else if (!(0, validators_1.validatePassword)(password) && !generateRandom) {
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.validations.INVALID_PASSWORD });
        }
        else if (!firstName) {
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.user.USER_DETAILS_MISSING });
        }
        else if ((!email && email != '' && (yield user_1.default.findOne({ email: email.toLowerCase() }))) || (yield user_1.default.findOne({ 'phoneNumber.number': phoneNumber.number }))) {
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.user.USER_ALREADY_EXISTS });
        }
        else if (password != confirmPassword && !generateRandom) {
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.validations.PASSWORDS_NOT_MATCH });
        }
        else if (!(phoneNumber === null || phoneNumber === void 0 ? void 0 : phoneNumber.countryCode) && !(phoneNumber === null || phoneNumber === void 0 ? void 0 : phoneNumber.number)) {
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.validations.INVALID_PHONE_NUMBER });
        }
        // else if (!registration) {
        //     res.status(HttpStatus.BAD_REQUEST).json({ message: HttpMessages.vehicle.VEHICLE_DETAILS_MISSING })
        // } else if (await User.findOne({ 'vehicle.registration': registration })) {
        //     res.status(HttpStatus.BAD_REQUEST).json({ message: HttpMessages.vehicle.VEHICLE_REGISTRATION_DUPLICATE })
        // }
        else if (!vehicles || vehicles.length == 0) {
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.vehicle.VEHICLE_DETAILS_MISSING });
        }
        else if (vehicles && (vehicles === null || vehicles === void 0 ? void 0 : vehicles.length) > 0 && (yield vehicle_1.default.findOne({ registration: { $in: vehicles === null || vehicles === void 0 ? void 0 : vehicles.map(vehicle => vehicle === null || vehicle === void 0 ? void 0 : vehicle.registration) } }))) {
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.vehicle.VEHICLE_REGISTRATION_DUPLICATE });
        }
        else {
            const generatedPassword = (0, generators_1.generateTemporaryPinCode)();
            const hashedPassword = yield (0, generators_1.generateHashPassword)(generateRandom ? generatedPassword.password : password);
            const user = new user_1.default({
                avatar,
                email: email ? email.toLowerCase() : '',
                // password: hashedPassword,
                temporaryPassword: hashedPassword,
                firstName,
                middleName,
                lastName,
                phoneNumber,
                electricCharging,
                designatedParkingSpot: designatedParkingSpot ? designatedParkingSpot : [],
                userSpot,
                parkingLots,
                mainParkingLot,
                disabled,
            });
            const userWithDesignatedSpot = yield user_1.default.findOne({ designatedParkingSpot: { $in: designatedParkingSpot } });
            const { startOfDay, endOfDay } = (0, dates_1.getStartAndEndOfDay)(new Date());
            const reservation = yield reservation_1.default.find({ parkingSpot: { $in: designatedParkingSpot }, reservationDate: { $gte: startOfDay, $lte: endOfDay }, $or: [{ parkingStatus: 'reserved' }, { parkingStatus: 'occupied' }] });
            if (userWithDesignatedSpot && designatedParkingSpot) {
                res.status(http_statuses_1.default.FORBIDDEN).json({ message: http_messages_1.default.parkingSpot.DESIGNATED_SPOT_ALREADY_ASSIGNED_TO_USER });
            }
            else if (reservation.length > 0 && designatedParkingSpot) {
                res.status(http_statuses_1.default.FORBIDDEN).json({ message: http_messages_1.default.parkingSpot.DESIGNATED_SPOT_HAS_RESERVATIONS });
            }
            else {
                const messageToSend = (0, generators_1.registrationSmsGenerator)(firstName, generateRandom, generatedPassword, password);
                if (designatedParkingSpot) {
                    const spots = yield Promise.all(designatedParkingSpot.map((spot) => __awaiter(void 0, void 0, void 0, function* () {
                        const date = (0, dates_1.getCorrectDateTime)(new Date());
                        const updated = yield parking_spot_1.default.findByIdAndUpdate(spot, { designatedTo: user._id, userPresenceDate: date }, { new: true });
                        if (updated) {
                            const reservation = new reservation_1.default({
                                user: user._id,
                                parkingLot: updated === null || updated === void 0 ? void 0 : updated.parkingLot,
                                level: updated === null || updated === void 0 ? void 0 : updated.level,
                                parkingSpot: updated._id,
                                reservationDate: date,
                                isElectricChargingRequired: false,
                                parkingStatus: 'reserved',
                            });
                            yield reservation.save();
                            return updated._id;
                        }
                        else
                            return null;
                    })));
                    if (spots.length > 0) {
                        const designated = yield Promise.all(spots === null || spots === void 0 ? void 0 : spots.filter(item => { if (item != null && item != undefined)
                            return item; }));
                        user.designatedParkingSpot = (designated === null || designated === void 0 ? void 0 : designated.length) > 0 ? designated : [];
                    }
                }
                if (vehicles && vehicles.length > 0) {
                    const createdVehicles = yield Promise.all(vehicles.map((vehicle) => __awaiter(void 0, void 0, void 0, function* () {
                        const created = new vehicle_1.default({
                            user: user._id,
                            make: vehicle === null || vehicle === void 0 ? void 0 : vehicle.make,
                            model: vehicle === null || vehicle === void 0 ? void 0 : vehicle.model,
                            registration: vehicle.registration.toUpperCase()
                        });
                        return yield created.save();
                    })));
                    user.vehicles = createdVehicles.map(vehicle => vehicle._id);
                }
                const savedUser = yield user.save();
                const messageSent = yield (0, email_messaging_1.sendMessage)(user.phoneNumber, messageToSend);
                res.status(http_statuses_1.default.CREATED).json({ user: savedUser, messageSent });
            }
        }
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.createUserService = createUserService;
const createMultipleUsersService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { fileUrl, lang = 'heb' } = req.body;
        const usersFromFile = yield (0, files_1.fileReaderUsers)(fileUrl);
        const users = yield Promise.all(usersFromFile.map((user, index) => __awaiter(void 0, void 0, void 0, function* () {
            return yield new Promise((resolve, reject) => __awaiter(void 0, void 0, void 0, function* () {
                var _a, _b, _c;
                try {
                    if (!user.email && user.email != '' && (yield user_1.default.findOne({ email: (_a = user.email) === null || _a === void 0 ? void 0 : _a.trim().toLowerCase() }))) {
                        reject({ type: 'email-already-exist', index, });
                    }
                    else if (yield user_1.default.findOne({ 'phoneNumber.number': (_b = user.phoneNumber) === null || _b === void 0 ? void 0 : _b.trim() })) {
                        reject({ type: 'phone-number-already-exist', index, });
                    }
                    else if (yield vehicle_1.default.findOne({ registration: (_c = user.registration) === null || _c === void 0 ? void 0 : _c.trim() })) {
                        reject({ type: 'vehicle-registration-already-exist', index, });
                    }
                    else {
                        const mainParkingLot = yield parking_lot_1.default.findOne({ name: user.mainParkingLot });
                        if (!mainParkingLot)
                            reject({ type: 'main-parking-lot-not-found', index, });
                        else {
                            const parkingLots = yield Promise.all(user.allowedParkingLots.map((parkingLot) => __awaiter(void 0, void 0, void 0, function* () {
                                try {
                                    const lot = yield parking_lot_1.default.findOne({ name: parkingLot });
                                    if (!lot)
                                        reject({ type: 'allowed-parking-lots-not-found', index, });
                                    else
                                        return lot;
                                }
                                catch (error) {
                                    reject(error);
                                }
                            })));
                            const designatedSpots = yield Promise.all(user.designatedSpots.map((spot) => __awaiter(void 0, void 0, void 0, function* () {
                                try {
                                    const parkingSpot = yield parking_spot_1.default.findOne({ parkingLot: { $in: parkingLots.map(item => item === null || item === void 0 ? void 0 : item._id) }, parkingCode: spot, designatedTo: null, spotType: 'designated' });
                                    if (!parkingSpot)
                                        reject({ type: 'designated-spot-not-found', index });
                                    else
                                        return parkingSpot;
                                }
                                catch (error) {
                                    reject(error);
                                }
                            })));
                            const generatedPassword = (0, generators_1.generateTemporaryPinCode)();
                            const hashedPassword = yield (0, generators_1.generateHashPassword)(generatedPassword.password);
                            const newUser = new user_1.default({
                                avatar: '',
                                email: user.email != '' && user.email != undefined ? user.email.trim().toLowerCase() : '',
                                temporaryPassword: hashedPassword,
                                firstName: user.firstName,
                                middleName: '',
                                lastName: user.lastName,
                                phoneNumber: {
                                    countryCode: '+972',
                                    number: user.phoneNumber,
                                },
                                electricCharging: user.electricCharging,
                                designatedParkingSpot: designatedSpots.map(item => item === null || item === void 0 ? void 0 : item._id),
                                parkingLots: parkingLots.map(item => item === null || item === void 0 ? void 0 : item._id),
                                mainParkingLot: mainParkingLot === null || mainParkingLot === void 0 ? void 0 : mainParkingLot._id,
                                disabled: user.accessible,
                                dateOfBirth: user.dateOfBirth,
                            });
                            const vehicle = new vehicle_1.default({
                                user: newUser._id,
                                make: user.make,
                                model: user.model,
                                registration: user.registration.toUpperCase(),
                            });
                            newUser.vehicles = [vehicle._id];
                            resolve({ user: newUser, vehicles: [vehicle] });
                        }
                    }
                }
                catch (error) {
                    reject(error);
                }
            }));
        })));
        const savedUsers = yield Promise.all(users.map((item) => __awaiter(void 0, void 0, void 0, function* () {
            var _d;
            yield Promise.all(item.user.designatedParkingSpot.map((spot) => __awaiter(void 0, void 0, void 0, function* () {
                var _e, _f;
                const parkingSpot = yield parking_spot_1.default.findByIdAndUpdate(spot === null || spot === void 0 ? void 0 : spot._id, { designatedTo: (_e = item === null || item === void 0 ? void 0 : item.user) === null || _e === void 0 ? void 0 : _e._id });
                const date = (0, dates_1.getCorrectDateTime)(new Date());
                const reservation = new reservation_1.default({
                    user: (_f = item === null || item === void 0 ? void 0 : item.user) === null || _f === void 0 ? void 0 : _f._id,
                    parkingLot: parkingSpot === null || parkingSpot === void 0 ? void 0 : parkingSpot.parkingLot,
                    level: parkingSpot === null || parkingSpot === void 0 ? void 0 : parkingSpot.level,
                    parkingSpot: parkingSpot === null || parkingSpot === void 0 ? void 0 : parkingSpot._id,
                    reservationDate: date,
                    isElectricChargingRequired: false,
                    parkingStatus: 'reserved',
                });
                yield reservation.save();
            })));
            yield Promise.all(item.vehicles.map((vehicle) => __awaiter(void 0, void 0, void 0, function* () {
                yield vehicle.save();
            })));
            return yield ((_d = item === null || item === void 0 ? void 0 : item.user) === null || _d === void 0 ? void 0 : _d.save());
        })));
        // res.status(HttpStatus.CREATED).json(users)
        res.status(http_statuses_1.default.CREATED).json(savedUsers);
    }
    catch (err) {
        const { fileUrl, lang = 'heb' } = req.body;
        yield (0, files_1.deleteFile)(fileUrl);
        if ((err === null || err === void 0 ? void 0 : err.type) == 'main-parking-lot-not-found')
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: 'Main parking lot not found at row ' + (Number(err === null || err === void 0 ? void 0 : err.index) + 1) });
        else if ((err === null || err === void 0 ? void 0 : err.type) == 'allowed-parking-lots-not-found')
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: 'Allowed parking lot not found at row ' + (Number(err === null || err === void 0 ? void 0 : err.index) + 1) });
        else if ((err === null || err === void 0 ? void 0 : err.type) == 'designated-spot-not-found')
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: 'Designated parking spot not found at row ' + (Number(err === null || err === void 0 ? void 0 : err.index) + 1) });
        else if ((err === null || err === void 0 ? void 0 : err.type) == 'email-already-exist')
            res.status(http_statuses_1.default.FORBIDDEN).json({ message: `${lang == 'heb' ? http_messages_1.default.user.EMAIL_ALREADY_EXISTS_HEB : http_messages_1.default.user.EMAIL_ALREADY_EXISTS} ${Number(err === null || err === void 0 ? void 0 : err.index) + 1}` });
        else if ((err === null || err === void 0 ? void 0 : err.type) == 'phone-number-already-exist')
            res.status(http_statuses_1.default.FORBIDDEN).json({ message: `${lang == 'heb' ? http_messages_1.default.user.PHONE_ALREADY_EXISTS_HEB : http_messages_1.default.user.PHONE_ALREADY_EXISTS} ${Number(err === null || err === void 0 ? void 0 : err.index) + 1}` });
        else if ((err === null || err === void 0 ? void 0 : err.type) == 'vehicle-registration-already-exist')
            res.status(http_statuses_1.default.FORBIDDEN).json({ message: `${lang == 'heb' ? http_messages_1.default.user.REG_ALREADY_EXISTS_HEB : http_messages_1.default.user.REG_ALREADY_EXISTS} ${Number(err === null || err === void 0 ? void 0 : err.index) + 1}` });
        else
            res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.createMultipleUsersService = createMultipleUsersService;
const getSingleUserService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const user = yield user_1.default.findById(_id).select(['-password', '-temporaryPassword']).populate({ path: 'designatedParkingSpot', populate: 'parkingLot' });
        if (user)
            res.status(http_statuses_1.default.OK).json(user);
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.user.USER_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getSingleUserService = getSingleUserService;
const getAdminLotsUsersService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { page = 1, limit = 10, type, parkingSpotType, isAvailable, search, sortBy = 'updated', sort = 'dsc' } = req.query;
        const sorting = sort == 'asc' ? 1 : -1;
        const admin = yield admin_1.default.findById(_id).populate('parkingLots').select('parkingLots');
        if (admin) {
            const { parkingLots } = admin;
            let filter = { parkingLots: { $in: parkingLots } };
            if (search) {
                const searchRegex = new RegExp(search, 'i'); // Case-insensitive regex
                filter.$or = [
                    { email: searchRegex },
                    { 'phoneNumber.number': searchRegex },
                    { firstName: searchRegex },
                    { middleName: searchRegex },
                    { lastName: searchRegex },
                    { 'vehicle.registration': searchRegex }
                ];
            }
            if (type == 'locked')
                filter.failedLoginAttempts = { $gte: 2 };
            else if (type == 'unlocked')
                filter.failedLoginAttempts = { $lte: 3 };
            if (parkingSpotType == 'designated')
                filter.designatedParkingSpot = { $ne: null };
            else if (parkingSpotType == 'regular')
                filter.designatedParkingSpot = null;
            const sortingFilter = {};
            if (sortBy == 'updated')
                sortingFilter.updatedAt = sorting;
            else if (sortBy == 'created')
                sortingFilter.createdAt = sorting;
            else if (sortBy == 'firstName')
                sortingFilter.firstName = sorting;
            else if (sortBy == 'middleName')
                sortingFilter.middleName = sorting;
            else if (sortBy == 'lastName')
                sortingFilter.lastName = sorting;
            else if (sortBy == 'registration')
                sortingFilter.vehicles = sorting;
            else if (sortBy == 'email')
                sortingFilter.email = sorting;
            else if (sortBy == 'phoneNumber')
                sortingFilter.phoneNumber = sorting;
            else if (sortBy == 'designated')
                sortingFilter.designatedParkingSpot = sorting;
            const totalCount = yield user_1.default.countDocuments(filter);
            const totalPages = Math.ceil(totalCount / limit);
            const users = yield user_1.default.find(filter)
                .sort(sortingFilter)
                .limit(limit)
                .skip(limit * (page - 1))
                .select(['-password', '-temporaryPassword', '-passwordUpdatedAt', '-temporaryPasswordExpiry',])
                .populate({ path: 'designatedParkingSpot', populate: { path: 'parkingLot', select: ['name'] }, })
                .populate({ path: 'parkingLots', select: ['name'] })
                .populate('vehicles');
            const usersWithVehicles = yield Promise.all(users.map((user) => __awaiter(void 0, void 0, void 0, function* () {
                const userWithDesignatedSpots = yield Promise.all(user.parkingLots.map((parkingLot) => __awaiter(void 0, void 0, void 0, function* () {
                    const designatedSpots = yield parking_spot_1.default.find({ parkingLot: parkingLot._id, spotType: 'designated', designatedTo: isAvailable ? null : { $ne: null } });
                    return Object.assign(Object.assign({}, parkingLot.toObject()), { designatedSpots });
                })));
                return Object.assign(Object.assign({}, user.toObject()), { parkingLots: userWithDesignatedSpots });
            })));
            res.status(http_statuses_1.default.OK).json({ totalPages, totalCount, users: usersWithVehicles });
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.admin.ADMIN_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getAdminLotsUsersService = getAdminLotsUsersService;
const getAllUsersService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { page = 1, limit = 10, type, parkingSpotType, isAvailable, search, sortBy = 'updated', sort = 'dsc' } = req.query;
        const sorting = sort == 'asc' ? 1 : -1;
        let filter = {};
        if (search) {
            const searchRegex = new RegExp(search, 'i'); // Case-insensitive regex
            if (searchRegex) {
                const sortingFilter = {};
                if (sortBy == 'registration')
                    sortingFilter.registration = sorting;
                const vehicles = yield vehicle_1.default.find({ $or: [{ registration: searchRegex }, { make: searchRegex }, { model: searchRegex }] }).sort(sortingFilter);
                filter.$or = [
                    { vehicles: { $in: vehicles.map(vehicle => vehicle.id) } }
                ];
            }
            filter.$or = [
                ...filter.$or,
                { email: searchRegex },
                { 'phoneNumber.number': searchRegex },
                { firstName: searchRegex },
                { middleName: searchRegex },
                { lastName: searchRegex },
                { 'vehicle.registration': searchRegex },
            ];
        }
        if (type == 'locked')
            filter.failedLoginAttempts = { $gt: 2 };
        else if (type == 'unlocked')
            filter.failedLoginAttempts = { $lt: 3 };
        if (parkingSpotType == 'designated')
            filter.designatedParkingSpot = { $ne: null };
        else if (parkingSpotType == 'regular')
            filter.designatedParkingSpot = null;
        const sortingFilter = {};
        if (sortBy == 'updated')
            sortingFilter.updatedAt = sorting;
        else if (sortBy == 'created')
            sortingFilter.createdAt = sorting;
        else if (sortBy == 'firstName')
            sortingFilter.firstName = sorting;
        else if (sortBy == 'middleName')
            sortingFilter.middleName = sorting;
        else if (sortBy == 'lastName')
            sortingFilter.lastName = sorting;
        else if (sortBy == 'registration')
            sortingFilter.vehicles = sorting;
        else if (sortBy == 'email')
            sortingFilter.email = sorting;
        else if (sortBy == 'phoneNumber')
            sortingFilter.phoneNumber = sorting;
        else if (sortBy == 'designated')
            sortingFilter.designatedParkingSpot = sorting;
        const totalCount = yield user_1.default.countDocuments(filter);
        const totalPages = Math.ceil(totalCount / limit);
        const users = yield user_1.default.find(filter).sort(sortingFilter)
            .limit(limit)
            .skip(limit * (page - 1))
            .select(['-password', '-temporaryPassword', '-passwordUpdatedAt', '-temporaryPasswordExpiry',])
            .populate({ path: 'designatedParkingSpot', populate: { path: 'parkingLot', select: ['name'] }, })
            .populate({ path: 'parkingLots', select: ['name'] })
            .populate({ path: 'vehicles', });
        const usersWithVehicles = yield Promise.all(users.map((user) => __awaiter(void 0, void 0, void 0, function* () {
            const userWithDesignatedSpots = yield Promise.all(user.parkingLots.map((parkingLot) => __awaiter(void 0, void 0, void 0, function* () {
                const designatedSpots = yield parking_spot_1.default.find({ parkingLot: parkingLot._id, spotType: 'designated', designatedTo: isAvailable ? null : { $ne: null } });
                return Object.assign(Object.assign({}, parkingLot.toObject()), { designatedSpots });
            })));
            return Object.assign(Object.assign({}, user.toObject()), { parkingLots: userWithDesignatedSpots });
        })));
        res.status(http_statuses_1.default.OK).json({ totalPages, totalCount, users: usersWithVehicles, });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getAllUsersService = getAllUsersService;
const getUsersReportService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { currentAdmin } = req.query;
        let filter = {};
        if (currentAdmin) {
            const admin = yield admin_1.default.findById(currentAdmin).populate('parkingLots').select('parkingLots');
            if (admin) {
                const { parkingLots } = admin;
                filter = { parkingLots: { $in: parkingLots } };
            }
        }
        const users = yield user_1.default.find(filter)
            .select(['firstName', 'phoneNumber', 'disabled', 'electricCharging', 'firstLoginDate', 'designatedParkingSpot', 'parkingLots', 'createdAt', 'updatedAt', 'deviceToken'])
            .populate({
            path: 'designatedParkingSpot',
            select: ['parkingCode', 'level'],
            populate: {
                path: 'level',
                select: ['name']
            }
        })
            .populate({ path: 'parkingLots', select: ['name'] });
        const { filePath } = yield (0, generators_1.generateUserReportCsv)(users);
        res.status(http_statuses_1.default.OK).json({ path: process.env.LIVE_BASE_URL + filePath });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getUsersReportService = getUsersReportService;
const updateUserService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { vehicles } = req.body;
        const found = yield user_1.default.findById(_id);
        if (!found)
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.user.USER_NOT_FOUND });
        else {
            let updatedVehicles = [];
            if (vehicles) {
                const data = yield updateVehiclesHelper(req, _id, vehicles);
                if (data.errors.length > 0)
                    res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.vehicle.VEHICLE_REGISTRATION_DUPLICATE });
                else {
                    const user = yield updateUserDetailsHelper(req);
                    yield updateDesignatedSpotsHelper(req);
                    updatedVehicles = data.vehicles;
                    res.status(http_statuses_1.default.OK).json({ user, vehicles: updatedVehicles });
                }
            }
            else {
                const user = yield updateUserDetailsHelper(req);
                yield updateDesignatedSpotsHelper(req);
                res.status(http_statuses_1.default.OK).json({ user, vehicles: [] });
            }
        }
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.updateUserService = updateUserService;
const loginUserService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { phoneNumber, password, deviceToken } = req.body;
        if (phoneNumber && password && deviceToken) {
            const user = yield user_1.default.findOne({ 'phoneNumber.number': phoneNumber });
            if (user) {
                if (user.failedLoginAttempts > 2) {
                    res.status(http_statuses_1.default.FORBIDDEN).json({ message: http_messages_1.default.user.ACCOUNT_LOCKED });
                }
                else {
                    const isValid = (0, validators_1.isPasswordValid)(password, user.password);
                    if (isValid) {
                        const isPasswordExpired = (0, validators_1.isDate90DaysOld)(new Date(user.passwordUpdatedAt));
                        if (isPasswordExpired)
                            res.status(http_statuses_1.default.FORBIDDEN).json({ message: http_messages_1.default.user.PASSWORD_EXPIRED });
                        else {
                            // user.temporaryPassword = null
                            // user.deviceToken = deviceToken
                            const savedUser = yield user_1.default.findByIdAndUpdate(user._id, Object.assign({ temporaryPassword: null, deviceToken, lastLogin: new Date() }, (!(user === null || user === void 0 ? void 0 : user.firstLoginDate) && { firstLoginDate: new Date() })), { new: true, omitUndefined: true }).select(['-password', '-temporaryPassword']).populate({ path: 'designatedParkingSpot', populate: 'parkingLot' });
                            // if (!user?.firstLoginDate) await User.findByIdAndUpdate(user._id, { firstLoginDate: new Date() })
                            // await user.save()
                            // const finalUser = await User.findById(user._id).select(['-password', '-temporaryPassword']).populate({ path: 'designatedParkingSpot', populate: 'parkingLot' })
                            const token = (0, authentication_1.signJwt)(user._id);
                            res.status(http_statuses_1.default.OK).json({ user: savedUser, token, });
                        }
                    }
                    else {
                        const isTemporaryPassword = user.temporaryPassword ? (0, validators_1.isPasswordValid)(password, user.temporaryPassword) : false;
                        if (isTemporaryPassword) {
                            res.status(http_statuses_1.default.ACCEPTED).json({ user: { phoneNumber: user.phoneNumber }, message: http_messages_1.default.user.IS_TEMPORARY_PASSWORD, });
                        }
                        else {
                            user.failedLoginAttempts++;
                            user.failedAttemptsDates.push(new Date());
                            yield user.save();
                            if (user.failedLoginAttempts > 2)
                                user.accountLockInstances.push(new Date());
                            yield user.save();
                            res.status(http_statuses_1.default.UNAUTHORIZED).json({ message: http_messages_1.default.user.FAILED_LOGIN_ATTEMPT + `You have ${3 - user.failedLoginAttempts} left.` });
                        }
                    }
                }
            }
            else
                res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.user.USER_NOT_FOUND });
        }
        else
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.validations.INVALID_LOGIN_CREDENTIALS });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.loginUserService = loginUserService;
const logoutUserService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const user = yield user_1.default.findByIdAndUpdate(_id, { deviceToken: null });
        if (user)
            res.status(http_statuses_1.default.OK).json({ message: http_messages_1.default.user.ACCOUNT_LOGGED_OUT });
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.user.USER_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.logoutUserService = logoutUserService;
const forgotPasswordService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { phoneNumber, isDev } = req.body;
        const user = yield user_1.default.findOne({ 'phoneNumber.number': phoneNumber.number });
        if (user) {
            // if (user.failedLoginAttempts > 2) {
            //     res.status(HttpStatus.FORBIDDEN).json({ message: HttpMessages.user.ACCOUNT_LOCKED })
            // } else {
            const { expiry, password } = (0, generators_1.generateTemporaryPinCode)();
            // console.log(password, 'password')
            yield user_1.default.findByIdAndUpdate(user._id, {
                temporaryPassword: yield (0, generators_1.generateHashPassword)(isDev ? 'Dev@12345' : password),
                temporaryPasswordExpiry: expiry,
                failedLoginAttempts: 0,
                failedAttemptsDates: []
            });
            if (isDev) {
                res.status(http_statuses_1.default.CREATED).json({ phoneNumber, message: http_messages_1.default.user.TEMPORARY_PASSWORD_CREATED_PHONE, });
            }
            else {
                const sms = `${user.firstName}, ברוך הבא לספוטפארק, אפליקציית ניהול החניונים החדשה של חברת החשמל.\n
לאחר הורדת האפליקציה החדשה מהחנות יש למלא את  שם המשתמש שלך (מספר הטלפון הנייד שלך) והסיסמה הזמנית שלך היא: ${isDev ? 'Dev@12345' : password}.
לאחר מכן, יש לשנות סיסמא כפי שמוסבר במצגת שקיבלת.\n
בכל בעיה שאתה נתקל אתה מוזמן לפנות לאדמיניסטרטור שלך כפי שמופיע במצגת שקיבלת.\n
לאחר ההתחברות לאפליקציה החדשה יש להסיר את האפליקציה הישנה שיצאה משימוש.\n                    
שים לב כי מספר החניה שלך נשאר ללא שינוי.\n                    
בהצלחה לכולנו!`;
                const messageSent = yield (0, email_messaging_1.sendMessage)(user.phoneNumber, sms);
                res.status(http_statuses_1.default.CREATED).json({ phoneNumber, message: http_messages_1.default.user.TEMPORARY_PASSWORD_CREATED_PHONE, messageSent, });
            }
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.user.USER_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.forgotPasswordService = forgotPasswordService;
const loginWithTemporaryPasswordService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { phoneNumber, temporaryPassword, newPassword, confirmPassword } = req.body;
        const user = yield user_1.default.findOne({ 'phoneNumber.number': phoneNumber.number });
        if (user) {
            if (user.failedLoginAttempts > 2) {
                res.status(http_statuses_1.default.FORBIDDEN).json({ message: http_messages_1.default.user.ACCOUNT_LOCKED });
            }
            else {
                const isValid = (0, validators_1.isPasswordValid)(temporaryPassword, user.temporaryPassword ? user.temporaryPassword : '');
                if (isValid) {
                    // if (new Date(user.temporaryPasswordExpiry) > new Date()) {
                    if (newPassword === confirmPassword) {
                        const hashedPassword = yield (0, generators_1.generateHashPassword)(newPassword);
                        user.password = hashedPassword;
                        user.temporaryPassword = null;
                        user.passwordUpdatedAt = Date.now();
                        yield user.save();
                        res.status(http_statuses_1.default.OK).json({ message: http_messages_1.default.user.PASSWORD_UPDATE_SUCCESS });
                    }
                    else
                        res.status(http_statuses_1.default.FORBIDDEN).json({ message: http_messages_1.default.validations.PASSWORDS_NOT_MATCH });
                    // } else res.status(HttpStatus.FORBIDDEN).json({ message: HttpMessages.validations.TEMP_PASSWORD_EXPIRED })
                }
                else
                    res.status(http_statuses_1.default.UNAUTHORIZED).json({ message: http_messages_1.default.user.FAILED_TEMPORARY_LOGIN });
            }
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.user.USER_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.loginWithTemporaryPasswordService = loginWithTemporaryPasswordService;
const resetPasswordService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { phoneNumber, oldPassword, newPassword, confirmPassword } = req.body;
        const user = yield user_1.default.findOne({ 'phoneNumber.number': phoneNumber.number });
        if (user) {
            if (user.failedLoginAttempts > 2) {
                res.status(http_statuses_1.default.FORBIDDEN).json({ message: http_messages_1.default.user.ACCOUNT_LOCKED });
            }
            else {
                const isValid = (0, validators_1.isPasswordValid)(oldPassword, user.password);
                if (isValid) {
                    if (newPassword === confirmPassword) {
                        const hashedPassword = yield (0, generators_1.generateHashPassword)(newPassword);
                        user.password = hashedPassword;
                        user.temporaryPassword = null;
                        user.passwordUpdatedAt = Date.now();
                        yield user.save();
                        res.status(http_statuses_1.default.OK).json({ message: http_messages_1.default.user.PASSWORD_UPDATE_SUCCESS });
                    }
                    else
                        res.status(http_statuses_1.default.FORBIDDEN).json({ message: http_messages_1.default.validations.PASSWORDS_NOT_MATCH });
                }
                else
                    res.status(http_statuses_1.default.UNAUTHORIZED).json({ message: http_messages_1.default.user.WRONG_OLD_PASSWORD });
            }
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.user.USER_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.resetPasswordService = resetPasswordService;
const unlockUserAccountService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const user = yield user_1.default.findByIdAndUpdate(_id, { failedLoginAttempts: 0, failedAttemptsDates: [], });
        if (user)
            res.status(http_statuses_1.default.OK).json({ message: http_messages_1.default.user.ACCOUNT_UNLOCKED });
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.user.USER_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.unlockUserAccountService = unlockUserAccountService;
const deleteAllUsersService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const users = yield user_1.default.find();
        const deleted = yield Promise.all(users.map((user) => __awaiter(void 0, void 0, void 0, function* () {
            return new Promise((resolve) => __awaiter(void 0, void 0, void 0, function* () {
                const { designatedParkingSpot } = user;
                let designatedFreed = null;
                if (designatedParkingSpot && designatedParkingSpot.length > 0) {
                    designatedFreed = yield parking_spot_1.default.updateMany({ _id: { $in: designatedParkingSpot } }, { designatedTo: null, userPresenceDate: null });
                }
                const reservationFreed = yield reservation_1.default.updateMany({ user: user._id, parkingStatus: { $in: [controllers_1.reservationEnums[0], controllers_1.reservationEnums[1]] } }, { parkingStatus: controllers_1.reservationEnums[4] });
                const vehiclesDeleted = yield vehicle_1.default.deleteMany({ user: user._id });
                const deletedUser = yield user_1.default.findByIdAndDelete(user._id);
                resolve({ deletedUser, reservationFreed, vehiclesDeleted, designatedFreed });
            }));
        })));
        res.status(http_statuses_1.default.OK).json({ message: http_messages_1.default.user.ALL_USERS_DELETED, deleted });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.deleteAllUsersService = deleteAllUsersService;
const deleteUserService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const user = yield user_1.default.findByIdAndDelete(_id);
        if (user) {
            const { designatedParkingSpot, vehicles } = user;
            if (designatedParkingSpot && designatedParkingSpot.length > 0) {
                const designatedFreed = yield parking_spot_1.default.updateMany({ _id: { $in: designatedParkingSpot } }, { designatedTo: null, userPresenceDate: null });
            }
            const reservationFreed = yield reservation_1.default.updateMany({ user: _id, parkingStatus: { $in: [controllers_1.reservationEnums[0], controllers_1.reservationEnums[1], controllers_1.reservationEnums[2]] } }, { parkingStatus: controllers_1.reservationEnums[4] });
            const deletedVehicles = yield vehicle_1.default.deleteMany({ $or: [{ user: _id }, { _id: { $in: vehicles } }] });
            res.status(http_statuses_1.default.OK).json({ message: http_messages_1.default.user.USER_DELETED_SUCCESS });
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.user.USER_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.deleteUserService = deleteUserService;
