"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteAnnouncementService = exports.updateAnnouncementService = exports.readUserAnnouncementsService = exports.getSingleAnnouncementService = exports.getUserAnnouncementsService = exports.getAllAnnouncementsService = exports.getAdminsAnnouncementsService = exports.createAnnouncementService = void 0;
const http_statuses_1 = __importDefault(require("../../services/constants/http-statuses"));
const http_messages_1 = __importDefault(require("../../services/constants/http-messages"));
const announcement_1 = __importDefault(require("../../models/announcement"));
const admin_1 = __importDefault(require("../../models/admin"));
const user_1 = __importDefault(require("../../models/user"));
const createAnnouncementService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { parkingLots, startDate, endDate, title, content, } = req.body;
        if (parkingLots && startDate && endDate && title && content) {
            if (new Date(startDate) > new Date(endDate))
                res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.announcement.ANNOUNCEMENT_WRONG_DATE });
            else {
                const announcement = new announcement_1.default({
                    parkingLots,
                    startDate,
                    endDate,
                    title,
                    content,
                    status: 'active',
                });
                yield announcement.save();
                res.status(http_statuses_1.default.CREATED).json({ announcement });
            }
        }
        else
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.announcement.ANNOUNCEMENT_DETAILS_MISSING });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.createAnnouncementService = createAnnouncementService;
const getAdminsAnnouncementsService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { page = 1, limit = 10, status, currentDate, } = req.query;
        if (_id) {
            const admin = yield admin_1.default.findById(_id).select(['parkingLots']);
            if (admin) {
                const { parkingLots } = admin;
                let filter = { parkingLots: { $in: parkingLots }, };
                if (status)
                    filter.status = status;
                if (currentDate) {
                    if (currentDate) {
                        filter = Object.assign(Object.assign({}, filter), { $or: [{ startDate: { $gte: currentDate } }, { endDate: { $gte: currentDate } }] });
                    }
                }
                const totalCount = yield announcement_1.default.countDocuments(filter);
                const totalPages = Math.ceil(totalCount / limit);
                const announcements = yield announcement_1.default.find(filter).sort({ updatedAt: -1, }).limit(limit).skip(limit * (page - 1)).populate({ path: 'parkingLots', select: ['name', 'avatar'] });
                res.status(http_statuses_1.default.OK).json({ totalPages, totalCount, announcements });
            }
            else
                res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.admin.ADMIN_NOT_FOUND });
        }
        else
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.admin.ADMIN_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getAdminsAnnouncementsService = getAdminsAnnouncementsService;
const getAllAnnouncementsService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { page = 1, limit = 10, status, currentDate, parkingLots } = req.query;
        let filter = {};
        if (status)
            filter.status = status;
        if (parkingLots && parkingLots.length > 0)
            filter.parkingLots = { $in: Array.isArray(parkingLots) ? parkingLots : [parkingLots] };
        if (currentDate) {
            if (currentDate) {
                filter = Object.assign(Object.assign({}, filter), { $or: [{ startDate: { $gte: currentDate } }, { endDate: { $gte: currentDate } }] });
            }
        }
        const totalCount = yield announcement_1.default.countDocuments(filter);
        const totalPages = Math.ceil(totalCount / limit);
        const announcements = yield announcement_1.default.find(filter).sort({ updatedAt: -1, }).limit(limit).skip(limit * (page - 1)).populate({ path: 'parkingLots', select: ['name', 'avatar'] });
        res.status(http_statuses_1.default.OK).json({ totalPages, totalCount, announcements });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getAllAnnouncementsService = getAllAnnouncementsService;
const getUserAnnouncementsService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { page = 1, limit = 10, status, currentDate, } = req.query;
        let filter = {};
        if (status)
            filter.status = status;
        if (currentDate) {
            filter = Object.assign(Object.assign({}, filter), { $or: [{ startDate: { $gte: currentDate } }, { endDate: { $gte: currentDate } }] });
        }
        const user = yield user_1.default.findById(_id);
        if (user) {
            const parkingLots = user.parkingLots;
            filter.parkingLots = { $in: parkingLots };
            const totalCount = yield announcement_1.default.countDocuments(filter);
            const totalPages = Math.ceil(totalCount / limit);
            const unreadCount = yield announcement_1.default.countDocuments(Object.assign(Object.assign({}, filter), { $or: [{ readBy: { $nin: [_id] } }, { readBy: undefined }] }));
            const announcements = yield announcement_1.default.find(filter).sort({ updatedAt: -1, }).limit(limit).skip(limit * (page - 1)).populate({ path: 'parkingLots', select: ['name', 'avatar'] });
            res.status(http_statuses_1.default.OK).json({ totalPages, totalCount, announcements, unreadCount });
        }
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getUserAnnouncementsService = getUserAnnouncementsService;
const getSingleAnnouncementService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const announcement = yield announcement_1.default.findById(_id);
        if (announcement)
            res.status(http_statuses_1.default.OK).json(announcement);
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.announcement.ANNOUNCEMENT_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getSingleAnnouncementService = getSingleAnnouncementService;
const readUserAnnouncementsService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { page = 1, limit = 10, } = req.query;
        const { status, currentDate, } = req.body;
        let filter = {};
        if (status)
            filter.status = status;
        const user = yield user_1.default.findById(_id);
        if (user) {
            const parkingLots = user.parkingLots;
            filter.parkingLots = { $in: parkingLots };
            const totalCount = yield announcement_1.default.countDocuments(filter);
            const totalPages = Math.ceil(totalCount / limit);
            const announcements = yield announcement_1.default.updateMany(Object.assign(Object.assign({}, filter), { $or: [{ readBy: { $nin: [_id] } }, { readBy: undefined }] }), { $push: { readBy: _id } });
            res.status(http_statuses_1.default.OK).json({ totalPages, totalCount, announcements, });
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.user.USER_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.readUserAnnouncementsService = readUserAnnouncementsService;
const updateAnnouncementService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { parkingLots, startDate, endDate, title, content, status } = req.body;
        const updateData = { parkingLots, startDate, endDate, title, content, status, };
        const announcement = yield announcement_1.default.findByIdAndUpdate(_id, updateData, { new: true, omitUndefined: true, });
        if (announcement)
            res.status(http_statuses_1.default.OK).json(announcement);
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.announcement.ANNOUNCEMENT_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.updateAnnouncementService = updateAnnouncementService;
const deleteAnnouncementService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const announcement = yield announcement_1.default.findByIdAndDelete(_id);
        if (announcement)
            res.status(http_statuses_1.default.OK).json({ message: http_messages_1.default.announcement.ANNOUNCEMENT_DELETED_SUCCESS });
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.announcement.ANNOUNCEMENT_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.deleteAnnouncementService = deleteAnnouncementService;
