"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.sendSmsToUser = exports.sendNotificationToUser = exports.uploadVideosToServer = exports.uploadFilesToServer = exports.uploadMultipleImages = void 0;
const express_1 = __importDefault(require("express"));
const authentication_1 = require("../../middlewares/authentication");
const uploaders_1 = require("../../middlewares/uploaders");
const http_messages_1 = __importDefault(require("../../services/constants/http-messages"));
const http_statuses_1 = __importDefault(require("../../services/constants/http-statuses"));
const user_1 = __importDefault(require("../../models/user"));
const email_messaging_1 = require("../../middlewares/email-messaging");
const notifications_1 = require("../../middlewares/notifications");
const app = (0, express_1.default)();
const uploadMultipleImages = app.post('/upload-images', [authentication_1.verifyToken, uploaders_1.uploadImages], (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    var _a;
    try {
        const urls = yield Promise.all((_a = req === null || req === void 0 ? void 0 : req.files) === null || _a === void 0 ? void 0 : _a.map((file) => file.path));
        if (urls.length > 0)
            res.status(http_statuses_1.default.CREATED).json(urls);
        else
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.upload.IMAGE_UPLOAD_FAILED });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
}));
exports.uploadMultipleImages = uploadMultipleImages;
const uploadFilesToServer = app.post('/upload-files', [authentication_1.verifyToken, uploaders_1.uploadFiles], (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    var _b;
    try {
        const urls = yield Promise.all((_b = req === null || req === void 0 ? void 0 : req.files) === null || _b === void 0 ? void 0 : _b.map((file) => file.path));
        if (urls.length > 0)
            res.status(http_statuses_1.default.CREATED).json(urls);
        else
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.upload.FILE_UPLOAD_FAILED });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
}));
exports.uploadFilesToServer = uploadFilesToServer;
const uploadVideosToServer = app.post('/upload-videos', [authentication_1.verifyToken, uploaders_1.uploadVideos], (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    var _c;
    try {
        const urls = yield Promise.all((_c = req === null || req === void 0 ? void 0 : req.files) === null || _c === void 0 ? void 0 : _c.map((file) => file.path));
        if (urls.length > 0)
            res.status(http_statuses_1.default.CREATED).json(urls);
        else
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.upload.VIDEO_UPLOAD_FAILED });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
}));
exports.uploadVideosToServer = uploadVideosToServer;
const sendSmsToUser = app.post('/send-message/:_id', authentication_1.verifyToken, (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { message, sendAnyway } = req.body;
        if (sendAnyway) {
            const send = yield (0, email_messaging_1.sendMessage)(null, message);
            if (send)
                res.status(http_statuses_1.default.OK).json({ message: http_messages_1.default.emailSmsNotification.SMS_SENT_SUCCESS, smsSent: send });
            else
                res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: http_messages_1.default.emailSmsNotification.SMS_SENT_FAILURE, smsSent: send });
        }
        else {
            const user = yield user_1.default.findById(_id).select('phoneNumber');
            if (user) {
                const send = yield (0, email_messaging_1.sendMessage)(user.phoneNumber, message);
                if (send)
                    res.status(http_statuses_1.default.OK).json({ message: http_messages_1.default.emailSmsNotification.SMS_SENT_SUCCESS, smsSent: send });
                else
                    res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: http_messages_1.default.emailSmsNotification.SMS_SENT_FAILURE, smsSent: send });
            }
            else
                res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.user.USER_NOT_FOUND });
        }
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
}));
exports.sendSmsToUser = sendSmsToUser;
const sendNotificationToUser = app.post('/send-notification/:_id', authentication_1.verifyToken, (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { message, deviceToken } = req.body;
        if (deviceToken) {
            const send = yield (0, notifications_1.sendNotification)(deviceToken, message, '', { reservation: 'asd' });
            if (send)
                res.status(http_statuses_1.default.OK).json({ message: http_messages_1.default.emailSmsNotification.NOTIFICATION_SENT_SUCCESS, notification: send });
            else
                res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: http_messages_1.default.emailSmsNotification.NOTIFICATION_SENT_FAILURE, smsSent: send });
        }
        else {
            const user = yield user_1.default.findById(_id).select(['deviceToken']);
            if (user && user.deviceToken) {
                const send = yield (0, notifications_1.sendNotification)(user.deviceToken, message, '', { reservation: 'asd' });
                if (send)
                    res.status(http_statuses_1.default.OK).json({ message: http_messages_1.default.emailSmsNotification.NOTIFICATION_SENT_SUCCESS, smsSent: send });
                else
                    res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: http_messages_1.default.emailSmsNotification.NOTIFICATION_SENT_FAILURE, smsSent: send });
            }
            else
                res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.user.USER_NOT_FOUND });
        }
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
}));
exports.sendNotificationToUser = sendNotificationToUser;
