"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteMessageTimeService = exports.updateMessageTimeService = exports.getSingleMessageTimeService = exports.getAllMessageTimesService = exports.createMessageTimeService = void 0;
const http_messages_1 = __importDefault(require("../../services/constants/http-messages"));
const http_statuses_1 = __importDefault(require("../../services/constants/http-statuses"));
const message_time_1 = __importDefault(require("../../models/message-time"));
const createMessageTimeService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { label, value, labelHeb, timeOfDay } = req.body;
        if (label && value && labelHeb && timeOfDay) {
            const messageTime = new message_time_1.default({
                label,
                value,
                labelHeb,
                timeOfDay
            });
            yield messageTime.save();
            res.status(http_statuses_1.default.CREATED).json(messageTime);
        }
        else
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.complaint.COMPLAINT_DETAILS_MISSING });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.createMessageTimeService = createMessageTimeService;
const getAllMessageTimesService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const messageTimes = yield message_time_1.default.find().sort({ value: -1, });
        res.status(http_statuses_1.default.OK).json(messageTimes);
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getAllMessageTimesService = getAllMessageTimesService;
const getSingleMessageTimeService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const messageTime = yield message_time_1.default.findById(_id);
        if (messageTime)
            res.status(http_statuses_1.default.OK).json(messageTime);
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.complaint.COMPLAINT_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getSingleMessageTimeService = getSingleMessageTimeService;
const updateMessageTimeService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { label, value, labelHeb, timeOfDay } = req.body;
        if (label && value && labelHeb && timeOfDay) {
            const messageTime = yield message_time_1.default.findByIdAndUpdate(_id, { label, value, labelHeb, timeOfDay }, { new: true, });
            if (messageTime)
                res.status(http_statuses_1.default.OK).json(messageTime);
            else
                res.status(http_statuses_1.default.NOT_FOUND).json({ message: 'Message time not found' });
        }
        else
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.complaint.COMPLAINT_DETAILS_MISSING });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.updateMessageTimeService = updateMessageTimeService;
const deleteMessageTimeService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const messageTime = yield message_time_1.default.findByIdAndDelete(_id);
        if (messageTime)
            res.status(http_statuses_1.default.OK).json({ message: 'Message time Deleted Successfully' });
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: 'Message time not found' });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.deleteMessageTimeService = deleteMessageTimeService;
