"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteReservationService = exports.changeReservedSpotService = exports.updateReservationService = exports.getSingleReservationService = exports.getSpotAvailabilityService = exports.getLevelReservationsService = exports.getParkingLotReservationsService = exports.getUserReservationsService = exports.getAllReservationsService = exports.createReservationService = void 0;
const index_1 = require("../../helpers/dates/index");
const http_messages_1 = __importDefault(require("../../services/constants/http-messages"));
const http_statuses_1 = __importDefault(require("../../services/constants/http-statuses"));
const dates_1 = require("../../helpers/dates");
const validators_1 = require("../../helpers/validators");
const parking_lot_1 = __importDefault(require("../../models/parking-lot"));
const parking_spot_1 = __importDefault(require("../../models/parking-spot"));
const reservation_1 = __importDefault(require("../../models/reservation"));
const email_messaging_1 = require("../../middlewares/email-messaging");
const user_1 = __importDefault(require("../../models/user"));
const moment_1 = __importDefault(require("moment"));
const absence_1 = __importDefault(require("../../models/absence"));
const notifications_1 = require("../../middlewares/notifications");
const notification_1 = require("../../services/constants/notification");
const generators_1 = require("../../helpers/generators");
const controllers_1 = require("./controllers");
const findAvailableSpots = (req, spotType, level) => __awaiter(void 0, void 0, void 0, function* () {
    if (spotType == 'designated')
        return yield findAvailableDesignatedSpots(req);
    else {
        const { parkingLot, reservationDate, } = req.body;
        const { startOfDay, endOfDay } = (0, dates_1.getStartAndEndOfDay)(reservationDate);
        const query = {
            parkingLot,
            spotType,
            isPermanent: false,
        };
        if (level)
            query.level = level;
        const parkingLotObject = yield parking_lot_1.default.findById(parkingLot);
        if (parkingLotObject) {
            const sort = parkingLotObject.allotmentMethod == 'manual' ? { _id: 1 } : { parkingCode: 1 };
            // const sort: {} = parkingLotObject.allotmentMethod == 'manual' ? { createdAt: 1 } : { parkingCode: 1 }
            const totalSpots = yield parking_spot_1.default.find(query).sort(sort);
            const reservedSpots = yield reservation_1.default.find({ parkingLot, reservationDate: { $gte: startOfDay, $lte: endOfDay }, parkingStatus: { $in: ['reserved', 'occupied'] } });
            const availableSpots = yield Promise.all(totalSpots.filter((spot) => {
                const isIncludedInReserved = reservedSpots.findIndex((reservedSpot) => reservedSpot.parkingSpot.toString() == spot._id.toString());
                if (isIncludedInReserved < 0)
                    return spot;
            }));
            return { availableSpots, reservedSpots };
        }
        else
            return { availableSpots: [], reservedSpots: [] };
    }
});
const findAvailableDesignatedSpots = (req) => __awaiter(void 0, void 0, void 0, function* () {
    const { parkingLot, reservationDate, } = req.body;
    const { startOfDay, endOfDay } = (0, dates_1.getStartAndEndOfDay)(reservationDate);
    const query = {
        parkingLot,
        spotType: 'designated',
        isPermanent: false,
    };
    const parkingLotObject = yield parking_lot_1.default.findById(parkingLot);
    if (parkingLotObject) {
        const sort = parkingLotObject.allotmentMethod == 'manual' ? { _id: 1 } : { parkingCode: 1 };
        // const sort: {} = parkingLotObject.allotmentMethod == 'manual' ? { createdAt: 1 } : { parkingCode: 1 }
        const totalSpots = yield parking_spot_1.default.find(query).sort(sort);
        const reservedSpots = yield reservation_1.default.find({ parkingLot, reservationDate: { $gte: startOfDay, $lte: endOfDay }, parkingStatus: { $in: ['reserved', 'occupied'] } });
        const possibleAvailable = yield Promise.all(totalSpots.filter((spot) => {
            const isIncludedInReserved = reservedSpots.findIndex((reservedSpot) => reservedSpot.parkingSpot.toString() == spot._id.toString());
            if (isIncludedInReserved < 0)
                return spot;
        }));
        const availableSpots = [];
        if (possibleAvailable.length > 0) {
            yield Promise.all(possibleAvailable.map((spot) => __awaiter(void 0, void 0, void 0, function* () {
                const findAbsence = yield absence_1.default.find({ designatedSpot: spot._id, startDate: { $lte: reservationDate }, endDate: { $gte: reservationDate }, status: 'pending' });
                if (findAbsence.length > 0 || spot.designatedTo == null) {
                    availableSpots.push(spot);
                }
            })));
        }
        return { availableSpots, reservedSpots };
    }
    else
        return { availableSpots: [], reservedSpots: [] };
});
const createElectricReservationWithTimeLimit = (electricSpot, otherAvailableSpot, lot, req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const { parkingLot, user, reservationDate, isElectricChargingRequired, isDisabled, language = "heb" } = req.body;
    const electricReservation = new reservation_1.default({
        user,
        parkingLot,
        level: electricSpot.level,
        parkingSpot: electricSpot._id,
        reservationDate: new Date(reservationDate),
        isElectricChargingRequired,
        isDisabled: isDisabled ? isDisabled : false,
        parkingStatus: 'reserved',
    });
    const regularReservation = new reservation_1.default({
        user,
        parkingLot,
        level: otherAvailableSpot.level,
        parkingSpot: otherAvailableSpot._id,
        reservationDate: new Date(reservationDate),
        isElectricChargingRequired,
        isDisabled: isDisabled ? isDisabled : false,
        parkingStatus: 'reserved',
        forMoving: true,
    });
    const savedReservation = yield regularReservation.save();
    electricReservation.moveTo = savedReservation._id;
    const save = yield electricReservation.save();
    regularReservation.moveFrom = save._id;
    yield regularReservation.save();
    const userFromDb = yield user_1.default.findById(user).select(['phoneNumber', 'firstName']);
    let sentSms = null;
    const messageSentForBlocking = yield checkIfSpotOuter(electricSpot, reservationDate);
    if (userFromDb && (userFromDb === null || userFromDb === void 0 ? void 0 : userFromDb.phoneNumber)) {
        const message = `A reservation has been made by ${userFromDb.firstName} for date ${(0, moment_1.default)(new Date(reservationDate)).format('YYYY-MM-DD')}. The parking lot has charging limit from ${lot.chargingTimeStart} to ${lot.chargingTimeEnd}. You will have to move your vehicle to the next assigned spot after the charge limit has been reached.`;
        sentSms = yield (0, email_messaging_1.sendMessage)(userFromDb.phoneNumber, message);
    }
    res.status(http_statuses_1.default.CREATED).json({ reservation: save, sentSms, messageSentForBlocking });
});
const getUserAbsenceStatusForDate = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    return new Promise((resolve) => __awaiter(void 0, void 0, void 0, function* () {
        const { _id } = req.params;
        const { checkDate, checkTime } = req.query;
        const user = yield user_1.default.findById(_id).populate({
            path: 'designatedParkingSpot',
            select: ['-level', '-spotType', '-designatedTo'],
            populate: {
                path: 'parkingLot',
                select: ['name', 'reservationTime', 'messageSendTime', 'absenceSendTime', 'designatedReservationDuration'],
            }
        }).select(['designatedParkingSpot']);
        const designatedParkingSpots = (user === null || user === void 0 ? void 0 : user.designatedParkingSpot) ? (Array.isArray(user === null || user === void 0 ? void 0 : user.designatedParkingSpot) ? user === null || user === void 0 ? void 0 : user.designatedParkingSpot : [user === null || user === void 0 ? void 0 : user.designatedParkingSpot]) : [];
        let designatedSpots = [];
        yield Promise.all(designatedParkingSpots.map((spot) => __awaiter(void 0, void 0, void 0, function* () {
            var _a, _b, _c, _d;
            const dateToCheck = ((_a = spot === null || spot === void 0 ? void 0 : spot.parkingLot) === null || _a === void 0 ? void 0 : _a.reservationTime) == 'day-before' ? (0, moment_1.default)(checkDate).add(1, 'days').toDate() : checkDate ? checkDate : new Date();
            const date = checkDate ? new Date(dateToCheck) : new Date();
            const { startOfDay, endOfDay } = (0, dates_1.getStartAndEndOfDay)(date);
            const userPresenceDate = spot.userPresenceDate;
            const isUserPresent = userPresenceDate != null && userPresenceDate >= startOfDay && userPresenceDate <= endOfDay;
            const absence = yield absence_1.default.find({ user: _id, designatedSpot: spot._id, startDate: { $lte: date }, endDate: { $gte: date }, status: 'pending' });
            if (absence.length == 0 && !isUserPresent && !spot.isPermanent) {
                const { yesterday } = (0, index_1.getTodayTomorrowYesterday)();
                const shabbatDateCheck = (0, index_1.getCorrectDateTime)(new Date());
                const isShabbat = yield (0, index_1.isJewishHolidayShabbat)(shabbatDateCheck);
                const holiday = yield (0, index_1.getJewishHolidayEndDate)(shabbatDateCheck);
                const holidayYesterday = yield (0, index_1.getJewishHolidayEndDate)(yesterday);
                // console.log(isShabbat, 'isShabbat')
                if (!(isShabbat === null || isShabbat === void 0 ? void 0 : isShabbat.isShabbat)) {
                    const timerDuration = (0, index_1.getTimeDifference)(spot === null || spot === void 0 ? void 0 : spot.parkingLot.absenceSendTime, spot === null || spot === void 0 ? void 0 : spot.parkingLot.messageSendTime);
                    const checkTimeDiffWithMessageTime = (0, index_1.getTimeDifference)(checkTime ? checkTime : '05:00', spot === null || spot === void 0 ? void 0 : spot.parkingLot.messageSendTime);
                    const absenceTimeDiffWithCheckTime = (0, index_1.getTimeDifference)(spot === null || spot === void 0 ? void 0 : spot.parkingLot.absenceSendTime, checkTime ? checkTime : '05:00');
                    // Check if the time is between mid night and 3 hours after
                    const isTimeAfterMidNight = (0, index_1.isTimeBetween)(checkTime ? checkTime : '05:00', '00:00', '03:00');
                    let isParkingLotTimeBeforeShabbat = false;
                    if (holiday) {
                        const timeAfterOneHourOfShabbat = (0, index_1.addTime)(holiday.endTime, '01:00');
                        isParkingLotTimeBeforeShabbat = (0, index_1.getTimeDifference)(spot === null || spot === void 0 ? void 0 : spot.parkingLot.messageSendTime, timeAfterOneHourOfShabbat).minutes < 0 ? true : false;
                        // console.log(timeAfterOneHourOfShabbat, isParkingLotTimeBeforeShabbat, getTimeDifference(spot?.parkingLot.messageSendTime, timeAfterOneHourOfShabbat).minutes)
                    }
                    if (checkTimeDiffWithMessageTime.minutes >= 0 && absenceTimeDiffWithCheckTime.minutes > 0 && !isParkingLotTimeBeforeShabbat)
                        designatedSpots.push(Object.assign(Object.assign({}, spot.toObject()), { status: false, timerDuration, timeLeft: absenceTimeDiffWithCheckTime }));
                    else if (holiday) {
                        const timeAfterOneHourOfShabbat = (0, index_1.addTime)(holiday.endTime, '01:00'); // Acts as messageSendTime
                        const parkingLotReservationTime = (0, index_1.getTimeDifference)((_b = spot === null || spot === void 0 ? void 0 : spot.parkingLot) === null || _b === void 0 ? void 0 : _b.messageSendTime, timeAfterOneHourOfShabbat);
                        const isParkingLotTimePassed = parkingLotReservationTime.minutes < 0 ? true : false;
                        if (((_c = spot === null || spot === void 0 ? void 0 : spot.parkingLot) === null || _c === void 0 ? void 0 : _c.reservationTime) == 'day-before' && isParkingLotTimePassed) {
                            const endTimeInShabbat = (0, index_1.addTime)(timeAfterOneHourOfShabbat, timerDuration.formatted); // Acts as absenceSendTime
                            const checkTimeDiffWithMessageTime = (0, index_1.getTimeDifference)(checkTime ? checkTime : '05:00', timeAfterOneHourOfShabbat);
                            const absenceTimeDiffWithCheckTime = (0, index_1.getTimeDifference)(endTimeInShabbat, checkTime ? checkTime : '05:00');
                            if (checkTimeDiffWithMessageTime.minutes >= 0) {
                                // console.log(absenceTimeDiffWithCheckTime,endTimeInShabbat, 'absencetime')
                                // if (checkTimeDiffWithMessageTime.minutes >= 0 && absenceTimeDiffWithCheckTime.minutes > 0){
                                designatedSpots.push(Object.assign(Object.assign({}, spot.toObject()), { parkingLot: Object.assign(Object.assign({}, spot.toObject().parkingLot), { messageSendTime: timeAfterOneHourOfShabbat }), status: false, timerDuration, timeLeft: absenceTimeDiffWithCheckTime }));
                            }
                        }
                    }
                    else if (isTimeAfterMidNight && holidayYesterday) {
                        const timeAfterOneHourOfShabbat = (0, index_1.addTime)(holidayYesterday.endTime, '01:00');
                        const durationEndTime = (0, index_1.addTime)(timeAfterOneHourOfShabbat, (_d = spot === null || spot === void 0 ? void 0 : spot.parkingLot) === null || _d === void 0 ? void 0 : _d.designatedReservationDuration);
                        const isDurationEndTimeAfterMidNight = (0, index_1.isTimeBetween)(durationEndTime, '00:00', '03:00');
                        if (isDurationEndTimeAfterMidNight) {
                            const absenceTimeDiffWithCheckTime = (0, index_1.getTimeDifference)(durationEndTime, checkTime ? checkTime : '05:00');
                            // console.log(isDurationEndTimeAfterMidNight, 'isDurationEndTimeAfterMidNight')
                            // console.log(timeAfterOneHourOfShabbat, 'timeAfterOneHourOfShabbat')
                            // console.log(durationEndTime, 'durationEndTime')
                            // console.log(absenceTimeDiffWithCheckTime, 'absenceTimeDiffWithCheckTime')
                            const date = (0, index_1.getCorrectDateTime)(new Date());
                            const { startOfDay, endOfDay } = (0, dates_1.getStartAndEndOfDay)(date);
                            const userPresenceDate = spot.userPresenceDate;
                            const isUserPresent = userPresenceDate != null && userPresenceDate >= startOfDay && userPresenceDate <= endOfDay;
                            const absence = yield absence_1.default.findOne({ user: _id, designatedSpot: spot._id, startDate: { $lte: date }, endDate: { $gte: date }, status: 'pending' });
                            // console.log(absence, 'absence', isUserPresent, 'isUserPresent')
                            if (absenceTimeDiffWithCheckTime.minutes > 0 && !absence && !isUserPresent) {
                                designatedSpots.push(Object.assign(Object.assign({}, spot.toObject()), { parkingLot: Object.assign(Object.assign({}, spot.toObject().parkingLot), { messageSendTime: timeAfterOneHourOfShabbat }), reservationDate: date, status: false, timerDuration, timeLeft: absenceTimeDiffWithCheckTime }));
                            }
                        }
                    }
                }
            }
        })));
        const sorted = designatedSpots.filter(spot => spot != null).sort((a, b) => {
            var _a, _b;
            const timeA = (_a = a === null || a === void 0 ? void 0 : a.parkingLot) === null || _a === void 0 ? void 0 : _a.messageSendTime;
            const timeB = (_b = b === null || b === void 0 ? void 0 : b.parkingLot) === null || _b === void 0 ? void 0 : _b.messageSendTime;
            if (timeA < timeB)
                return -1;
            if (timeA > timeB)
                return 1;
            return 0;
        });
        resolve({ designatedSpots: sorted });
    }));
});
const checkIfSpotHasReservations = (spot, reservationDate) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { startOfDay, endOfDay } = (0, dates_1.getStartAndEndOfDay)(reservationDate);
        const reservations = yield reservation_1.default.countDocuments({
            parkingSpot: spot === null || spot === void 0 ? void 0 : spot._id,
            reservationDate: { $gte: startOfDay, $lte: endOfDay },
            parkingStatus: { $in: [controllers_1.reservationEnums[0], controllers_1.reservationEnums[1]] }
        });
        if (reservations > 0)
            return true;
        else
            return false;
    }
    catch (error) {
        return true;
    }
});
const checkIfSpotOuter = (spot, reservationDate) => __awaiter(void 0, void 0, void 0, function* () {
    var _e, _f, _g, _h, _j, _k, _l, _m, _o, _p, _q, _r, _s, _t, _u, _v, _w, _x, _y, _z, _0, _1, _2, _3;
    if (spot.outerToSpot) {
        const { startOfDay, endOfDay } = (0, dates_1.getStartAndEndOfDay)(reservationDate);
        const outer = yield reservation_1.default.findOne({
            parkingSpot: spot._id,
            reservationDate: { $gte: startOfDay, $lte: endOfDay },
            parkingStatus: { $in: [controllers_1.reservationEnums[0], controllers_1.reservationEnums[1]] }
        }).populate({ path: 'user', select: ['firstName', 'lastName', 'phoneNumber', 'vehicle', 'vehicles', 'deviceToken'], populate: 'vehicles' })
            .populate({ path: 'parkingLot', select: ['name'] });
        const inner = yield reservation_1.default.findOne({
            parkingSpot: spot.outerToSpot,
            reservationDate: { $gte: startOfDay, $lte: endOfDay },
            parkingStatus: { $in: [controllers_1.reservationEnums[0], controllers_1.reservationEnums[1]] }
        }).populate({ path: 'user', select: ['firstName', 'lastName', 'phoneNumber', 'vehicle', 'vehicles', 'deviceToken'], populate: 'vehicles' })
            .populate({ path: 'parkingLot', select: ['name'] });
        if (!outer || !inner)
            return null;
        else if (((_e = outer === null || outer === void 0 ? void 0 : outer.user) === null || _e === void 0 ? void 0 : _e.phoneNumber) && ((_f = inner === null || inner === void 0 ? void 0 : inner.user) === null || _f === void 0 ? void 0 : _f.phoneNumber)) {
            const messageForInner = (0, generators_1.doubleParkingNotificationTextGenerator)(((_h = (_g = outer === null || outer === void 0 ? void 0 : outer.user) === null || _g === void 0 ? void 0 : _g.vehicles) === null || _h === void 0 ? void 0 : _h.length) > 0 ? (_k = (_j = outer === null || outer === void 0 ? void 0 : outer.user) === null || _j === void 0 ? void 0 : _j.vehicles[0]) === null || _k === void 0 ? void 0 : _k.registration : '', (_l = inner === null || inner === void 0 ? void 0 : inner.parkingLot) === null || _l === void 0 ? void 0 : _l.name, `${(_m = outer === null || outer === void 0 ? void 0 : outer.user) === null || _m === void 0 ? void 0 : _m.firstName} ${(_o = outer === null || outer === void 0 ? void 0 : outer.user) === null || _o === void 0 ? void 0 : _o.lastName}`, (_p = outer === null || outer === void 0 ? void 0 : outer.user) === null || _p === void 0 ? void 0 : _p.phoneNumber);
            const messageForOuter = (0, generators_1.doubleParkingNotificationTextGenerator)(((_r = (_q = inner === null || inner === void 0 ? void 0 : inner.user) === null || _q === void 0 ? void 0 : _q.vehicles) === null || _r === void 0 ? void 0 : _r.length) > 0 ? (_t = (_s = inner === null || inner === void 0 ? void 0 : inner.user) === null || _s === void 0 ? void 0 : _s.vehicles[0]) === null || _t === void 0 ? void 0 : _t.registration : '', (_u = outer === null || outer === void 0 ? void 0 : outer.parkingLot) === null || _u === void 0 ? void 0 : _u.name, `${(_v = inner === null || inner === void 0 ? void 0 : inner.user) === null || _v === void 0 ? void 0 : _v.firstName} ${(_w = inner === null || inner === void 0 ? void 0 : inner.user) === null || _w === void 0 ? void 0 : _w.lastName}`, (_x = inner === null || inner === void 0 ? void 0 : inner.user) === null || _x === void 0 ? void 0 : _x.phoneNumber);
            const sentToInner = ((_y = inner === null || inner === void 0 ? void 0 : inner.user) === null || _y === void 0 ? void 0 : _y.deviceToken) ? (0, notifications_1.sendNotification)((_z = inner === null || inner === void 0 ? void 0 : inner.user) === null || _z === void 0 ? void 0 : _z.deviceToken, messageForInner, notification_1.NotificationTypes.outerSpotReservation, { reservationId: inner._id.toString() }, [(_0 = inner === null || inner === void 0 ? void 0 : inner.user) === null || _0 === void 0 ? void 0 : _0._id]) : null;
            const sentToOuter = ((_1 = outer === null || outer === void 0 ? void 0 : outer.user) === null || _1 === void 0 ? void 0 : _1.deviceToken) ? (0, notifications_1.sendNotification)((_2 = outer === null || outer === void 0 ? void 0 : outer.user) === null || _2 === void 0 ? void 0 : _2.deviceToken, messageForOuter, notification_1.NotificationTypes.outerSpotReservation, { reservationId: outer._id.toString() }, [(_3 = outer === null || outer === void 0 ? void 0 : outer.user) === null || _3 === void 0 ? void 0 : _3._id]) : null;
            return { sentToInner, sentToOuter };
        }
        else
            return null;
    }
    else
        return null;
});
const createReservationService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { parkingLot, user, reservationDate, isElectricChargingRequired, location, language = "heb", parkingSpot, keepDesignatedSpot } = req.body;
        if (parkingLot && user && reservationDate && isElectricChargingRequired != undefined && (location === null || location === void 0 ? void 0 : location.latitude) && (location === null || location === void 0 ? void 0 : location.longitude)) {
            const { startOfDay, endOfDay } = (0, dates_1.getStartAndEndOfDay)(reservationDate);
            const parkingLotReservations = yield reservation_1.default.find({
                user,
                parkingLot,
                forMoving: false,
                parkingStatus: { $in: [controllers_1.reservationEnums[0], controllers_1.reservationEnums[1]] },
                $and: [{ reservationDate: { $gte: startOfDay } }, { reservationDate: { $lte: endOfDay } }]
            });
            const userReservations = yield reservation_1.default.find({
                user,
                forMoving: false,
                parkingStatus: { $in: [controllers_1.reservationEnums[0], controllers_1.reservationEnums[1]] },
                $and: [{ reservationDate: { $gte: startOfDay } }, { reservationDate: { $lte: endOfDay } }]
            });
            let absenceOnDesignated = null;
            // const designatedCheck = await ParkingSpot.findOne({
            //     parkingLot,
            //     designatedTo: user,
            //     // isPermanent: true
            // })
            const designatedSpot = yield parking_spot_1.default.findOne({
                parkingLot,
                designatedTo: user,
                // isPermanent: false
            });
            if (designatedSpot) {
                absenceOnDesignated = yield absence_1.default.findOne({ designatedSpot, status: 'pending', startDate: { $lte: reservationDate }, endDate: { $gte: reservationDate } });
            }
            if (parkingLotReservations.length > 0) {
                res.status(http_statuses_1.default.FORBIDDEN).json({ message: language == 'heb' ? http_messages_1.default.reservation.RESERVATION_ALREADY_IN_PARKING_LOT_HEB : http_messages_1.default.reservation.RESERVATION_ALREADY_IN_PARKING_LOT });
            }
            else if (userReservations.length >= 2) {
                res.status(http_statuses_1.default.FORBIDDEN).json({ message: language == 'heb' ? http_messages_1.default.reservation.USER_MAX_RESERVATION_HEB : http_messages_1.default.reservation.USER_MAX_RESERVATION });
            }
            else if (designatedSpot && (designatedSpot.isPermanent || (absenceOnDesignated == null && !keepDesignatedSpot))) {
                res.status(http_statuses_1.default.FORBIDDEN).json({ message: language == 'heb' ? http_messages_1.default.reservation.DESIGNATED_ALREADY_IN_PARKING_LOT_HEB : http_messages_1.default.reservation.DESIGNATED_ALREADY_IN_PARKING_LOT });
            }
            else {
                const lot = yield parking_lot_1.default.findById(parkingLot);
                if (lot) {
                    const isInRadius = (0, validators_1.isCoordinatesInRadius)(lot.location, location, lot.maxReservationDistance);
                    if (isInRadius || keepDesignatedSpot) {
                        if (lot.allotmentMethod == 'user-selected') {
                            const spot = yield parking_spot_1.default.findById(parkingSpot);
                            if (spot) {
                                if (yield checkIfSpotHasReservations(spot, reservationDate))
                                    res.status(http_statuses_1.default.FORBIDDEN).json({ message: language == 'heb' ? http_messages_1.default.reservation.RESERVATION_SELECTED_SPOT_ALREADY_BOOKED_HEB : http_messages_1.default.reservation.RESERVATION_SELECTED_SPOT_ALREADY_BOOKED });
                                else {
                                    if (lot.chargingTimeLimit && spot.spotType == 'electric') {
                                        const { availableSpots: regularAvailable, } = yield findAvailableSpots(req, 'regular', spot.level);
                                        if (regularAvailable.length > 0) {
                                            yield createElectricReservationWithTimeLimit(spot, regularAvailable[0], lot, req, res);
                                        }
                                        else {
                                            const { availableSpots: designatedAvailable, } = yield findAvailableSpots(req, 'designated', spot.level);
                                            if (designatedAvailable.length > 0) {
                                                yield createElectricReservationWithTimeLimit(spot, designatedAvailable[0], lot, req, res);
                                            }
                                            else {
                                                const electricSpotsError = language == "heb" ? http_messages_1.default.reservation.RESERVATION_SPOTS_ALL_BOOKED_ELECTRIC_HEB : http_messages_1.default.reservation.RESERVATION_SPOTS_ALL_BOOKED_ELECTRIC;
                                                const normalSpotsError = language == "heb" ? http_messages_1.default.reservation.RESERVATION_EXTRA_SPOT_NOT_FOUND_HEB : http_messages_1.default.reservation.RESERVATION_EXTRA_SPOT_NOT_FOUND;
                                                res.status(http_statuses_1.default.NOT_FOUND).json({ message: isElectricChargingRequired ? electricSpotsError : normalSpotsError });
                                            }
                                        }
                                    }
                                    else if (spot.spotType == 'designated') {
                                        const date = reservationDate ? new Date(reservationDate) : new Date();
                                        const absence = yield absence_1.default.find({ designatedSpot: spot._id, startDate: { $lte: date }, endDate: { $gte: date }, status: 'pending' });
                                        if (absence.length > 0 || spot.designatedTo == null || String(spot.designatedTo) == user) {
                                            const { startOfDay, endOfDay } = (0, dates_1.getStartAndEndOfDay)(reservationDate);
                                            const reservations = yield reservation_1.default.find({ user: { $ne: user }, parkingSpot: spot._id, reservationDate: { $gte: startOfDay, $lte: endOfDay }, parkingStatus: { $in: ['reserved', 'occupied'] } });
                                            if (reservations.length > 0)
                                                res.status(http_statuses_1.default.FORBIDDEN).json({ message: language == 'heb' ? http_messages_1.default.reservation.RESERVATION_SELECTED_SPOT_ALREADY_BOOKED_HEB : http_messages_1.default.reservation.RESERVATION_SELECTED_SPOT_ALREADY_BOOKED });
                                            else {
                                                const newReservation = new reservation_1.default({
                                                    user,
                                                    parkingLot,
                                                    level: spot.level,
                                                    parkingSpot: spot._id,
                                                    reservationDate: new Date(reservationDate),
                                                    isElectricChargingRequired,
                                                    parkingStatus: 'reserved',
                                                });
                                                const savedReservation = yield newReservation.save();
                                                const messageSentForBlocking = yield checkIfSpotOuter(spot, reservationDate);
                                                if (keepDesignatedSpot) {
                                                    const parkingSpot = yield parking_spot_1.default.findByIdAndUpdate(spot._id, { userPresenceDate: reservationDate }, { omitUndefined: true, new: true });
                                                    res.status(http_statuses_1.default.CREATED).json({ reservation: savedReservation, designatedSpot: parkingSpot, messageSentForBlocking });
                                                }
                                                else
                                                    res.status(http_statuses_1.default.CREATED).json({ reservation: savedReservation, designatedSpot: spot, messageSentForBlocking });
                                            }
                                        }
                                        else
                                            res.status(http_statuses_1.default.FORBIDDEN).json({ message: language == 'heb' ? http_messages_1.default.reservation.RESERVATION_SPOT_NOT_RELEASED_HEB : http_messages_1.default.reservation.RESERVATION_SPOT_NOT_RELEASED });
                                    }
                                    else {
                                        const newReservation = new reservation_1.default({
                                            user,
                                            parkingLot,
                                            level: spot.level,
                                            parkingSpot: spot._id,
                                            reservationDate: new Date(reservationDate),
                                            isElectricChargingRequired,
                                            parkingStatus: 'reserved',
                                        });
                                        const savedReservation = yield newReservation.save();
                                        const messageSentForBlocking = yield checkIfSpotOuter(spot, reservationDate);
                                        res.status(http_statuses_1.default.CREATED).json({ reservation: savedReservation, messageSentForBlocking });
                                    }
                                }
                            }
                            else
                                res.status(http_statuses_1.default.NOT_FOUND).json({ message: language == 'heb' ? http_messages_1.default.parkingSpot.PARKING_SPOT_NOT_FOUND_HEB : http_messages_1.default.parkingSpot.PARKING_SPOT_NOT_FOUND });
                        }
                    }
                    else {
                        const radiusError = language == "heb" ? http_messages_1.default.reservation.RESERVATION_NOT_IN_RESERVATION_RADIUS_HEB : http_messages_1.default.reservation.RESERVATION_NOT_IN_RESERVATION_RADIUS;
                        res.status(http_statuses_1.default.FORBIDDEN).json({ message: radiusError });
                    }
                }
                else
                    res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.parkingLot.PARKING_LOT_NOT_FOUND });
            }
        }
        else
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.reservation.RESERVATION_DETAILS_MISSING });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.createReservationService = createReservationService;
const getAllReservationsService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { page = 1, limit = 10, parkingStatus } = req.query;
        let filter = {};
        if (parkingStatus)
            filter['parkingStatus'] = parkingStatus;
        const totalCount = yield reservation_1.default.countDocuments(filter);
        const totalPages = Math.ceil(totalCount / limit);
        const reservations = yield reservation_1.default.find(filter).sort({ updatedAt: -1 }).limit(limit).skip(limit * (page - 1));
        res.status(http_statuses_1.default.OK).json({ totalPages, totalCount, reservations });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getAllReservationsService = getAllReservationsService;
const getUserReservationsService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id: user } = req.params;
        const { page = 1, limit = 10, parkingStatus } = req.query;
        if (user) {
            const { startOfDay } = (0, dates_1.getStartAndEndOfDay)(new Date());
            let filter = { user, forMoving: false, parkingStatus: { $nin: [controllers_1.reservationEnums[3], controllers_1.reservationEnums[4],] }, reservationDate: { $gte: startOfDay } };
            if (parkingStatus)
                filter['parkingStatus'] = parkingStatus;
            const totalCount = yield reservation_1.default.countDocuments(filter);
            const totalPages = Math.ceil(totalCount / limit);
            const reservations = yield reservation_1.default.find(filter).sort({ updatedAt: -1 }).limit(limit).skip(limit * (page - 1))
                .populate({ path: 'parkingLot', select: ['name',] })
                .populate({ path: 'level', select: ['entrance'] })
                .populate({ path: 'parkingSpot', select: ['parkingCode'] });
            const { designatedSpots } = yield getUserAbsenceStatusForDate(req, res);
            res.status(http_statuses_1.default.OK).json({ totalPages, totalCount, reservations, designatedSpots });
        }
        else
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.reservation.RESERVATION_DETAILS_MISSING });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getUserReservationsService = getUserReservationsService;
const getParkingLotReservationsService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id: parkingLot } = req.params;
        const { page = 1, limit = 10, parkingStatus } = req.query;
        if (parkingLot) {
            let filter = { parkingLot };
            if (parkingStatus)
                filter['parkingStatus'] = parkingStatus;
            const totalCount = yield reservation_1.default.countDocuments(filter);
            const totalPages = Math.ceil(totalCount / limit);
            const reservations = yield reservation_1.default.find(filter).sort({ updatedAt: -1 }).limit(limit).skip(limit * (page - 1));
            res.status(http_statuses_1.default.OK).json({ totalPages, totalCount, reservations });
        }
        else
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.reservation.RESERVATION_DETAILS_MISSING });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getParkingLotReservationsService = getParkingLotReservationsService;
const getLevelReservationsService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id: level } = req.params;
        const { page = 1, limit = 10, parkingStatus } = req.query;
        if (level) {
            let filter = { level };
            if (parkingStatus)
                filter['parkingStatus'] = parkingStatus;
            const totalCount = yield reservation_1.default.countDocuments(filter);
            const totalPages = Math.ceil(totalCount / limit);
            const reservations = yield reservation_1.default.find(filter).sort({ updatedAt: -1 }).limit(limit).skip(limit * (page - 1));
            res.status(http_statuses_1.default.OK).json({ totalPages, totalCount, reservations });
        }
        else
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.reservation.RESERVATION_DETAILS_MISSING });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getLevelReservationsService = getLevelReservationsService;
const getSpotAvailabilityService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { reservationDate, language = 'heb' } = req.query;
        const parkingSpot = yield parking_spot_1.default.findById(_id);
        if (parkingSpot) {
            const date = reservationDate ? new Date(reservationDate) : new Date();
            const { startOfDay, endOfDay } = (0, dates_1.getStartAndEndOfDay)(date);
            const reservations = yield reservation_1.default.find({ parkingSpot, reservationDate: { $gte: new Date(startOfDay), $lte: new Date(endOfDay) }, parkingStatus: { $in: ['reserved', 'occupied'] } });
            if ((reservations === null || reservations === void 0 ? void 0 : reservations.length) > 0)
                res.status(http_statuses_1.default.FORBIDDEN).json({ message: language == 'heb' ? http_messages_1.default.reservation.RESERVATION_SELECTED_SPOT_ALREADY_BOOKED_HEB : http_messages_1.default.reservation.RESERVATION_SELECTED_SPOT_ALREADY_BOOKED });
            else
                res.status(http_statuses_1.default.OK).json({ isAvailable: true });
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.parkingSpot.PARKING_SPOT_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getSpotAvailabilityService = getSpotAvailabilityService;
const getSingleReservationService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const reservation = yield reservation_1.default.findById(_id)
            .populate('parkingLot', ['name', 'accessMapFile', 'avatar', 'chargingTimeLimit', 'chargingTimeStart', 'chargingTimeEnd', 'reservationTime', 'messageSendTime', 'messageSendTimeOfDay',])
            .populate('level', ['level', 'map', 'entrance',])
            .populate('parkingSpot', ['parkingCode', 'name', 'avatar', 'spotType', 'innerToSpot', 'outerToSpot',])
            .populate({ path: 'moveTo', populate: { path: 'parkingSpot', select: ['parkingCode', 'name', 'avatar',] }, select: ['parkingSpot', 'reservationDate', 'parkingStatus'] })
            .populate({ path: 'moveFrom', populate: { path: 'parkingSpot', select: ['parkingCode', 'name', 'avatar',] }, select: ['parkingSpot', 'reservationDate', 'parkingStatus'] });
        if (reservation) {
            const outerReservation = () => __awaiter(void 0, void 0, void 0, function* () {
                if (reservation.parkingSpot.innerToSpot) {
                    const outerSpot = yield parking_spot_1.default.findById(reservation.parkingSpot.innerToSpot);
                    if (outerSpot) {
                        const { startOfDay, endOfDay } = (0, dates_1.getStartAndEndOfDay)(reservation.reservationDate);
                        const outerReservation = yield reservation_1.default.findOne({
                            parkingLot: reservation.parkingLot,
                            level: reservation.level,
                            parkingSpot: outerSpot._id,
                            reservationDate: { $gte: startOfDay, $lte: endOfDay },
                            parkingStatus: { $in: [controllers_1.reservationEnums[0], controllers_1.reservationEnums[1]] }
                        }).populate({ path: 'user', select: ['phoneNumber', 'firstName', 'lastName', 'vehicle', 'vehicles'], populate: 'vehicles' });
                        if (outerReservation)
                            return outerReservation;
                        else
                            return null;
                    }
                    else
                        return null;
                }
                else
                    return null;
            });
            const innerReservation = () => __awaiter(void 0, void 0, void 0, function* () {
                if (reservation.parkingSpot.outerToSpot) {
                    const innerSpot = yield parking_spot_1.default.findById(reservation.parkingSpot.outerToSpot);
                    if (innerSpot) {
                        const { startOfDay, endOfDay } = (0, dates_1.getStartAndEndOfDay)(reservation.reservationDate);
                        const innerReservation = yield reservation_1.default.findOne({
                            parkingLot: reservation.parkingLot,
                            level: reservation.level,
                            parkingSpot: innerSpot._id,
                            reservationDate: { $gte: startOfDay, $lte: endOfDay },
                            parkingStatus: { $in: [controllers_1.reservationEnums[0], controllers_1.reservationEnums[1]] }
                        }).populate({ path: 'user', select: ['phoneNumber', 'firstName', 'lastName', 'vehicle', 'vehicles'], populate: 'vehicles' });
                        if (innerReservation)
                            return innerReservation;
                        else
                            return null;
                    }
                    else
                        return null;
                }
                else
                    return null;
            });
            res.status(http_statuses_1.default.OK).json({ reservation, outerReservation: yield outerReservation(), innerReservation: yield innerReservation() });
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.reservation.RESERVATION_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getSingleReservationService = getSingleReservationService;
const updateReservationService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    var _4, _5;
    try {
        const { _id } = req.params;
        const { parkingStatus, user, reservationDate } = req.body;
        if (controllers_1.reservationEnums.includes(parkingStatus)) {
            const reservation = yield reservation_1.default.findById(_id).populate({ path: 'parkingSpot', select: ['designatedTo', 'spotType'] });
            if (reservation) {
                if (parkingStatus == controllers_1.reservationEnums[2]) {
                    if (reservation.moveTo) {
                        reservation.parkingStatus = controllers_1.reservationEnums[2];
                        yield reservation_1.default.findByIdAndUpdate(reservation.moveTo, { parkingStatus: 'occupied' });
                        yield reservation.save();
                    }
                    else
                        res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.reservation.RESERVATION_INVALID_STATUS });
                }
                if (controllers_1.reservationEnums.includes(parkingStatus)) {
                    reservation.parkingStatus = parkingStatus;
                    yield reservation.save();
                    if ((reservation === null || reservation === void 0 ? void 0 : reservation.moveTo) && (parkingStatus == controllers_1.reservationEnums[3] || parkingStatus == controllers_1.reservationEnums[4]))
                        yield reservation_1.default.findByIdAndUpdate(reservation === null || reservation === void 0 ? void 0 : reservation.moveTo, { parkingStatus });
                    if (parkingStatus == controllers_1.reservationEnums[3] && user && ((_5 = (_4 = reservation === null || reservation === void 0 ? void 0 : reservation.parkingSpot) === null || _4 === void 0 ? void 0 : _4.designatedTo) === null || _5 === void 0 ? void 0 : _5.toString()) == user) {
                        const { startOfDay, endOfDay } = (0, dates_1.getStartAndEndOfDay)(reservationDate);
                        yield parking_spot_1.default.findByIdAndUpdate(reservation === null || reservation === void 0 ? void 0 : reservation.parkingSpot, { userPresenceDate: null }, { new: true });
                        const absence = new absence_1.default({
                            user: user,
                            designatedSpot: reservation === null || reservation === void 0 ? void 0 : reservation.parkingSpot._id,
                            startDate: startOfDay,
                            endDate: endOfDay,
                            type: 'system-generated',
                            note: 'הזמנת מקום חניה הושלמה על ידי המשתמש.',
                        });
                        yield absence.save();
                    }
                    res.status(http_statuses_1.default.OK).json(reservation);
                }
                else
                    res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.reservation.RESERVATION_INVALID_STATUS });
            }
            else
                res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.reservation.RESERVATION_NOT_FOUND });
        }
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.updateReservationService = updateReservationService;
const changeReservedSpotService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { language = "heb", parkingSpot } = req.body;
        const reservation = yield reservation_1.default.findById(_id);
        if (reservation) {
            const spot = yield parking_spot_1.default.findById(parkingSpot);
            if (spot) {
                if (yield checkIfSpotHasReservations(spot, reservation.reservationDate))
                    res.status(http_statuses_1.default.FORBIDDEN).json({ message: language == 'heb' ? http_messages_1.default.reservation.RESERVATION_SELECTED_SPOT_ALREADY_BOOKED_HEB : http_messages_1.default.reservation.RESERVATION_SELECTED_SPOT_ALREADY_BOOKED });
                else {
                    const saved = yield reservation_1.default.findByIdAndUpdate(_id, { parkingSpot });
                    res.status(http_statuses_1.default.OK).json(saved);
                }
            }
            else
                res.status(http_statuses_1.default.NOT_FOUND).json({ message: language == 'heb' ? http_messages_1.default.parkingSpot.PARKING_SPOT_NOT_FOUND_HEB : http_messages_1.default.parkingSpot.PARKING_SPOT_NOT_FOUND });
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: language == 'heb' ? http_messages_1.default.reservation.RESERVATION_NOT_FOUND_HEB : http_messages_1.default.reservation.RESERVATION_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.changeReservedSpotService = changeReservedSpotService;
const deleteReservationService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const reservation = yield reservation_1.default.findByIdAndDelete(_id);
        if (reservation)
            res.status(http_statuses_1.default.OK).json({ message: http_messages_1.default.reservation.RESERVATION_DELETED_SUCCESS });
        else
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.reservation.RESERVATION_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.deleteReservationService = deleteReservationService;
