"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteVehicleService = exports.updateVehicleService = exports.searchFromRegistrationService = exports.getUserFromRegistrationService = exports.getUserVehiclesService = exports.getSingleVehicleService = exports.getAllVehiclesService = exports.createVehicleService = void 0;
const http_messages_1 = __importDefault(require("../../services/constants/http-messages"));
const http_statuses_1 = __importDefault(require("../../services/constants/http-statuses"));
const vehicle_1 = __importDefault(require("../../models/vehicle"));
const createVehicleService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { user, make, model, registration } = req.body;
        if (user && make && model && registration) {
            const vehicle = new vehicle_1.default({
                user,
                make,
                model,
                registration: registration.toUpperCase(),
            });
            yield vehicle.save();
            res.status(http_statuses_1.default.CREATED).json(vehicle);
        }
        else
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.vehicle.VEHICLE_DETAILS_MISSING });
    }
    catch (err) {
        if ((err === null || err === void 0 ? void 0 : err.code) == 11000)
            res.status(http_statuses_1.default.FORBIDDEN).json({ message: http_messages_1.default.vehicle.VEHICLE_REGISTRATION_DUPLICATE });
        else
            res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.createVehicleService = createVehicleService;
const getAllVehiclesService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { page = 1, limit = 10 } = req.query;
        const totalCount = yield vehicle_1.default.countDocuments();
        const totalPages = Math.ceil(totalCount / limit);
        const vehicles = yield vehicle_1.default.find().sort({ updatedAt: -1 }).limit(limit).skip(limit * (page - 1));
        res.status(http_statuses_1.default.OK).json({ totalPages, totalCount, vehicles });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getAllVehiclesService = getAllVehiclesService;
const getSingleVehicleService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const vehicle = yield vehicle_1.default.findById(_id);
        if (vehicle)
            res.status(http_statuses_1.default.OK).json(vehicle);
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.vehicle.VEHICLE_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getSingleVehicleService = getSingleVehicleService;
const getUserVehiclesService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { page = 1, limit = 10 } = req.query;
        const totalCount = yield vehicle_1.default.countDocuments({ user: _id });
        const totalPages = Math.ceil(totalCount / limit);
        const vehicles = yield vehicle_1.default.find({ user: _id }).sort({ updatedAt: -1 }).limit(limit).skip(limit * (page - 1));
        res.status(http_statuses_1.default.OK).json({ totalPages, totalCount, vehicles });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getUserVehiclesService = getUserVehiclesService;
const getUserFromRegistrationService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { registration } = req.params;
        const query = { registration: { $regex: new RegExp(registration.toUpperCase(), 'i') } };
        const vehicle = yield vehicle_1.default.findOne(query).sort({ updatedAt: -1 }).populate({ path: 'user', select: 'phoneNumber', });
        if (vehicle)
            res.status(http_statuses_1.default.OK).json(vehicle);
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.user.USER_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getUserFromRegistrationService = getUserFromRegistrationService;
const searchFromRegistrationService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { registration } = req.query;
        if (!registration)
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.vehicle.VEHICLE_DETAILS_MISSING });
        else {
            const regex = new RegExp(registration, 'i');
            const vehicles = yield vehicle_1.default.find({ registration: { $regex: regex } });
            // const users = await User.find({ 'vehicle.registration': { $regex: regex } });
            // const vehicles = await Promise.all(users.map(user => user.vehicle))
            res.status(http_statuses_1.default.OK).json(vehicles);
        }
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.searchFromRegistrationService = searchFromRegistrationService;
const updateVehicleService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { make, model, registration } = req.body;
        if (make && model && registration) {
            const vehicle = yield vehicle_1.default.findByIdAndUpdate(_id, { make, model, registration, }, { new: true });
            res.status(http_statuses_1.default.OK).json(vehicle);
        }
        else
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.vehicle.VEHICLE_DETAILS_MISSING });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.updateVehicleService = updateVehicleService;
const deleteVehicleService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const vehicle = yield vehicle_1.default.findByIdAndDelete(_id);
        if (vehicle)
            res.status(http_statuses_1.default.OK).json({ message: http_messages_1.default.vehicle.VEHICLE_DELETED_SUCCESS });
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.vehicle.VEHICLE_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.deleteVehicleService = deleteVehicleService;
