"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.unlockAdminAccountService = exports.resetPasswordService = exports.loginWithTemporaryPasswordService = exports.forgotPasswordService = exports.loginAdminService = exports.updateAdminService = exports.deleteAdminService = exports.getAllAdminsService = exports.getParkingLotAdminsService = exports.getSingleAdminService = exports.createAdminService = void 0;
const generators_1 = require("../../helpers/generators");
const validators_1 = require("../../helpers/validators");
const email_messaging_1 = require("../../middlewares/email-messaging");
const admin_1 = __importDefault(require("../../models/admin"));
const http_messages_1 = __importDefault(require("../../services/constants/http-messages"));
const http_statuses_1 = __importDefault(require("../../services/constants/http-statuses"));
const messages_1 = __importDefault(require("../../services/constants/messages"));
const parking_lot_1 = __importDefault(require("../../models/parking-lot"));
const authentication_1 = require("../../middlewares/authentication");
const createAdminService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { email, password, confirmPassword, firstName, middleName, lastName, avatar, role, phoneNumber, generateRandom, currentAdmin } = req.body;
        if (!(0, validators_1.validateEmail)(email)) {
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.validations.INVALID_EMAIL });
        }
        else if (!(0, validators_1.validatePassword)(password) && !generateRandom) {
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.validations.INVALID_PASSWORD });
        }
        else if (!firstName) {
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.admin.ADMIN_DETAILS_MISSING });
        }
        else if (yield admin_1.default.findOne({ email: String(email).toLowerCase() })) {
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.admin.ADMIN_ALREADY_EXISTS });
        }
        else if (password != confirmPassword && !generateRandom) {
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.validations.PASSWORDS_NOT_MATCH });
        }
        else {
            const currAdmin = yield admin_1.default.findById(currentAdmin);
            const generatedPassword = (0, generators_1.generateTemporaryPassword)();
            const hashedPassword = yield (0, generators_1.generateHashPassword)(generateRandom ? generatedPassword.password : password);
            const admin = new admin_1.default({
                avatar,
                email: String(email).toLowerCase(),
                password: hashedPassword,
                firstName,
                middleName,
                lastName,
                role: (currAdmin === null || currAdmin === void 0 ? void 0 : currAdmin.role) == 'sub-admin' ? 'sub-admin' : role,
                phoneNumber,
            });
            const savedAdmin = yield admin.save();
            const emailSent = yield (0, email_messaging_1.sendEmail)(email, messages_1.default.emailSubjects.createAdmin, messages_1.default.emailDetails.createAdmin + `Link: ${process.env.LIVE_BASE_URL}\nEmail: ${email}\nPassword: ${generateRandom ? generatedPassword.password : password}`);
            if (generateRandom)
                res.status(http_statuses_1.default.CREATED).json(Object.assign(Object.assign({}, savedAdmin.toObject()), { password: generatedPassword, emailSent }));
            else
                res.status(http_statuses_1.default.CREATED).json(savedAdmin);
        }
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.createAdminService = createAdminService;
const getSingleAdminService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const admin = yield admin_1.default.findById(_id).select(['-password', '-temporaryPassword']);
        if (admin)
            res.status(http_statuses_1.default.OK).json(admin);
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.admin.ADMIN_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getSingleAdminService = getSingleAdminService;
const getParkingLotAdminsService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const parkingLot = yield parking_lot_1.default.findById(_id).populate({ path: 'admins', select: ['firstName', 'lastName', 'phoneNumber'] }).select(['admins']);
        if (parkingLot) {
            const { admins } = parkingLot;
            res.status(http_statuses_1.default.OK).json(admins);
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.parkingLot.PARKING_LOT_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getParkingLotAdminsService = getParkingLotAdminsService;
const getAllAdminsService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { currentAdmin, page = 1, limit = 10, filter } = req.query;
        const query = {
            _id: { $ne: currentAdmin },
        };
        if (filter)
            query.role = filter;
        if (currentAdmin) {
            const admin = yield admin_1.default.findById(currentAdmin);
            if ((admin === null || admin === void 0 ? void 0 : admin.role) == 'sub-admin')
                query.role = 'sub-admin';
        }
        const totalCount = yield admin_1.default.countDocuments(query);
        const totalPages = Math.ceil(totalCount / limit);
        const admins = yield admin_1.default.find(query).sort({ updatedAt: -1 }).limit(limit).skip(limit * (page - 1)).select(['-password', '-temporaryPassword']);
        res.status(http_statuses_1.default.OK).json({ totalPages, totalCount, admins });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getAllAdminsService = getAllAdminsService;
const deleteAdminService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const admin = yield admin_1.default.findById(_id);
        if (admin) {
            if (admin.parkingLots.length > 0) {
                res.status(http_statuses_1.default.FORBIDDEN).json({ message: http_messages_1.default.admin.ADMIN_DELETE_DEPENDENCIES, });
            }
            else {
                yield admin_1.default.findByIdAndDelete(_id);
                res.status(http_statuses_1.default.OK).json({ message: http_messages_1.default.admin.ADMIN_DELETED_SUCCESS });
            }
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.admin.ADMIN_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.deleteAdminService = deleteAdminService;
const updateAdminService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { firstName, middleName, lastName, avatar, role } = req.body;
        const admin = yield admin_1.default.findByIdAndUpdate(_id, { firstName, middleName, lastName, avatar, role }, { omitUndefined: true, new: true, });
        if (admin)
            res.status(http_statuses_1.default.OK).json(admin);
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.admin.ADMIN_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.updateAdminService = updateAdminService;
const loginAdminService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { email, password } = req.body;
        if (!(0, validators_1.validateEmail)(email)) {
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.validations.INVALID_EMAIL });
        }
        else {
            const admin = yield admin_1.default.findOne({ email: String(email).toLowerCase() });
            if (admin) {
                if (admin.failedLoginAttempts > 2) {
                    res.status(http_statuses_1.default.FORBIDDEN).json({ message: http_messages_1.default.admin.ACCOUNT_LOCKED });
                }
                else {
                    const isValid = (0, validators_1.isPasswordValid)(password, admin.password);
                    if (isValid) {
                        const isPasswordExpired = (0, validators_1.isDate90DaysOld)(new Date(admin.passwordUpdatedAt));
                        if (isPasswordExpired) {
                            res.status(http_statuses_1.default.FORBIDDEN).json({ message: http_messages_1.default.admin.PASSWORD_EXPIRED });
                        }
                        else {
                            admin.temporaryPassword = null;
                            yield admin.save();
                            const newAdmin = yield admin_1.default.findById(admin._id).select(['-password', '-temporaryPassword']);
                            const token = (0, authentication_1.signJwt)(admin === null || admin === void 0 ? void 0 : admin._id);
                            res.status(http_statuses_1.default.OK).json({ admin: newAdmin, token });
                        }
                    }
                    else {
                        admin.failedLoginAttempts++;
                        admin.failedAttemptsDates.push(new Date());
                        yield admin.save();
                        if (admin.failedLoginAttempts > 2)
                            admin.accountLockInstances.push(new Date());
                        yield admin.save();
                        res.status(http_statuses_1.default.UNAUTHORIZED).json({ message: http_messages_1.default.admin.FAILED_LOGIN_ATTEMPT + `You have ${3 - admin.failedLoginAttempts} left.` });
                    }
                }
            }
            else
                res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.admin.ADMIN_NOT_FOUND });
        }
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.loginAdminService = loginAdminService;
const forgotPasswordService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { email, isDev } = req.body;
        if (!(0, validators_1.validateEmail)(email)) {
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.validations.INVALID_EMAIL });
        }
        else {
            const admin = yield admin_1.default.findOne({ email });
            if (admin) {
                if (admin.failedLoginAttempts > 2) {
                    res.status(http_statuses_1.default.FORBIDDEN).json({ message: http_messages_1.default.admin.ACCOUNT_LOCKED });
                }
                else {
                    const { expiry, password } = (0, generators_1.generateTemporaryPassword)();
                    admin.temporaryPassword = yield (0, generators_1.generateHashPassword)(isDev ? 'Dev@12345' : password);
                    admin.temporaryPasswordExpiry = expiry;
                    yield admin.save();
                    yield (0, email_messaging_1.sendEmail)(admin.email, messages_1.default.emailSubjects.temporaryPasswordCreated, messages_1.default.emailDetails.temporaryPasswordCreated + `Email: ${admin.email}\nPassword: ${isDev ? 'Dev@12345' : password}`);
                    res.status(http_statuses_1.default.CREATED).json({ email, message: http_messages_1.default.admin.TEMPORARY_PASSWORD_CREATED });
                }
            }
            else
                res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.admin.ADMIN_NOT_FOUND });
        }
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.forgotPasswordService = forgotPasswordService;
const loginWithTemporaryPasswordService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { email, temporaryPassword, newPassword, confirmPassword } = req.body;
        if (!(0, validators_1.validateEmail)(email)) {
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.validations.INVALID_EMAIL });
        }
        else {
            const admin = yield admin_1.default.findOne({ email: String(email).toLowerCase() });
            if (admin) {
                if (admin.failedLoginAttempts > 2) {
                    res.status(http_statuses_1.default.FORBIDDEN).json({ message: http_messages_1.default.admin.ACCOUNT_LOCKED });
                }
                else {
                    const isValid = (0, validators_1.isPasswordValid)(temporaryPassword, admin.temporaryPassword);
                    if (isValid) {
                        if (new Date(admin.temporaryPasswordExpiry) > new Date()) {
                            if (newPassword === confirmPassword) {
                                const hashedPassword = yield (0, generators_1.generateHashPassword)(newPassword);
                                admin.password = hashedPassword;
                                admin.temporaryPassword = null;
                                admin.passwordUpdatedAt = Date.now();
                                yield admin.save();
                                res.status(http_statuses_1.default.OK).json({ message: http_messages_1.default.admin.PASSWORD_UPDATE_SUCCESS });
                            }
                            else
                                res.status(http_statuses_1.default.FORBIDDEN).json({ message: http_messages_1.default.validations.PASSWORDS_NOT_MATCH });
                        }
                        else
                            res.status(http_statuses_1.default.FORBIDDEN).json({ message: http_messages_1.default.validations.TEMP_PASSWORD_EXPIRED });
                    }
                    else
                        res.status(http_statuses_1.default.UNAUTHORIZED).json({ message: http_messages_1.default.admin.FAILED_TEMPORARY_LOGIN });
                }
            }
            else
                res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.admin.ADMIN_NOT_FOUND });
        }
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.loginWithTemporaryPasswordService = loginWithTemporaryPasswordService;
const resetPasswordService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { email, oldPassword, newPassword, confirmPassword } = req.body;
        if (!(0, validators_1.validateEmail)(email)) {
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.validations.INVALID_EMAIL });
        }
        else {
            const admin = yield admin_1.default.findOne({ email });
            if (admin) {
                if (admin.failedLoginAttempts > 2) {
                    res.status(http_statuses_1.default.FORBIDDEN).json({ message: http_messages_1.default.admin.ACCOUNT_LOCKED });
                }
                else {
                    const isValid = (0, validators_1.isPasswordValid)(oldPassword, admin.password);
                    if (isValid) {
                        if (newPassword === confirmPassword) {
                            const hashedPassword = yield (0, generators_1.generateHashPassword)(newPassword);
                            admin.password = hashedPassword;
                            admin.temporaryPassword = null;
                            admin.passwordUpdatedAt = Date.now();
                            yield admin.save();
                            res.status(http_statuses_1.default.OK).json({ message: http_messages_1.default.admin.PASSWORD_UPDATE_SUCCESS });
                        }
                        else
                            res.status(http_statuses_1.default.FORBIDDEN).json({ message: http_messages_1.default.validations.PASSWORDS_NOT_MATCH });
                    }
                    else
                        res.status(http_statuses_1.default.UNAUTHORIZED).json({ message: http_messages_1.default.admin.WRONG_OLD_PASSWORD });
                }
            }
            else
                res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.admin.ADMIN_NOT_FOUND });
        }
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.resetPasswordService = resetPasswordService;
const unlockAdminAccountService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const admin = yield admin_1.default.findByIdAndUpdate(_id, { failedLoginAttempts: 0, failedAttemptsDates: [], });
        if (admin)
            res.status(http_statuses_1.default.OK).json({ message: http_messages_1.default.admin.ACCOUNT_UNLOCKED });
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.admin.ADMIN_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.unlockAdminAccountService = unlockAdminAccountService;
