"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteLevelService = exports.updateLotLevelService = exports.getParkingLotLevelAvailableSpotsService = exports.getParkingLotLevelsService = exports.getSingleLevelService = exports.getAllLotLevelsService = exports.createLotLevelService = void 0;
const http_messages_1 = __importDefault(require("../../services/constants/http-messages"));
const http_statuses_1 = __importDefault(require("../../services/constants/http-statuses"));
const generators_1 = require("../../helpers/generators");
const parking_lot_1 = __importDefault(require("../../models/parking-lot"));
const lot_level_1 = __importDefault(require("../../models/lot-level"));
const parking_spot_1 = __importDefault(require("../../models/parking-spot"));
const reservation_1 = __importDefault(require("../../models/reservation"));
const controllers_1 = require("../reservations/controllers");
const helpers_1 = require("../../helpers");
const validators_1 = require("../../helpers/validators");
const createCustomSpotsHelper = (req, res, lot, createNewLevel) => __awaiter(void 0, void 0, void 0, function* () {
    return new Promise((resolve, reject) => __awaiter(void 0, void 0, void 0, function* () {
        try {
            const { _id } = req.params;
            const { parkingLot, level, name, map, entrance, regularParkingSpots, designatedParkingSpots, electricParkingSpots, disabledParkingSpots, doubleParkingSpots, frozenParkingSpots, language = 'heb' } = req.body;
            const newLevel = new lot_level_1.default({
                parkingLot,
                name,
                level,
                map,
                entrance,
            });
            const levelCode = (0, generators_1.generateLevelCode)(level);
            let regularSpotNumbers = [];
            let designatedSpotNumbers = [];
            let electricSpotNumbers = [];
            let disabledSpotNumbers = [];
            let doubleSpotNumbers = [];
            let frozenSpotNumbers = [];
            if (regularParkingSpots)
                regularSpotNumbers = String(regularParkingSpots.match(/\d+/g)).split(',').filter(num => num !== '');
            if (designatedParkingSpots)
                designatedSpotNumbers = String(designatedParkingSpots.match(/\d+/g)).split(',').filter(num => num !== '');
            if (electricParkingSpots)
                electricSpotNumbers = String(electricParkingSpots.match(/\d+/g)).split(',').filter(num => num !== '');
            if (disabledParkingSpots)
                disabledSpotNumbers = String(disabledParkingSpots.match(/\d+/g)).split(',').filter(num => num !== '');
            if (doubleParkingSpots)
                doubleSpotNumbers = String(doubleParkingSpots.match(/\d+/g)).split(',').filter(num => num !== '');
            if (frozenParkingSpots)
                frozenSpotNumbers = String(frozenParkingSpots.match(/\d+/g)).split(',').filter(num => num !== '');
            if ((0, validators_1.parkingCodesOverLapCheck)(regularSpotNumbers, designatedSpotNumbers, electricSpotNumbers, disabledSpotNumbers))
                res.status(http_statuses_1.default.BAD_REQUEST).json({ message: language == 'heb' ? http_messages_1.default.parkingSpot.FAILURE_CREATING_DUPLICATE_VALUES_HEB : http_messages_1.default.parkingSpot.FAILURE_CREATING_DUPLICATE_VALUES });
            else {
                const createdRegular = yield Promise.all(regularSpotNumbers.map((spotNumber) => __awaiter(void 0, void 0, void 0, function* () {
                    if (doubleSpotNumbers.includes(spotNumber)) {
                        const parkingSpotA = new parking_spot_1.default({
                            parkingLot: lot._id,
                            level: createNewLevel ? newLevel._id : _id,
                            spotType: 'regular',
                            parkingCode: `${levelCode}${spotNumber}-A`,
                            isPermanent: false,
                        });
                        const parkingSpotB = new parking_spot_1.default({
                            parkingLot: lot._id,
                            level: createNewLevel ? newLevel._id : _id,
                            spotType: 'regular',
                            parkingCode: `${levelCode}${spotNumber}-B`,
                            outerToSpot: parkingSpotA._id,
                            isPermanent: false,
                        });
                        parkingSpotA.innerToSpot = parkingSpotB._id;
                        return [parkingSpotA, parkingSpotB];
                    }
                    else {
                        const parkingSpot = new parking_spot_1.default({
                            parkingLot: lot._id,
                            level: createNewLevel ? newLevel._id : _id,
                            spotType: 'regular',
                            parkingCode: `${levelCode}${spotNumber}`,
                            isPermanent: false,
                        });
                        return parkingSpot;
                    }
                })));
                const createdDesignated = yield Promise.all(designatedSpotNumbers.map((spotNumber) => __awaiter(void 0, void 0, void 0, function* () {
                    const index = yield Promise.resolve(createdRegular.flat().findIndex(item => (item === null || item === void 0 ? void 0 : item.parkingCode) == `${levelCode}${spotNumber}`));
                    if (index < 0) {
                        if (doubleSpotNumbers.includes(spotNumber)) {
                            const parkingSpotA = new parking_spot_1.default({
                                parkingLot: lot._id,
                                level: createNewLevel ? newLevel._id : _id,
                                spotType: 'designated',
                                parkingCode: `${levelCode}${spotNumber}-A`,
                                isPermanent: frozenSpotNumbers.includes(spotNumber) ? true : false,
                            });
                            const parkingSpotB = new parking_spot_1.default({
                                parkingLot: lot._id,
                                level: createNewLevel ? newLevel._id : _id,
                                spotType: 'designated',
                                parkingCode: `${levelCode}${spotNumber}-B`,
                                outerToSpot: parkingSpotA._id,
                                isPermanent: frozenSpotNumbers.includes(spotNumber) ? true : false,
                            });
                            parkingSpotA.innerToSpot = parkingSpotB._id;
                            return [parkingSpotA, parkingSpotB];
                        }
                        else {
                            const parkingSpot = new parking_spot_1.default({
                                parkingLot: lot._id,
                                level: createNewLevel ? newLevel._id : _id,
                                spotType: 'designated',
                                parkingCode: `${levelCode}${spotNumber}`,
                                isPermanent: frozenSpotNumbers.includes(spotNumber) ? true : false,
                            });
                            return parkingSpot;
                        }
                    }
                })));
                const createdElectric = yield Promise.all(electricSpotNumbers.map((spotNumber) => __awaiter(void 0, void 0, void 0, function* () {
                    const index = yield Promise.resolve(createdDesignated.flat().findIndex(item => (item === null || item === void 0 ? void 0 : item.parkingCode) == `${levelCode}${spotNumber}`));
                    const _index = yield Promise.resolve(createdRegular.flat().findIndex(item => (item === null || item === void 0 ? void 0 : item.parkingCode) == `${levelCode}${spotNumber}`));
                    if (index < 0 && _index < 0) {
                        if (doubleSpotNumbers.includes(spotNumber)) {
                            const parkingSpotA = new parking_spot_1.default({
                                parkingLot: lot._id,
                                level: createNewLevel ? newLevel._id : _id,
                                spotType: 'electric',
                                parkingCode: `${levelCode}${spotNumber}-A`,
                                isPermanent: false,
                            });
                            const parkingSpotB = new parking_spot_1.default({
                                parkingLot: lot._id,
                                level: createNewLevel ? newLevel._id : _id,
                                spotType: 'electric',
                                parkingCode: `${levelCode}${spotNumber}-B`,
                                outerToSpot: parkingSpotA._id,
                                isPermanent: false,
                            });
                            parkingSpotA.innerToSpot = parkingSpotB._id;
                            return [parkingSpotA, parkingSpotB];
                        }
                        else {
                            const parkingSpot = new parking_spot_1.default({
                                parkingLot: lot._id,
                                level: createNewLevel ? newLevel._id : _id,
                                spotType: 'electric',
                                parkingCode: `${levelCode}${spotNumber}`,
                            });
                            return parkingSpot;
                        }
                    }
                })));
                const createdDisabled = yield Promise.all(disabledSpotNumbers.map((spotNumber) => __awaiter(void 0, void 0, void 0, function* () {
                    const index = yield Promise.resolve(createdDesignated.flat().findIndex(item => (item === null || item === void 0 ? void 0 : item.parkingCode) == `${levelCode}${spotNumber}`));
                    const _index = yield Promise.resolve(createdRegular.flat().findIndex(item => (item === null || item === void 0 ? void 0 : item.parkingCode) == `${levelCode}${spotNumber}`));
                    if (index < 0 && _index < 0) {
                        if (index < 0 && _index < 0) {
                            if (doubleSpotNumbers.includes(spotNumber)) {
                                const parkingSpotA = new parking_spot_1.default({
                                    parkingLot: lot._id,
                                    level: createNewLevel ? newLevel._id : _id,
                                    spotType: 'disabled',
                                    parkingCode: `${levelCode}${spotNumber}-A`,
                                    isPermanent: false,
                                });
                                const parkingSpotB = new parking_spot_1.default({
                                    parkingLot: lot._id,
                                    level: createNewLevel ? newLevel._id : _id,
                                    spotType: 'disabled',
                                    parkingCode: `${levelCode}${spotNumber}-B`,
                                    outerToSpot: parkingSpotA._id,
                                    isPermanent: false,
                                });
                                parkingSpotA.innerToSpot = parkingSpotB._id;
                                return [parkingSpotA, parkingSpotB];
                            }
                            else {
                                const parkingSpot = new parking_spot_1.default({
                                    parkingLot: lot._id,
                                    level: createNewLevel ? newLevel._id : _id,
                                    spotType: 'disabled',
                                    parkingCode: `${levelCode}${spotNumber}`,
                                    isPermanent: false,
                                });
                                return parkingSpot;
                            }
                        }
                    }
                })));
                const nullValues = [...createdRegular, ...createdElectric, ...createdDesignated, ...createdDisabled].some(value => value == null);
                if (nullValues)
                    res.status(http_statuses_1.default.BAD_REQUEST).json({ message: language == 'heb' ? http_messages_1.default.parkingSpot.FAILURE_CREATING_DUPLICATE_VALUES_HEB : http_messages_1.default.parkingSpot.FAILURE_CREATING_DUPLICATE_VALUES });
                else {
                    if (createNewLevel)
                        yield newLevel.save();
                    if (createNewLevel != undefined && createNewLevel == false) {
                        // if (regularSpotNumbers.length > 0) await ParkingSpot.deleteMany({ level: _id, spotType: 'regular' })
                        // if (designatedSpotNumbers.length > 0) await ParkingSpot.deleteMany({ level: _id, spotType: 'designated' })
                        // if (electricSpotNumbers.length > 0) await ParkingSpot.deleteMany({ level: _id, spotType: 'electric' })
                        // if (disabledSpotNumbers.length > 0) await ParkingSpot.deleteMany({ level: _id, spotType: 'disabled' })
                        // if (doubleSpotNumbers.length > 0) await ParkingSpot.deleteMany({ level: _id, spotType: 'double' })
                        yield parking_spot_1.default.deleteMany({ level: _id, spotType: 'regular' });
                        yield parking_spot_1.default.deleteMany({ level: _id, spotType: 'designated' });
                        yield parking_spot_1.default.deleteMany({ level: _id, spotType: 'electric' });
                        yield parking_spot_1.default.deleteMany({ level: _id, spotType: 'disabled' });
                        yield parking_spot_1.default.deleteMany({ level: _id, spotType: 'double' });
                    }
                    const parkingSpots = yield Promise.all([...createdRegular.flat(), ...createdElectric.flat(), ...createdDesignated.flat(), ...createdDisabled.flat()].filter((spot) => __awaiter(void 0, void 0, void 0, function* () {
                        if (spot != null) {
                            setTimeout(() => __awaiter(void 0, void 0, void 0, function* () {
                                if (Array.isArray(spot)) {
                                    yield Promise.all(spot.map((s) => __awaiter(void 0, void 0, void 0, function* () { return yield s.save(); })));
                                    return spot;
                                }
                                else {
                                    yield spot.save();
                                    return spot;
                                }
                            }), 100);
                        }
                    })));
                    resolve({ newLevel, parkingSpots });
                }
            }
        }
        catch (err) {
            reject(err);
        }
    }));
});
const processUpdateSpots = (req, res, existingLevel) => __awaiter(void 0, void 0, void 0, function* () {
    return new Promise((resolve, reject) => __awaiter(void 0, void 0, void 0, function* () {
        try {
            const { _id } = req.params;
            const { parkingLot, level, name, map, entrance, regularParkingSpots, designatedParkingSpots, electricParkingSpots, disabledParkingSpots, doubleParkingSpots, frozenParkingSpots, language = 'heb', } = req.body;
            const levelCode = (0, generators_1.generateLevelCode)(level);
            let regularSpotNumbers = [];
            let designatedSpotNumbers = [];
            let electricSpotNumbers = [];
            let disabledSpotNumbers = [];
            let doubleSpotNumbers = [];
            let frozenSpotNumbers = [];
            let regularSpotCodes = [];
            let designatedSpotCodes = [];
            let electricSpotCodes = [];
            let disabledSpotCodes = [];
            let frozenSpotCodes = [];
            if (regularParkingSpots)
                regularSpotNumbers = String(regularParkingSpots.match(/\d+/g)).split(',').filter(num => num !== '');
            if (designatedParkingSpots)
                designatedSpotNumbers = String(designatedParkingSpots.match(/\d+/g)).split(',').filter(num => num !== '');
            if (electricParkingSpots)
                electricSpotNumbers = String(electricParkingSpots.match(/\d+/g)).split(',').filter(num => num !== '');
            if (disabledParkingSpots)
                disabledSpotNumbers = String(disabledParkingSpots.match(/\d+/g)).split(',').filter(num => num !== '');
            if (doubleParkingSpots)
                doubleSpotNumbers = String(doubleParkingSpots.match(/\d+/g)).split(',').filter(num => num !== '');
            if (frozenParkingSpots)
                frozenSpotNumbers = String(frozenParkingSpots.match(/\d+/g)).split(',').filter(num => num !== '');
            if ((0, validators_1.parkingCodesOverLapCheck)(regularSpotNumbers, designatedSpotNumbers, electricSpotNumbers, disabledSpotNumbers))
                res.status(http_statuses_1.default.BAD_REQUEST).json({ message: language == 'heb' ? http_messages_1.default.parkingSpot.FAILURE_UPDATING_DUPLICATE_VALUES_HEB : http_messages_1.default.parkingSpot.FAILURE_UPDATING_DUPLICATE_VALUES });
            else {
                regularSpotNumbers.map((num) => {
                    if (doubleSpotNumbers.includes(num)) {
                        regularSpotCodes.push(`${levelCode}${num}-A`);
                        regularSpotCodes.push(`${levelCode}${num}-B`);
                    }
                    else
                        regularSpotCodes.push(`${levelCode}${num}`);
                });
                designatedSpotNumbers.map(num => {
                    if (doubleSpotNumbers.includes(num)) {
                        designatedSpotCodes.push(`${levelCode}${num}-A`);
                        designatedSpotCodes.push(`${levelCode}${num}-B`);
                    }
                    else
                        designatedSpotCodes.push(`${levelCode}${num}`);
                });
                electricSpotNumbers.map(num => {
                    if (doubleSpotNumbers.includes(num)) {
                        electricSpotCodes.push(`${levelCode}${num}-A`);
                        electricSpotCodes.push(`${levelCode}${num}-B`);
                    }
                    else
                        electricSpotCodes.push(`${levelCode}${num}`);
                });
                disabledSpotNumbers.map(num => {
                    if (doubleSpotNumbers.includes(num)) {
                        disabledSpotCodes.push(`${levelCode}${num}-A`);
                        disabledSpotCodes.push(`${levelCode}${num}-B`);
                    }
                    else
                        disabledSpotCodes.push(`${levelCode}${num}`);
                });
                frozenSpotNumbers.map(num => {
                    if (doubleSpotNumbers.includes(num)) {
                        frozenSpotCodes.push(`${levelCode}${num}-A`);
                        frozenSpotCodes.push(`${levelCode}${num}-B`);
                    }
                    else
                        frozenSpotCodes.push(`${levelCode}${num}`);
                });
                const existingRegularString = (yield parking_spot_1.default.find({ level: existingLevel === null || existingLevel === void 0 ? void 0 : existingLevel._id, spotType: 'regular', }).sort({ parkingCode: 1 }).select('parkingCode')).map(spot => spot.parkingCode);
                const existingDesignatedString = (yield parking_spot_1.default.find({ level: existingLevel === null || existingLevel === void 0 ? void 0 : existingLevel._id, spotType: 'designated', }).sort({ parkingCode: 1 }).select('parkingCode')).map(spot => spot.parkingCode);
                const existingElectricString = (yield parking_spot_1.default.find({ level: existingLevel === null || existingLevel === void 0 ? void 0 : existingLevel._id, spotType: 'electric', }).sort({ parkingCode: 1 }).select('parkingCode')).map(spot => spot.parkingCode);
                const existingDisabledString = (yield parking_spot_1.default.find({ level: existingLevel === null || existingLevel === void 0 ? void 0 : existingLevel._id, spotType: 'disabled', }).sort({ parkingCode: 1 }).select('parkingCode')).map(spot => spot.parkingCode);
                const existingFrozenString = (yield parking_spot_1.default.find({ level: existingLevel === null || existingLevel === void 0 ? void 0 : existingLevel._id, spotType: 'designated', isPermanent: true, }).sort({ parkingCode: 1 }).select('parkingCode')).map(spot => spot.parkingCode);
                const regularStatus = (0, validators_1.compareOldAndNewParkingCodes)(existingRegularString, regularSpotCodes);
                const designatedStatus = (0, validators_1.compareOldAndNewParkingCodes)(existingDesignatedString, designatedSpotCodes);
                const electricStatus = (0, validators_1.compareOldAndNewParkingCodes)(existingElectricString, electricSpotCodes);
                const disabledStatus = (0, validators_1.compareOldAndNewParkingCodes)(existingDisabledString, disabledSpotCodes);
                const frozenStatus = (0, validators_1.compareOldAndNewParkingCodes)(existingFrozenString, frozenSpotCodes);
                const createdRegular = [];
                const createdDesignated = [];
                const createdElectric = [];
                const createdDisabled = [];
                if (regularStatus) {
                    if (regularStatus.removed.length > 0) {
                        yield Promise.all(regularStatus.removed.map((parkingCode) => __awaiter(void 0, void 0, void 0, function* () {
                            const deletedSpot = yield parking_spot_1.default.findOneAndDelete({ level: existingLevel === null || existingLevel === void 0 ? void 0 : existingLevel._id, spotType: 'regular', parkingCode, });
                            if (deletedSpot) {
                                const filter = { parkingSpot: deletedSpot._id, parkingStatus: { $in: [controllers_1.reservationEnums[0], controllers_1.reservationEnums[1], controllers_1.reservationEnums[2]] } };
                                const deletedReservations = yield reservation_1.default.deleteMany(filter);
                            }
                        })));
                    }
                    if (regularStatus.added.length > 0) {
                        const { singleSpots, doubleSpots } = (0, validators_1.separateDoubleAndSingleSpots)(regularStatus.added);
                        if (singleSpots.length > 0) {
                            yield Promise.all(singleSpots.map((parkingCode) => __awaiter(void 0, void 0, void 0, function* () {
                                const parkingSpot = new parking_spot_1.default({
                                    parkingLot: parkingLot,
                                    level: existingLevel._id,
                                    spotType: 'regular',
                                    parkingCode: parkingCode,
                                    isPermanent: false,
                                });
                                createdRegular.push(parkingSpot);
                            })));
                        }
                        if (doubleSpots.length > 0) {
                            yield Promise.all(doubleSpots.map((spots) => __awaiter(void 0, void 0, void 0, function* () {
                                const parkingSpotA = new parking_spot_1.default({
                                    parkingLot: parkingLot,
                                    level: existingLevel._id,
                                    spotType: 'regular',
                                    parkingCode: spots[0],
                                    isPermanent: false,
                                });
                                const parkingSpotB = new parking_spot_1.default({
                                    parkingLot: parkingLot,
                                    level: existingLevel._id,
                                    spotType: 'regular',
                                    parkingCode: spots[1],
                                    outerToSpot: parkingSpotA._id,
                                    isPermanent: false,
                                });
                                parkingSpotA.innerToSpot = parkingSpotB._id;
                                createdRegular.push(parkingSpotA);
                                createdRegular.push(parkingSpotB);
                            })));
                        }
                    }
                }
                if (designatedStatus || frozenStatus) {
                    const existingRegularString = (yield parking_spot_1.default.find({ level: existingLevel === null || existingLevel === void 0 ? void 0 : existingLevel._id, spotType: 'regular', }).sort({ parkingCode: 1 }).select('parkingCode')).map(spot => spot.parkingCode);
                    if (designatedStatus.removed.length > 0 || frozenStatus.removed.length > 0) {
                        yield Promise.all(designatedStatus.removed.map((parkingCode) => __awaiter(void 0, void 0, void 0, function* () {
                            const deletedSpot = yield parking_spot_1.default.findOneAndDelete({ level: existingLevel === null || existingLevel === void 0 ? void 0 : existingLevel._id, spotType: 'designated', parkingCode, });
                            if (deletedSpot) {
                                const deletedReservations = yield reservation_1.default.deleteMany({ parkingSpot: deletedSpot._id, parkingStatus: { $in: [controllers_1.reservationEnums[0], controllers_1.reservationEnums[1], controllers_1.reservationEnums[2]] } });
                            }
                        })));
                        yield Promise.all(frozenStatus.removed.map((parkingCode) => __awaiter(void 0, void 0, void 0, function* () {
                            const updatedSpot = yield parking_spot_1.default.findOneAndUpdate({
                                level: existingLevel === null || existingLevel === void 0 ? void 0 : existingLevel._id,
                                spotType: 'designated',
                                parkingCode,
                                isPermanent: true
                            }, { isPermanent: false });
                            if (updatedSpot) {
                                const deletedReservations = yield reservation_1.default.deleteMany({ parkingSpot: updatedSpot._id, parkingStatus: { $in: [controllers_1.reservationEnums[0], controllers_1.reservationEnums[1], controllers_1.reservationEnums[2]] } });
                            }
                        })));
                    }
                    if (designatedStatus.added.length > 0 || frozenStatus.added.length > 0) {
                        if (designatedStatus.added.length > 0) {
                            const { singleSpots, doubleSpots } = (0, validators_1.separateDoubleAndSingleSpots)(designatedStatus.added);
                            if (singleSpots.length > 0) {
                                yield Promise.all(singleSpots.map((parkingCode) => __awaiter(void 0, void 0, void 0, function* () {
                                    const index = yield Promise.resolve(createdRegular.findIndex(item => (item === null || item === void 0 ? void 0 : item.parkingCode) == parkingCode));
                                    const regularExistenceCheck = !existingRegularString.includes(parkingCode);
                                    const isPermanent = frozenSpotCodes.includes(parkingCode) ? true : false;
                                    if (index < 0 && regularExistenceCheck) {
                                        const parkingSpot = new parking_spot_1.default({
                                            parkingLot: parkingLot,
                                            level: existingLevel._id,
                                            spotType: 'designated',
                                            parkingCode: parkingCode,
                                            isPermanent,
                                        });
                                        createdDesignated.push(parkingSpot);
                                    }
                                    else
                                        createdDesignated.push(null);
                                })));
                            }
                            if (doubleSpots.length > 0) {
                                yield Promise.all(doubleSpots.map((spots) => __awaiter(void 0, void 0, void 0, function* () {
                                    const index = yield Promise.resolve(createdRegular.findIndex(item => (item === null || item === void 0 ? void 0 : item.parkingCode) == spots[0] || (item === null || item === void 0 ? void 0 : item.parkingCode) == spots[1]));
                                    const regularExistenceCheck = !existingRegularString.includes(spots[0]) || !existingRegularString.includes(spots[1]);
                                    const isPermanent = frozenSpotCodes.includes(spots[0]) || frozenSpotCodes.includes(spots[1]) ? true : false;
                                    if (index < 0 && regularExistenceCheck) {
                                        const parkingSpotA = new parking_spot_1.default({
                                            parkingLot: parkingLot,
                                            level: existingLevel._id,
                                            spotType: 'designated',
                                            parkingCode: spots[0],
                                            isPermanent,
                                        });
                                        const parkingSpotB = new parking_spot_1.default({
                                            parkingLot: parkingLot,
                                            level: existingLevel._id,
                                            spotType: 'designated',
                                            parkingCode: spots[1],
                                            outerToSpot: parkingSpotA._id,
                                            isPermanent,
                                        });
                                        parkingSpotA.innerToSpot = parkingSpotB._id;
                                        createdDesignated.push(parkingSpotA);
                                        createdDesignated.push(parkingSpotB);
                                    }
                                    else
                                        createdDesignated.push(null);
                                })));
                            }
                        }
                        if (frozenStatus.added.length > 0) {
                            const { singleSpots, doubleSpots } = (0, validators_1.separateDoubleAndSingleSpots)(frozenStatus.added);
                            if (singleSpots.length > 0) {
                                yield Promise.all(singleSpots.map((parkingCode) => __awaiter(void 0, void 0, void 0, function* () {
                                    const index = yield Promise.resolve(createdRegular.findIndex(item => (item === null || item === void 0 ? void 0 : item.parkingCode) == parkingCode));
                                    const regularExistenceCheck = !existingRegularString.includes(parkingCode);
                                    if (index < 0 && regularExistenceCheck) {
                                        const isPermanent = frozenSpotCodes.includes(parkingCode) ? true : false;
                                        const update = yield parking_spot_1.default.findOne({
                                            parkingLot: parkingLot,
                                            level: existingLevel._id,
                                            spotType: 'designated',
                                            parkingCode: parkingCode,
                                        });
                                        if (update) {
                                            update.isPermanent = isPermanent;
                                            createdDesignated.push(update);
                                        }
                                        else {
                                            const findInCreated = yield Promise.resolve(createdDesignated.findIndex(item => (item === null || item === void 0 ? void 0 : item.parkingCode) == parkingCode));
                                            if (findInCreated >= 0)
                                                createdDesignated[findInCreated].set({ isPermanent });
                                            else {
                                                const parkingSpot = new parking_spot_1.default({
                                                    parkingLot: parkingLot,
                                                    level: existingLevel._id,
                                                    spotType: 'designated',
                                                    parkingCode: parkingCode,
                                                    isPermanent: isPermanent,
                                                });
                                                createdDesignated.push(parkingSpot);
                                            }
                                        }
                                    }
                                    else
                                        createdDesignated.push(null);
                                })));
                            }
                            if (doubleSpots.length > 0) {
                                yield Promise.all(doubleSpots.map((spots) => __awaiter(void 0, void 0, void 0, function* () {
                                    const index = yield Promise.resolve(createdRegular.findIndex(item => (item === null || item === void 0 ? void 0 : item.parkingCode) == spots[0] || (item === null || item === void 0 ? void 0 : item.parkingCode) == spots[1]));
                                    const regularExistenceCheck = !existingRegularString.includes(spots[0]) || !existingRegularString.includes(spots[1]);
                                    if (index < 0 && regularExistenceCheck) {
                                        const isPermanent = frozenSpotCodes.includes(spots[0]) || frozenSpotCodes.includes(spots[1]) ? true : false;
                                        const updateA = yield parking_spot_1.default.findOne({
                                            parkingLot: parkingLot,
                                            level: existingLevel._id,
                                            spotType: 'designated',
                                            parkingCode: spots[0],
                                        });
                                        if (updateA) {
                                            updateA.isPermanent = isPermanent;
                                            const updateB = yield parking_spot_1.default.findOne({
                                                parkingLot: parkingLot,
                                                level: existingLevel._id,
                                                spotType: 'designated',
                                                parkingCode: spots[1],
                                            });
                                            if (updateB) {
                                                updateB.isPermanent = isPermanent;
                                                createdDesignated.push(updateA);
                                                createdDesignated.push(updateB);
                                            }
                                        }
                                        else {
                                            const findAInCreated = yield Promise.resolve(createdDesignated.findIndex(item => (item === null || item === void 0 ? void 0 : item.parkingCode) == spots[0]));
                                            if (findAInCreated >= 0)
                                                createdDesignated[findAInCreated].set({ isPermanent });
                                            const findBInCreated = yield Promise.resolve(createdDesignated.findIndex(item => (item === null || item === void 0 ? void 0 : item.parkingCode) == spots[1]));
                                            if (findBInCreated >= 0)
                                                createdDesignated[findBInCreated].set({ isPermanent });
                                            if (findAInCreated < 0 && findBInCreated < 0) {
                                                const parkingSpotA = new parking_spot_1.default({
                                                    parkingLot: parkingLot,
                                                    level: existingLevel._id,
                                                    spotType: 'designated',
                                                    parkingCode: spots[0],
                                                    isPermanent: frozenSpotNumbers.includes((0, validators_1.extractNumberFromParkingCode)(spots[0])) ? true : false,
                                                });
                                                const parkingSpotB = new parking_spot_1.default({
                                                    parkingLot: parkingLot,
                                                    level: existingLevel._id,
                                                    spotType: 'designated',
                                                    parkingCode: spots[1],
                                                    outerToSpot: parkingSpotA._id,
                                                    isPermanent: frozenSpotNumbers.includes((0, validators_1.extractNumberFromParkingCode)(spots[1])) ? true : false,
                                                });
                                                parkingSpotA.innerToSpot = parkingSpotB._id;
                                                createdDesignated.push(parkingSpotA);
                                                createdDesignated.push(parkingSpotB);
                                            }
                                        }
                                    }
                                    else
                                        createdDesignated.push(null);
                                })));
                            }
                        }
                    }
                }
                if (electricStatus) {
                    const existingRegularString = (yield parking_spot_1.default.find({ level: existingLevel === null || existingLevel === void 0 ? void 0 : existingLevel._id, spotType: 'regular', }).sort({ parkingCode: 1 }).select('parkingCode')).map(spot => spot.parkingCode);
                    const existingDesignatedString = (yield parking_spot_1.default.find({ level: existingLevel === null || existingLevel === void 0 ? void 0 : existingLevel._id, spotType: 'designated', }).sort({ parkingCode: 1 }).select('parkingCode')).map(spot => spot.parkingCode);
                    if (electricStatus.removed.length > 0) {
                        yield Promise.all(electricStatus.removed.map((parkingCode) => __awaiter(void 0, void 0, void 0, function* () {
                            const deletedSpot = yield parking_spot_1.default.findOneAndDelete({ level: existingLevel === null || existingLevel === void 0 ? void 0 : existingLevel._id, spotType: 'electric', parkingCode, });
                            if (deletedSpot) {
                                const deletedReservations = yield reservation_1.default.deleteMany({ parkingSpot: deletedSpot._id, parkingStatus: { $in: [controllers_1.reservationEnums[0], controllers_1.reservationEnums[1], controllers_1.reservationEnums[2]] } });
                            }
                        })));
                    }
                    if (electricStatus.added.length > 0) {
                        const { singleSpots, doubleSpots } = (0, validators_1.separateDoubleAndSingleSpots)(electricStatus.added);
                        if (singleSpots.length > 0) {
                            yield Promise.all(singleSpots.map((parkingCode) => __awaiter(void 0, void 0, void 0, function* () {
                                const index = yield Promise.resolve(createdRegular.findIndex(item => (item === null || item === void 0 ? void 0 : item.parkingCode) == parkingCode));
                                const regularExistenceCheck = !existingRegularString.includes(parkingCode);
                                const designatedExistenceCheck = !existingDesignatedString.includes(parkingCode);
                                if (index < 0 && regularExistenceCheck && designatedExistenceCheck) {
                                    const parkingSpot = new parking_spot_1.default({
                                        parkingLot: parkingLot,
                                        level: existingLevel._id,
                                        spotType: 'electric',
                                        parkingCode: parkingCode,
                                        isPermanent: false,
                                    });
                                    createdElectric.push(parkingSpot);
                                }
                                else
                                    createdElectric.push(null);
                            })));
                        }
                        if (doubleSpots.length > 0) {
                            yield Promise.all(doubleSpots.map((spots) => __awaiter(void 0, void 0, void 0, function* () {
                                const index = yield Promise.resolve(createdRegular.findIndex(item => (item === null || item === void 0 ? void 0 : item.parkingCode) == spots[0] || (item === null || item === void 0 ? void 0 : item.parkingCode) == spots[1]));
                                const regularExistenceCheck = !existingRegularString.includes(spots[0]) || !existingRegularString.includes(spots[1]);
                                const designatedExistenceCheck = !existingDesignatedString.includes(spots[0]) || !existingDesignatedString.includes(spots[1]);
                                if (index < 0 && regularExistenceCheck && designatedExistenceCheck) {
                                    const parkingSpotA = new parking_spot_1.default({
                                        parkingLot: parkingLot,
                                        level: existingLevel._id,
                                        spotType: 'electric',
                                        parkingCode: spots[0],
                                        isPermanent: false,
                                    });
                                    const parkingSpotB = new parking_spot_1.default({
                                        parkingLot: parkingLot,
                                        level: existingLevel._id,
                                        spotType: 'electric',
                                        parkingCode: spots[1],
                                        outerToSpot: parkingSpotA._id,
                                        isPermanent: false,
                                    });
                                    parkingSpotA.innerToSpot = parkingSpotB._id;
                                    createdElectric.push(parkingSpotA);
                                    createdElectric.push(parkingSpotB);
                                }
                                else
                                    createdElectric.push(null);
                            })));
                        }
                    }
                }
                if (disabledStatus) {
                    const existingRegularString = (yield parking_spot_1.default.find({ level: existingLevel === null || existingLevel === void 0 ? void 0 : existingLevel._id, spotType: 'regular', }).sort({ parkingCode: 1 }).select('parkingCode')).map(spot => spot.parkingCode);
                    const existingDesignatedString = (yield parking_spot_1.default.find({ level: existingLevel === null || existingLevel === void 0 ? void 0 : existingLevel._id, spotType: 'designated', }).sort({ parkingCode: 1 }).select('parkingCode')).map(spot => spot.parkingCode);
                    const existingElectricString = (yield parking_spot_1.default.find({ level: existingLevel === null || existingLevel === void 0 ? void 0 : existingLevel._id, spotType: 'electric', }).sort({ parkingCode: 1 }).select('parkingCode')).map(spot => spot.parkingCode);
                    if (disabledStatus.removed.length > 0) {
                        yield Promise.all(disabledStatus.removed.map((parkingCode) => __awaiter(void 0, void 0, void 0, function* () {
                            const deletedSpot = yield parking_spot_1.default.findOneAndDelete({ level: existingLevel === null || existingLevel === void 0 ? void 0 : existingLevel._id, spotType: 'electric', parkingCode, });
                            if (deletedSpot) {
                                const deletedReservations = yield reservation_1.default.deleteMany({ parkingSpot: deletedSpot._id, parkingStatus: { $in: [controllers_1.reservationEnums[0], controllers_1.reservationEnums[1], controllers_1.reservationEnums[2]] } });
                            }
                        })));
                    }
                    if (disabledStatus.added.length > 0) {
                        const { singleSpots, doubleSpots } = (0, validators_1.separateDoubleAndSingleSpots)(disabledStatus.added);
                        if (singleSpots.length > 0) {
                            yield Promise.all(singleSpots.map((parkingCode) => __awaiter(void 0, void 0, void 0, function* () {
                                const index = yield Promise.resolve(createdRegular.findIndex(item => (item === null || item === void 0 ? void 0 : item.parkingCode) == parkingCode));
                                const regularExistenceCheck = !existingRegularString.includes(parkingCode);
                                const designatedExistenceCheck = !existingDesignatedString.includes(parkingCode);
                                const electricExistenceCheck = !existingElectricString.includes(parkingCode);
                                if (index < 0 && regularExistenceCheck && designatedExistenceCheck && electricExistenceCheck) {
                                    const parkingSpot = new parking_spot_1.default({
                                        parkingLot: parkingLot,
                                        level: existingLevel._id,
                                        spotType: 'disabled',
                                        parkingCode: parkingCode,
                                        isPermanent: false,
                                    });
                                    createdDisabled.push(parkingSpot);
                                }
                                else
                                    createdDisabled.push(null);
                            })));
                        }
                        if (doubleSpots.length > 0) {
                            yield Promise.all(doubleSpots.map((spots) => __awaiter(void 0, void 0, void 0, function* () {
                                const index = yield Promise.resolve(createdRegular.findIndex(item => (item === null || item === void 0 ? void 0 : item.parkingCode) == spots[0] || (item === null || item === void 0 ? void 0 : item.parkingCode) == spots[1]));
                                const regularExistenceCheck = !existingRegularString.includes(spots[0]) || !existingRegularString.includes(spots[1]);
                                const designatedExistenceCheck = !existingDesignatedString.includes(spots[0]) || !existingDesignatedString.includes(spots[1]);
                                const electricExistenceCheck = !existingElectricString.includes(spots[0]) || !existingElectricString.includes(spots[1]);
                                if (index < 0 && regularExistenceCheck && designatedExistenceCheck) {
                                    const parkingSpotA = new parking_spot_1.default({
                                        parkingLot: parkingLot,
                                        level: existingLevel._id,
                                        spotType: 'disabled',
                                        parkingCode: spots[0],
                                        isPermanent: false,
                                    });
                                    const parkingSpotB = new parking_spot_1.default({
                                        parkingLot: parkingLot,
                                        level: existingLevel._id,
                                        spotType: 'disabled',
                                        parkingCode: spots[1],
                                        outerToSpot: parkingSpotA._id,
                                        isPermanent: false,
                                    });
                                    parkingSpotA.innerToSpot = parkingSpotB._id;
                                    createdDisabled.push(parkingSpotA);
                                    createdDisabled.push(parkingSpotB);
                                }
                                else
                                    createdDisabled.push(null);
                            })));
                        }
                    }
                }
                const nullValues = [...createdRegular, ...createdDesignated, ...createdElectric, ...createdDisabled].some(value => value == null);
                if (nullValues)
                    res.status(http_statuses_1.default.BAD_REQUEST).json({ message: language == 'heb' ? http_messages_1.default.parkingSpot.FAILURE_UPDATING_DUPLICATE_VALUES_HEB : http_messages_1.default.parkingSpot.FAILURE_UPDATING_DUPLICATE_VALUES });
                else {
                    const parkingSpots = yield Promise.all([...createdRegular, ...createdDesignated, ...createdElectric, ...createdDisabled].filter((spot) => __awaiter(void 0, void 0, void 0, function* () {
                        if (spot != null) {
                            setTimeout(() => __awaiter(void 0, void 0, void 0, function* () {
                                if (Array.isArray(spot)) {
                                    yield Promise.all(spot.map((s) => __awaiter(void 0, void 0, void 0, function* () { return yield s.save(); })));
                                    return spot;
                                }
                                else {
                                    yield spot.save();
                                    return spot;
                                }
                            }), 100);
                        }
                    })));
                    resolve({ newLevel: null, parkingSpots });
                }
            }
        }
        catch (err) {
            reject(err);
        }
    }));
});
const updateCustomSpotsHelper = (req, res, lot) => __awaiter(void 0, void 0, void 0, function* () {
    return new Promise((resolve, reject) => __awaiter(void 0, void 0, void 0, function* () {
        try {
            const { _id, } = req.params;
            const { parkingLot, level, name, entrance, map } = req.body;
            const foundLevel = yield lot_level_1.default.findByIdAndUpdate(_id, { parkingLot, level, name, entrance, map }, { new: true, omitUndefined: true });
            if (foundLevel) {
                // const { parkingSpots } = await createCustomSpotsHelper(req, res, lot, false)
                const { parkingSpots } = yield processUpdateSpots(req, res, foundLevel);
                res.status(http_statuses_1.default.OK).json(parkingSpots);
            }
            else
                res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.lotLevel.LEVEL_NOT_FOUND });
        }
        catch (err) {
            reject(err);
        }
    }));
});
const createLotLevelService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { parkingLot, level, map, entrance, name } = req.body;
        if (parkingLot && level && map && entrance && name) {
            const lot = yield parking_lot_1.default.findById(parkingLot);
            if (lot) {
                const existingLevel = yield lot_level_1.default.findOne({ parkingLot, level });
                if (existingLevel) {
                    res.status(http_statuses_1.default.FORBIDDEN).json({ message: http_messages_1.default.lotLevel.LEVEL_ALREADY_EXIST + `Level: ${level}.` });
                }
                else {
                    const { newLevel, parkingSpots } = yield createCustomSpotsHelper(req, res, lot, true);
                    res.status(http_statuses_1.default.CREATED).json({ level: newLevel, parkingSpots });
                }
            }
            else
                return res.status(http_statuses_1.default.NOT_FOUND).json({ messages: http_messages_1.default.parkingLot.PARKING_LOT_NOT_FOUND });
        }
        else
            res.status(http_statuses_1.default.BAD_REQUEST).json({ messages: http_messages_1.default.lotLevel.LEVEL_DETAILS_MISSING });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ messages: err });
    }
});
exports.createLotLevelService = createLotLevelService;
const getAllLotLevelsService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { page = 1, limit = 10, } = req.query;
        const totalCount = yield lot_level_1.default.countDocuments();
        const totalPages = Math.ceil(totalCount / limit);
        const lotLevels = yield lot_level_1.default.find().sort({ updatedAt: -1 }).limit(limit).skip(limit * (page - 1));
        if (lotLevels.length > 0) {
            const counts = yield Promise.all(lotLevels.map((lotLevel) => __awaiter(void 0, void 0, void 0, function* () {
                const regular = yield parking_spot_1.default.countDocuments({ level: lotLevel._id, spotType: 'regular' });
                const designated = yield parking_spot_1.default.countDocuments({ level: lotLevel._id, spotType: 'designated' });
                const electric = yield parking_spot_1.default.countDocuments({ level: lotLevel._id, spotType: 'electric' });
                return Object.assign(Object.assign({}, lotLevel.toObject()), { regular,
                    designated,
                    electric });
            })));
            res.status(http_statuses_1.default.OK).json({ totalPages, totalCount, levels: counts });
        }
        else
            res.status(http_statuses_1.default.OK).json({ totalPages, totalCount, levels: lotLevels });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getAllLotLevelsService = getAllLotLevelsService;
const getSingleLevelService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const level = yield lot_level_1.default.findById(_id);
        if (level)
            res.status(http_statuses_1.default.OK).json(level);
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.lotLevel.LEVEL_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getSingleLevelService = getSingleLevelService;
const getParkingLotLevelsService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { page = 1, limit = 10, getSpots = true, sort = 'dsc', } = req.query;
        const levels = yield lot_level_1.default.find({ parkingLot: _id }).sort({ level: sort == 'asc' ? 1 : -1 }).limit(limit).skip(limit * (page - 1));
        const totalCount = yield lot_level_1.default.countDocuments({ parkingLot: _id });
        const totalPages = Math.ceil(totalCount / limit);
        if (getSpots == true || getSpots == 'true') {
            const sort = { _id: 1 };
            const allLevels = yield Promise.all(levels.map((level) => __awaiter(void 0, void 0, void 0, function* () {
                const regular = yield parking_spot_1.default.countDocuments({ level, spotType: 'regular', outerToSpot: null });
                const regularString = (yield parking_spot_1.default.find({ level, spotType: 'regular', outerToSpot: null }).sort(sort).select('parkingCode')).map(spot => { var _a; return (_a = spot.parkingCode) === null || _a === void 0 ? void 0 : _a.split('-')[0].trim(); }).join(',');
                const designated = yield parking_spot_1.default.countDocuments({ level, spotType: 'designated', outerToSpot: null });
                const designatedString = (yield parking_spot_1.default.find({ level, spotType: 'designated', outerToSpot: null }).sort(sort).select('parkingCode')).map(spot => { var _a; return (_a = spot.parkingCode) === null || _a === void 0 ? void 0 : _a.split('-')[0].trim(); }).join(',');
                const electric = yield parking_spot_1.default.countDocuments({ level, spotType: 'electric', outerToSpot: null });
                const electricString = (yield parking_spot_1.default.find({ level, spotType: 'electric', outerToSpot: null }).sort(sort).select('parkingCode')).map(spot => { var _a; return (_a = spot.parkingCode) === null || _a === void 0 ? void 0 : _a.split('-')[0].trim(); }).join(',');
                const disabled = yield parking_spot_1.default.countDocuments({ level, spotType: 'disabled', outerToSpot: null });
                const disabledString = (yield parking_spot_1.default.find({ level, spotType: 'disabled', outerToSpot: null }).sort(sort).select('parkingCode')).map(spot => { var _a; return (_a = spot.parkingCode) === null || _a === void 0 ? void 0 : _a.split('-')[0].trim(); }).join(',');
                const double = yield parking_spot_1.default.countDocuments({ level, outerToSpot: { $ne: null } });
                const doubleString = (yield parking_spot_1.default.find({ level, outerToSpot: { $ne: null } }).sort(sort).select('parkingCode')).map(spot => { var _a; return (_a = spot.parkingCode) === null || _a === void 0 ? void 0 : _a.split('-')[0].trim(); }).join(',');
                const frozen = yield parking_spot_1.default.countDocuments({ level, spotType: 'designated', isPermanent: true, outerToSpot: null });
                const frozenString = (yield parking_spot_1.default.find({ level, spotType: 'designated', isPermanent: true, outerToSpot: null }).sort(sort).select('parkingCode')).map(spot => { var _a; return (_a = spot.parkingCode) === null || _a === void 0 ? void 0 : _a.split('-')[0].trim(); }).join(',');
                return Object.assign(Object.assign({}, level.toObject()), { regular,
                    regularString,
                    designated,
                    designatedString,
                    electric,
                    electricString,
                    disabled,
                    disabledString,
                    double,
                    doubleString,
                    frozen,
                    frozenString });
            })));
            res.status(http_statuses_1.default.OK).json({ totalPages, totalCount, levels: allLevels });
        }
        else
            res.status(http_statuses_1.default.OK).json({ totalPages, totalCount, levels });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getParkingLotLevelsService = getParkingLotLevelsService;
const getParkingLotLevelAvailableSpotsService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { page = 1, limit = 10, sort = 'dsc', } = req.query;
        const levels = yield lot_level_1.default.find({ parkingLot: _id })
            .sort({ level: sort == 'asc' ? 1 : -1 })
            .limit(limit)
            .skip(limit * (page - 1));
        const levelCounts = yield Promise.all(levels.map((level) => __awaiter(void 0, void 0, void 0, function* () {
            const { totalCount } = yield (0, helpers_1.findAvailableSpots)(req, level._id);
            return Object.assign(Object.assign({}, level.toObject()), { count: totalCount });
        })));
        res.status(http_statuses_1.default.OK).json(levelCounts);
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getParkingLotLevelAvailableSpotsService = getParkingLotLevelAvailableSpotsService;
const updateLotLevelService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { parkingLot, level, } = req.body;
        const existingLevel = yield lot_level_1.default.findOne({ parkingLot, level });
        if (existingLevel && (existingLevel === null || existingLevel === void 0 ? void 0 : existingLevel._id) != _id)
            res.status(http_statuses_1.default.FORBIDDEN).json({ message: http_messages_1.default.lotLevel.LEVEL_ALREADY_EXIST });
        else {
            // const levelReservations = await Reservation.countDocuments({ level: _id, parkingStatus: { $in: [reservationEnums[0], reservationEnums[1], reservationEnums[2]] } })
            // const levelDesignated = await ParkingSpot.find({ level: _id, designatedTo: { $ne: null } })
            // if (levelReservations > 0) {
            // await Reservation.deleteMany({ level: _id, parkingStatus: { $in: [reservationEnums[0], reservationEnums[1], reservationEnums[2]] } })
            // }
            // if (levelDesignated.length > 0) {
            //     const designatedIds: Schema.Types.ObjectId[] = []
            //     const designatedUserIds: Schema.Types.ObjectId[] = []
            //     await Promise.all(levelDesignated.map((parkingSpot) => {
            //         designatedIds.push(parkingSpot._id)
            //         designatedUserIds.push(parkingSpot.designatedTo)
            //     }))
            //     await User.updateMany(
            //         { _id: { $in: designatedUserIds } }, // Filter condition
            //         { $pull: { designatedParkingSpot: { $in: designatedIds } } } // Update operation
            //     );
            //     await ParkingSpot.updateMany({ level: _id, designatedTo: { $ne: null } }, { designatedTo: null })
            // }
            const foundLot = yield parking_lot_1.default.findById(parkingLot);
            const { parkingSpots, } = yield updateCustomSpotsHelper(req, res, foundLot);
            res.status(http_statuses_1.default.OK).json(parkingSpots);
        }
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.updateLotLevelService = updateLotLevelService;
const deleteLevelService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const levelReservations = yield reservation_1.default.countDocuments({ level: _id, parkingStatus: { $in: [controllers_1.reservationEnums[0], controllers_1.reservationEnums[1]] } });
        if (levelReservations > 0)
            res.status(http_statuses_1.default.FORBIDDEN).json({ message: http_messages_1.default.lotLevel.DELETE_FAILURE_REASON_UNCOMPLETED_RESERVATION });
        else {
            const level = yield lot_level_1.default.findByIdAndDelete(_id);
            if (level) {
                yield parking_spot_1.default.deleteMany({ level: level._id });
                res.status(http_statuses_1.default.OK).json({ message: http_messages_1.default.lotLevel.LEVEL_DELETED_SUCCESS });
            }
            else {
                res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.lotLevel.LEVEL_NOT_FOUND });
            }
        }
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.deleteLevelService = deleteLevelService;
