"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteParkingLotService = exports.updateParkingLotService = exports.getParkingLotCountsService = exports.getUserParkingLotAdminsService = exports.getUserParkingLotsService = exports.getAdminParkingLotsService = exports.getSingleParkingLotService = exports.getReportService = exports.getAllParkingLotsService = exports.createParkingLotService = void 0;
const http_messages_1 = __importDefault(require("../../services/constants/http-messages"));
const http_statuses_1 = __importDefault(require("../../services/constants/http-statuses"));
const admin_1 = __importDefault(require("../../models/admin"));
const parking_lot_1 = __importDefault(require("../../models/parking-lot"));
const lot_level_1 = __importDefault(require("../../models/lot-level"));
const parking_spot_1 = __importDefault(require("../../models/parking-spot"));
const reservation_1 = __importDefault(require("../../models/reservation"));
const dates_1 = require("../../helpers/dates");
const user_1 = __importDefault(require("../../models/user"));
const controllers_1 = require("../reservations/controllers");
const generators_1 = require("../../helpers/generators");
const createParkingLotService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { name, chargingTimeLimit, chargingTimeStart, chargingTimeEnd, location, admins, allotmentMethod, allotmentFile, accessMapFile, avatar, maxReservationDistance, messageSendTime, absenceSendTime, messageSendTimeOfDay, reservationTime, smsSendTime, designatedReservationDuration, sendPushNotification, } = req.body;
        if (name &&
            (chargingTimeLimit == true || chargingTimeLimit == false) &&
            location &&
            admins &&
            avatar &&
            maxReservationDistance > 0 &&
            allotmentMethod &&
            accessMapFile &&
            messageSendTime &&
            absenceSendTime &&
            designatedReservationDuration &&
            // messageSendTimeOfDay &&
            reservationTime) {
            const parkingLot = new parking_lot_1.default({
                name,
                avatar,
                chargingTimeLimit,
                chargingTimeStart,
                chargingTimeEnd,
                location,
                allotmentMethod,
                allotmentFile: allotmentFile ? allotmentFile : null,
                accessMapFile,
                admins,
                maxReservationDistance,
                messageSendTime,
                absenceSendTime,
                messageSendTimeOfDay,
                reservationTime,
                smsSendTime,
                designatedReservationDuration,
                sendPushNotification,
            });
            yield Promise.all(admins.map((admin) => __awaiter(void 0, void 0, void 0, function* () {
                const saved = yield admin_1.default.findByIdAndUpdate(admin, { $push: { parkingLots: parkingLot._id } });
                return saved;
            })));
            yield parkingLot.save();
            // let createdLevels: any[] = []
            if (parkingLot.allotmentMethod == 'manual') {
                // createdLevels = await createManualSpotsHelper(parkingLot._id, parkingLot)
                // res.status(HttpStatus.CREATED).json({ parkingLot, levels: createdLevels });
                res.status(http_statuses_1.default.CREATED).json({ parkingLot });
            }
            else
                res.status(http_statuses_1.default.CREATED).json({ parkingLot });
        }
        else
            res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.parkingLot.PARKING_LOT_DETAILS_MISSING });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.createParkingLotService = createParkingLotService;
const getAllParkingLotsService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { page = 1, limit = 10, currentAdmin } = req.query;
        let filter = {};
        if (currentAdmin) {
            const admin = yield admin_1.default.findById(currentAdmin);
            if ((admin === null || admin === void 0 ? void 0 : admin.role) == 'sub-admin') {
                filter.admins = { $in: [currentAdmin] };
            }
        }
        const totalCount = yield parking_lot_1.default.countDocuments(filter);
        const totalPages = Math.ceil(totalCount / limit);
        const parkingLots = yield parking_lot_1.default.find(filter).sort({ updatedAt: -1 }).limit(limit).skip(limit * (page - 1));
        if (parkingLots.length > 0) {
            const counts = yield Promise.all(parkingLots.map((parkingLot) => __awaiter(void 0, void 0, void 0, function* () {
                // const lot = await Reservation.countDocuments({ parkingLot: parkingLot._id })
                const regular = yield parking_spot_1.default.countDocuments({ parkingLot: parkingLot._id, spotType: 'regular' });
                const designated = yield parking_spot_1.default.countDocuments({ parkingLot: parkingLot._id, spotType: 'designated' });
                const electric = yield parking_spot_1.default.countDocuments({ parkingLot: parkingLot._id, spotType: 'electric' });
                const disabled = yield parking_spot_1.default.countDocuments({ parkingLot: parkingLot._id, spotType: 'disabled' });
                return Object.assign(Object.assign({}, parkingLot.toObject()), { regular,
                    designated,
                    electric,
                    disabled });
            })));
            res.status(http_statuses_1.default.OK).json({ totalPages, totalCount, parkingLots: counts });
        }
        else
            res.status(http_statuses_1.default.OK).json({ totalPages, totalCount, parkingLots: [] });
        // res.status(HttpStatus.OK).json({ totalPages, totalCount, parkingLots });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getAllParkingLotsService = getAllParkingLotsService;
const getReportService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { startDate = new Date(), endDate = new Date(), } = req.query;
        let filter = { parkingLot: _id };
        const parkingLot = yield parking_lot_1.default.findById(_id);
        if (parkingLot) {
            const sort = { parkingCode: 1 };
            const { startOfDay, } = (0, dates_1.getStartAndEndOfDay)(startDate);
            const { endOfDay } = (0, dates_1.getStartAndEndOfDay)(endDate);
            const parkingSpots = yield parking_spot_1.default.find(filter).sort(sort).populate({ path: 'designatedTo', select: ['firstName'], }).populate({ path: 'level', select: ['name'] }).populate({ path: 'parkingLot', select: ['name'] });
            const reservations = yield reservation_1.default.find({ parkingLot: _id, parkingStatus: { $nin: [controllers_1.reservationEnums[3], controllers_1.reservationEnums[4]] }, reservationDate: { $gte: startOfDay, $lte: endOfDay } }).select(['reservationDate', 'parkingSpot']).populate({ path: 'user', select: ['firstName'] });
            const finalSpots = yield Promise.all(parkingSpots.map((spot) => __awaiter(void 0, void 0, void 0, function* () {
                var _a, _b;
                const reservationIndex = yield Promise.resolve(reservations.findIndex((reservation) => String(reservation.parkingSpot) == String(spot._id)));
                const spotType = (spot === null || spot === void 0 ? void 0 : spot.spotType) == 'regular' ? 'רגיל' : (spot === null || spot === void 0 ? void 0 : spot.spotType) == 'designated' ? 'שמור' : (spot === null || spot === void 0 ? void 0 : spot.spotType) == 'electric' ? 'חשמלי' : 'נגיש';
                if (reservationIndex < 0)
                    return Object.assign(Object.assign({}, spot === null || spot === void 0 ? void 0 : spot.toObject()), { spotType, reservation: null, status: spot.designatedTo ? 'לא פנוי' : 'פנוי', 
                        // status: spot.designatedTo ? 'שמור' : 'פנוי',
                        isOccupied: 'לא תפוסה', isPermanent: (spot === null || spot === void 0 ? void 0 : spot.isPermanent) ? 'כן' : 'לא' });
                else
                    return Object.assign(Object.assign({}, spot === null || spot === void 0 ? void 0 : spot.toObject()), { spotType, reservation: reservations[reservationIndex], status: 'לא פנוי', isOccupied: (_b = (_a = reservations[reservationIndex]) === null || _a === void 0 ? void 0 : _a.user) === null || _b === void 0 ? void 0 : _b.firstName, 
                        // isOccupied: 'תפוס',
                        isPermanent: (spot === null || spot === void 0 ? void 0 : spot.isPermanent) ? 'כן' : 'לא' });
            })));
            const { filePath } = yield (0, generators_1.generateParkingLotReportCsv)(finalSpots);
            res.status(http_statuses_1.default.OK).json({ path: process.env.LIVE_BASE_URL + filePath });
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.parkingLot.PARKING_LOT_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getReportService = getReportService;
const getSingleParkingLotService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const parkingLot = yield parking_lot_1.default.findById(_id).populate({ path: 'admins', select: ['email', 'firstName', 'lastName', 'role'] });
        if (parkingLot)
            res.status(http_statuses_1.default.OK).json(parkingLot);
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.parkingLot.PARKING_LOT_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getSingleParkingLotService = getSingleParkingLotService;
const getAdminParkingLotsService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { page = 1, limit = 10 } = req.query;
        const admin = yield admin_1.default.findById(_id).populate('parkingLots').select('parkingLots').sort({ updatedAt: -1 }).limit(limit).skip(limit * (page - 1));
        if (admin) {
            const { parkingLots } = admin;
            res.status(http_statuses_1.default.OK).json({ parkingLots });
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.admin.ADMIN_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getAdminParkingLotsService = getAdminParkingLotsService;
const getUserParkingLotsService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { page = 1, limit = 10 } = req.query;
        const user = yield user_1.default.findById(_id)
            .populate({
            path: 'parkingLots',
            options: { sort: { updatedAt: -1 } }
        });
        if (user) {
            let { parkingLots } = user;
            const mainParkingLotIndex = parkingLots.findIndex((lot) => lot._id.equals(user.mainParkingLot));
            if (mainParkingLotIndex !== -1) {
                const mainParkingLot = parkingLots.splice(mainParkingLotIndex, 1)[0];
                parkingLots.unshift(mainParkingLot);
            }
            res.status(http_statuses_1.default.OK).json({ parkingLots });
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.user.USER_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getUserParkingLotsService = getUserParkingLotsService;
const getUserParkingLotAdminsService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const user = yield user_1.default.findById(_id)
            .populate({
            path: 'parkingLots',
            options: { sort: { updatedAt: -1 }, },
            select: ['admins'],
            populate: { path: 'admins', select: ['phoneNumber', 'firstName', 'lastName', 'middleName', 'role'] },
        });
        if (user) {
            let { parkingLots } = user;
            const mainParkingLotIndex = parkingLots.findIndex((lot) => lot._id.equals(user.mainParkingLot));
            if (mainParkingLotIndex !== -1) {
                const mainParkingLot = parkingLots.splice(mainParkingLotIndex, 1)[0];
                parkingLots.unshift(mainParkingLot);
            }
            const admins = [];
            yield Promise.all(parkingLots.map((lot) => __awaiter(void 0, void 0, void 0, function* () {
                yield Promise.all(lot.admins.map((admin) => admins.push(admin)));
            })));
            res.status(http_statuses_1.default.OK).json(admins);
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.user.USER_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getUserParkingLotAdminsService = getUserParkingLotAdminsService;
const getParkingLotCountsService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { page = 1, limit = 10, } = req.query;
        const { startOfDay, endOfDay } = (0, dates_1.getStartAndEndOfDay)(new Date());
        let filter = {};
        if (_id) {
            const admin = yield admin_1.default.findById(_id);
            if ((admin === null || admin === void 0 ? void 0 : admin.role) == 'sub-admin') {
                filter.admins = { $in: [admin._id] };
            }
        }
        const parkingLots = yield parking_lot_1.default.find(filter).sort({ updatedAt: -1 }).limit(limit).skip(limit * (page - 1)).select(['name', 'location',]);
        const totalCount = yield parking_lot_1.default.countDocuments(filter);
        const totalPages = Math.ceil(totalCount / limit);
        if (parkingLots.length > 0) {
            const counts = yield Promise.all(parkingLots.map((parkingLot) => __awaiter(void 0, void 0, void 0, function* () {
                const lot = yield reservation_1.default.countDocuments({ parkingLot: parkingLot._id, reservationDate: { $gte: startOfDay, $lte: endOfDay }, parkingStatus: { $in: [controllers_1.reservationEnums[0], controllers_1.reservationEnums[1]] } });
                const total = yield parking_spot_1.default.countDocuments({ parkingLot: parkingLot._id, });
                return {
                    parkingLot,
                    total: total,
                    reserved: lot,
                };
            })));
            res.status(http_statuses_1.default.OK).json({ totalPages, totalCount, counts });
        }
        else
            res.status(http_statuses_1.default.OK).json([]);
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getParkingLotCountsService = getParkingLotCountsService;
const updateParkingLotService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    var _c;
    try {
        const { _id } = req.params;
        const { name, chargingTimeLimit, chargingTimeStart, chargingTimeEnd, location, admins, allotmentMethod, allotmentFile, accessMapFile, avatar, maxReservationDistance, smsSendTime, messageSendTime, absenceSendTime, messageSendTimeOfDay, reservationTime, designatedReservationDuration, sendPushNotification, } = req.body;
        const beforeUpdate = yield parking_lot_1.default.findById(_id);
        const parkingLot = yield parking_lot_1.default.findByIdAndUpdate(_id, {
            name,
            chargingTimeLimit,
            chargingTimeStart,
            chargingTimeEnd,
            location,
            admins,
            allotmentMethod,
            allotmentFile,
            accessMapFile,
            avatar,
            maxReservationDistance,
            smsSendTime,
            messageSendTime,
            absenceSendTime,
            messageSendTimeOfDay,
            reservationTime,
            designatedReservationDuration,
            sendPushNotification,
        }, { omitUndefined: true, new: true, });
        if (parkingLot) {
            if (admins) {
                if (beforeUpdate && ((_c = beforeUpdate === null || beforeUpdate === void 0 ? void 0 : beforeUpdate.admins) === null || _c === void 0 ? void 0 : _c.length) > 0) {
                    yield Promise.all(beforeUpdate === null || beforeUpdate === void 0 ? void 0 : beforeUpdate.admins.map((admin) => __awaiter(void 0, void 0, void 0, function* () {
                        const removing = yield admin_1.default.findByIdAndUpdate(admin, { $pull: { parkingLots: parkingLot._id } }, { new: true });
                        return removing;
                    })));
                }
                yield Promise.all(admins.map((admin) => __awaiter(void 0, void 0, void 0, function* () {
                    const saved = yield admin_1.default.findByIdAndUpdate(admin, { $push: { parkingLots: parkingLot._id } }, { new: true });
                    return saved;
                })));
            }
            res.status(http_statuses_1.default.OK).json(parkingLot);
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.parkingLot.PARKING_LOT_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.updateParkingLotService = updateParkingLotService;
const deleteParkingLotService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const parkingLotReservations = yield reservation_1.default.countDocuments({ parkingLot: _id, parkingStatus: { $in: [controllers_1.reservationEnums[0], controllers_1.reservationEnums[1]] } });
        const designatedAssigned = yield parking_spot_1.default.countDocuments({ parkingLot: _id, designatedTo: { $ne: null } });
        if (parkingLotReservations > 0)
            res.status(http_statuses_1.default.FORBIDDEN).json({ message: http_messages_1.default.parkingLot.DELETE_FAILURE_REASON_UNCOMPLETED_RESERVATION });
        else if (designatedAssigned > 0)
            res.status(http_statuses_1.default.FORBIDDEN).json({ message: http_messages_1.default.parkingLot.DELETE_FAILURE_REASON_DESIGNATED });
        else {
            const parkingLot = yield parking_lot_1.default.findByIdAndDelete(_id);
            if (parkingLot) {
                yield lot_level_1.default.deleteMany({ parkingLot: _id });
                yield parking_spot_1.default.deleteMany({ parkingLot: _id });
                yield parking_lot_1.default.updateMany({ admins: { $in: parkingLot.admins } }, { $pull: { parkingLots: _id } });
                // await Promise.all(parkingLot.admins.map(async (admin: Types.ObjectId) => {
                //     const removed = await Admin.findByIdAndUpdate(admin, { $pull: { parkingLots: _id } });
                // }))
                res.status(http_statuses_1.default.OK).json({ message: http_messages_1.default.parkingLot.PARKING_LOT_DELETED_SUCCESS });
            }
            else
                res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.parkingLot.PARKING_LOT_NOT_FOUND });
        }
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.deleteParkingLotService = deleteParkingLotService;
