"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteParkingSpotService = exports.updateParkingSpotService = exports.getLevelAvailableSpotsService = exports.getLotLevelSpotsService = exports.getLotOccupancyService = exports.getLotMonitorSpotsService = exports.getParkingLotParkingSpotsService = exports.getSingleParkingSpotService = exports.getAllParkingSpotsService = exports.createParkingSpotService = void 0;
const http_messages_1 = __importDefault(require("../../services/constants/http-messages"));
const http_statuses_1 = __importDefault(require("../../services/constants/http-statuses"));
const parking_lot_1 = __importDefault(require("../../models/parking-lot"));
const lot_level_1 = __importDefault(require("../../models/lot-level"));
const parking_spot_1 = __importDefault(require("../../models/parking-spot"));
const absence_1 = __importDefault(require("../../models/absence"));
const dates_1 = require("../../helpers/dates");
const reservation_1 = __importDefault(require("../../models/reservation"));
const controllers_1 = require("../reservations/controllers");
const helpers_1 = require("../../helpers");
const createParkingSpotService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { parkingLot, name, avatar, level, parkingCode, spotType, } = req.body;
        if (parkingLot && level && parkingCode) {
            const lot = yield parking_lot_1.default.findById(parkingLot);
            if (lot) {
                const parkingLevel = yield lot_level_1.default.findById(level);
                if (parkingLevel) {
                    const code = yield parking_spot_1.default.findOne({ parkingLot, level, parkingCode, });
                    if (code)
                        res.status(http_statuses_1.default.BAD_REQUEST).json({ message: http_messages_1.default.parkingSpot.PARKING_CODE_ALREADY_IN_DB, });
                    else {
                        const parkingSpot = new parking_spot_1.default({
                            name,
                            avatar,
                            parkingLot,
                            level,
                            spotType,
                            parkingCode,
                        });
                        yield parkingSpot.save();
                        res.status(http_statuses_1.default.CREATED).json(parkingSpot);
                    }
                }
                else
                    res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.lotLevel.LEVEL_NOT_FOUND });
            }
            else
                res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.parkingLot.PARKING_LOT_NOT_FOUND });
        }
        else
            res.status(http_statuses_1.default.BAD_REQUEST).json({
                message: http_messages_1.default.parkingSpot.PARKING_SPOT_DETAILS_MISSING,
            });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.createParkingSpotService = createParkingSpotService;
const getAllParkingSpotsService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { page = 1, limit = 10, type, isAvailable, } = req.query;
        let filter = {};
        if (type != undefined)
            filter.spotType = type;
        if (type == "designated" && (isAvailable == "true" || isAvailable == true))
            filter.designatedTo = null;
        if (type == "designated" && (isAvailable == "false" || isAvailable == false))
            filter.designatedTo = { $ne: null };
        const totalCount = yield parking_spot_1.default.countDocuments(filter);
        const totalPages = Math.ceil(totalCount / limit);
        const parkingSpots = yield parking_spot_1.default.find(filter).sort({ updatedAt: -1 }).limit(limit).skip(limit * (page - 1));
        res.status(http_statuses_1.default.OK).json({ totalPages, totalCount, parkingSpots });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getAllParkingSpotsService = getAllParkingSpotsService;
const getSingleParkingSpotService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const parkingSpot = yield parking_spot_1.default.findById(_id);
        if (parkingSpot) {
            res.status(http_statuses_1.default.OK).json(parkingSpot);
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.parkingSpot.PARKING_SPOT_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getSingleParkingSpotService = getSingleParkingSpotService;
const getParkingLotParkingSpotsService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { page = 1, limit = 10, type, isAvailable, sortParkingCodes, } = req.query;
        let filter = { parkingLot: _id };
        if (type != undefined)
            filter.spotType = type;
        if (type == "designated" && (isAvailable == "true" || isAvailable == true))
            filter.designatedTo = null;
        if (type == "designated" && (isAvailable == "false" || isAvailable == false))
            filter.designatedTo = { $ne: null };
        const parkingLot = yield parking_lot_1.default.findById(_id);
        if (parkingLot) {
            const sort = sortParkingCodes == "true" || sortParkingCodes == true ? { parkingCode: 1 } : { _id: 1 };
            // const sort: {} = parkingLot.allotmentMethod == 'manual' ? { createdAt: 1 } : { parkingCode: 1 }
            const totalCount = yield parking_spot_1.default.countDocuments(filter);
            const totalPages = Math.ceil(totalCount / limit);
            const parkingSpots = yield parking_spot_1.default.find(filter).sort(sort).limit(limit).skip(limit * (page - 1));
            res.status(http_statuses_1.default.OK).json({ totalPages, totalCount, parkingSpots });
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.parkingLot.PARKING_LOT_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getParkingLotParkingSpotsService = getParkingLotParkingSpotsService;
// export const getLotMonitorSpotsService = async (req: Request, res: Response) => {
//     try {
//         const { _id }: { _id?: string } = req.params;
//         const { startDate = new Date(), endDate = new Date(), }: { startDate?: Date, endDate?: Date, } = req.query
//         let filter: any = { parkingLot: _id }
//         const parkingLot = await ParkingLot.findById(_id)
//         if (parkingLot) {
//             const sort: {} = { parkingCode: 1 }
//             const { startOfDay, } = getStartAndEndOfDay(startDate)
//             const { endOfDay } = getStartAndEndOfDay(endDate)
//             const levels = await Level.find({ parkingLot: _id })
//             const parkingSpots: any = await ParkingSpot.find(filter).sort(sort).populate({
//                 path: 'designatedTo',
//                 select: ['-password', '-temporaryPassword'],
//                 populate: [{
//                     path: 'parkingLots',
//                 }, {
//                     path: 'designatedParkingSpot',
//                     populate: 'parkingLot'
//                 }, {
//                     path: 'vehicles'
//                 }]
//             })
//             const reservations: any = await Reservation.find({ parkingStatus: { $nin: [reservationEnums[3], reservationEnums[4]] }, reservationDate: { $gte: startOfDay, $lte: endOfDay } })
//                 .populate({
//                     path: 'user',
//                     select: ['-password', '-temporaryPassword'],
//                     populate: [{
//                         path: 'parkingLots',
//                     }, {
//                         path: 'designatedParkingSpot',
//                         populate: 'parkingLot'
//                     }, {
//                         path: 'vehicles'
//                     }]
//                 })
//             const finalSpots = await Promise.all(parkingSpots.map(async (spot: any) => {
//                 const reservationIndex = await Promise.resolve(reservations.findIndex((reservation: any) => String(reservation.parkingSpot) == String(spot._id)))
//                 const absence = await Absence.findOne({ designatedSpot: spot?._id, startDate: { $lte: startDate }, endDate: { $gte: endDate } })
//                 if (reservationIndex < 0) {
//                     if (spot?.designatedTo?.parkingLots && spot?.designatedTo?.parkingLots?.length > 0) {
//                         const userWithDesignatedSpots = await Promise.all(spot?.designatedTo?.parkingLots.map(async (parkingLot: any) => {
//                             const designatedSpots = await ParkingSpot.find({ parkingLot: parkingLot._id, spotType: 'designated', designatedTo: spot?.designatedTo?._id });
//                             return {
//                                 ...parkingLot.toObject(),
//                                 designatedSpots
//                             };
//                         }))
//                         const designatedTo = spot?.designatedTo ? { ...spot?.designatedTo?.toObject(), parkingLots: userWithDesignatedSpots } : null
//                         return {
//                             ...spot.toObject(),
//                             status: absence && designatedTo ? 'released' : 'designated-not-released',
//                             designatedTo,
//                             reservedBy: null,
//                         }
//                     } else return {
//                         ...spot.toObject(),
//                         status: 'free',
//                         reservedBy: null,
//                     }
//                 }
//                 else {
//                     const userWithDesignatedSpots = await Promise.all(reservations[reservationIndex].user.parkingLots.map(async (parkingLot: any) => {
//                         const designatedSpots = await ParkingSpot.find({ parkingLot: parkingLot._id, spotType: 'designated', designatedTo: null });
//                         return {
//                             ...parkingLot.toObject(),
//                             designatedSpots
//                         };
//                     }))
//                     return {
//                         ...spot.toObject(),
//                         status: 'reserved',
//                         reservedBy: {
//                             ...reservations[reservationIndex].user.toObject(),
//                             parkingLots: userWithDesignatedSpots
//                         },
//                     }
//                 }
//             }))
//             let totalCount = 0
//             let totalReservedCount = 0
//             let totalFreeCount = 0
//             let totalElectricCount = 0
//             let totalDesignatedCount = 0
//             let totalRegularCount = 0
//             let totalDisabledCount = 0
//             const levelsWithSpots = await Promise.all(levels.map(async (level) => {
//                 const spots = await Promise.all(finalSpots.filter((spot) => String(spot.level) == String(level._id)))
//                 const reserved = await Promise.all(spots.filter((spot) => spot.status == 'reserved'))
//                 const free = await Promise.all(spots.filter((spot) => spot.status == 'free'))
//                 const electric = await Promise.all(spots.filter((spot) => spot.spotType == 'electric'))
//                 const designated = await Promise.all(spots.filter((spot) => spot.spotType == 'designated'))
//                 const designatedReleased = await Promise.all(spots.filter((spot) => spot.spotType == 'designated' && spot?.status == 'released'))
//                 const regular = await Promise.all(spots.filter((spot) => spot.spotType == 'regular'))
//                 const disabled = await Promise.all(spots.filter((spot) => spot.spotType == 'disabled'))
//                 totalCount = totalCount + spots.length
//                 totalReservedCount = totalReservedCount + reserved.length
//                 totalFreeCount = totalFreeCount + free.length + designatedReleased?.length
//                 totalElectricCount = totalElectricCount + electric.length
//                 totalDesignatedCount = totalDesignatedCount + designated.length
//                 totalRegularCount = totalRegularCount + regular.length
//                 totalDisabledCount = totalDisabledCount + disabled.length
//                 return {
//                     level: {
//                         ...level.toObject(),
//                         totalCount: spots.length,
//                         reservedCount: reserved.length,
//                         electricCount: electric.length,
//                         designatedCount: designated.length,
//                         regularCount: regular.length,
//                         disabledCount: disabled.length,
//                         freeCount: free.length + designatedReleased?.length,
//                         // freeCount: free.length,
//                     },
//                     parkingSpots: spots.sort(customSort)
//                 }
//             }))
//             res.status(HttpStatus.OK).json({
//                 totalCount,
//                 totalReservedCount,
//                 totalFreeCount,
//                 totalElectricCount,
//                 totalDesignatedCount,
//                 totalRegularCount,
//                 totalDisabledCount,
//                 levels: levelsWithSpots,
//             });
//         } else res.status(HttpStatus.NOT_FOUND).json({ message: HttpMessages.parkingLot.PARKING_LOT_NOT_FOUND })
//     } catch (err) {
//         res.status(HttpStatus.INTERNAL_SERVER_ERROR).json({ message: err })
//     }
// }
const getLotMonitorSpotsService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { startDate = new Date(), endDate = new Date(), sortBy = "parkingCode", sort = "asc", } = req.query;
        let filter = { parkingLot: _id };
        const parkingLot = yield parking_lot_1.default.findById(_id);
        if (parkingLot) {
            const sorting = sort == "asc" ? 1 : -1;
            const sortingFilter = {};
            if (sortBy == "parkingCode")
                sortingFilter.parkingCode = sorting;
            if (sortBy == "level")
                sortingFilter.name = sorting;
            if (sortBy == "spotType")
                sortingFilter.spotType = sorting;
            const { startOfDay } = (0, dates_1.getStartAndEndOfDay)(startDate);
            const { endOfDay } = (0, dates_1.getStartAndEndOfDay)(endDate);
            const levels = yield lot_level_1.default.find({ parkingLot: _id }).sort(sortBy == "level" ? sortingFilter : {});
            const parkingSpots = yield parking_spot_1.default.find(filter)
                .sort(sortingFilter)
                .populate({
                path: "designatedTo",
                select: ["-password", "-temporaryPassword"],
                populate: [{
                        path: "parkingLots",
                    }, {
                        path: "designatedParkingSpot",
                        populate: "parkingLot",
                    }, {
                        path: "vehicles",
                    },],
            });
            const reservations = yield reservation_1.default.find({ parkingStatus: { $nin: [controllers_1.reservationEnums[3], controllers_1.reservationEnums[4]] }, reservationDate: { $gte: startOfDay, $lte: endOfDay }, }).populate({
                path: "user",
                select: ["-password", "-temporaryPassword"],
                populate: [{
                        path: "parkingLots",
                    }, {
                        path: "designatedParkingSpot",
                        populate: "parkingLot",
                    }, {
                        path: "vehicles",
                    },],
            });
            const finalSpots = yield Promise.all(parkingSpots.map((spot) => __awaiter(void 0, void 0, void 0, function* () {
                var _a, _b, _c, _d, _e;
                const reservationIndex = yield Promise.resolve(reservations.findIndex((reservation) => String(reservation.parkingSpot) == String(spot._id)));
                const absence = yield absence_1.default.findOne({
                    designatedSpot: spot === null || spot === void 0 ? void 0 : spot._id,
                    startDate: { $lte: startDate },
                    endDate: { $gte: endDate },
                });
                if (reservationIndex < 0) {
                    if (((_a = spot === null || spot === void 0 ? void 0 : spot.designatedTo) === null || _a === void 0 ? void 0 : _a.parkingLots) && ((_c = (_b = spot === null || spot === void 0 ? void 0 : spot.designatedTo) === null || _b === void 0 ? void 0 : _b.parkingLots) === null || _c === void 0 ? void 0 : _c.length) > 0) {
                        const userWithDesignatedSpots = yield Promise.all((_d = spot === null || spot === void 0 ? void 0 : spot.designatedTo) === null || _d === void 0 ? void 0 : _d.parkingLots.map((parkingLot) => __awaiter(void 0, void 0, void 0, function* () {
                            var _f;
                            const designatedSpots = yield parking_spot_1.default.find({ parkingLot: parkingLot._id, spotType: "designated", designatedTo: (_f = spot === null || spot === void 0 ? void 0 : spot.designatedTo) === null || _f === void 0 ? void 0 : _f._id, });
                            return Object.assign(Object.assign({}, parkingLot.toObject()), { designatedSpots });
                        })));
                        const designatedTo = (spot === null || spot === void 0 ? void 0 : spot.designatedTo) ? Object.assign(Object.assign({}, (_e = spot === null || spot === void 0 ? void 0 : spot.designatedTo) === null || _e === void 0 ? void 0 : _e.toObject()), { parkingLots: userWithDesignatedSpots }) : null;
                        return Object.assign(Object.assign({}, spot.toObject()), { status: absence && designatedTo ? "released" : "designated-not-released", designatedTo, reservedBy: null });
                    }
                    else
                        return Object.assign(Object.assign({}, spot.toObject()), { status: "free", reservedBy: null });
                }
                else {
                    const userWithDesignatedSpots = yield Promise.all(reservations[reservationIndex].user.parkingLots.map((parkingLot) => __awaiter(void 0, void 0, void 0, function* () {
                        const designatedSpots = yield parking_spot_1.default.find({
                            parkingLot: parkingLot._id,
                            spotType: "designated",
                            designatedTo: null,
                        });
                        return Object.assign(Object.assign({}, parkingLot.toObject()), { designatedSpots });
                    })));
                    return Object.assign(Object.assign({}, spot.toObject()), { status: "reserved", reservedBy: Object.assign(Object.assign({}, reservations[reservationIndex].user.toObject()), { parkingLots: userWithDesignatedSpots }) });
                }
            })));
            let totalCount = 0;
            let totalReservedCount = 0;
            let totalFreeCount = 0;
            let totalElectricCount = 0;
            let totalDesignatedCount = 0;
            let totalRegularCount = 0;
            let totalDisabledCount = 0;
            let designatedCounts = {
                free: 0,
                reserved: 0,
                taken: 0,
                frozen: 0,
                released: 0,
            };
            let electricCounts = {
                free: 0,
                reserved: 0,
                taken: 0,
            };
            let regularCounts = {
                free: 0,
                reserved: 0,
                taken: 0,
            };
            let disabledCounts = {
                free: 0,
                reserved: 0,
                taken: 0,
            };
            const levelsWithSpots = yield Promise.all(levels.map((level) => __awaiter(void 0, void 0, void 0, function* () {
                const spots = finalSpots.filter(spot => String(spot.level) == String(level._id));
                const reserved = spots.filter(spot => {
                    if (spot.spotType == "designated" && spot.status == "reserved")
                        designatedCounts.reserved = designatedCounts.reserved + 1;
                    else if (spot.spotType == "electric" && spot.status == "reserved")
                        electricCounts.reserved = electricCounts.reserved + 1;
                    else if (spot.spotType == "regular" && spot.status == "reserved")
                        regularCounts.reserved = regularCounts.reserved + 1;
                    else if (spot.spotType == "disabled" && spot.status == "reserved")
                        disabledCounts.reserved = disabledCounts.reserved + 1;
                    return spot.status == "reserved";
                });
                const free = spots.filter(spot => {
                    if (spot.spotType == "designated" && spot.status == "free")
                        designatedCounts.free = designatedCounts.free + 1;
                    else if (spot.spotType == "electric" && spot.status == "free")
                        electricCounts.free = electricCounts.free + 1;
                    else if (spot.spotType == "regular" && spot.status == "free")
                        regularCounts.free = regularCounts.free + 1;
                    else if (spot.spotType == "disabled" && spot.status == "free")
                        disabledCounts.free = disabledCounts.free + 1;
                    return spot.status == "free";
                });
                const electric = spots.filter(spot => spot.spotType == "electric");
                const designated = spots.filter(spot => {
                    if (spot.spotType == "designated") {
                        if (spot.isPermanent)
                            designatedCounts.frozen = designatedCounts.frozen + 1;
                        return spot;
                    }
                });
                const designatedReleased = spots.filter(spot => {
                    if (spot.spotType == "designated" && (spot === null || spot === void 0 ? void 0 : spot.status) == "released") {
                        designatedCounts.released = designatedCounts.released + 1;
                        return spot;
                    }
                });
                const regular = spots.filter(spot => spot.spotType == "regular");
                const disabled = spots.filter(spot => spot.spotType == "disabled");
                totalCount = totalCount + spots.length;
                totalReservedCount = totalReservedCount + reserved.length;
                totalFreeCount = totalFreeCount + free.length + (designatedReleased === null || designatedReleased === void 0 ? void 0 : designatedReleased.length);
                totalElectricCount = totalElectricCount + electric.length;
                totalDesignatedCount = totalDesignatedCount + designated.length;
                totalRegularCount = totalRegularCount + regular.length;
                totalDisabledCount = totalDisabledCount + disabled.length;
                return {
                    level: Object.assign(Object.assign({}, level.toObject()), { totalCount: spots.length, reservedCount: reserved.length, electricCount: electric.length, designatedCount: designated.length, regularCount: regular.length, disabledCount: disabled.length, freeCount: free.length + (designatedReleased === null || designatedReleased === void 0 ? void 0 : designatedReleased.length) }),
                    parkingSpots: spots.sort(sortBy == "parkingCode" && sort == "dsc"
                        ? helpers_1.customSortDescending
                        : sortBy == "parkingCode" && sort == "asc"
                            ? helpers_1.customSort
                            : sortBy == "firstName" && sort == "asc"
                                ? helpers_1.customSortNameAscending
                                : sortBy == "firstName" && sort == "dsc"
                                    ? helpers_1.customSortNameDescending
                                    : sortBy == "firstNameReservedBy" && sort == "asc"
                                        ? helpers_1.customSortNameAscending
                                        : sortBy == "firstNameReservedBy" && sort == "dsc"
                                            ? helpers_1.customSortNameDescending
                                            : sortBy == "status" && sort == "asc"
                                                ? helpers_1.customSortStatusAscending
                                                : sortBy == "status" && sort == "dsc"
                                                    ? helpers_1.customSortStatusDescending
                                                    : // sortBy == 'phoneNumber' && sort == 'asc' ? customSortPhoneNumberAscending :
                                                        // sortBy == 'phoneNumber' && sort == 'dsc' ? customSortPhoneNumberDescending :
                                                        sortBy == "phoneNumberDesignated" && sort == "asc"
                                                            ? helpers_1.customSortPhoneNumberDesignatedToAscending
                                                            : sortBy == "phoneNumberDesignated" && sort == "dsc"
                                                                ? helpers_1.customSortPhoneNumberDesignatedToDescending
                                                                : sortBy == "registration" && sort == "asc"
                                                                    ? helpers_1.customSortRegistrationAscending
                                                                    : sortBy == "registration" && sort == "dsc"
                                                                        ? helpers_1.customSortRegistrationDescending
                                                                        : (a, b) => 0),
                };
            })));
            res.status(http_statuses_1.default.OK).json({
                totalCounts: {
                    total: totalCount,
                    reserved: totalReservedCount,
                    free: totalFreeCount,
                    electric: totalElectricCount,
                    designated: totalDesignatedCount,
                    regular: totalRegularCount,
                    disabled: totalDisabledCount,
                },
                counts: {
                    designated: designatedCounts,
                    electric: electricCounts,
                    regular: regularCounts,
                    disabled: disabledCounts,
                },
                levels: levelsWithSpots,
            });
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.parkingLot.PARKING_LOT_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getLotMonitorSpotsService = getLotMonitorSpotsService;
const getLotOccupancyService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { startDate = new Date(), endDate = new Date(), sortBy = "parkingCode", sort = "asc", } = req.query;
        let filter = { parkingLot: _id };
        const parkingLot = yield parking_lot_1.default.findById(_id);
        if (parkingLot) {
            const sorting = sort == "asc" ? 1 : -1;
            const sortingFilter = {};
            if (sortBy == "parkingCode")
                sortingFilter.parkingCode = sorting;
            if (sortBy == "level")
                sortingFilter.name = sorting;
            if (sortBy == "spotType")
                sortingFilter.spotType = sorting;
            const { startOfDay } = (0, dates_1.getStartAndEndOfDay)(startDate);
            const { endOfDay } = (0, dates_1.getStartAndEndOfDay)(endDate);
            const levels = yield lot_level_1.default.find({ parkingLot: _id }).sort(sortBy == "level" ? sortingFilter : {});
            const parkingSpots = yield parking_spot_1.default.find(filter)
                .sort(sortingFilter)
                .populate({
                path: "designatedTo",
                select: ["-password", "-temporaryPassword"],
                populate: [{
                        path: "parkingLots",
                    }, {
                        path: "designatedParkingSpot",
                        populate: "parkingLot",
                    }, {
                        path: "vehicles",
                    },],
            });
            const reservations = yield reservation_1.default.find({
                parkingStatus: { $nin: [controllers_1.reservationEnums[3], controllers_1.reservationEnums[4]] },
                reservationDate: { $gte: startOfDay, $lte: endOfDay },
            }).populate({
                path: "user",
                select: ["-password", "-temporaryPassword"],
                populate: [{
                        path: "parkingLots",
                    }, {
                        path: "designatedParkingSpot",
                        populate: "parkingLot",
                    }, {
                        path: "vehicles",
                    },],
            });
            const finalSpots = yield Promise.all(parkingSpots.map((spot) => __awaiter(void 0, void 0, void 0, function* () {
                var _g, _h, _j, _k, _l;
                const reservationIndex = yield Promise.resolve(reservations.findIndex((reservation) => String(reservation.parkingSpot) == String(spot._id)));
                const absence = yield absence_1.default.findOne({ designatedSpot: spot === null || spot === void 0 ? void 0 : spot._id, startDate: { $lte: startDate }, endDate: { $gte: endDate }, });
                if (reservationIndex < 0) {
                    if (((_g = spot === null || spot === void 0 ? void 0 : spot.designatedTo) === null || _g === void 0 ? void 0 : _g.parkingLots) && ((_j = (_h = spot === null || spot === void 0 ? void 0 : spot.designatedTo) === null || _h === void 0 ? void 0 : _h.parkingLots) === null || _j === void 0 ? void 0 : _j.length) > 0) {
                        const userWithDesignatedSpots = yield Promise.all((_k = spot === null || spot === void 0 ? void 0 : spot.designatedTo) === null || _k === void 0 ? void 0 : _k.parkingLots.map((parkingLot) => __awaiter(void 0, void 0, void 0, function* () {
                            var _m;
                            const designatedSpots = yield parking_spot_1.default.find({ parkingLot: parkingLot._id, spotType: "designated", designatedTo: (_m = spot === null || spot === void 0 ? void 0 : spot.designatedTo) === null || _m === void 0 ? void 0 : _m._id, });
                            return Object.assign(Object.assign({}, parkingLot.toObject()), { designatedSpots });
                        })));
                        const designatedTo = (spot === null || spot === void 0 ? void 0 : spot.designatedTo) ? Object.assign(Object.assign({}, (_l = spot === null || spot === void 0 ? void 0 : spot.designatedTo) === null || _l === void 0 ? void 0 : _l.toObject()), { parkingLots: userWithDesignatedSpots }) : null;
                        return Object.assign(Object.assign({}, spot.toObject()), { status: absence && designatedTo ? "released" : "designated-not-released", designatedTo, reservedBy: null });
                    }
                    else
                        return Object.assign(Object.assign({}, spot.toObject()), { status: "free", reservedBy: null });
                }
                else {
                    const userWithDesignatedSpots = yield Promise.all(reservations[reservationIndex].user.parkingLots.map((parkingLot) => __awaiter(void 0, void 0, void 0, function* () {
                        const designatedSpots = yield parking_spot_1.default.find({ parkingLot: parkingLot._id, spotType: "designated", designatedTo: null, });
                        return Object.assign(Object.assign({}, parkingLot.toObject()), { designatedSpots });
                    })));
                    return Object.assign(Object.assign({}, spot.toObject()), { status: "reserved", reservedBy: Object.assign(Object.assign({}, reservations[reservationIndex].user.toObject()), { parkingLots: userWithDesignatedSpots }) });
                }
            })));
            let totalCount = 0;
            let totalReservedCount = 0;
            let totalFreeCount = 0;
            let totalElectricCount = 0;
            let totalDesignatedCount = 0;
            let totalRegularCount = 0;
            let totalDisabledCount = 0;
            let designatedCounts = {
                free: 0,
                reserved: 0,
                taken: 0,
                frozen: 0,
                released: 0,
            };
            let electricCounts = {
                free: 0,
                reserved: 0,
                taken: 0,
            };
            let regularCounts = {
                free: 0,
                reserved: 0,
                taken: 0,
            };
            let disabledCounts = {
                free: 0,
                reserved: 0,
                taken: 0,
            };
            const levelsWithSpots = yield Promise.all(levels.map((level) => __awaiter(void 0, void 0, void 0, function* () {
                const spots = finalSpots.filter(spot => String(spot.level) == String(level._id));
                const reserved = spots.filter(spot => {
                    if (spot.spotType == "designated" && spot.status == "reserved")
                        designatedCounts.reserved = designatedCounts.reserved + 1;
                    else if (spot.spotType == "electric" && spot.status == "reserved")
                        electricCounts.reserved = electricCounts.reserved + 1;
                    else if (spot.spotType == "regular" && spot.status == "reserved")
                        regularCounts.reserved = regularCounts.reserved + 1;
                    else if (spot.spotType == "disabled" && spot.status == "reserved")
                        disabledCounts.reserved = disabledCounts.reserved + 1;
                    return spot.status == "reserved";
                });
                const free = spots.filter(spot => {
                    if (spot.spotType == "designated" && spot.status == "free")
                        designatedCounts.free = designatedCounts.free + 1;
                    else if (spot.spotType == "electric" && spot.status == "free")
                        electricCounts.free = electricCounts.free + 1;
                    else if (spot.spotType == "regular" && spot.status == "free")
                        regularCounts.free = regularCounts.free + 1;
                    else if (spot.spotType == "disabled" && spot.status == "free")
                        disabledCounts.free = disabledCounts.free + 1;
                    return spot.status == "free";
                });
                const electric = spots.filter(spot => spot.spotType == "electric");
                const designated = spots.filter(spot => {
                    if (spot.spotType == "designated") {
                        if (spot.isPermanent)
                            designatedCounts.frozen = designatedCounts.frozen + 1;
                        return spot;
                    }
                });
                const designatedReleased = spots.filter(spot => {
                    if (spot.spotType == "designated" && (spot === null || spot === void 0 ? void 0 : spot.status) == "released") {
                        designatedCounts.released = designatedCounts.released + 1;
                        return spot;
                    }
                });
                const regular = spots.filter(spot => spot.spotType == "regular");
                const disabled = spots.filter(spot => spot.spotType == "disabled");
                totalCount = totalCount + spots.length;
                totalReservedCount = totalReservedCount + reserved.length;
                totalFreeCount = totalFreeCount + free.length + (designatedReleased === null || designatedReleased === void 0 ? void 0 : designatedReleased.length);
                totalElectricCount = totalElectricCount + electric.length;
                totalDesignatedCount = totalDesignatedCount + designated.length;
                totalRegularCount = totalRegularCount + regular.length;
                totalDisabledCount = totalDisabledCount + disabled.length;
                return [
                    ...spots.map(spot => {
                        const lev = levels.find(level => String(spot.level) == String(level._id));
                        spot.level = Object.assign(Object.assign({}, lev === null || lev === void 0 ? void 0 : lev.toObject()), { totalCount: spots.length, reservedCount: reserved.length, electricCount: electric.length, designatedCount: designated.length, regularCount: regular.length, disabledCount: disabled.length, freeCount: free.length + (designatedReleased === null || designatedReleased === void 0 ? void 0 : designatedReleased.length) });
                        return spot;
                    }),
                ];
            })));
            let finalResponse = null;
            const flatData = levelsWithSpots.flat();
            finalResponse = flatData.sort(sortBy == "parkingCode" && sort == "dsc"
                ? helpers_1.customSortDescending
                : sortBy == "parkingCode" && sort == "asc"
                    ? helpers_1.customSort
                    : sortBy == "firstName" && sort == "asc"
                        ? helpers_1.customSortNameAscending
                        : sortBy == "firstName" && sort == "dsc"
                            ? helpers_1.customSortNameDescending
                            : sortBy == "firstNameReservedBy" && sort == "asc"
                                ? helpers_1.customSortNameReservedByAscending
                                : sortBy == "firstNameReservedBy" && sort == "dsc"
                                    ? helpers_1.customSortNameReservedByDescending
                                    : sortBy == "status" && sort == "asc"
                                        ? helpers_1.customSortStatusAscending
                                        : sortBy == "status" && sort == "dsc"
                                            ? helpers_1.customSortStatusDescending
                                            : // sortBy == 'phoneNumber' && sort == 'asc' ? customSortPhoneNumberAscending :
                                                // sortBy == 'phoneNumber' && sort == 'dsc' ? customSortPhoneNumberDescending :
                                                sortBy == "phoneNumberDesignated" && sort == "asc"
                                                    ? helpers_1.customSortPhoneNumberDesignatedToAscending
                                                    : sortBy == "phoneNumberDesignated" && sort == "dsc"
                                                        ? helpers_1.customSortPhoneNumberDesignatedToDescending
                                                        : sortBy == "registration" && sort == "asc"
                                                            ? helpers_1.customSortRegistrationAscending
                                                            : sortBy == "registration" && sort == "dsc"
                                                                ? helpers_1.customSortRegistrationDescending
                                                                : (a, b) => 0);
            if (sortBy == "spotType" && sort == "asc")
                finalResponse = flatData.sort((a, b) => a.spotType.localeCompare(b.spotType));
            else if (sortBy == "spotType" && sort == "dsc")
                finalResponse = flatData.sort((a, b) => b.spotType.localeCompare(a.spotType));
            else if (sortBy == "level" && sort == "asc")
                finalResponse = flatData.sort((a, b) => a.level.name.localeCompare(b.level.name));
            else if (sortBy == "level" && sort == "asc")
                finalResponse = flatData.sort((a, b) => b.level.name.localeCompare(a.level.name));
            else if (sortBy == "status" && sort == "asc")
                finalResponse = flatData.sort((a, b) => a.status.localeCompare(b.status));
            else if (sortBy == "status" && sort == "dsc")
                finalResponse = flatData.sort((a, b) => b.status.localeCompare(a.status));
            res.status(http_statuses_1.default.OK).json({
                totalCounts: {
                    total: totalCount,
                    reserved: totalReservedCount,
                    free: totalFreeCount,
                    electric: totalElectricCount,
                    designated: totalDesignatedCount,
                    regular: totalRegularCount,
                    disabled: totalDisabledCount,
                },
                counts: {
                    designated: designatedCounts,
                    electric: electricCounts,
                    regular: regularCounts,
                    disabled: disabledCounts,
                },
                parkingSpots: finalResponse || levelsWithSpots.flat(),
            });
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.parkingLot.PARKING_LOT_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getLotOccupancyService = getLotOccupancyService;
const getLotLevelSpotsService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { page = 1, limit = 10, type, isAvailable, isDisabled, isElectric, } = req.query;
        let filter = { level: _id };
        if (type != undefined)
            filter.spotType = type;
        if (type == "designated" && isAvailable == "true")
            filter.designatedTo = null;
        if (type == "designated" && isAvailable == "false")
            filter.designatedTo = { $ne: null };
        if (isDisabled == "false")
            filter.spotType = Object.assign(Object.assign({}, filter.spotType), { $nin: ["disabled"] });
        if (isElectric == "false")
            filter.spotType = Object.assign(Object.assign({}, filter.spotType), { $nin: ["electric"] });
        if (isDisabled == "false" && isElectric == "false")
            filter.spotType = { $nin: ["disabled", "electric"] };
        const level = yield lot_level_1.default.findById(_id);
        if (level) {
            const parkingLot = yield parking_lot_1.default.findById(level.parkingLot);
            if (parkingLot) {
                const sort = { _id: 1 };
                const totalCount = yield parking_spot_1.default.countDocuments(filter);
                const totalPages = Math.ceil(totalCount / limit);
                const parkingSpots = yield parking_spot_1.default.find(filter).sort(sort).limit(limit).skip(limit * (page - 1));
                if (parkingSpots) {
                    res.status(http_statuses_1.default.OK).json({ totalPages, totalCount, parkingSpots });
                }
                else
                    res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.lotLevel.LEVEL_NOT_FOUND });
            }
            else
                res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.parkingLot.PARKING_LOT_NOT_FOUND });
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.lotLevel.LEVEL_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getLotLevelSpotsService = getLotLevelSpotsService;
const getLevelAvailableSpotsService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const level = yield lot_level_1.default.findById(_id);
        if (level) {
            const parkingLot = yield parking_lot_1.default.findById(level.parkingLot);
            if (parkingLot) {
                const { availableSpots, totalCount } = yield (0, helpers_1.findAvailableSpots)(req);
                res.status(http_statuses_1.default.OK).json({ parkingSpots: availableSpots, totalCount });
            }
            else
                res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.parkingLot.PARKING_LOT_NOT_FOUND });
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.lotLevel.LEVEL_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.getLevelAvailableSpotsService = getLevelAvailableSpotsService;
const updateParkingSpotService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const { name, avatar, userPresenceDate, } = req.body;
        const parkingSpot = yield parking_spot_1.default.findByIdAndUpdate(_id, { name, avatar, userPresenceDate }, { omitUndefined: true, new: true });
        if (parkingSpot) {
            let response = { parkingSpot };
            if (userPresenceDate == null) {
                const absence = new absence_1.default({ user: parkingSpot.designatedTo, designatedSpot: parkingSpot._id, startDate: new Date().setUTCHours(0, 0, 0, 0), endDate: new Date().setUTCHours(23, 59, 59, 999), type: "system-generated", });
                const saved = yield absence.save();
                if (saved)
                    response.absence = saved;
            }
            res.status(http_statuses_1.default.OK).json(response);
        }
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.parkingSpot.PARKING_SPOT_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.updateParkingSpotService = updateParkingSpotService;
const deleteParkingSpotService = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    try {
        const { _id } = req.params;
        const parkingSpot = yield parking_spot_1.default.findByIdAndDelete(_id);
        if (parkingSpot)
            res.status(http_statuses_1.default.OK).json({ message: http_messages_1.default.parkingSpot.PARKING_SPOT_DELETED_SUCCESS, });
        else
            res.status(http_statuses_1.default.NOT_FOUND).json({ message: http_messages_1.default.parkingSpot.PARKING_SPOT_NOT_FOUND });
    }
    catch (err) {
        res.status(http_statuses_1.default.INTERNAL_SERVER_ERROR).json({ message: err });
    }
});
exports.deleteParkingSpotService = deleteParkingSpotService;
