import moment from "moment";
import Holiday, { IHoliday } from "../../models/holiday";
export interface ShabbatInfo {
    isShabbat: boolean;
    endDate: {
        minutes: number;
        hours: number;
        day: number;
        month: number;
        year: number;
        date: Date | null;
    };
}
// export const getStartAndEndOfDay = (d: Date): { startOfDay: Date, endOfDay: Date } => {
//     const date = new Date(d);
//     // date.setUTCHours(0, 0, 0, 1);
//     // const startOfDay = date.toISOString();
//     // date.setUTCHours(23, 59, 59, 999);
//     // const endOfDay = date.toISOString();
//     const startOfDay = new Date(date.getFullYear(), date.getMonth(), date.getDate(), 0, 0, 1);
//     const endOfDay = new Date(date.getFullYear(), date.getMonth(), date.getDate(), 23, 59, 59);
//     return { startOfDay, endOfDay };
// }
export const getStartAndEndOfDay = (d: Date): { startOfDay: Date, endOfDay: Date } => {
    const date = new Date(d);
    const year = date.getUTCFullYear();
    const month = date.getUTCMonth();
    const day = date.getUTCDate();
    // Set start of day to midnight in UTC
    const startOfDay = new Date(Date.UTC(year, month, day, 0, 0, 1));
    // Set end of day to one millisecond before midnight in UTC
    const endOfDay = new Date(Date.UTC(year, month, day, 23, 59, 59, 999));
    return { startOfDay, endOfDay };
}

export const getTimeRelation = (startTime: string, timeToMatch: string): 'before' | 'within-2-hours' | 'within-1-hour' | 'within-3-hours' | 'within-4-hours' | 'after' => {
    const [startHour, startMinute] = startTime.split(':').map(Number);
    const [matchHour, matchMinute] = timeToMatch.split(':').map(Number);

    const startTimeInMinutes = startHour * 60 + startMinute;
    const matchTimeInMinutes = matchHour * 60 + matchMinute;
    const difference = startTimeInMinutes - matchTimeInMinutes;
    if (difference >= 0 && difference < 60) {
        return 'within-1-hour';
    } else if (difference > 0 && difference < 120) {
        return 'within-2-hours';
    } else if (difference > 0 && difference < 180) {
        return 'within-3-hours';
    } else if (difference > 0 && difference < 240) {
        return 'within-4-hours';
    } else if (difference < 0) {
        return 'before';
    } else {
        return 'after';
    }
}

export const getTimeDifference = (startTime: string, timeToMatch: string): { minutes: number; formatted: string } => {
    const [startHour, startMinute] = startTime.split(':').map(Number);
    const [matchHour, matchMinute] = timeToMatch.split(':').map(Number);
    const startTimeInMinutes = startHour * 60 + startMinute;
    const matchTimeInMinutes = matchHour * 60 + matchMinute;
    const difference = startTimeInMinutes - matchTimeInMinutes;
    const diffHours = Math.floor(Math.abs(difference) / 60);
    const diffMinutes = Math.abs(difference) % 60;
    const formatted = (difference < 0 ? '-' : '') + String(diffHours).padStart(2, '0') + ':' + String(diffMinutes).padStart(2, '0');
    return { minutes: difference, formatted: formatted };
};

export const addTime = (time1: string, time2: string): string => {
    const [hour1, minute1] = time1.split(':').map(Number);
    const [hour2, minute2] = time2.split(':').map(Number);
    let totalMinutes = (hour1 + hour2) * 60 + minute1 + minute2;
    const hours = (Math.floor(totalMinutes / 60)) % 24; // Ensure hours stay within 24-hour format
    const minutes = totalMinutes % 60;
    const formatted = String(hours).padStart(2, '0') + ':' + String(minutes).padStart(2, '0');
    return formatted;
};
export const isTimeBetween = (time: string, startTime: string, endTime: string): boolean => {
    const [hour, minute] = time.split(':').map(Number);
    const [startHour, startMinute] = startTime.split(':').map(Number);
    const [endHour, endMinute] = endTime.split(':').map(Number);
    // Convert time to minutes for easier comparison
    const timeInMinutes = hour * 60 + minute;
    const startTimeInMinutes = startHour * 60 + startMinute;
    const endTimeInMinutes = endHour * 60 + endMinute;
    // Check if time falls between start and end times
    if (startTimeInMinutes <= endTimeInMinutes) {
        return timeInMinutes >= startTimeInMinutes && timeInMinutes <= endTimeInMinutes;
    } else {
        return timeInMinutes >= startTimeInMinutes || timeInMinutes <= endTimeInMinutes;
    }
};

export const isParkingLotTimeBeforeCurrent = (parkingLotTime: string, currentTime: string): boolean => {
    const [parkingLotHours, parkingLotMinutes] = parkingLotTime.split(':').map(Number);
    const [currentHours, currentMinutes] = currentTime.split(':').map(Number);
    if (parkingLotHours < currentHours || (parkingLotHours === currentHours && parkingLotMinutes < currentMinutes)) {
        return true;
    } else {
        return false;
    }
}

export const getCorrectDateTime = (date: Date): Date => {
    // console.log(date, 'date')
    // console.log(convertToServerLocalTime(date), 'convertToServerLocalTime')
    // const hours = Number(moment(date).format('HH'))
    // const minutes = Number(moment(date).format('mm'))
    // date.setUTCHours(hours, minutes, 0)
    // date.setUTCMilliseconds(0)
    return convertToServerLocalTime(date)
}

export const isJewishHolidayShabbat = async (now: Date): Promise<ShabbatInfo | null> => {
    try {
        const oneHourBefore = new Date(now)
        const date = now
        const sendAfterDate = new Date(date)
        sendAfterDate.setUTCHours(sendAfterDate.getUTCHours() + 1)
        // oneHourBefore.setUTCMinutes(oneHourBefore.getUTCMinutes() - 1)
        oneHourBefore.setUTCHours(oneHourBefore.getUTCHours() - 1)
        const holiday = await Holiday.findOne({ startDate: { $lte: date }, endDate: { $gte: date } })
        // console.log(holiday, 'holiday1')
        if (!holiday) {
            // console.log('isJewishHolidayShabbat=>', 'currentDate:', date, 'oneHourBefore:', oneHourBefore)
            const holiday = await Holiday.findOne({ startDate: { $lte: date }, endDate: { $gt: oneHourBefore } })
            // console.log(holiday, 'holiday2')
            if (holiday) {
                return {
                    isShabbat: true,
                    endDate: {
                        day: sendAfterDate.getUTCDate(),
                        month: sendAfterDate.getUTCMonth() + 1,
                        year: sendAfterDate.getUTCFullYear(),
                        hours: sendAfterDate.getUTCHours(),
                        minutes: sendAfterDate.getUTCMinutes(),
                        date: sendAfterDate,
                    }
                }
            } else return {
                isShabbat: false,
                endDate: {
                    day: 0,
                    month: 0,
                    year: 0,
                    hours: 0,
                    minutes: 0,
                    date: null
                }
            }

        } else return {
            isShabbat: true,
            endDate: {
                day: sendAfterDate.getUTCDate(),
                month: sendAfterDate.getUTCMonth() + 1,
                year: sendAfterDate.getUTCFullYear(),
                hours: sendAfterDate.getUTCHours(),
                minutes: sendAfterDate.getUTCMinutes(),
                date: sendAfterDate,
            }
        }
    } catch (error) {
        console.error('Error finding holiday:', error);
        return null
    }
}

export const getJewishHolidayEndDate = async (now: Date): Promise<null | IHoliday> => {
    const { startOfDay, endOfDay } = getStartAndEndOfDay(now)
    return await Holiday.findOne({ endDate: { $gte: startOfDay, $lte: endOfDay } })
}

export const getTodayTomorrowYesterday = () => {
    const today = getCorrectDateTime(new Date())
    const yesterday = getCorrectDateTime(new Date())
    yesterday.setUTCDate(yesterday.getUTCDate() - 1)
    const tomorrow = getCorrectDateTime(new Date())
    tomorrow.setUTCDate(tomorrow.getUTCDate() + 1)
    return { yesterday, today, tomorrow }
}

export const convertToServerLocalTime = (utcDateString: Date) => {
    const utcDateTime = moment(utcDateString);
    const localDateTime = utcDateTime.local();
    const date = new Date();
    date.setUTCFullYear(localDateTime.year())
    date.setUTCMonth(localDateTime.month())
    date.setUTCDate(localDateTime.date())
    date.setUTCHours(localDateTime.hours())
    date.setUTCMinutes(localDateTime.minute())
    date.setUTCSeconds(localDateTime.second())
    date.setUTCMilliseconds(localDateTime.millisecond())
    return date;
}
