import fs from 'fs'
import { unlink } from 'fs/promises'
import csv from 'csv-parser';

interface IFileDataRow {
  level: number,
  regular_spots: string | any,
  designated_spots: string | any,
  electric_spots: string | any,
  entrance: number,
  map: string,
}

export interface IFileUserRow {
  "﻿email": string,
  email: string,
  phoneNumber: string,
  firstName: string,
  lastName: string,
  dateOfBirth: string,
  electricCharging: boolean | string,
  accessible: boolean | string,
  make: string,
  model: string,
  registration: string,
  mainParkingLot: string,
  allowedParkingLots: string | any,
  designatedSpots: string | any,
  permanentDesignated: string | any,
}

export const fileReader = (filePath: string): Promise<IFileDataRow[]> => {
  return new Promise((resolve, reject) => {
    const data: IFileDataRow[] = []
    fs.createReadStream(filePath)
      .pipe(csv())
      .on('data', (row: IFileDataRow) => {
        row.regular_spots = row.regular_spots.split(',').map((spot: any) => spot.trim());
        row.designated_spots = row.designated_spots.split(',').map((spot: any) => spot.trim());
        row.electric_spots = row.electric_spots.split(',').map((spot: any) => spot.trim());
        data.push(row);
      })
      .on('end', () => {
        resolve(data)
      })
      .on('error', (err: any) => {
        reject(err)
      })
  })
}
export const fileReaderUsers = (filePath: string): Promise<IFileUserRow[]> => {
  return new Promise((resolve, reject) => {
    const data: IFileUserRow[] = []
    fs.createReadStream(filePath)
      .pipe(csv())
      .on('data', (row: IFileUserRow) => {
        // console.log(row.dateOfBirth.replace(/\//g, "-"))
        const dob = new Date()
        // console.log(dob,'dob')
        dob.setUTCHours(12)
        dob.setUTCMinutes(0)
        dob.setUTCSeconds(0)
        row.allowedParkingLots = row.allowedParkingLots.split(',').map((spot: any) => spot.trim()).filter((spot: string) => spot !== '')
        row.designatedSpots = row.designatedSpots.split(',').map((spot: any) => spot.trim()).filter((spot: string) => spot !== '')
        // row.permanentDesignated = row.permanentDesignated.split(',').map((spot: any) => spot.trim()).filter((spot: string) => spot !== '')
        row.dateOfBirth = dob.toISOString()
        row.electricCharging = row.electricCharging == 'FALSE' ? false : true
        row.accessible = row.accessible == 'FALSE' ? false : true
        data.push(row);
      })
      .on('end', () => {
        resolve(data)
      })
      .on('error', (err: any) => {
        reject(err)
      })
  })
}

export const deleteFile = async (filePath: string): Promise<any> => {
  try {
    return await unlink(filePath)
  } catch (err) {
    return err
  }
}