import { Request, Response, NextFunction, } from 'express';
import mongoose from 'mongoose'
import jwt from 'jsonwebtoken'
import HttpMessages from '../../services/constants/http-messages';
import HttpStatus from '../../services/constants/http-statuses';
import Messages from '../../services/constants/messages';
declare const process: {
    env: {
        JWT_SECRET: string,
    }
}
declare module 'express-serve-static-core' {
    interface Request {
        headers: {
            authorization: string
        }
    }
}
interface ICustomRequest extends Request {
    headers: {
        authorization: string
    }
}
export const signJwt = (_id: mongoose.Schema.Types.ObjectId): string => {
    return jwt.sign({ _id }, process.env.JWT_SECRET, { algorithm: 'HS384', expiresIn: "365 days" })
}

export const verifyToken = (req: ICustomRequest, res: Response, next: NextFunction) => {
    try {
        const decoded = jwt.verify(req.headers.authorization?.split('Bearer ')[1], process.env.JWT_SECRET)
        next()
    } catch (err) {
        res.status(HttpStatus.UNAUTHORIZED).json({ message: HttpMessages.validations.INVALID_TOKEN })
    }
}