import { DataMessagePayload } from "firebase-admin/lib/messaging/messaging-api";
import { FirebaseService } from "../../services/firebase";
import Notification from "../../models/notification";
import { Schema, Types } from "mongoose";

const sendNotification = async (deviceToken: string | string[], message: string, notificationType?: string, data?: DataMessagePayload | null, userIds?: Schema.Types.ObjectId[]) => {
    const body = {
        notificationType: notificationType ? notificationType : '',
        ...(data && { ...data }),
    }
    const notification = await FirebaseService.messaging().sendToDevice(deviceToken, {
        data: body,
        notification: {
            title: 'SpotPark',
            body: message,
            sound: 'default',
        },
    });
    const dbNotification = new Notification({
        deviceTokens: deviceToken,
        users: userIds ? userIds : [],
        title: 'SpotPark',
        message,
        body,
        response: notification
    })
    await dbNotification.save()
    return notification
}
export { sendNotification }