import multer from 'multer'
import fs from 'fs'
import path from 'path'
import { mbToBytes, fileFilter, imageFileFilter, videoFilter } from '../../helpers/uploaders';
const imagesStorage = multer.diskStorage({
    destination: (req: Request | any, file: Express.Multer.File, cb: any) => {
        const filePath = path.join(__dirname, "../../../public/images")
        // const filePath = path.join(__dirname, "../../public/images")
        fs.mkdirSync(filePath, { recursive: true })
        cb(null, filePath);
    },
    filename: (req: Request | any, file: Express.Multer.File, cb: any) => {
        const name = `${path.parse(file.originalname).name}-${Date.now()}${Date.now()}${path.extname(file.originalname)}`;
        cb(null, name);
    },
});
const filesStorage = multer.diskStorage({
    destination: (req: Request | any, file: any, cb: any) => {
        const filePath = path.join(__dirname, "../../../public/files")
        // const filePath = path.join(__dirname, "../../public/files")
        fs.mkdirSync(filePath, { recursive: true })
        cb(null, filePath);
    },
    filename: (req: Request | any, file: Express.Multer.File, cb: any) => {
        const name = `${path.parse(file.originalname).name}-${Date.now()}${Date.now()}${path.extname(file.originalname)}`;
        cb(null, name);
    },
});
const videosStorage = multer.diskStorage({
    destination: (req: Request | any, file: Express.Multer.File, cb: any) => {
        const filePath = path.join(__dirname, "../../../public/videos")
        // const filePath = path.join(__dirname, "../../public/videos")
        fs.mkdirSync(filePath, { recursive: true })
        cb(null, filePath);
    },
    filename: (req: Request | any, file: Express.Multer.File, cb: any) => {
        const name = `${path.parse(file.originalname).name}-${Date.now()}${Date.now()}${path.extname(file.originalname)}`;
        cb(null, name);
    },
});
const imageUploader = multer({
    storage: imagesStorage,
    limits: { fileSize: mbToBytes(5) },
    fileFilter: imageFileFilter
});
const fileUploader = multer({
    storage: filesStorage,
    limits: { fileSize: mbToBytes(5) },
    fileFilter,
});
const videoUploader = multer({
    storage: videosStorage,
    limits: { fileSize: mbToBytes(25) },
    fileFilter: videoFilter,
});

export const uploadImages = imageUploader.array('images', 10)

export const uploadFiles = fileUploader.array('files', 10)

export const uploadVideos = videoUploader.array('videos', 10)