import { Schema, Model, model } from "mongoose";

export interface IAbsenceInterface {
    user: Schema.Types.ObjectId,
    designatedSpot: Schema.Types.ObjectId,
    startDate: Date | null,
    endDate: Date | null,
    note: string,
    status: 'completed' | 'pending' | 'canceled',
    type: 'system-generated' | 'user-created'
}

const absenceSchema: Schema<IAbsenceInterface> = new Schema({
    user: {
        type: Schema.Types.ObjectId,
        ref: 'User',
        required: true,
    },
    designatedSpot: {
        type: Schema.Types.ObjectId,
        ref: 'ParkingSpot',
        required: true,
    },
    startDate: {
        type: Date,
        required: true,
        default: null,
    },
    endDate: {
        type: Date,
        required: true,
        default: null,
    },
    note: {
        type: String,
        required: false,
        default: 'לא נחוץ.',
    },
    status: {
        type: String,
        required: false,
        default: 'pending',
        enum: ['pending', 'completed', 'canceled'],
    },
    type: {
        type: String,
        required: false,
        default: 'system-generated',
        enum: ['user-created', 'system-generated'],
    },
}, { timestamps: true })

const Absence: Model<IAbsenceInterface> = model('Absence', absenceSchema)

export default Absence