import { Schema, Model, Types, model } from "mongoose";
export interface IAdminInterface {
    _id: Schema.Types.ObjectId ,
    email: string,
    phoneNumber: {
        countryCode: string,
        number: number,
    },
    password: string,
    avatar: string,
    temporaryPassword: string | null | any,
    firstName: string,
    middleName: string,
    lastName: string,
    failedLoginAttempts: number,
    failedAttemptsDates: Date[],
    passwordUpdatedAt: Date | number,
    temporaryPasswordExpiry: Date,
    parkingLots: Types.ObjectId[],
    role: string,
    accountLockInstances: Date[],
    createdAt?: Date,
    updatedAt?: Date,
}
const adminSchema: Schema<IAdminInterface> = new Schema({
    email: {
        type: String,
        required: true,
        unique: true,
    },
    phoneNumber: {
        countryCode: {
            type: String,
            required: true,
        },
        number: {
            type: String,
            required: true,
        },
    },
    password: {
        type: String,
        required: true,
    },
    avatar: {
        type: String,
        required: false,
        default: null,
    },
    temporaryPassword: {
        type: String,
        required: false,
        default: null,
    },
    firstName: {
        type: String,
        required: true,
        default: "",
    },
    middleName: {
        type: String,
        required: false,
        default: "",
    },
    lastName: {
        type: String,
        required: false,
        default: "",
    },
    failedLoginAttempts: {
        type: Number,
        default: 0,
    },
    failedAttemptsDates: [{
        type: Date,
        default: []
    }],
    passwordUpdatedAt: {
        type: Date,
        default: Date.now,
    },
    temporaryPasswordExpiry: {
        type: Date,
    },
    parkingLots: [{
        type: Schema.Types.ObjectId,
        ref: 'ParkingLot',
        required: true,
        default: [],
    }],
    role: {
        type: String,
        required: true,
        enum: ['admin', 'sub-admin'],
        default: 'admin',
    },
    accountLockInstances: [{
        type: Date,
        default: []
    }],
}, { timestamps: true })

const Admin: Model<IAdminInterface> = model('Admin', adminSchema)

export default Admin