import { Schema, Model, model } from "mongoose";

export interface IAnnouncementInterface {
    parkingLots: Schema.Types.ObjectId,
    startDate: Date,
    endDate: Date,
    title: string,
    content: string,
    status: string,
    readBy: Schema.Types.ObjectId[]
}

const announcementSchema: Schema<IAnnouncementInterface> = new Schema({
    parkingLots: {
        type: [Schema.Types.ObjectId],
        ref: 'ParkingLot',
        required: true,
    },
    startDate: {
        type: Date,
        required: true,
        default: Date.now(),
    },
    endDate: {
        type: Date,
        required: true,
        default: Date.now(),
    },
    title: {
        type: String,
        required: false,
        default: '',
    },
    content: {
        type: String,
        required: true,
    },
    status: {
        type: String,
        required: true,
        default: 'active',
        enum: ['active', 'archived', 'complete',],
    },
    readBy: {
        type: [Schema.Types.ObjectId],
        ref: 'User',
        default: [],
    }
}, { timestamps: true })

const Announcement: Model<IAnnouncementInterface> = model('Announcement', announcementSchema)

export default Announcement