import { Schema, Model, model } from "mongoose";
import { Request, Response } from "express";

export interface ILogSchema {
    user?: Schema.Types.ObjectId,
    parkingLot?: Schema.Types.ObjectId,
    level?: Schema.Types.ObjectId,
    parkingSpot?: Schema.Types.ObjectId,
    reservation?: Schema.Types.ObjectId,
    absence?: Schema.Types.ObjectId,
    announcement?: Schema.Types.ObjectId,
    notification?: Schema.Types.ObjectId,
    request?: object,
    input: object,
    response?: object,
    type: string,
    status: 'error' | 'success',
    statusCode: number,
}

const logSchema: Schema<ILogSchema> = new Schema({
    user: {
        ref: 'User',
        type: Schema.Types.ObjectId
    },
    parkingLot: {
        ref: 'ParkingLot',
        type: Schema.Types.ObjectId
    },
    level: {
        ref: 'Level',
        type: Schema.Types.ObjectId
    },
    parkingSpot: {
        ref: 'ParkingSpot',
        type: Schema.Types.ObjectId
    },
    reservation: {
        ref: 'Reservation',
        type: Schema.Types.ObjectId
    },
    absence: {
        ref: 'Absence',
        type: Schema.Types.ObjectId
    },
    announcement: {
        ref: 'Announcement',
        type: Schema.Types.ObjectId
    },
    notification: {
        ref: 'Notification',
        type: Schema.Types.ObjectId
    },
    request: {
        type: Schema.Types.Mixed
    },
    input: {
        type: Schema.Types.Mixed,
    },
    response: {
        type: Schema.Types.Mixed
    },
    type: {
        type: String,
    },
    status: {
        type: String,
        enum: ['error', 'success'],
        required: true
    },
    statusCode: {
        type: Number,
        required: false,
    },
}, { timestamps: true })

const Log: Model<ILogSchema> = model('Log', logSchema)

export default Log