import { DataMessagePayload } from "firebase-admin/lib/messaging/messaging-api";
import { Schema, Model, model } from "mongoose";

export interface INotificationInterface {
    deviceTokens: string[],
    users: Schema.Types.ObjectId[],
    title: string,
    message: string,
    body: DataMessagePayload | null,
    response: any,
}

const notificationSchema: Schema<INotificationInterface> = new Schema({
    deviceTokens: [{
        type: Schema.Types.String,
        required: false,
        default: []
    }],
    users: [{
        type: Schema.Types.ObjectId,
        ref: 'User',
        required: false,
        default: null,
    }],
    title: {
        type: String,
        default: '',
        required: false,
    },
    message: {
        type: String,
        default: '',
        required: false,
    },
    body: {
        type: {},
        default: null,
        required: false,
    },
    response: {
        type: {},
        default: null,
        required: false,
    },
}, { timestamps: true })

const Notification: Model<INotificationInterface> = model('Notification', notificationSchema)

export default Notification