import { Schema, Model, model } from "mongoose";
export interface IParkingLotInterface {
    name: string;
    chargingTimeLimit: boolean;
    chargingTimeStart: string;
    chargingTimeEnd: string;
    location: { name: string, latitude: number, longitude: number };
    admins: Schema.Types.ObjectId[];
    allotmentMethod: 'random' | 'manual' | 'user-selected';
    allotmentFile: string;
    accessMapFile: string;
    avatar: string;
    maxReservationDistance: number;
    messageSendTime: string;
    absenceSendTime: string;
    messageSendTimeOfDay: 'before-evening' | 'morning';
    reservationTime: 'day-before' | 'same-day';
    smsSendTime: string;
    designatedReservationDuration: string;
    sendPushNotification?: boolean;
}
const parkingLotSchema: Schema<IParkingLotInterface> = new Schema({
    name: {
        type: String,
        required: true,
    },
    chargingTimeLimit: {
        type: Boolean,
        required: true,
    },
    chargingTimeStart: {
        type: String,
        required: false,
        default: null,
    },
    chargingTimeEnd: {
        type: String,
        required: false,
        default: null,
    },
    location: {
        name: {
            type: String,
            required: true,
        },
        latitude: {
            type: Number,
            required: true,
        },
        longitude: {
            type: Number,
            required: true,
        },
    },
    admins: [{
        type: Schema.Types.ObjectId,
        ref: 'Admin',
        required: true,
        default: [],
    }],
    allotmentMethod: {
        type: String,
        required: false,
        enum: ['random', 'manual', 'user-selected'],
        default: 'random',
    },
    allotmentFile: {
        type: String,
        required: false,
        default: null,
    },
    accessMapFile: {
        type: String,
        required: false,
        default: null,
    },
    avatar: {
        type: String,
        required: true,
    },
    maxReservationDistance: {
        type: Number,
        required: true,
        default: 1,
    },
    messageSendTime: {
        type: String,
        required: true,
        default: '05:00',
    },
    absenceSendTime: {
        type: String,
        required: true,
        default: '08:00',
    },
    messageSendTimeOfDay: {
        type: String,
        required: false,
        default: 'morning',
    },
    reservationTime: {
        type: String,
        required: true,
        default: 'same-day',
        enum: ['same-day', 'day-before']
    },
    smsSendTime: {
        type: String,
        required: false,
    },
    designatedReservationDuration: {
        type: String,
        required: true,
        default: '03:00',    // Hours
    },
    sendPushNotification: {
        type: Boolean,
        required: false,
        default: true,
    },
}, { timestamps: true })

const ParkingLot: Model<IParkingLotInterface> = model('ParkingLot', parkingLotSchema)

export default ParkingLot