import { Schema, Model, model } from "mongoose";
export interface IParkingSpotInterface {
    _id: Schema.Types.ObjectId,
    parkingLot: Schema.Types.ObjectId,
    level: Schema.Types.ObjectId,
    spotType: 'regular' | 'designated' | 'electric' | 'disabled',
    designatedTo: Schema.Types.ObjectId,
    name: string,
    avatar: string,
    parkingCode: string,
    userPresenceDate: Date | null,
    outerToSpot: Schema.Types.ObjectId,
    innerToSpot: Schema.Types.ObjectId,
    isPermanent: boolean,
}
const parkingSpotSchema: Schema<IParkingSpotInterface> = new Schema({
    parkingLot: {
        type: Schema.Types.ObjectId,
        ref: 'ParkingLot',
        required: true,
    },
    level: {
        type: Schema.Types.ObjectId,
        ref: 'Level',
        required: true,
    },
    spotType: {
        type: String,
        required: true,
        default: 'regular',
        enum: ['regular', 'designated', 'electric', 'disabled'],
    },
    designatedTo: {
        type: Schema.Types.ObjectId,
        ref: 'User',
        required: false,
        default: null,
    },
    name: {
        type: String,
        required: false,
    },
    avatar: {
        type: String,
        required: false,
    },
    parkingCode: {
        type: String,
        required: true,
    },
    userPresenceDate: {
        type: Date,
        default: null,
        required: false,
    },
    outerToSpot: {
        type: Schema.Types.ObjectId,
        ref: 'ParkingSpot',
        required: false,
        default: null,
    },
    innerToSpot: {
        type: Schema.Types.ObjectId,
        ref: 'ParkingSpot',
        required: false,
        default: null,
    },
    isPermanent: {
        type: Schema.Types.Boolean,
        required: false,
        default: false,
    }
}, { timestamps: true })

const ParkingSpot: Model<IParkingSpotInterface> = model('ParkingSpot', parkingSpotSchema)

export default ParkingSpot