import { IUserInterface } from './user';
import { Schema, Model, model } from "mongoose";
export interface IReservationInterface {
    _id: Schema.Types.ObjectId,
    user: IUserInterface | Schema.Types.ObjectId,
    parkingLot: Schema.Types.ObjectId,
    level: Schema.Types.ObjectId,
    parkingSpot: Schema.Types.ObjectId,
    reservationDate: Date,
    isElectricChargingRequired: boolean,
    isDisabled?: boolean,
    forMoving: boolean,
    parkingStatus: string,
    moveTo: Schema.Types.ObjectId | null,
    moveFrom: Schema.Types.ObjectId | null,
}
const reservationSchema: Schema<IReservationInterface> = new Schema({
    user: {
        type: Schema.Types.ObjectId,
        ref: 'User',
        required: true,
    },
    parkingLot: {
        type: Schema.Types.ObjectId,
        ref: 'ParkingLot',
        required: true
    },
    level: {
        type: Schema.Types.ObjectId,
        ref: 'Level',
        required: true
    },
    parkingSpot: {
        type: Schema.Types.ObjectId,
        ref: 'ParkingSpot',
        required: true,
    },
    reservationDate: {
        type: Date,
        required: true,
    },
    isElectricChargingRequired: {
        type: Boolean,
        required: true,
        default: false,
    },
    isDisabled: {
        type: Boolean,
        required: false,
        default: false,
    },
    forMoving: {
        type: Boolean,
        required: false,
        default: false,
    },
    parkingStatus: {
        type: String,
        required: true,
        default: 'reserved',
        enum: ['reserved', 'occupied', 'moved', 'completed', 'canceled'],
    },
    moveTo: {
        type: Schema.Types.ObjectId,
        ref: 'Reservation',
        required: false,
        default: null,
    },
    moveFrom: {
        type: Schema.Types.ObjectId,
        ref: 'Reservation',
        required: false,
        default: null,
    },
}, { timestamps: true })

const Reservation: Model<IReservationInterface> = model('Reservation', reservationSchema)

export default Reservation