import { Schema, Model, model, Types } from "mongoose";
import { IVehicle, vehiclesSchema } from "./vehicle";
export interface IUserInterface {
    _id: Schema.Types.ObjectId;
    email: string;
    phoneNumber: { countryCode: string; number: string };
    password: string;
    avatar: string;
    temporaryPassword: string | null;
    firstName: string;
    middleName: string;
    lastName: string;
    failedLoginAttempts: number;
    failedAttemptsDates: Date[];
    passwordUpdatedAt: Date | number;
    temporaryPasswordExpiry: Date | number;
    electricCharging: boolean;
    designatedParkingSpot: (Schema.Types.ObjectId | null)[];
    userSpot: string;
    accountLockInstances: Date[];
    parkingLots?: Schema.Types.ObjectId[];
    mainParkingLot?: Schema.Types.ObjectId | null;
    disabled?: boolean;
    deviceToken?: string;
    vehicle: IVehicle;
    dateOfBirth?: Date;
    firstLoginDate?: Date | null;
    vehicles: Types.ObjectId[];
    lastLogin: Date | null;
    isGuest: boolean;
}

const userSchema: Schema<IUserInterface> = new Schema({
    email: {
        type: String,
        required: false,
        unique: false,
    },
    phoneNumber: {
        countryCode: {
            type: String,
            required: true,
        },
        number: {
            type: String,
            required: true,
            unique: true,
        },
    },
    password: {
        type: String,
        required: false,
        default: '',
    },
    avatar: {
        type: String,
        required: false,
        default: null,
    },
    temporaryPassword: {
        type: String,
        required: false,
        default: null,
    },
    firstName: {
        type: String,
        required: true,
    },
    middleName: {
        type: String,
        required: false,
    },
    lastName: {
        type: String,
        required: false,
    },
    failedLoginAttempts: {
        type: Number,
        default: 0,
    },
    failedAttemptsDates: [{
        type: Date,
        default: []
    }],
    passwordUpdatedAt: {
        type: Date,
        default: Date.now,
    },
    temporaryPasswordExpiry: {
        type: Date,
    },
    electricCharging: {
        type: Boolean,
        required: false,
        default: false,
    },
    designatedParkingSpot: [{
        type: Schema.Types.ObjectId,
        ref: 'ParkingSpot',
        required: false,
        default: [],
    }],
    userSpot: {
        type: Schema.Types.String,
        required: false,
        default: null,
    },
    accountLockInstances: [{
        type: Date,
        default: []
    }],
    parkingLots: [{
        type: Schema.Types.ObjectId,
        default: [],
        ref: 'ParkingLot',
    }],
    mainParkingLot: {
        type: Schema.Types.ObjectId,
        default: null,
        ref: 'ParkingLot',
    },
    disabled: {
        type: Schema.Types.Boolean,
        required: false,
        default: false,
    },
    deviceToken: {
        type: Schema.Types.String,
        required: false,
        default: null,
    },
    vehicle: vehiclesSchema,
    dateOfBirth: {
        type: Date,
        required: false,
        default: Date.now()
    },
    firstLoginDate: {
        type: Date,
        required: false,
        default: null
    },
    vehicles: [{
        ref: 'Vehicle',
        type: Schema.Types.ObjectId,
        default: []
    }],
    lastLogin: {
        type: Date,
        required: false,
        default: null,
    },
    isGuest: {
        type: Schema.Types.Boolean,
        required: false,
        default: false,
    }
}, { timestamps: true })

const User: Model<IUserInterface> = model('User', userSchema)

export default User