import express, { Request, Response, Application, } from 'express';
import { verifyToken } from '../../middlewares/authentication';
import { uploadImages, uploadFiles, uploadVideos } from '../../middlewares/uploaders';
import HttpMessages from '../../services/constants/http-messages';
import HttpStatus from '../../services/constants/http-statuses';
import Messages from '../../services/constants/messages';
import User from '../../models/user';
import { sendMessage } from '../../middlewares/email-messaging';
import { sendNotification } from '../../middlewares/notifications';
const app: Application = express()
declare module 'express-serve-static-core' {
    interface Request {
        files: Express.Multer.File[];
    }
}
interface ICustomRequest extends Request {
    files: Express.Multer.File[];
}
const uploadMultipleImages = app.post('/upload-images', [verifyToken, uploadImages], async (req: ICustomRequest, res: Response) => {
    try {
        const urls = await Promise.all(req?.files?.map((file: any) => file.path))
        if (urls.length > 0) res.status(HttpStatus.CREATED).json(urls)
        else res.status(HttpStatus.BAD_REQUEST).json({ message: HttpMessages.upload.IMAGE_UPLOAD_FAILED })
    } catch (err) {
        res.status(HttpStatus.INTERNAL_SERVER_ERROR).json({ message: err })
    }
})

const uploadFilesToServer = app.post('/upload-files', [verifyToken, uploadFiles], async (req: ICustomRequest, res: Response) => {
    try {
        const urls = await Promise.all(req?.files?.map((file: any) => file.path))
        if (urls.length > 0) res.status(HttpStatus.CREATED).json(urls)
        else res.status(HttpStatus.BAD_REQUEST).json({ message: HttpMessages.upload.FILE_UPLOAD_FAILED })
    } catch (err) {
        res.status(HttpStatus.INTERNAL_SERVER_ERROR).json({ message: err })
    }
})

const uploadVideosToServer = app.post('/upload-videos', [verifyToken, uploadVideos], async (req: ICustomRequest, res: Response) => {
    try {
        const urls = await Promise.all(req?.files?.map((file: any) => file.path))
        if (urls.length > 0) res.status(HttpStatus.CREATED).json(urls)
        else res.status(HttpStatus.BAD_REQUEST).json({ message: HttpMessages.upload.VIDEO_UPLOAD_FAILED })
    } catch (err) {
        res.status(HttpStatus.INTERNAL_SERVER_ERROR).json({ message: err })
    }
})

const sendSmsToUser = app.post('/send-message/:_id', verifyToken, async (req: Request, res: Response) => {
    try {
        const { _id } = req.params
        const { message, sendAnyway } = req.body
        if (sendAnyway) {
            const send = await sendMessage(null, message)
            if (send) res.status(HttpStatus.OK).json({ message: HttpMessages.emailSmsNotification.SMS_SENT_SUCCESS, smsSent: send })
            else res.status(HttpStatus.INTERNAL_SERVER_ERROR).json({ message: HttpMessages.emailSmsNotification.SMS_SENT_FAILURE, smsSent: send })
        } else {
            const user = await User.findById(_id).select('phoneNumber')
            if (user) {
                const send = await sendMessage(user.phoneNumber, message)
                if (send) res.status(HttpStatus.OK).json({ message: HttpMessages.emailSmsNotification.SMS_SENT_SUCCESS, smsSent: send })
                else res.status(HttpStatus.INTERNAL_SERVER_ERROR).json({ message: HttpMessages.emailSmsNotification.SMS_SENT_FAILURE, smsSent: send })
            } else res.status(HttpStatus.NOT_FOUND).json({ message: HttpMessages.user.USER_NOT_FOUND })
        }
    } catch (err) {
        res.status(HttpStatus.INTERNAL_SERVER_ERROR).json({ message: err })
    }
})

const sendNotificationToUser = app.post('/send-notification/:_id', verifyToken, async (req: Request, res: Response) => {
    try {
        const { _id } = req.params
        const { message, deviceToken } = req.body
        if (deviceToken) {
            const send = await sendNotification(deviceToken, message, '', { reservation: 'asd' })
            if (send) res.status(HttpStatus.OK).json({ message: HttpMessages.emailSmsNotification.NOTIFICATION_SENT_SUCCESS, notification: send })
            else res.status(HttpStatus.INTERNAL_SERVER_ERROR).json({ message: HttpMessages.emailSmsNotification.NOTIFICATION_SENT_FAILURE, smsSent: send })
        } else {
            const user = await User.findById(_id).select(['deviceToken'])
            if (user && user.deviceToken) {
                const send = await sendNotification(user.deviceToken, message, '', { reservation: 'asd' })
                if (send) res.status(HttpStatus.OK).json({ message: HttpMessages.emailSmsNotification.NOTIFICATION_SENT_SUCCESS, smsSent: send })
                else res.status(HttpStatus.INTERNAL_SERVER_ERROR).json({ message: HttpMessages.emailSmsNotification.NOTIFICATION_SENT_FAILURE, smsSent: send })
            } else res.status(HttpStatus.NOT_FOUND).json({ message: HttpMessages.user.USER_NOT_FOUND })
        }
    } catch (err) {
        res.status(HttpStatus.INTERNAL_SERVER_ERROR).json({ message: err })
    }
})

export {
    uploadMultipleImages,
    uploadFilesToServer,
    uploadVideosToServer,
    sendNotificationToUser,
    sendSmsToUser,
}