import { Request, Response, } from 'express';
import HttpMessages from '../../services/constants/http-messages';
import HttpStatus from '../../services/constants/http-statuses';
import Holiday from '../../models/holiday';

export const createHolidayService = async (req: Request, res: Response) => {
    try {
        const { name, startDate, endDate, startTime, endTime }: { name?: string, startDate: Date, endDate: Date, startTime: string, endTime: string } = req.body;
        if (startDate && endDate && startTime && endTime) {
            const holiday = new Holiday({
                name,
                startDate,
                endDate,
                startTime,
                endTime
            })
            await holiday.save()
            res.status(HttpStatus.CREATED).json(holiday)
        } else res.status(HttpStatus.BAD_REQUEST).json({ message: HttpMessages.holiday.HOLIDAY_DETAILS_MISSING })
    } catch (err) {
        res.status(HttpStatus.INTERNAL_SERVER_ERROR).json({ message: err })
    }
}

export const getAllHolidaysService = async (req: Request, res: Response) => {
    try {
        const holidays = await Holiday.find().sort({ startDate: 1, })
        res.status(HttpStatus.OK).json(holidays);
    } catch (err) {
        res.status(HttpStatus.INTERNAL_SERVER_ERROR).json({ message: err })
    }
}

export const getSingleHolidayService = async (req: Request, res: Response) => {
    try {
        const { _id }: { _id?: string } = req.params;
        const holiday = await Holiday.findById(_id);
        if (holiday) res.status(HttpStatus.OK).json(holiday);
        else res.status(HttpStatus.NOT_FOUND).json({ message: HttpMessages.complaint.COMPLAINT_NOT_FOUND })
    } catch (err) {
        res.status(HttpStatus.INTERNAL_SERVER_ERROR).json({ message: err })
    }
}

export const updateHolidayService = async (req: Request, res: Response) => {
    try {
        const { _id }: { _id?: string } = req.params
        const { name, startDate, endDate, startTime, endTime }: { name?: string, startDate?: Date, endDate?: Date, startTime?: string, endTime?: string } = req.body;
        const holiday = await Holiday.findByIdAndUpdate(_id, {
            name,
            startDate,
            endDate,
            startTime,
            endTime
        }, { new: true, omitUndefined: true })
        if (holiday) res.status(HttpStatus.OK).json(holiday)
        else res.status(HttpStatus.NOT_FOUND).json({ message: HttpMessages.holiday.HOLIDAY_NOT_FOUND })
    } catch (err) {
        res.status(HttpStatus.INTERNAL_SERVER_ERROR).json({ message: err })
    }
}

export const deleteHolidayService = async (req: Request, res: Response) => {
    try {
        const { _id }: { _id?: string } = req.params;
        const holiday = await Holiday.findByIdAndDelete(_id);
        if (holiday) res.status(HttpStatus.OK).json({ message: HttpMessages.holiday.HOLIDAY_DELETED_SUCCESS });
        else res.status(HttpStatus.NOT_FOUND).json({ message: HttpMessages.holiday.HOLIDAY_NOT_FOUND })
    } catch (err) {
        res.status(HttpStatus.INTERNAL_SERVER_ERROR).json({ message: err })
    }
}