import express, { Application, } from 'express';
import { verifyToken } from '../../middlewares/authentication';
import { createMultipleUsersService, createUserService, deleteAllUsersService, deleteUserService, forgotPasswordService, getAdminLotsUsersService, getAllUsersService, getSingleUserService, getUsersReportService, loginUserService, loginWithTemporaryPasswordService, logoutUserService, resetPasswordService, unlockUserAccountService, updateUserService } from './services';
const app: Application = express()

export const createUser = app.post('/', verifyToken, createUserService)

export const createMultipleUsers = app.post('/create-multiple', verifyToken, createMultipleUsersService)

export const getSingleUser = app.get('/user/:_id', verifyToken, getSingleUserService)

export const getAdminLotsUsers = app.get('/admin-users/:_id', verifyToken, getAdminLotsUsersService)

export const getAllUsers = app.get('/', verifyToken, getAllUsersService)

export const getUsersReport = app.get('/report', verifyToken, getUsersReportService)

export const updateUser = app.put('/:_id', verifyToken, updateUserService)

export const loginUser = app.post('/login', loginUserService)

export const logoutUser = app.patch('/logout/:_id', logoutUserService)

export const forgotPassword = app.post('/forgot-password', forgotPasswordService)

export const loginWithTemporaryPassword = app.post('/login-temporary-password', loginWithTemporaryPasswordService)

export const resetPassword = app.patch('/reset-password', resetPasswordService)

export const unlockUserAccount = app.patch('/unlock-account/:_id', verifyToken, unlockUserAccountService)

export const deleteAllUsers = app.delete('/', verifyToken, deleteAllUsersService)

export const deleteUser = app.delete('/:_id', verifyToken, deleteUserService)