import express, { Application, } from 'express';
import { verifyToken } from '../../middlewares/authentication';
import { changeReservedSpotService, createReservationService, deleteReservationService, getAllReservationsService, getLevelReservationsService, getParkingLotReservationsService, getSingleReservationService, getSpotAvailabilityService, getUserReservationsService, updateReservationService } from './services';
const app: Application = express()
export const reservationEnums = ['reserved', 'occupied', 'moved', 'completed', 'canceled']

export const createReservation = app.post('/', verifyToken, createReservationService)

export const getAllReservations = app.get('/', verifyToken, getAllReservationsService)

export const getUserReservations = app.get('/user/:_id', verifyToken, getUserReservationsService)

export const getParkingLotReservations = app.get('/lot/:_id', verifyToken, getParkingLotReservationsService)

export const getLevelReservations = app.get('/level/:_id', verifyToken, getLevelReservationsService)

export const getSpotAvailability = app.get('/spot-availability/:_id', verifyToken, getSpotAvailabilityService)

export const getSingleReservation = app.get('/:_id', verifyToken, getSingleReservationService)

export const updateReservation = app.patch('/:_id', verifyToken, updateReservationService)

export const changeReservedSpot = app.put('/change-spot/:_id', verifyToken, changeReservedSpotService)

export const deleteReservation = app.delete('/:_id', verifyToken, deleteReservationService)