
import { Request, Response, } from 'express';
import HttpMessages from '../../services/constants/http-messages';
import HttpStatus from '../../services/constants/http-statuses';
import { generateLevelCode, } from '../../helpers/generators'
import ParkingLot from '../../models/parking-lot';
import Level, { ILevelInterface } from '../../models/lot-level';
import ParkingSpot from '../../models/parking-spot';
import Reservation from '../../models/reservation';
import { reservationEnums } from '../reservations/controllers';
import { findAvailableSpots } from '../../helpers';
import { compareOldAndNewParkingCodes, extractNumberFromParkingCode, parkingCodesOverLapCheck, separateDoubleAndSingleSpots } from '../../helpers/validators';
const createCustomSpotsHelper = async (req: Request, res: Response, lot: any, createNewLevel?: boolean): Promise<{ newLevel: any, parkingSpots: any[] }> => {
    return new Promise(async (resolve, reject) => {
        try {
            const { _id }: { _id?: string } = req.params
            const {
                parkingLot,
                level,
                name,
                map,
                entrance,
                regularParkingSpots,
                designatedParkingSpots,
                electricParkingSpots,
                disabledParkingSpots,
                doubleParkingSpots,
                frozenParkingSpots,
                language = 'heb'
            }: {
                parkingLot: string,
                level: number,
                name: string,
                map: string,
                entrance: number,
                regularParkingSpots: string,
                designatedParkingSpots: string,
                electricParkingSpots: string,
                disabledParkingSpots: string,
                doubleParkingSpots: string,
                frozenParkingSpots: string,
                language: string
            } = req.body
            const newLevel = new Level({
                parkingLot,
                name,
                level,
                map,
                entrance,
            })
            const levelCode = generateLevelCode(level)
            let regularSpotNumbers: string[] = []
            let designatedSpotNumbers: string[] = []
            let electricSpotNumbers: string[] = []
            let disabledSpotNumbers: string[] = []
            let doubleSpotNumbers: string[] = []
            let frozenSpotNumbers: string[] = []
            if (regularParkingSpots) regularSpotNumbers = String(regularParkingSpots.match(/\d+/g)).split(',').filter(num => num !== '')
            if (designatedParkingSpots) designatedSpotNumbers = String(designatedParkingSpots.match(/\d+/g)).split(',').filter(num => num !== '')
            if (electricParkingSpots) electricSpotNumbers = String(electricParkingSpots.match(/\d+/g)).split(',').filter(num => num !== '')
            if (disabledParkingSpots) disabledSpotNumbers = String(disabledParkingSpots.match(/\d+/g)).split(',').filter(num => num !== '')
            if (doubleParkingSpots) doubleSpotNumbers = String(doubleParkingSpots.match(/\d+/g)).split(',').filter(num => num !== '')
            if (frozenParkingSpots) frozenSpotNumbers = String(frozenParkingSpots.match(/\d+/g)).split(',').filter(num => num !== '')
            if (parkingCodesOverLapCheck(regularSpotNumbers, designatedSpotNumbers, electricSpotNumbers, disabledSpotNumbers))
                res.status(HttpStatus.BAD_REQUEST).json({ message: language == 'heb' ? HttpMessages.parkingSpot.FAILURE_CREATING_DUPLICATE_VALUES_HEB : HttpMessages.parkingSpot.FAILURE_CREATING_DUPLICATE_VALUES })
            else {
                const createdRegular = await Promise.all(regularSpotNumbers.map(async (spotNumber) => {
                    if (doubleSpotNumbers.includes(spotNumber)) {
                        const parkingSpotA = new ParkingSpot({
                            parkingLot: lot._id,
                            level: createNewLevel ? newLevel._id : _id,
                            spotType: 'regular',
                            parkingCode: `${levelCode}${spotNumber}-A`,
                            isPermanent: false,
                        });
                        const parkingSpotB = new ParkingSpot({
                            parkingLot: lot._id,
                            level: createNewLevel ? newLevel._id : _id,
                            spotType: 'regular',
                            parkingCode: `${levelCode}${spotNumber}-B`,
                            outerToSpot: parkingSpotA._id,
                            isPermanent: false,
                        });
                        parkingSpotA.innerToSpot = parkingSpotB._id
                        return [parkingSpotA, parkingSpotB]
                    } else {
                        const parkingSpot = new ParkingSpot({
                            parkingLot: lot._id,
                            level: createNewLevel ? newLevel._id : _id,
                            spotType: 'regular',
                            parkingCode: `${levelCode}${spotNumber}`,
                            isPermanent: false,
                        });
                        return parkingSpot
                    }
                }))
                const createdDesignated = await Promise.all(designatedSpotNumbers.map(async (spotNumber) => {
                    const index = await Promise.resolve(createdRegular.flat().findIndex(item => item?.parkingCode == `${levelCode}${spotNumber}`))
                    if (index < 0) {
                        if (doubleSpotNumbers.includes(spotNumber)) {
                            const parkingSpotA = new ParkingSpot({
                                parkingLot: lot._id,
                                level: createNewLevel ? newLevel._id : _id,
                                spotType: 'designated',
                                parkingCode: `${levelCode}${spotNumber}-A`,
                                isPermanent: frozenSpotNumbers.includes(spotNumber) ? true : false,
                            });
                            const parkingSpotB = new ParkingSpot({
                                parkingLot: lot._id,
                                level: createNewLevel ? newLevel._id : _id,
                                spotType: 'designated',
                                parkingCode: `${levelCode}${spotNumber}-B`,
                                outerToSpot: parkingSpotA._id,
                                isPermanent: frozenSpotNumbers.includes(spotNumber) ? true : false,
                            });
                            parkingSpotA.innerToSpot = parkingSpotB._id
                            return [parkingSpotA, parkingSpotB]
                        }
                        else {
                            const parkingSpot = new ParkingSpot({
                                parkingLot: lot._id,
                                level: createNewLevel ? newLevel._id : _id,
                                spotType: 'designated',
                                parkingCode: `${levelCode}${spotNumber}`,
                                isPermanent: frozenSpotNumbers.includes(spotNumber) ? true : false,
                            });
                            return parkingSpot
                        }
                    }
                }))
                const createdElectric = await Promise.all(electricSpotNumbers.map(async (spotNumber) => {
                    const index = await Promise.resolve(createdDesignated.flat().findIndex(item => item?.parkingCode == `${levelCode}${spotNumber}`))
                    const _index = await Promise.resolve(createdRegular.flat().findIndex(item => item?.parkingCode == `${levelCode}${spotNumber}`))
                    if (index < 0 && _index < 0) {
                        if (doubleSpotNumbers.includes(spotNumber)) {
                            const parkingSpotA = new ParkingSpot({
                                parkingLot: lot._id,
                                level: createNewLevel ? newLevel._id : _id,
                                spotType: 'electric',
                                parkingCode: `${levelCode}${spotNumber}-A`,
                                isPermanent: false,
                            });
                            const parkingSpotB = new ParkingSpot({
                                parkingLot: lot._id,
                                level: createNewLevel ? newLevel._id : _id,
                                spotType: 'electric',
                                parkingCode: `${levelCode}${spotNumber}-B`,
                                outerToSpot: parkingSpotA._id,
                                isPermanent: false,
                            });
                            parkingSpotA.innerToSpot = parkingSpotB._id
                            return [parkingSpotA, parkingSpotB]
                        }
                        else {
                            const parkingSpot = new ParkingSpot({
                                parkingLot: lot._id,
                                level: createNewLevel ? newLevel._id : _id,
                                spotType: 'electric',
                                parkingCode: `${levelCode}${spotNumber}`,
                            });
                            return parkingSpot
                        }
                    }
                }))
                const createdDisabled = await Promise.all(disabledSpotNumbers.map(async (spotNumber) => {
                    const index = await Promise.resolve(createdDesignated.flat().findIndex(item => item?.parkingCode == `${levelCode}${spotNumber}`))
                    const _index = await Promise.resolve(createdRegular.flat().findIndex(item => item?.parkingCode == `${levelCode}${spotNumber}`))
                    if (index < 0 && _index < 0) {
                        if (index < 0 && _index < 0) {
                            if (doubleSpotNumbers.includes(spotNumber)) {
                                const parkingSpotA = new ParkingSpot({
                                    parkingLot: lot._id,
                                    level: createNewLevel ? newLevel._id : _id,
                                    spotType: 'disabled',
                                    parkingCode: `${levelCode}${spotNumber}-A`,
                                    isPermanent: false,
                                });
                                const parkingSpotB = new ParkingSpot({
                                    parkingLot: lot._id,
                                    level: createNewLevel ? newLevel._id : _id,
                                    spotType: 'disabled',
                                    parkingCode: `${levelCode}${spotNumber}-B`,
                                    outerToSpot: parkingSpotA._id,
                                    isPermanent: false,
                                });
                                parkingSpotA.innerToSpot = parkingSpotB._id
                                return [parkingSpotA, parkingSpotB]
                            }
                            else {
                                const parkingSpot = new ParkingSpot({
                                    parkingLot: lot._id,
                                    level: createNewLevel ? newLevel._id : _id,
                                    spotType: 'disabled',
                                    parkingCode: `${levelCode}${spotNumber}`,
                                    isPermanent: false,
                                });
                                return parkingSpot
                            }
                        }
                    }
                }))
                const nullValues = [...createdRegular, ...createdElectric, ...createdDesignated, ...createdDisabled].some(value => value == null)
                if (nullValues) res.status(HttpStatus.BAD_REQUEST).json({ message: language == 'heb' ? HttpMessages.parkingSpot.FAILURE_CREATING_DUPLICATE_VALUES_HEB : HttpMessages.parkingSpot.FAILURE_CREATING_DUPLICATE_VALUES })
                else {
                    if (createNewLevel) await newLevel.save()
                    if (createNewLevel != undefined && createNewLevel == false) {
                        // if (regularSpotNumbers.length > 0) await ParkingSpot.deleteMany({ level: _id, spotType: 'regular' })
                        // if (designatedSpotNumbers.length > 0) await ParkingSpot.deleteMany({ level: _id, spotType: 'designated' })
                        // if (electricSpotNumbers.length > 0) await ParkingSpot.deleteMany({ level: _id, spotType: 'electric' })
                        // if (disabledSpotNumbers.length > 0) await ParkingSpot.deleteMany({ level: _id, spotType: 'disabled' })
                        // if (doubleSpotNumbers.length > 0) await ParkingSpot.deleteMany({ level: _id, spotType: 'double' })
                        await ParkingSpot.deleteMany({ level: _id, spotType: 'regular' })
                        await ParkingSpot.deleteMany({ level: _id, spotType: 'designated' })
                        await ParkingSpot.deleteMany({ level: _id, spotType: 'electric' })
                        await ParkingSpot.deleteMany({ level: _id, spotType: 'disabled' })
                        await ParkingSpot.deleteMany({ level: _id, spotType: 'double' })
                    }
                    const parkingSpots = await Promise.all([...createdRegular.flat(), ...createdElectric.flat(), ...createdDesignated.flat(), ...createdDisabled.flat()].filter(async (spot) => {
                        if (spot != null) {
                            setTimeout(async () => {
                                if (Array.isArray(spot)) {
                                    await Promise.all(spot.map(async (s) => await s.save()))
                                    return spot
                                } else {
                                    await spot.save()
                                    return spot
                                }
                            }, 100);
                        }
                    }))
                    resolve({ newLevel, parkingSpots })
                }
            }
        } catch (err) {
            reject(err)
        }
    })
}

const processUpdateSpots = async (req: Request, res: Response, existingLevel: ILevelInterface): Promise<{ newLevel: any, parkingSpots: any[] }> => {
    return new Promise(async (resolve, reject) => {
        try {
            const { _id }: { _id?: string } = req.params
            const {
                parkingLot,
                level,
                name,
                map,
                entrance,
                regularParkingSpots,
                designatedParkingSpots,
                electricParkingSpots,
                disabledParkingSpots,
                doubleParkingSpots,
                frozenParkingSpots,
                language = 'heb',
            }: {
                parkingLot: string,
                level: number,
                name: string,
                map: string,
                entrance: number,
                regularParkingSpots: string,
                designatedParkingSpots: string,
                electricParkingSpots: string,
                disabledParkingSpots: string,
                doubleParkingSpots: string,
                frozenParkingSpots: string,
                language: string,
            } = req.body
            const levelCode = generateLevelCode(level)

            let regularSpotNumbers: string[] = []
            let designatedSpotNumbers: string[] = []
            let electricSpotNumbers: string[] = []
            let disabledSpotNumbers: string[] = []
            let doubleSpotNumbers: string[] = []
            let frozenSpotNumbers: string[] = []

            let regularSpotCodes: any[] = []
            let designatedSpotCodes: string[] = []
            let electricSpotCodes: string[] = []
            let disabledSpotCodes: string[] = []
            let frozenSpotCodes: string[] = []

            if (regularParkingSpots) regularSpotNumbers = String(regularParkingSpots.match(/\d+/g)).split(',').filter(num => num !== '')
            if (designatedParkingSpots) designatedSpotNumbers = String(designatedParkingSpots.match(/\d+/g)).split(',').filter(num => num !== '')
            if (electricParkingSpots) electricSpotNumbers = String(electricParkingSpots.match(/\d+/g)).split(',').filter(num => num !== '')
            if (disabledParkingSpots) disabledSpotNumbers = String(disabledParkingSpots.match(/\d+/g)).split(',').filter(num => num !== '')
            if (doubleParkingSpots) doubleSpotNumbers = String(doubleParkingSpots.match(/\d+/g)).split(',').filter(num => num !== '')
            if (frozenParkingSpots) frozenSpotNumbers = String(frozenParkingSpots.match(/\d+/g)).split(',').filter(num => num !== '')

            if (parkingCodesOverLapCheck(regularSpotNumbers, designatedSpotNumbers, electricSpotNumbers, disabledSpotNumbers))
                res.status(HttpStatus.BAD_REQUEST).json({ message: language == 'heb' ? HttpMessages.parkingSpot.FAILURE_UPDATING_DUPLICATE_VALUES_HEB : HttpMessages.parkingSpot.FAILURE_UPDATING_DUPLICATE_VALUES })
            else {
                regularSpotNumbers.map((num) => {
                    if (doubleSpotNumbers.includes(num)) {
                        regularSpotCodes.push(`${levelCode}${num}-A`)
                        regularSpotCodes.push(`${levelCode}${num}-B`)
                    } else regularSpotCodes.push(`${levelCode}${num}`)
                })
                designatedSpotNumbers.map(num => {
                    if (doubleSpotNumbers.includes(num)) {
                        designatedSpotCodes.push(`${levelCode}${num}-A`)
                        designatedSpotCodes.push(`${levelCode}${num}-B`)
                    } else designatedSpotCodes.push(`${levelCode}${num}`)
                })
                electricSpotNumbers.map(num => {
                    if (doubleSpotNumbers.includes(num)) {
                        electricSpotCodes.push(`${levelCode}${num}-A`)
                        electricSpotCodes.push(`${levelCode}${num}-B`)
                    } else electricSpotCodes.push(`${levelCode}${num}`)
                })
                disabledSpotNumbers.map(num => {
                    if (doubleSpotNumbers.includes(num)) {
                        disabledSpotCodes.push(`${levelCode}${num}-A`)
                        disabledSpotCodes.push(`${levelCode}${num}-B`)
                    } else disabledSpotCodes.push(`${levelCode}${num}`)
                })
                frozenSpotNumbers.map(num => {
                    if (doubleSpotNumbers.includes(num)) {
                        frozenSpotCodes.push(`${levelCode}${num}-A`)
                        frozenSpotCodes.push(`${levelCode}${num}-B`)
                    } else frozenSpotCodes.push(`${levelCode}${num}`)
                })

                const existingRegularString = (await ParkingSpot.find({ level: existingLevel?._id, spotType: 'regular', }).sort({ parkingCode: 1 }).select('parkingCode')).map(spot => spot.parkingCode)
                const existingDesignatedString = (await ParkingSpot.find({ level: existingLevel?._id, spotType: 'designated', }).sort({ parkingCode: 1 }).select('parkingCode')).map(spot => spot.parkingCode)
                const existingElectricString = (await ParkingSpot.find({ level: existingLevel?._id, spotType: 'electric', }).sort({ parkingCode: 1 }).select('parkingCode')).map(spot => spot.parkingCode)
                const existingDisabledString = (await ParkingSpot.find({ level: existingLevel?._id, spotType: 'disabled', }).sort({ parkingCode: 1 }).select('parkingCode')).map(spot => spot.parkingCode)
                const existingFrozenString = (await ParkingSpot.find({ level: existingLevel?._id, spotType: 'designated', isPermanent: true, }).sort({ parkingCode: 1 }).select('parkingCode')).map(spot => spot.parkingCode)

                const regularStatus = compareOldAndNewParkingCodes(existingRegularString, regularSpotCodes)
                const designatedStatus = compareOldAndNewParkingCodes(existingDesignatedString, designatedSpotCodes)
                const electricStatus = compareOldAndNewParkingCodes(existingElectricString, electricSpotCodes)
                const disabledStatus = compareOldAndNewParkingCodes(existingDisabledString, disabledSpotCodes)
                const frozenStatus = compareOldAndNewParkingCodes(existingFrozenString, frozenSpotCodes)

                const createdRegular: any[] = []
                const createdDesignated: any[] = []
                const createdElectric: any[] = []
                const createdDisabled: any[] = []

                if (regularStatus) {
                    if (regularStatus.removed.length > 0) {
                        await Promise.all(regularStatus.removed.map(async (parkingCode) => {
                            const deletedSpot = await ParkingSpot.findOneAndDelete({ level: existingLevel?._id, spotType: 'regular', parkingCode, })
                            if (deletedSpot) {
                                const filter = { parkingSpot: deletedSpot._id, parkingStatus: { $in: [reservationEnums[0], reservationEnums[1], reservationEnums[2]] } }
                                const deletedReservations = await Reservation.deleteMany(filter)
                            }
                        }))
                    }
                    if (regularStatus.added.length > 0) {
                        const { singleSpots, doubleSpots } = separateDoubleAndSingleSpots(regularStatus.added)
                        if (singleSpots.length > 0) {
                            await Promise.all(singleSpots.map(async (parkingCode) => {
                                const parkingSpot = new ParkingSpot({
                                    parkingLot: parkingLot,
                                    level: existingLevel._id,
                                    spotType: 'regular',
                                    parkingCode: parkingCode,
                                    isPermanent: false,
                                })
                                createdRegular.push(parkingSpot)
                            }))
                        }
                        if (doubleSpots.length > 0) {
                            await Promise.all(doubleSpots.map(async (spots) => {
                                const parkingSpotA = new ParkingSpot({
                                    parkingLot: parkingLot,
                                    level: existingLevel._id,
                                    spotType: 'regular',
                                    parkingCode: spots[0],
                                    isPermanent: false,
                                });
                                const parkingSpotB = new ParkingSpot({
                                    parkingLot: parkingLot,
                                    level: existingLevel._id,
                                    spotType: 'regular',
                                    parkingCode: spots[1],
                                    outerToSpot: parkingSpotA._id,
                                    isPermanent: false,
                                });
                                parkingSpotA.innerToSpot = parkingSpotB._id
                                createdRegular.push(parkingSpotA)
                                createdRegular.push(parkingSpotB)
                            }))
                        }
                    }
                }
                if (designatedStatus || frozenStatus) {
                    const existingRegularString = (await ParkingSpot.find({ level: existingLevel?._id, spotType: 'regular', }).sort({ parkingCode: 1 }).select('parkingCode')).map(spot => spot.parkingCode)
                    if (designatedStatus.removed.length > 0 || frozenStatus.removed.length > 0) {
                        await Promise.all(designatedStatus.removed.map(async (parkingCode) => {
                            const deletedSpot = await ParkingSpot.findOneAndDelete({ level: existingLevel?._id, spotType: 'designated', parkingCode, })
                            if (deletedSpot) {
                                const deletedReservations = await Reservation.deleteMany({ parkingSpot: deletedSpot._id, parkingStatus: { $in: [reservationEnums[0], reservationEnums[1], reservationEnums[2]] } })
                            }
                        }))
                        await Promise.all(frozenStatus.removed.map(async (parkingCode) => {
                            const updatedSpot = await ParkingSpot.findOneAndUpdate({
                                level: existingLevel?._id,
                                spotType: 'designated',
                                parkingCode,
                                isPermanent: true
                            }, { isPermanent: false })
                            if (updatedSpot) {
                                const deletedReservations = await Reservation.deleteMany({ parkingSpot: updatedSpot._id, parkingStatus: { $in: [reservationEnums[0], reservationEnums[1], reservationEnums[2]] } })
                            }
                        }))
                    }
                    if (designatedStatus.added.length > 0 || frozenStatus.added.length > 0) {
                        if (designatedStatus.added.length > 0) {
                            const { singleSpots, doubleSpots } = separateDoubleAndSingleSpots(designatedStatus.added)
                            if (singleSpots.length > 0) {
                                await Promise.all(singleSpots.map(async (parkingCode) => {
                                    const index = await Promise.resolve(createdRegular.findIndex(item => item?.parkingCode == parkingCode))
                                    const regularExistenceCheck = !existingRegularString.includes(parkingCode)
                                    const isPermanent = frozenSpotCodes.includes(parkingCode) ? true : false
                                    if (index < 0 && regularExistenceCheck) {
                                        const parkingSpot = new ParkingSpot({
                                            parkingLot: parkingLot,
                                            level: existingLevel._id,
                                            spotType: 'designated',
                                            parkingCode: parkingCode,
                                            isPermanent,
                                        })
                                        createdDesignated.push(parkingSpot)
                                    } else createdDesignated.push(null)
                                }))
                            }
                            if (doubleSpots.length > 0) {
                                await Promise.all(doubleSpots.map(async (spots) => {
                                    const index = await Promise.resolve(createdRegular.findIndex(item => item?.parkingCode == spots[0] || item?.parkingCode == spots[1]))
                                    const regularExistenceCheck = !existingRegularString.includes(spots[0]) || !existingRegularString.includes(spots[1])
                                    const isPermanent = frozenSpotCodes.includes(spots[0]) || frozenSpotCodes.includes(spots[1]) ? true : false
                                    if (index < 0 && regularExistenceCheck) {
                                        const parkingSpotA = new ParkingSpot({
                                            parkingLot: parkingLot,
                                            level: existingLevel._id,
                                            spotType: 'designated',
                                            parkingCode: spots[0],
                                            isPermanent,
                                        });
                                        const parkingSpotB = new ParkingSpot({
                                            parkingLot: parkingLot,
                                            level: existingLevel._id,
                                            spotType: 'designated',
                                            parkingCode: spots[1],
                                            outerToSpot: parkingSpotA._id,
                                            isPermanent,
                                        });
                                        parkingSpotA.innerToSpot = parkingSpotB._id
                                        createdDesignated.push(parkingSpotA)
                                        createdDesignated.push(parkingSpotB)
                                    } else createdDesignated.push(null)
                                }))
                            }
                        }
                        if (frozenStatus.added.length > 0) {
                            const { singleSpots, doubleSpots } = separateDoubleAndSingleSpots(frozenStatus.added)
                            if (singleSpots.length > 0) {
                                await Promise.all(singleSpots.map(async (parkingCode) => {
                                    const index = await Promise.resolve(createdRegular.findIndex(item => item?.parkingCode == parkingCode))
                                    const regularExistenceCheck = !existingRegularString.includes(parkingCode)
                                    if (index < 0 && regularExistenceCheck) {
                                        const isPermanent = frozenSpotCodes.includes(parkingCode) ? true : false
                                        const update = await ParkingSpot.findOne({
                                            parkingLot: parkingLot,
                                            level: existingLevel._id,
                                            spotType: 'designated',
                                            parkingCode: parkingCode,
                                        })
                                        if (update) {
                                            update.isPermanent = isPermanent
                                            createdDesignated.push(update)
                                        }
                                        else {
                                            const findInCreated = await Promise.resolve(createdDesignated.findIndex(item => item?.parkingCode == parkingCode))
                                            if (findInCreated >= 0) createdDesignated[findInCreated].set({ isPermanent })
                                            else {
                                                const parkingSpot = new ParkingSpot({
                                                    parkingLot: parkingLot,
                                                    level: existingLevel._id,
                                                    spotType: 'designated',
                                                    parkingCode: parkingCode,
                                                    isPermanent: isPermanent,
                                                })
                                                createdDesignated.push(parkingSpot)
                                            }
                                        }
                                    } else createdDesignated.push(null)
                                }))
                            }
                            if (doubleSpots.length > 0) {
                                await Promise.all(doubleSpots.map(async (spots) => {
                                    const index = await Promise.resolve(createdRegular.findIndex(item => item?.parkingCode == spots[0] || item?.parkingCode == spots[1]))
                                    const regularExistenceCheck = !existingRegularString.includes(spots[0]) || !existingRegularString.includes(spots[1])
                                    if (index < 0 && regularExistenceCheck) {
                                        const isPermanent = frozenSpotCodes.includes(spots[0]) || frozenSpotCodes.includes(spots[1]) ? true : false
                                        const updateA = await ParkingSpot.findOne({
                                            parkingLot: parkingLot,
                                            level: existingLevel._id,
                                            spotType: 'designated',
                                            parkingCode: spots[0],
                                        })
                                        if (updateA) {
                                            updateA.isPermanent = isPermanent

                                            const updateB = await ParkingSpot.findOne({
                                                parkingLot: parkingLot,
                                                level: existingLevel._id,
                                                spotType: 'designated',
                                                parkingCode: spots[1],
                                            })
                                            if (updateB) {
                                                updateB.isPermanent = isPermanent
                                                createdDesignated.push(updateA)
                                                createdDesignated.push(updateB)
                                            }
                                        } else {
                                            const findAInCreated = await Promise.resolve(createdDesignated.findIndex(item => item?.parkingCode == spots[0]))
                                            if (findAInCreated >= 0) createdDesignated[findAInCreated].set({ isPermanent })
                                            const findBInCreated = await Promise.resolve(createdDesignated.findIndex(item => item?.parkingCode == spots[1]))
                                            if (findBInCreated >= 0) createdDesignated[findBInCreated].set({ isPermanent })
                                            if (findAInCreated < 0 && findBInCreated < 0) {
                                                const parkingSpotA = new ParkingSpot({
                                                    parkingLot: parkingLot,
                                                    level: existingLevel._id,
                                                    spotType: 'designated',
                                                    parkingCode: spots[0],
                                                    isPermanent: frozenSpotNumbers.includes(extractNumberFromParkingCode(spots[0])) ? true : false,
                                                });
                                                const parkingSpotB = new ParkingSpot({
                                                    parkingLot: parkingLot,
                                                    level: existingLevel._id,
                                                    spotType: 'designated',
                                                    parkingCode: spots[1],
                                                    outerToSpot: parkingSpotA._id,
                                                    isPermanent: frozenSpotNumbers.includes(extractNumberFromParkingCode(spots[1])) ? true : false,
                                                });
                                                parkingSpotA.innerToSpot = parkingSpotB._id
                                                createdDesignated.push(parkingSpotA)
                                                createdDesignated.push(parkingSpotB)
                                            }
                                        }
                                    } else createdDesignated.push(null)
                                }))
                            }
                        }
                    }
                }
                if (electricStatus) {
                    const existingRegularString = (await ParkingSpot.find({ level: existingLevel?._id, spotType: 'regular', }).sort({ parkingCode: 1 }).select('parkingCode')).map(spot => spot.parkingCode)
                    const existingDesignatedString = (await ParkingSpot.find({ level: existingLevel?._id, spotType: 'designated', }).sort({ parkingCode: 1 }).select('parkingCode')).map(spot => spot.parkingCode)
                    if (electricStatus.removed.length > 0) {
                        await Promise.all(electricStatus.removed.map(async (parkingCode) => {
                            const deletedSpot = await ParkingSpot.findOneAndDelete({ level: existingLevel?._id, spotType: 'electric', parkingCode, })
                            if (deletedSpot) {
                                const deletedReservations = await Reservation.deleteMany({ parkingSpot: deletedSpot._id, parkingStatus: { $in: [reservationEnums[0], reservationEnums[1], reservationEnums[2]] } })
                            }
                        }))
                    }
                    if (electricStatus.added.length > 0) {
                        const { singleSpots, doubleSpots } = separateDoubleAndSingleSpots(electricStatus.added)
                        if (singleSpots.length > 0) {
                            await Promise.all(singleSpots.map(async (parkingCode) => {
                                const index = await Promise.resolve(createdRegular.findIndex(item => item?.parkingCode == parkingCode))
                                const regularExistenceCheck = !existingRegularString.includes(parkingCode)
                                const designatedExistenceCheck = !existingDesignatedString.includes(parkingCode)
                                if (index < 0 && regularExistenceCheck && designatedExistenceCheck) {
                                    const parkingSpot = new ParkingSpot({
                                        parkingLot: parkingLot,
                                        level: existingLevel._id,
                                        spotType: 'electric',
                                        parkingCode: parkingCode,
                                        isPermanent: false,
                                    })
                                    createdElectric.push(parkingSpot)
                                } else createdElectric.push(null)
                            }))
                        }
                        if (doubleSpots.length > 0) {
                            await Promise.all(doubleSpots.map(async (spots) => {
                                const index = await Promise.resolve(createdRegular.findIndex(item => item?.parkingCode == spots[0] || item?.parkingCode == spots[1]))
                                const regularExistenceCheck = !existingRegularString.includes(spots[0]) || !existingRegularString.includes(spots[1])
                                const designatedExistenceCheck = !existingDesignatedString.includes(spots[0]) || !existingDesignatedString.includes(spots[1])
                                if (index < 0 && regularExistenceCheck && designatedExistenceCheck) {
                                    const parkingSpotA = new ParkingSpot({
                                        parkingLot: parkingLot,
                                        level: existingLevel._id,
                                        spotType: 'electric',
                                        parkingCode: spots[0],
                                        isPermanent: false,
                                    });
                                    const parkingSpotB = new ParkingSpot({
                                        parkingLot: parkingLot,
                                        level: existingLevel._id,
                                        spotType: 'electric',
                                        parkingCode: spots[1],
                                        outerToSpot: parkingSpotA._id,
                                        isPermanent: false,
                                    });
                                    parkingSpotA.innerToSpot = parkingSpotB._id
                                    createdElectric.push(parkingSpotA)
                                    createdElectric.push(parkingSpotB)
                                } else createdElectric.push(null)
                            }))
                        }
                    }
                }
                if (disabledStatus) {
                    const existingRegularString = (await ParkingSpot.find({ level: existingLevel?._id, spotType: 'regular', }).sort({ parkingCode: 1 }).select('parkingCode')).map(spot => spot.parkingCode)
                    const existingDesignatedString = (await ParkingSpot.find({ level: existingLevel?._id, spotType: 'designated', }).sort({ parkingCode: 1 }).select('parkingCode')).map(spot => spot.parkingCode)
                    const existingElectricString = (await ParkingSpot.find({ level: existingLevel?._id, spotType: 'electric', }).sort({ parkingCode: 1 }).select('parkingCode')).map(spot => spot.parkingCode)
                    if (disabledStatus.removed.length > 0) {
                        await Promise.all(disabledStatus.removed.map(async (parkingCode) => {
                            const deletedSpot = await ParkingSpot.findOneAndDelete({ level: existingLevel?._id, spotType: 'electric', parkingCode, })
                            if (deletedSpot) {
                                const deletedReservations = await Reservation.deleteMany({ parkingSpot: deletedSpot._id, parkingStatus: { $in: [reservationEnums[0], reservationEnums[1], reservationEnums[2]] } })
                            }
                        }))
                    }
                    if (disabledStatus.added.length > 0) {
                        const { singleSpots, doubleSpots } = separateDoubleAndSingleSpots(disabledStatus.added)
                        if (singleSpots.length > 0) {
                            await Promise.all(singleSpots.map(async (parkingCode) => {
                                const index = await Promise.resolve(createdRegular.findIndex(item => item?.parkingCode == parkingCode))
                                const regularExistenceCheck = !existingRegularString.includes(parkingCode)
                                const designatedExistenceCheck = !existingDesignatedString.includes(parkingCode)
                                const electricExistenceCheck = !existingElectricString.includes(parkingCode)
                                if (index < 0 && regularExistenceCheck && designatedExistenceCheck && electricExistenceCheck) {
                                    const parkingSpot = new ParkingSpot({
                                        parkingLot: parkingLot,
                                        level: existingLevel._id,
                                        spotType: 'disabled',
                                        parkingCode: parkingCode,
                                        isPermanent: false,
                                    })
                                    createdDisabled.push(parkingSpot)
                                } else createdDisabled.push(null)
                            }))
                        }
                        if (doubleSpots.length > 0) {
                            await Promise.all(doubleSpots.map(async (spots) => {
                                const index = await Promise.resolve(createdRegular.findIndex(item => item?.parkingCode == spots[0] || item?.parkingCode == spots[1]))
                                const regularExistenceCheck = !existingRegularString.includes(spots[0]) || !existingRegularString.includes(spots[1])
                                const designatedExistenceCheck = !existingDesignatedString.includes(spots[0]) || !existingDesignatedString.includes(spots[1])
                                const electricExistenceCheck = !existingElectricString.includes(spots[0]) || !existingElectricString.includes(spots[1])
                                if (index < 0 && regularExistenceCheck && designatedExistenceCheck) {
                                    const parkingSpotA = new ParkingSpot({
                                        parkingLot: parkingLot,
                                        level: existingLevel._id,
                                        spotType: 'disabled',
                                        parkingCode: spots[0],
                                        isPermanent: false,
                                    });
                                    const parkingSpotB = new ParkingSpot({
                                        parkingLot: parkingLot,
                                        level: existingLevel._id,
                                        spotType: 'disabled',
                                        parkingCode: spots[1],
                                        outerToSpot: parkingSpotA._id,
                                        isPermanent: false,
                                    });
                                    parkingSpotA.innerToSpot = parkingSpotB._id
                                    createdDisabled.push(parkingSpotA)
                                    createdDisabled.push(parkingSpotB)
                                } else createdDisabled.push(null)
                            }))
                        }
                    }
                }

                const nullValues = [...createdRegular, ...createdDesignated, ...createdElectric, ...createdDisabled].some(value => value == null)
                if (nullValues) res.status(HttpStatus.BAD_REQUEST).json({ message: language == 'heb' ? HttpMessages.parkingSpot.FAILURE_UPDATING_DUPLICATE_VALUES_HEB : HttpMessages.parkingSpot.FAILURE_UPDATING_DUPLICATE_VALUES })
                else {
                    const parkingSpots = await Promise.all([...createdRegular, ...createdDesignated, ...createdElectric, ...createdDisabled].filter(async (spot) => {
                        if (spot != null) {
                            setTimeout(async () => {
                                if (Array.isArray(spot)) {
                                    await Promise.all(spot.map(async (s) => await s.save()))
                                    return spot
                                } else {
                                    await spot.save()
                                    return spot
                                }
                            }, 100);
                        }
                    }))
                    resolve({ newLevel: null, parkingSpots })
                }
            }
        } catch (err) {
            reject(err)
        }
    })
}

const updateCustomSpotsHelper = async (req: Request, res: Response, lot?: any): Promise<{ newLevel: any, parkingSpots: any[] }> => {
    return new Promise(async (resolve, reject) => {
        try {
            const { _id, }: { _id?: string, } = req.params
            const { parkingLot, level, name, entrance, map }: { parkingLot?: string, level?: number, name?: string, entrance?: string, map?: string } = req.body
            const foundLevel = await Level.findByIdAndUpdate(_id, { parkingLot, level, name, entrance, map }, { new: true, omitUndefined: true })
            if (foundLevel) {
                // const { parkingSpots } = await createCustomSpotsHelper(req, res, lot, false)
                const { parkingSpots } = await processUpdateSpots(req, res, foundLevel)
                res.status(HttpStatus.OK).json(parkingSpots)
            } else res.status(HttpStatus.NOT_FOUND).json({ message: HttpMessages.lotLevel.LEVEL_NOT_FOUND })
        } catch (err) {
            reject(err)
        }
    })
}

export const createLotLevelService = async (req: Request, res: Response) => {
    try {
        const { parkingLot, level, map, entrance, name }: { parkingLot: string, level: number, map: string, entrance: number, name: string } = req.body
        if (parkingLot && level && map && entrance && name) {
            const lot = await ParkingLot.findById(parkingLot)
            if (lot) {
                const existingLevel = await Level.findOne({ parkingLot, level })
                if (existingLevel) {
                    res.status(HttpStatus.FORBIDDEN).json({ message: HttpMessages.lotLevel.LEVEL_ALREADY_EXIST + `Level: ${level}.` })
                } else {
                    const { newLevel, parkingSpots } = await createCustomSpotsHelper(req, res, lot, true)
                    res.status(HttpStatus.CREATED).json({ level: newLevel, parkingSpots })
                }
            } else return res.status(HttpStatus.NOT_FOUND).json({ messages: HttpMessages.parkingLot.PARKING_LOT_NOT_FOUND })
        } else res.status(HttpStatus.BAD_REQUEST).json({ messages: HttpMessages.lotLevel.LEVEL_DETAILS_MISSING })
    } catch (err) {
        res.status(HttpStatus.INTERNAL_SERVER_ERROR).json({ messages: err })
    }
}
export const getAllLotLevelsService = async (req: Request, res: Response) => {
    try {
        const { page = 1, limit = 10, }: { page?: number, limit?: number } = req.query
        const totalCount = await Level.countDocuments();
        const totalPages = Math.ceil(totalCount / limit);
        const lotLevels = await Level.find().sort({ updatedAt: -1 }).limit(limit).skip(limit * (page - 1))
        if (lotLevels.length > 0) {
            const counts = await Promise.all(lotLevels.map(async (lotLevel) => {
                const regular = await ParkingSpot.countDocuments({ level: lotLevel._id, spotType: 'regular' })
                const designated = await ParkingSpot.countDocuments({ level: lotLevel._id, spotType: 'designated' })
                const electric = await ParkingSpot.countDocuments({ level: lotLevel._id, spotType: 'electric' })
                return {
                    ...lotLevel.toObject(),
                    regular,
                    designated,
                    electric,
                }
            }))
            res.status(HttpStatus.OK).json({ totalPages, totalCount, levels: counts })
        } else res.status(HttpStatus.OK).json({ totalPages, totalCount, levels: lotLevels })

    } catch (err) {
        res.status(HttpStatus.INTERNAL_SERVER_ERROR).json({ message: err })
    }
}
export const getSingleLevelService = async (req: Request, res: Response) => {
    try {
        const { _id }: { _id?: string } = req.params
        const level = await Level.findById(_id)
        if (level) res.status(HttpStatus.OK).json(level)
        else res.status(HttpStatus.NOT_FOUND).json({ message: HttpMessages.lotLevel.LEVEL_NOT_FOUND })
    } catch (err) {
        res.status(HttpStatus.INTERNAL_SERVER_ERROR).json({ message: err })
    }
}
export const getParkingLotLevelsService = async (req: Request, res: Response) => {
    try {
        const { _id }: { _id?: string } = req.params
        const { page = 1, limit = 10, getSpots = true, sort = 'dsc', }: { page?: number, limit?: number, getSpots?: boolean | string, sort?: string } = req.query
        const levels = await Level.find({ parkingLot: _id }).sort({ level: sort == 'asc' ? 1 : -1 }).limit(limit).skip(limit * (page - 1))
        const totalCount = await Level.countDocuments({ parkingLot: _id });
        const totalPages = Math.ceil(totalCount / limit);
        if (getSpots == true || getSpots == 'true') {
            const sort: {} = { _id: 1 }
            const allLevels = await Promise.all(levels.map(async (level) => {
                const regular = await ParkingSpot.countDocuments({ level, spotType: 'regular', outerToSpot: null })
                const regularString = (await ParkingSpot.find({ level, spotType: 'regular', outerToSpot: null }).sort(sort).select('parkingCode')).map(spot => spot.parkingCode?.split('-')[0].trim()).join(',')
                const designated = await ParkingSpot.countDocuments({ level, spotType: 'designated', outerToSpot: null })
                const designatedString = (await ParkingSpot.find({ level, spotType: 'designated', outerToSpot: null }).sort(sort).select('parkingCode')).map(spot => spot.parkingCode?.split('-')[0].trim()).join(',')
                const electric = await ParkingSpot.countDocuments({ level, spotType: 'electric', outerToSpot: null })
                const electricString = (await ParkingSpot.find({ level, spotType: 'electric', outerToSpot: null }).sort(sort).select('parkingCode')).map(spot => spot.parkingCode?.split('-')[0].trim()).join(',')
                const disabled = await ParkingSpot.countDocuments({ level, spotType: 'disabled', outerToSpot: null })
                const disabledString = (await ParkingSpot.find({ level, spotType: 'disabled', outerToSpot: null }).sort(sort).select('parkingCode')).map(spot => spot.parkingCode?.split('-')[0].trim()).join(',')
                const double = await ParkingSpot.countDocuments({ level, outerToSpot: { $ne: null } })
                const doubleString = (await ParkingSpot.find({ level, outerToSpot: { $ne: null } }).sort(sort).select('parkingCode')).map(spot => spot.parkingCode?.split('-')[0].trim()).join(',')
                const frozen = await ParkingSpot.countDocuments({ level, spotType: 'designated', isPermanent: true, outerToSpot: null })
                const frozenString = (await ParkingSpot.find({ level, spotType: 'designated', isPermanent: true, outerToSpot: null }).sort(sort).select('parkingCode')).map(spot => spot.parkingCode?.split('-')[0].trim()).join(',')
                return {
                    ...level.toObject(),
                    regular,
                    regularString,
                    designated,
                    designatedString,
                    electric,
                    electricString,
                    disabled,
                    disabledString,
                    double,
                    doubleString,
                    frozen,
                    frozenString,
                }
            }))
            res.status(HttpStatus.OK).json({ totalPages, totalCount, levels: allLevels })
        } else res.status(HttpStatus.OK).json({ totalPages, totalCount, levels })
    } catch (err) {
        res.status(HttpStatus.INTERNAL_SERVER_ERROR).json({ message: err })
    }
}
export const getParkingLotLevelAvailableSpotsService = async (req: Request, res: Response) => {
    try {
        const { _id }: { _id?: string } = req.params
        const { page = 1, limit = 10, sort = 'dsc', }: { page?: number, limit?: number, sort?: string } = req.query
        const levels = await Level.find({ parkingLot: _id })
            .sort({ level: sort == 'asc' ? 1 : -1 })
            .limit(limit)
            .skip(limit * (page - 1))
        const levelCounts = await Promise.all(levels.map(async (level) => {
            const { totalCount } = await findAvailableSpots(req, level._id)
            return {
                ...level.toObject(),
                count: totalCount,
            }
        }))
        res.status(HttpStatus.OK).json(levelCounts)
    } catch (err) {
        res.status(HttpStatus.INTERNAL_SERVER_ERROR).json({ message: err })
    }
}
export const updateLotLevelService = async (req: Request, res: Response) => {
    try {
        const { _id }: { _id?: string } = req.params
        const { parkingLot, level, }: { parkingLot: string, level: number, } = req.body
        const existingLevel = await Level.findOne({ parkingLot, level })
        if (existingLevel && existingLevel?._id != _id) res.status(HttpStatus.FORBIDDEN).json({ message: HttpMessages.lotLevel.LEVEL_ALREADY_EXIST })
        else {
            // const levelReservations = await Reservation.countDocuments({ level: _id, parkingStatus: { $in: [reservationEnums[0], reservationEnums[1], reservationEnums[2]] } })
            // const levelDesignated = await ParkingSpot.find({ level: _id, designatedTo: { $ne: null } })
            // if (levelReservations > 0) {
            // await Reservation.deleteMany({ level: _id, parkingStatus: { $in: [reservationEnums[0], reservationEnums[1], reservationEnums[2]] } })
            // }
            // if (levelDesignated.length > 0) {
            //     const designatedIds: Schema.Types.ObjectId[] = []
            //     const designatedUserIds: Schema.Types.ObjectId[] = []
            //     await Promise.all(levelDesignated.map((parkingSpot) => {
            //         designatedIds.push(parkingSpot._id)
            //         designatedUserIds.push(parkingSpot.designatedTo)
            //     }))
            //     await User.updateMany(
            //         { _id: { $in: designatedUserIds } }, // Filter condition
            //         { $pull: { designatedParkingSpot: { $in: designatedIds } } } // Update operation
            //     );
            //     await ParkingSpot.updateMany({ level: _id, designatedTo: { $ne: null } }, { designatedTo: null })
            // }
            const foundLot = await ParkingLot.findById(parkingLot)
            const { parkingSpots, } = await updateCustomSpotsHelper(req, res, foundLot)
            res.status(HttpStatus.OK).json(parkingSpots)
        }
    } catch (err) {
        res.status(HttpStatus.INTERNAL_SERVER_ERROR).json({ message: err })
    }
}
export const deleteLevelService = async (req: Request, res: Response) => {
    try {
        const { _id }: { _id?: string } = req.params
        const levelReservations = await Reservation.countDocuments({ level: _id, parkingStatus: { $in: [reservationEnums[0], reservationEnums[1]] } })
        if (levelReservations > 0) res.status(HttpStatus.FORBIDDEN).json({ message: HttpMessages.lotLevel.DELETE_FAILURE_REASON_UNCOMPLETED_RESERVATION })
        else {
            const level = await Level.findByIdAndDelete(_id)
            if (level) {
                await ParkingSpot.deleteMany({ level: level._id })
                res.status(HttpStatus.OK).json({ message: HttpMessages.lotLevel.LEVEL_DELETED_SUCCESS })
            } else {
                res.status(HttpStatus.NOT_FOUND).json({ message: HttpMessages.lotLevel.LEVEL_NOT_FOUND })
            }
        }
    } catch (err) {
        res.status(HttpStatus.INTERNAL_SERVER_ERROR).json({ message: err })
    }
}