import { Request, Response, } from 'express';
import HttpMessages from '../../services/constants/http-messages';
import HttpStatus from '../../services/constants/http-statuses';
import MessageTime from '../../models/message-time';
export const createMessageTimeService = async (req: Request, res: Response) => {
    try {
        const { label, value, labelHeb, timeOfDay }: { label: string, value: string, labelHeb: string, timeOfDay: string } = req.body;
        if (label && value && labelHeb && timeOfDay) {
            const messageTime = new MessageTime({
                label,
                value,
                labelHeb,
                timeOfDay
            })
            await messageTime.save()
            res.status(HttpStatus.CREATED).json(messageTime)
        } else res.status(HttpStatus.BAD_REQUEST).json({ message: HttpMessages.complaint.COMPLAINT_DETAILS_MISSING })
    } catch (err) {
        res.status(HttpStatus.INTERNAL_SERVER_ERROR).json({ message: err })
    }
}

export const getAllMessageTimesService = async (req: Request, res: Response) => {
    try {
        const messageTimes = await MessageTime.find().sort({ value: -1, })
        res.status(HttpStatus.OK).json(messageTimes);
    } catch (err) {
        res.status(HttpStatus.INTERNAL_SERVER_ERROR).json({ message: err })
    }
}

export const getSingleMessageTimeService = async (req: Request, res: Response) => {
    try {
        const { _id }: { _id?: string } = req.params;
        const messageTime = await MessageTime.findById(_id);
        if (messageTime) res.status(HttpStatus.OK).json(messageTime);
        else res.status(HttpStatus.NOT_FOUND).json({ message: HttpMessages.complaint.COMPLAINT_NOT_FOUND })
    } catch (err) {
        res.status(HttpStatus.INTERNAL_SERVER_ERROR).json({ message: err })
    }
}

export const updateMessageTimeService = async (req: Request, res: Response) => {
    try {
        const { _id }: { _id?: string } = req.params
        const { label, value, labelHeb, timeOfDay }: { label: string, value: string, labelHeb: string, timeOfDay: string } = req.body;
        if (label && value && labelHeb && timeOfDay) {
            const messageTime = await MessageTime.findByIdAndUpdate(_id, { label, value, labelHeb, timeOfDay }, { new: true, })
            if (messageTime) res.status(HttpStatus.OK).json(messageTime)
            else res.status(HttpStatus.NOT_FOUND).json({ message: 'Message time not found' })
        } else res.status(HttpStatus.BAD_REQUEST).json({ message: HttpMessages.complaint.COMPLAINT_DETAILS_MISSING })
    } catch (err) {
        res.status(HttpStatus.INTERNAL_SERVER_ERROR).json({ message: err })
    }
}

export const deleteMessageTimeService = async (req: Request, res: Response) => {
    try {
        const { _id }: { _id?: string } = req.params;
        const messageTime = await MessageTime.findByIdAndDelete(_id);
        if (messageTime) res.status(HttpStatus.OK).json({ message: 'Message time Deleted Successfully' });
        else res.status(HttpStatus.NOT_FOUND).json({ message: 'Message time not found' })
    } catch (err) {
        res.status(HttpStatus.INTERNAL_SERVER_ERROR).json({ message: err })
    }
}